Package: gridpattern
Type: Package
Title: 'grid' Pattern Grobs
Version: 1.4.2
Authors@R: c(
        person("Trevor L.", "Davis", role=c("aut", "cre"), email="trevor.l.davis@gmail.com",
               comment = c(ORCID = "0000-0001-6341-4639")),
        person("Mike", "FC", role = "aut", comment = "Code/docs adapted from ggpattern"),
        person("ggplot2 authors", role = "ctb", comment = "some utility functions copied from ggplot2"))
Description: Provides 'grid' grobs that fill in a user-defined area with various patterns.  Includes enhanced versions of the geometric and image-based patterns originally contained in the 'ggpattern' package as well as original 'pch', 'polygon_tiling', 'regular_polygon', 'rose', 'text', 'wave', and 'weave' patterns plus support for custom user-defined patterns.
URL: https://trevorldavis.com/R/gridpattern/,
        https://github.com/trevorld/gridpattern
BugReports: https://github.com/trevorld/gridpattern/issues
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.4.0)
Imports: glue, grDevices, grid, memoise, png, rlang, sf, utils
Suggests: ambient, aqp, aRtsy, ggplot2 (>= 3.5.0), gtable, knitr,
        magick (>= 2.7.4), ragg (>= 1.2.0), rmarkdown, scales, svglite
        (>= 2.1.0), testthat, Unicode, vdiffr (>= 1.0.6)
VignetteBuilder: knitr, rmarkdown
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-22 22:48:11 UTC; trevor
Author: Trevor L. Davis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>),
  Mike FC [aut] (Code/docs adapted from ggpattern),
  ggplot2 authors [ctb] (some utility functions copied from ggplot2)
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-23 10:00:20 UTC

# Generated by roxygen2: do not edit by hand

S3method(makeContent,alpha_mask)
S3method(makeContent,clipping_path)
S3method(makeContent,pattern)
S3method(print,pattern_hex)
S3method(print,pattern_square)
S3method(print,pattern_weave)
S3method(update_pattern_alpha,GridPattern)
S3method(update_pattern_alpha,GridTilingPattern)
S3method(update_pattern_alpha,default)
S3method(update_pattern_alpha,list)
export(alphaMaskGrob)
export(clippingPathGrob)
export(grid.pattern)
export(grid.pattern_aRtsy)
export(grid.pattern_ambient)
export(grid.pattern_circle)
export(grid.pattern_crosshatch)
export(grid.pattern_fill)
export(grid.pattern_gradient)
export(grid.pattern_image)
export(grid.pattern_magick)
export(grid.pattern_none)
export(grid.pattern_pch)
export(grid.pattern_placeholder)
export(grid.pattern_plasma)
export(grid.pattern_polygon_tiling)
export(grid.pattern_regular_polygon)
export(grid.pattern_rose)
export(grid.pattern_stripe)
export(grid.pattern_text)
export(grid.pattern_wave)
export(grid.pattern_weave)
export(guess_has_R4.1_features)
export(mean_col)
export(names_aRtsy)
export(names_hex)
export(names_magick)
export(names_magick_intensity)
export(names_magick_stripe)
export(names_pattern)
export(names_placeholder)
export(names_polygon_tiling)
export(names_square)
export(names_weave)
export(patternFill)
export(patternGrob)
export(pattern_hex)
export(pattern_square)
export(pattern_weave)
export(reset_image_cache)
export(star_angle)
export(star_scale)
export(update_alpha)
import(grid)
importFrom(glue,glue)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.capabilities)
importFrom(grDevices,dev.capture)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(rlang,"%||%")
importFrom(rlang,abort)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)

# gridpattern <img src="man/figures/logo.png" align="right" width="200px" alt="gridpattern hex sticker">

[![CRAN Status Badge](https://www.r-pkg.org/badges/version/gridpattern)](https://cran.r-project.org/package=gridpattern)
[![R-CMD-check](https://github.com/trevorld/gridpattern/actions/workflows/R-CMD-check.yaml/badge.svg?branch=main)](https://github.com/trevorld/gridpattern/actions)
[![Coverage Status](https://codecov.io/github/trevorld/gridpattern/branch/main/graph/badge.svg)](https://app.codecov.io/github/trevorld/gridpattern?branch=main)

### Table of Contents

* [Overview](#overview)
* [Installation](#installation)
* [Examples](#examples)
* [Using these patterns with the "ggpattern" package](#ggpattern)
* [Using these patterns with the "ggplot2" package](#ggplot2)
* [Using these patterns with the "piecepackr" package](#piecepackr)

## <a name="overview">Overview</a>

`gridpattern` provides [grid.pattern()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html) and
[patternGrob()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html)
functions to use with R's grid graphics system. They fill in a
user-specified boundary path with a user-specified pattern. These pattern grobs include
enhanced versions of the patterns originally contained within
[Mike FC](https://github.com/coolbutuseless)'s awesome
[ggpattern](https://github.com/trevorld/ggpattern) package
as well as original "pch", "polygon_tiling", "regular_polygon", "rose", "text", "wave", and "weave" patterns.

`{gridpattern}` currently provides `{grid}` grob support for the following patterns:

1. [ambient](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_ambient.html):
    noise array patterns powered by the [ambient](https://cran.r-project.org/package=ambient) package
1. [aRtsy](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_aRtsy.html):
    patterns powered by the [aRtsy](https://cran.r-project.org/package=aRtsy) package
1. [circle](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_circle.html): circle geometry patterns
1. [crosshatch](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_crosshatch.html): crosshatch geometry patterns
1. [fill](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_fill.html): simple fill patterns
1. [gradient](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_gradient.html): gradient array/geometry patterns
1. [hatch](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_hatch.html): heraldic hatching patterns
1. [image](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_image.html): image array patterns
1. [line](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_line.html): line geometry patterns
1. [magick](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_magick.html): imagemagick array patterns
1. [none](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_none.html): draws nothing
1. [placeholder](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_placeholder.html): placeholder image array patterns
1. [plasma](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_plasma.html): plasma array patterns
1. [pch](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_pch.html): plotting character geometry patterns
1. [polygon_tiling](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_polygon_tiling.html): polygon tiling patterns
1. [regular_polygon](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_regular_polygon.html): regular polygon geometry patterns
1. [rose](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_rose.html): rose curve array/geometry patterns
1. [stripe](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_stripe.html): stripe geometry patterns
1. [text](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_text.html): text array/geometry patterns
1. [wave](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_wave.html): wave geometry patterns
1. [weave](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_weave.html): weave geometry patterns
1. [custom geometry-based patterns](https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html)
1. [custom array-based patterns](https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html)

## <a name="installation">Installation</a>

To install the development version use the following command in R:



``` r
remotes::install_github("trevorld/gridpattern")
```

## <a name="examples">Examples</a>




``` r
library("grid")
library("gridpattern")
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
```


``` r
grid.pattern_circle(x_hex, y_hex, density = 0.5, grid = "hex_circle",
                    fill = c("blue", "yellow", "red"))
```

<img src="man/figures/README-circle-1.png" alt="The 'circle' pattern"  />

``` r
grid.pattern_regular_polygon(x_hex, y_hex, shape = c("convex4", "star8", "circle"),
                             colour = "black", fill = c("blue", "yellow", "red"),
                             density = c(0.45, 0.42, 0.4), spacing = 0.08, angle = 0)
```

<img src="man/figures/README-regular_star-1.png" alt="The 'regular_polygon' pattern"  />

``` r
grid.pattern_regular_polygon(x_hex, y_hex, shape = "convex6", grid = "hex",
                             color = "transparent", fill = c("white", "grey", "black"),
                             density = 1.0, spacing = 0.1)
```

<img src="man/figures/README-regular_hex-1.png" alt="The 'regular_polygon' pattern used to create a hexagonal tiling"  />

``` r
gp <- gpar(fill = c("yellow", "blue", "red"))
grid.pattern_polygon_tiling(x_hex, y_hex, type = "truncated_hexagonal",
                            spacing = 0.15, gp = gp)
```

<img src="man/figures/README-truncated_hexagonal-1.png" alt="The 'polygon_tiling' pattern used to create a truncated hexagonal tiling"  />

``` r
blue <- grDevices::rgb(0.35, 0.70, 0.90)
yellow <- grDevices::rgb(0.95, 0.90, 0.25)
red <- grDevices::rgb(0.80, 0.40, 0.00)
green <- grDevices::rgb(0.00, 0.60, 0.50)
grid.rect(gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
gp <- gpar(fill = green, col = "black")
scale <- star_scale(12, 30)
grid.pattern_regular_polygon(shape = "star12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp,
                             scale = scale)
```

<img src="man/figures/README-rhombitrihexagonal-1.png" alt="Using several patterns to create a rhombitrihexgonal tiling variation"  />

``` r
gp <- gpar(fill = c("blue", "red", "yellow", "green"), col = "black")
grid.newpage()
grid.pattern_rose(x_hex, y_hex,
                  spacing = 0.18, density = 0.5, angle = 0,
                  frequency = c(2, 6/4, 5/4, 3/7), gp = gp)
```

<img src="man/figures/README-rose-1.png" alt="The 'rose' pattern"  />

``` r
playing_card_symbols <- c("\u2660", "\u2665", "\u2666", "\u2663")
grid.pattern_text(x_hex, y_hex,
                 shape = playing_card_symbols,
                 colour = c("black", "red", "red", "black"),
                 size = 24, spacing = 0.12, angle = 0)
```

<img src="man/figures/README-text-1.png" alt="The 'text' pattern with playing card suits"  />

``` r
grid.pattern_wave(x_hex, y_hex, colour = "black", type = "sine",
                  fill = c("red", "blue"), density = 0.4,
                  spacing = 0.15, angle = 0,
                  amplitude = 0.05, frequency = 1 / 0.15)
```

<img src="man/figures/README-wave-1.png" alt="The 'wave' pattern"  />

``` r
grid.pattern_weave(x_hex, y_hex, type = "satin",
             colour = "black", fill = "lightblue", fill2 =  "yellow",
             density = 0.3)
```

<img src="man/figures/README-weave-1.png" alt="Using the 'weave' pattern to make a 'satin' weave"  />

## <a name="ggpattern">Using these patterns with the "ggpattern" package</a>

`ggpattern` automatically supports all patterns provided by ``gridpattern`` since `ggpattern` v0.2.0.
Here is an example of using the "stripe" pattern:


``` r
library("ggpattern") # remotes::install_github("trevorld/ggpattern")
library("ggplot2", warn.conflicts = FALSE)
library("gridpattern")
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col_pattern(aes(fill = trt), colour = 'black',
                     pattern = 'stripe',
                     pattern_fill = list(c("grey30" , "grey70", "white", "grey70")))
```

<img src="man/figures/README-ggpattern-1.png" alt="A bar chart using 'ggpattern' with builtin patterned fills"  />

One can also create [custom patterns](https://trevorldavis.com/R/ggpattern/dev/articles/developing-patterns.html) for use with `ggpattern`.
Here is an example of creating a "tiling3" pattern that creates
three-color polygon tilings using the ``fill`` colour, the ``pattern_fill`` colour,
and their "average" color.


``` r
tiling3_pattern <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    args <- as.list(params)
    args <- args[grep("^pattern_", names(args))]

    # hexagonal tiling using "regular_polygon" pattern
    args$pattern <- "polygon_tiling"

    # three-color tiling using `fill`, `pattern_fill` and their "average"
    avg_col <- gridpattern::mean_col(params$fill, params$pattern_fill)
    args$pattern_fill <- c(params$fill, avg_col, args$pattern_fill)

    args$x <- boundary_df$x
    args$y <- boundary_df$y
    args$id <- boundary_df$id
    args$prefix <- ""

    do.call(gridpattern::patternGrob, args)
}

options(ggpattern_geometry_funcs = list(tiling3 = tiling3_pattern))
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col_pattern(aes(fill = trt, pattern_type = trt),
                     pattern = 'tiling3', pattern_angle = 45) +
    scale_pattern_type_manual(values = c("hexagonal", "tetrakis_square", "rhombille")) +
    theme(legend.key.size = unit(1.5, 'cm'))
```

<img src="man/figures/README-hex_ggpattern-1.png" alt="A bar chart using 'ggpattern' with custom patterned fills"  />

## <a name="ggplot2">Using these patterns with the "ggplot2" package</a>

Although use of the ["ggpattern" package](#ggpattern) package is still highly recommended one can directly use pattern/gradient fills in base `ggplot2` geoms since v3.5.0.
[`patternFill()`](https://trevorldavis.com/R/gridpattern/dev/reference/patternFill.html) wraps `grid::pattern()` and [patternGrob()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html) and is one way to use `gridpattern` patterns directly with `ggplot2`:


``` r
library("ggplot2", warn.conflicts = FALSE)
library("gridpattern")
pal <- grDevices::palette()
herringbone <- patternFill("polygon_tiling", type = "herringbone", fill = pal[2])
truncated_hexagonal <- patternFill("polygon_tiling", type = "truncated_hexagonal", fill = pal[3])
pythagorean <- patternFill("polygon_tiling", type = "pythagorean", fill = pal[4])
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col(fill = list(herringbone, truncated_hexagonal, pythagorean))
```

<img src="man/figures/README-ggplot2-1.png" alt="A bar chart using 'ggplot2' with 'patternFill()' 'fill' values"  />

## <a name="piecepackr">Using these patterns with the "piecepackr" package</a>

[piecepackr](https://github.com/piecepackr/piecepackr) allows the use of [custom grob functions](https://trevorldavis.com/piecepackr/custom-grob-functions.html) to completely customize the appearance of one's game pieces.  `{piecepackr}` comes with a variety of convenience functions such as `pp_shape()` to facilitate creating custom game pieces using custom grob functions.  Here is an example of creating "patterned" checkers filled with uniform polygon tilings by using ``pp_shape()`` objects' `pattern()` method powered by `{gridpattern}`:


``` r
library("grid")
library("gridpattern")
library("piecepackr")

tilings <- c("hexagonal", "snub_square", "pythagorean",
             "truncated_square", "triangular", "trihexagonal")
patternedCheckerGrobFn <- function(piece_side, suit, rank, cfg) {
    opt <- cfg$get_piece_opt(piece_side, suit, rank)
    shape <- pp_shape(opt$shape, opt$shape_t, opt$shape_r, opt$back)
    gp_pattern <- gpar(col=opt$suit_color, fill=c(opt$background_color, "white"))
    pattern_grob <- shape$pattern("polygon_tiling", type = tilings[suit],
                                  spacing = 0.3, name = "pattern",
                                  gp = gp_pattern, angle = 0)
    gp_border <- gpar(col=opt$border_color, fill=NA, lex=opt$border_lex)
    border_grob <- shape$shape(gp=gp_border, name = "border")
    grobTree(pattern_grob, border_grob)
}
checkers1 <- as.list(game_systems()$checkers1)
checkers1$grob_fn.bit <- patternedCheckerGrobFn
checkers1 <- pp_cfg(checkers1)

x1 <- c(1:3, 1:2, 1)
x2 <- c(6:8, 7:8, 8)
df <- tibble::tibble(piece_side = c("board_face", rep_len("bit_back", 24L)),
                     suit = c(6L, rep(c(1L, 3L, 4L, 5L), each = 6L)),
                     rank = 8L,
                     x = c(4.5, x1, rev(x1), x2, rev(x2)),
                     y = c(4.5, rep(c(1,1,1, 2,2, 3, 6, 7,7, 8,8,8), 2)))

pmap_piece(df, cfg=checkers1, default.units="in")
```

<img src="man/figures/README-piecepackr-1.png" alt="A board game diagram using 'piecepackr' with custom patterned game pieces"  />

# gridpattern <img src="man/figures/logo.png" align="right" width="200px" alt="gridpattern hex sticker">

[![CRAN Status Badge](https://www.r-pkg.org/badges/version/gridpattern)](https://cran.r-project.org/package=gridpattern)
[![R-CMD-check](https://github.com/trevorld/gridpattern/actions/workflows/R-CMD-check.yaml/badge.svg?branch=main)](https://github.com/trevorld/gridpattern/actions)
[![Coverage Status](https://codecov.io/github/trevorld/gridpattern/branch/main/graph/badge.svg)](https://app.codecov.io/github/trevorld/gridpattern?branch=main)

### Table of Contents

* [Overview](#overview)
* [Installation](#installation)
* [Examples](#examples)
* [Using these patterns with the "ggpattern" package](#ggpattern)
* [Using these patterns with the "ggplot2" package](#ggplot2)
* [Using these patterns with the "piecepackr" package](#piecepackr)

## <a name="overview">Overview</a>

`gridpattern` provides [grid.pattern()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html) and
[patternGrob()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html)
functions to use with R's grid graphics system. They fill in a
user-specified boundary path with a user-specified pattern. These pattern grobs include
enhanced versions of the patterns originally contained within
[Mike FC](https://github.com/coolbutuseless)'s awesome
[ggpattern](https://github.com/trevorld/ggpattern) package
as well as original "pch", "polygon_tiling", "regular_polygon", "rose", "text", "wave", and "weave" patterns.

`{gridpattern}` currently provides `{grid}` grob support for the following patterns:

1. [ambient](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_ambient.html):
    noise array patterns powered by the [ambient](https://cran.r-project.org/package=ambient) package
1. [aRtsy](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_aRtsy.html):
    patterns powered by the [aRtsy](https://cran.r-project.org/package=aRtsy) package
1. [circle](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_circle.html): circle geometry patterns
1. [crosshatch](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_crosshatch.html): crosshatch geometry patterns
1. [fill](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_fill.html): simple fill patterns
1. [gradient](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_gradient.html): gradient array/geometry patterns
1. [image](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_image.html): image array patterns
1. [magick](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_magick.html): imagemagick array patterns
1. [none](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_none.html): draws nothing
1. [placeholder](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_placeholder.html): placeholder image array patterns
1. [plasma](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_plasma.html): plasma array patterns
1. [pch](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_pch.html): plotting character geometry patterns
1. [polygon_tiling](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_polygon_tiling.html): polygon tiling patterns
1. [regular_polygon](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_regular_polygon.html): regular polygon geometry patterns
1. [rose](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_rose.html): rose curve array/geometry patterns
1. [stripe](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_stripe.html): stripe geometry patterns
1. [text](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_text.html): text array/geometry patterns
1. [wave](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_wave.html): wave geometry patterns
1. [weave](https://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern_weave.html): weave geometry patterns
1. [custom geometry-based patterns](https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html)
1. [custom array-based patterns](https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html)

## <a name="installation">Installation</a>

To install the development version use the following command in R:



``` r
remotes::install_github("trevorld/gridpattern")
```

## <a name="examples">Examples</a>




``` r
library("grid")
library("gridpattern")
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
```


``` r
grid.pattern_circle(x_hex, y_hex, density = 0.5, grid = "hex_circle",
                    fill = c("blue", "yellow", "red"))
```

<img src="man/figures/README-circle-1.png" alt="The 'circle' pattern"  />

``` r
grid.pattern_regular_polygon(x_hex, y_hex, shape = c("convex4", "star8", "circle"),
                             colour = "black", fill = c("blue", "yellow", "red"),
                             density = c(0.45, 0.42, 0.4), spacing = 0.08, angle = 0)
```

<img src="man/figures/README-regular_star-1.png" alt="The 'regular_polygon' pattern"  />

``` r
grid.pattern_regular_polygon(x_hex, y_hex, shape = "convex6", grid = "hex",
                             color = "transparent", fill = c("white", "grey", "black"),
                             density = 1.0, spacing = 0.1)
```

<img src="man/figures/README-regular_hex-1.png" alt="The 'regular_polygon' pattern used to create a hexagonal tiling"  />

``` r
gp <- gpar(fill = c("yellow", "blue", "red"))
grid.pattern_polygon_tiling(x_hex, y_hex, type = "truncated_hexagonal",
                            spacing = 0.15, gp = gp)
```

<img src="man/figures/README-truncated_hexagonal-1.png" alt="The 'polygon_tiling' pattern used to create a truncated hexagonal tiling"  />

``` r
blue <- grDevices::rgb(0.35, 0.70, 0.90)
yellow <- grDevices::rgb(0.95, 0.90, 0.25)
red <- grDevices::rgb(0.80, 0.40, 0.00)
green <- grDevices::rgb(0.00, 0.60, 0.50)
grid.rect(gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3,
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
gp <- gpar(fill = green, col = "black")
scale <- star_scale(12, 30)
grid.pattern_regular_polygon(shape = "star12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp,
                             scale = scale)
```

<img src="man/figures/README-rhombitrihexagonal-1.png" alt="Using several patterns to create a rhombitrihexgonal tiling variation"  />

``` r
gp <- gpar(fill = c("blue", "red", "yellow", "green"), col = "black")
grid.newpage()
grid.pattern_rose(x_hex, y_hex,
                  spacing = 0.18, density = 0.5, angle = 0,
                  frequency = c(2, 6/4, 5/4, 3/7), gp = gp)
```

<img src="man/figures/README-rose-1.png" alt="The 'rose' pattern"  />

``` r
playing_card_symbols <- c("\u2660", "\u2665", "\u2666", "\u2663")
grid.pattern_text(x_hex, y_hex,
                 shape = playing_card_symbols,
                 colour = c("black", "red", "red", "black"),
                 size = 24, spacing = 0.12, angle = 0)
```

<img src="man/figures/README-text-1.png" alt="The 'text' pattern with playing card suits"  />

``` r
grid.pattern_wave(x_hex, y_hex, colour = "black", type = "sine",
                  fill = c("red", "blue"), density = 0.4,
                  spacing = 0.15, angle = 0,
                  amplitude = 0.05, frequency = 1 / 0.15)
```

<img src="man/figures/README-wave-1.png" alt="The 'wave' pattern"  />

``` r
grid.pattern_weave(x_hex, y_hex, type = "satin",
             colour = "black", fill = "lightblue", fill2 =  "yellow",
             density = 0.3)
```

<img src="man/figures/README-weave-1.png" alt="Using the 'weave' pattern to make a 'satin' weave"  />

## <a name="ggpattern">Using these patterns with the "ggpattern" package</a>

`ggpattern` automatically supports all patterns provided by ``gridpattern`` since `ggpattern` v0.2.0.
Here is an example of using the "stripe" pattern:


``` r
library("ggpattern") # remotes::install_github("trevorld/ggpattern")
library("ggplot2", warn.conflicts = FALSE)
library("gridpattern")
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col_pattern(aes(fill = trt), colour = 'black',
                     pattern = 'stripe',
                     pattern_fill = list(c("grey30" , "grey70", "white", "grey70")))
```

<img src="man/figures/README-ggpattern-1.png" alt="A bar chart using 'ggpattern' with builtin patterned fills"  />

One can also create [custom patterns](https://trevorldavis.com/R/ggpattern/dev/articles/developing-patterns.html) for use with `ggpattern`.
Here is an example of creating a "tiling3" pattern that creates
three-color polygon tilings using the ``fill`` colour, the ``pattern_fill`` colour,
and their "average" color.


``` r
tiling3_pattern <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    args <- as.list(params)
    args <- args[grep("^pattern_", names(args))]

    # hexagonal tiling using "regular_polygon" pattern
    args$pattern <- "polygon_tiling"

    # three-color tiling using `fill`, `pattern_fill` and their "average"
    avg_col <- gridpattern::mean_col(params$fill, params$pattern_fill)
    args$pattern_fill <- c(params$fill, avg_col, args$pattern_fill)

    args$x <- boundary_df$x
    args$y <- boundary_df$y
    args$id <- boundary_df$id
    args$prefix <- ""

    do.call(gridpattern::patternGrob, args)
}

options(ggpattern_geometry_funcs = list(tiling3 = tiling3_pattern))
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col_pattern(aes(fill = trt, pattern_type = trt),
                     pattern = 'tiling3', pattern_angle = 45) +
    scale_pattern_type_manual(values = c("hexagonal", "tetrakis_square", "rhombille")) +
    theme(legend.key.size = unit(1.5, 'cm'))
```

<img src="man/figures/README-hex_ggpattern-1.png" alt="A bar chart using 'ggpattern' with custom patterned fills"  />

## <a name="ggplot2">Using these patterns with the "ggplot2" package</a>

Although use of the ["ggpattern" package](#ggpattern) package is still highly recommended one can directly use pattern/gradient fills in base `ggplot2` geoms since v3.5.0.
[`patternFill()`](https://trevorldavis.com/R/gridpattern/dev/reference/patternFill.html) wraps `grid::pattern()` and [patternGrob()](http://trevorldavis.com/R/gridpattern/dev/reference/grid.pattern.html) and is one way to use `gridpattern` patterns directly with `ggplot2`:


``` r
library("ggplot2", warn.conflicts = FALSE)
library("gridpattern")
pal <- grDevices::palette()
herringbone <- patternFill("polygon_tiling", type = "herringbone", fill = pal[2])
truncated_hexagonal <- patternFill("polygon_tiling", type = "truncated_hexagonal", fill = pal[3])
pythagorean <- patternFill("polygon_tiling", type = "pythagorean", fill = pal[4])
df <- data.frame(trt = c("a", "b", "c"), outcome = c(2.3, 1.9, 3.2))
ggplot(df, aes(trt, outcome)) +
    geom_col(fill = list(herringbone, truncated_hexagonal, pythagorean))
```

<img src="man/figures/README-ggplot2-1.png" alt="A bar chart using 'ggplot2' with 'patternFill()' 'fill' values"  />

## <a name="piecepackr">Using these patterns with the "piecepackr" package</a>

[piecepackr](https://github.com/piecepackr/piecepackr) allows the use of [custom grob functions](https://trevorldavis.com/piecepackr/custom-grob-functions.html) to completely customize the appearance of one's game pieces.  `{piecepackr}` comes with a variety of convenience functions such as `pp_shape()` to facilitate creating custom game pieces using custom grob functions.  Here is an example of creating "patterned" checkers filled with uniform polygon tilings by using ``pp_shape()`` objects' `pattern()` method powered by `{gridpattern}`:


``` r
library("grid")
library("gridpattern")
library("piecepackr")

tilings <- c("hexagonal", "snub_square", "pythagorean",
             "truncated_square", "triangular", "trihexagonal")
patternedCheckerGrobFn <- function(piece_side, suit, rank, cfg) {
    opt <- cfg$get_piece_opt(piece_side, suit, rank)
    shape <- pp_shape(opt$shape, opt$shape_t, opt$shape_r, opt$back)
    gp_pattern <- gpar(col=opt$suit_color, fill=c(opt$background_color, "white"))
    pattern_grob <- shape$pattern("polygon_tiling", type = tilings[suit],
                                  spacing = 0.3, name = "pattern",
                                  gp = gp_pattern, angle = 0)
    gp_border <- gpar(col=opt$border_color, fill=NA, lex=opt$border_lex)
    border_grob <- shape$shape(gp=gp_border, name = "border")
    grobTree(pattern_grob, border_grob)
}
checkers1 <- as.list(game_systems()$checkers1)
checkers1$grob_fn.bit <- patternedCheckerGrobFn
checkers1 <- pp_cfg(checkers1)

x1 <- c(1:3, 1:2, 1)
x2 <- c(6:8, 7:8, 8)
df <- tibble::tibble(piece_side = c("board_face", rep_len("bit_back", 24L)),
                     suit = c(6L, rep(c(1L, 3L, 4L, 5L), each = 6L)),
                     rank = 8L,
                     x = c(4.5, x1, rev(x1), x2, rev(x2)),
                     y = c(4.5, rep(c(1,1,1, 2,2, 3, 6, 7,7, 8,8,8), 2)))

pmap_piece(df, cfg=checkers1, default.units="in")
```

<img src="man/figures/README-piecepackr-1.png" alt="A board game diagram using 'piecepackr' with custom patterned game pieces"  />

## -----------------------------------------------------------------------------
library("grid")
library("gridpattern")
add_borders <- function() {
    pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
}
x_sq <- c(0, 0, 1, 1)
y_sq <- c(0, 1, 1, 0)
# colorblind accessible scheme https://jfly.uni-koeln.de/color/
blue <- grDevices::rgb(0.35, 0.70, 0.90)
yellow <- grDevices::rgb(0.95, 0.90, 0.25)
red <- grDevices::rgb(0.80, 0.40, 0.00)
green <- grDevices::rgb(0.00, 0.60, 0.50)
orange <- grDevices::rgb(0.90, 0.60, 0.00)

## ----fig.alt = "Four different uniform colorings of a triangular tiling"------
grid.triangular_tiling <- function(...) {
    grid.pattern_regular_polygon(..., shape = "convex3", density = 1, grid = "hex",
                                 spacing = 0.2, angle = 0, rot = 180)
}
# 1 color uniform triangular tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 2 color uniform triangular tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 3 color uniform triangular tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 4 color uniform triangular tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(red, blue, green), col = "black")
grid.triangular_tiling(gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Four different uniform colorings of a square tiling"----------
# 2 color uniform square tiling (1212)
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, 
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 2 color uniform square tiling (1122)
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, type = "horizontal",
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 3 color uniform square tiling (1213)
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1,
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 4 color uniform square tiling (1234)
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue, green), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1,
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Four different uniform colorings of a hexagonal tiling"-------
grid.hexagonal_tiling <- function(..., shape = "convex6", spacing = 0.2) {
    scale <- star_scale(6, 30)
    grid.pattern_regular_polygon(..., shape = shape, density = 1, grid = "hex",
                                 spacing = spacing, scale = scale, angle = 0)
}
# 1 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# One of several 2 color uniform hexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(red, yellow), col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# 3 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# (4.6_{π/6})^3
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.hexagonal_tiling(shape = "star6", spacing = 0.3, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Three different uniform colorings of an elongated triangular tiling"----
grid.elongated_triangular_tiling <- function(...) {
    grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
    grid.pattern_regular_polygon(..., shape = rep(c("convex4", "convex3"), each = 2), 
                                 density = rep(c(1.41, 1.15), each = 2), 
                                 grid = "elongated_triangle",
                                 type = "square_tiling", subtype = "3412",
                                 spacing = 0.2, angle = 0, 
                                 rot = rep(c(45, 0), each = 2))
}
# 1 color elongated triangular tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()

# 2 color elongated triangular tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(red, red, yellow, yellow), col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()

# 3 color elongated triangular tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(red, blue, yellow, yellow), col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Three different uniform colorings of a snub square tiling"----
grid.snub_square <- function(..., gp_sq, gp_sq2 = gp_sq, gp_tri = gp_sq) {
    scale_star <- star_scale(4, 90 + 60, external = TRUE)
    grid.polygon(x_sq, y_sq, gp = gp_sq)
    grid.pattern_regular_polygon(shape = "star4", scale = scale_star,
                                 angle = 0, rot = 15, spacing = 0.2,
                                 density = 1.4, gp = gp_tri)
    grid.pattern_regular_polygon(shape = "convex4", scale = scale_star,
                                 angle = 0, rot = 60, spacing = 0.2,
                                 density = scale_star * 1.4, gp = gp_sq2)
}

# 2 color uniform snub square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp_y <- gpar(fill = yellow, col = "black")
gp_r <- gpar(fill = red, col = "black")
grid.snub_square(gp_sq = gp_y, gp_tri = gp_r)
popViewport()

# 3 color uniform snub square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp_b <- gpar(fill = blue, col = "black")
grid.snub_square(gp_sq = gp_r, gp_sq2 = gp_b, gp_tri = gp_y)
popViewport()

# 1 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.snub_square(gp_sq = gp_y)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()

## ----fig.alt = "Four different uniform colorings of a truncated square tiling"----
# 2 color uniform truncated square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
popViewport()

# 3 color uniform truncated square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = blue, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
popViewport()

# 3 color uniform truncated square tiling with star
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
gp <- gpar(fill = orange, col = "black")
scale <- star_scale(8, 60, external = TRUE)
grid.pattern_regular_polygon(shape = "star8", density = 1.082, rot = 22.5, 
                             angle = 0, scale = scale, spacing = 0.3, gp = gp)
popViewport()

# 4 color uniform truncated square tiling with an alternating star
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = blue, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
gp <- gpar(fill = orange, col = c("black"))
grid.pattern_regular_polygon(shape = c("null", "star8"), 
                             density = 1.082, rot = 22.5, 
                             angle = 0, scale = scale, spacing = 0.3, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "A truncated hexagonal tiling and a related 'star12' variation"----
# 2 color uniform truncated hexagonal tiling
grid.truncated_hexagonal_tiling <- function(..., shape = "convex12") {
    scale <- star_scale(12, 60, external = TRUE)
    grid.pattern_regular_polygon(..., shape = shape, density = 1.034, spacing = 0.3,
                                 angle = 0, rot = 15, scale = scale, 
                                 grid = "hex_circle")
}
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.truncated_hexagonal_tiling(gp = gp)
popViewport()

# Add in a "convex12" star within
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.truncated_hexagonal_tiling(gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.truncated_hexagonal_tiling(gp = gp, shape = "star12")
popViewport()
add_borders()

## ----fig.alt = "A trihexagonal tiling plus 'star6', 'star3', and 'convex3' variations"----
grid.trihexagonal_tiling <- function(..., shape = "convex6", density = 1) {
    grid.pattern_regular_polygon(..., shape = shape, density = density, grid = "hex_circle",
                                 spacing = 0.2, rot = 30, angle = 0)
}
# 2 color uniform trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.trihexagonal_tiling(gp = gp)
popViewport()

# 'star6' trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
scale <- star_scale(6, 120, external = TRUE)
grid.trihexagonal_tiling(shape = "star6", scale = scale, gp = gp)
popViewport()

# 'star3' trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
scale <- star_scale(3, 30)
grid.trihexagonal_tiling(shape = "star3", scale = scale, density=1.57, gp = gp)
popViewport()

# 'convex3' trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
grid.trihexagonal_tiling(shape = "convex3", density=1.33, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Three different colorings of a snub trihexagonal tiling"------
grid.snub_trihexagonal <- function(..., gp_tri, gp_tri2 = gp_tri, gp_hex = gp_tri) {
    scale_star <- star_scale(6, 60 + 60, external = TRUE)
    grid.polygon(x_sq, y_sq, gp = gp_tri)
    grid.pattern_regular_polygon(shape = "star6", scale = scale_star,
                                 grid = "hex_circle",
                                 angle = 0, rot = 19, spacing = 0.2,
                                 density = 1.305, gp = gp_tri2)
    grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle",
                                 angle = 0, rot = 19 + 30, spacing = 0.2,
                                 density = scale_star * 1.305, gp = gp_hex)
}

# 3 color uniform snub square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp_y <- gpar(fill = yellow, col = "black")
gp_r <- gpar(fill = red, col = "black")
gp_b <- gpar(fill = blue, col = "black")
grid.snub_trihexagonal(gp_tri = gp_b, gp_tri2 = gp_r, gp_hex = gp_y)
popViewport()

# 2 color snub trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.snub_trihexagonal(gp_tri = gp_y, gp_hex = gp_r)
popViewport()

# 1 color snub trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.snub_trihexagonal(gp_tri = gp_y)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()

## ----fig.alt = "Two different colorings of a truncated trihexagonal tiling"----
# truncated trihexagonal
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3,
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle", 
                             density = 0.75, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# truncated trihexagonal with 5 colors
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = green, col = NA))
gp <- gpar(fill = orange, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle", 
                             density = 0.75, spacing = 0.3, angle = 0, gp = gp)
popViewport()
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
popViewport()
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()

## ----fig.alt = "Two different colorings of a rhombitrihexagonal tiling plus a 'star12' variation"----
# rhombitrihexagonal
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# rhombitrihexagonal with 5 colors
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = green, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = orange, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# 3.4.6.3.12* rhombitrihexagonal plus star
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
gp <- gpar(fill = green, col = "black")
scale <- star_scale(12, 30)
grid.pattern_regular_polygon(shape = "star12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp,
                             scale = scale)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()

## ----fig.alt = "Four uniform tilings with star polygons"----------------------
# 12.12.4*
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow, blue), col = "black")
grid.pattern_regular_polygon(shape="convex12", rot=15, density=1.035, 
                             angle=0, gp = gp, spacing=0.3)
popViewport()

# 12.3*.12.3*
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow), col = "black")
grid.pattern_regular_polygon(shape="convex12", density=1, grid = "hex_circle",
                             angle=0, gp = gp, spacing=0.4)
popViewport()

# 4.4*.4**
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow, blue), col = "black")
gp <- gpar(fill=yellow)
scale <- star_scale(4, 30)
grid.pattern_regular_polygon(shape="star4", density=1.60, angle=0, scale=scale, 
                             rot=-9.5, spacing=0.3, gp = gp)
popViewport()

# 3.3.8*.4**.8*
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
sp <- 0.35
gp <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, angle = 0, 
                             spacing = sp, gp = gp)
gp <- gpar(fill = yellow, col = "black")
scale <- star_scale(4, 60)
grid.pattern_regular_polygon(shape = "star4", density = 1.14, rot = 45, 
                             xoffset = sp / 2, yoffset = sp / 2,
                             angle = 0, scale = scale, spacing = sp, gp = gp)
gp <- gpar(fill = blue, col = "black")
scale <- star_scale(8, 15)
grid.pattern_regular_polygon(shape = "star8", density = 1.082, rot = 22.5, 
                             angle = 0, scale = 0.30, spacing = sp, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Four more uniform tilings with star polygons"-----------------
# 3.6*.6**
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
scale <- star_scale(6, 30)
grid.pattern_regular_polygon(shape="star6", density=1.30, angle=0, scale=scale, rot=-22.0, 
                             spacing=0.3, grid="hex", gp=gp)
popViewport()

# 4.6.4*.6
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
scale <- star_scale(4, 120, external = TRUE)
grid.pattern_regular_polygon(shape=c("star4", "convex4"), density=c(1.2, 0.8), 
                             scale = scale, 
                             spacing = 0.2, gp = gp, angle=45)
popViewport()

# 9.3.9.3*
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = blue, col = NA)
grid.pattern_stripe(spacing = 0.4, density = 0.35, angle = 0,
                    grid = "hex_circle", gp = gp, yoffset = 0.096)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex9", density = 1.01, angle = 0, 
                             grid = "hex_circle", spacing = 0.4, gp = gp)
popViewport()

# 8.4*.8.4*
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = c("convex8", "null"), density = 1.41, 
                             angle = 0, spacing = 0.2, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Two more uniform tilings with star polygons"------------------
# 18.18.3*
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = blue, col = NA))
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex18", density = 1, 
                             grid = "hex_circle", angle = 0,
                             spacing = 0.2, gp = gp)
popViewport()

# Only uniform if you consider the larger squares to be four-pointed stars...
# 4.8*.4**.8*
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = blue, col = NA))
gp <- gpar(fill = c(yellow, red), col = "black")
scale <- star_scale(8, 90, external = TRUE)
grid.pattern_regular_polygon(shape = c("star8", "convex4"), scale = scale,
                             density = c(1.53, 1.2), rot = c(22.5, 0), 
                             angle = 0, spacing = 0.2, gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Four different herringbone tilings"---------------------------
grid.herringbone_tiling <- function(..., subtype = "2/2(1)") {
    grid.pattern_weave(..., type = "twill", subtype = subtype, density = 1, 
                       spacing = 0.1, angle = 45)
}
# 1 color 2:1 herringbone 
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp)
popViewport()

# 2 color 2:1 herringbone
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, fill2 = red)
popViewport()

# 1 color 3:1 herringbone
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, subtype = "3/3(1)")
popViewport()

# 2 color 3:1 herringbone
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, fill2 = red, subtype = "3/3(1)")
popViewport()
add_borders()

## ----fig.alt = "Four different colorings of a Pythagorean tiling"-------------
grid.pythagorean_tiling <- function(...) {
    grid.pattern_regular_polygon(..., shape = "convex4", density = 1.222, 
                                 rot = 15, spacing = 0.2, angle = -15)
}
# 1 color Pythagorean tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 2 color Pythagorean tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 3 color Pythagorean tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 4 color Pythagorean tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(red, blue, green), col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()
add_borders()

## ----fig.alt = "Three different colorings of a rhombille tiling"--------------
grid.rhombille_tiling <- function(gp1, gp2 = gp1, gp3 = gp2) {
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot = -120, spacing = 0.2,
                       grid = "hex", gp = gp3)
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot =  120, spacing = 0.2,
                       grid = "hex", gp = gp2)
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot = 0, spacing = 0.2,
                       grid = "hex", gp = gp1)
}
gp1 <- gpar(fill = yellow, col = "black")
gp2 <- gpar(fill = blue, col = "black")
gp3 <- gpar(fill = red, col = "black")
# 1 color rhombille tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.rhombille_tiling(gp1)
popViewport()
# 2 color rhombille tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.rhombille_tiling(gp1, gp2)
popViewport()
# 3 color rhombille tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.rhombille_tiling(gp1, gp2, gp3)
popViewport()
add_borders()

## ----fig.alt = "Four different colorings of a tetrakis square tiling"---------
grid.tetrakis_tiling <- function(gp1, gp2 = gp1, gp3 = gp2, gp4 = gp1) {
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1, 
                                 angle = 0, rot=0, gp = gp1, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1, 
                                 angle = 0, rot=90, gp = gp4, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1,
                                 angle = 0, rot=180, gp = gp1, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1,
                                 angle = 0, rot=270, gp = gp4, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=0, gp = gp2, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=90, gp = gp3, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=180, gp = gp2, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=270, gp = gp3, spacing = 0.25)

}
gp1 <- gpar(fill = yellow, col = "black")
gp2 <- gpar(fill = blue, col = "black")
gp3 <- gpar(fill = red, col = "black")
gp4 <- gpar(fill = green, col = "black")
# 1 color tetrakis square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1)
popViewport()
# 2 color tetrakis square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2)
popViewport()
# 3 color tetrakis square tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2, gp3)
popViewport()
# 4 color tetrakis square tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2, gp3, gp4)
popViewport()
add_borders()

## ----fig.alt = "Three different tilings using rhombi"-------------------------
# 2*.2**.2*.2**
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
sp <- 0.3
ang <- 63.5
scale <- 0.5
gpy <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpy)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpy,
                             xoffset = 0.5 * sp)
gpr <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpr,
                             xoffset = 0.25 * sp, yoffset = 0.5 * sp)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpr,
                             xoffset = 0.75 * sp, yoffset = 0.5 * sp)
popViewport()

# 4.2*.4.2**
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
dens_sq <- 0.73
scale <- star_scale(2, 60)
dens_rh <- 0.88
gpy <- gpar(fill = yellow, col = "black")
gpb <- gpar(fill = blue, col = "black")
gpr <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "convex4", scale = scale, angle = 0,
                             rot = 60,
                             density = dens_sq, spacing = sp, gp = gpb)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = 0,
                             density = dens_rh, spacing = sp, gp = gpy,
                             rot = 45, xoffset = 0.5 * sp)
grid.pattern_regular_polygon(shape = "convex4", scale = scale, angle = 0,
                             rot = -60, yoffset = 0.5 * sp, xoffset = 0.5 * sp,
                             density = dens_sq, spacing = sp, gp = gpr)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = 0,
                             density = dens_rh, spacing = sp, gp = gpy,
                             rot = -45, yoffset = 0.5 * sp)
popViewport()

# Use a "star12" instead of a "convex12" in truncated hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.truncated_hexagonal_tiling(gp = gp, shape = "star12")
popViewport()
add_borders()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>"
)

## ----setup--------------------------------------------------------------------
library("gridpattern")

## ----echo = FALSE-------------------------------------------------------------
x <- read.csv(textConnection("
x,  y,  id
0,  0,   1
1,  0,   1
1,  1,   1
0,  1,   1
0,  0,   2
2,  0,   2
2,  1,   2
0,  1,   2"))

knitr::kable(x, caption = "example data in 'polygon_df' format")

## ----eval = FALSE-------------------------------------------------------------
# options(ggpattern_array_funcs    = list(your_pattern_name = your_pattern_function))
# options(ggpattern_geometry_funcs = list(your_pattern_name = your_pattern_function))

## -----------------------------------------------------------------------------
create_pattern_polygon <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    x <- boundary_df$x
    y <- boundary_df$y
    id <- boundary_df$id

    alpha <- ifelse(is.na(params$pattern_alpha), 1, params$pattern_alpha)
    gp <- grid::gpar(alpha = alpha, 
                     col = params$pattern_colour,
                     fill = params$pattern_fill,
                     lty = params$pattern_linetype,
                     lwd = params$pattern_linewidth)

    grid::polygonGrob(x = x, y = y, id = id, default.units = "npc", gp = gp)
}

## -----------------------------------------------------------------------------
options(ggpattern_geometry_funcs = list(polygon = create_pattern_polygon))

## ----fig.alt = "Three rectangles each filled with a single color"-------------
grid.pattern("polygon", fill = "red", size = 4, linetype = "dashed", 
             x = c(0.05, 0.05, 0.305, 0.305), y = c(0.05, 0.305, 0.305, 0.05))
grid.pattern("polygon", fill = "green", alpha = 0.2, 
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("polygon", fill = "blue", colour = "grey",
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))

## -----------------------------------------------------------------------------
create_pattern_complex <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    args <- as.list(params)
    args <- args[grep("^pattern_", names(args))]
    args$x <- boundary_df$x
    args$y <- boundary_df$y
    args$id <- boundary_df$id
    args$prefix <- ""

    args_stripe <- args
    args_stripe$pattern <- "stripe"
    args_stripe$pattern_density <- 0.5 * args$pattern_density
    args_stripe$pattern_spacing <- 2 * args$pattern_spacing
    grob_stripe <- do.call(gridpattern::patternGrob, args_stripe)

    args_circle <- args
    args_circle$pattern <- "regular_polygon"
    args_circle$pattern_shape <- c("circle", "null")
    args_circle$pattern_yoffset <- args$pattern_spacing + args$pattern_yoffset
    args_circle$pattern_type = "horizontal"
    grob_circle <- do.call(gridpattern::patternGrob, args_circle)

    args_gradient <- args
    args_gradient$pattern <- "gradient"
    args_gradient$pattern_fill <- "#00000070"
    args_gradient$pattern_fill2 <- "#FFFFFF70"
    args_gradient$pattern_orientation <- "vertical"
    grob_gradient <- do.call(gridpattern::patternGrob, args_gradient)

    grid::grobTree(grob_stripe, grob_circle, grob_gradient)
}

## -----------------------------------------------------------------------------
options(ggpattern_geometry_funcs = list(complex = create_pattern_complex))

## ----fig.alt = "Three rectangles each with a 'stripe', 'circle', and 'gradient' pattern"----
grid.pattern("complex", fill = "red", angle = 45, spacing = 0.05, density = 0.3,
             x = c(0.0, 0.0, 0.3, 0.3), y = c(0.0, 0.3, 0.3, 0.0))
grid.pattern("complex", fill = "green", angle = 45, spacing = 0.2, density = 0.2,
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("complex", fill = "blue", angle = 45, spacing = 0.1, density = 0.3,
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))
grid::grid.polygon(x = c(0.0, 0.0, 0.3, 0.3, 0.35, 0.35, 0.65, 0.65, 0.7, 0.7, 1.0, 1.0),
                   y = c(0.0, 0.3, 0.3, 0.0, 0.35, 0.65, 0.65, 0.35, 0.7, 1.0, 1.0, 0.7), 
                   id = rep(1:3, each = 4), 
                   gp = grid::gpar(col = "black", fill = NA, lwd=4))

## -----------------------------------------------------------------------------
create_pattern_simple <- function(width, height, params, legend) {
  
  # Ensure the selected pattern is sane.
  choice <- params$pattern_type
  if (is.null(choice) || is.na(choice) || !is.character(choice)) {
    choice <- 'a'
  }
  
  # Choose the values with which to fill the array
  values <- switch(
    choice,
    a = rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3),
    b = rep(c(1, 0, 0, 1, 0.5, 0.5, 1, 1, 0, 0, 0, 0, 0, 0.5), each = 7),
    c = rep(seq(0, 1, 0.05), each = 7),
    rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3)
  )
  # Create an RGBA array of the requested dimensions
  simple_array <- array(values, dim = c(height, width, 4))

  simple_array
}

## -----------------------------------------------------------------------------
options(ggpattern_array_funcs = list(simple = create_pattern_simple))

## ----fig.alt = "Three rectangles each filled with an 'array' pattern"---------
grid::grid.polygon(x = c(0, 0, 1, 1), y = c(0, 1, 1, 0), 
                   gp = grid::gpar(col=NA, fill="grey"))
grid.pattern("simple", type = "a",
             x = c(0.0, 0.0, 0.3, 0.3), y = c(0.0, 0.3, 0.3, 0.0))
grid.pattern("simple", type = "b",
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("simple", type = "c",
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))

#' Compute average color
#'
#' `mean_col()` computes an average color.
#'
#' We currently compute an average color
#' by using the quadratic mean of the colors' RGBA values.
#'
#' @param ... Colors to average
#' @return A color string of 9 characters: `"#"` followed by the
#'         red, blue, green, and alpha values in hexadecimal.
#' @examples
#'  mean_col("black", "white")
#'  mean_col(c("black", "white"))
#'  mean_col("red", "blue")
#' @seealso [mix_col()] for subtractive (pigment) mixing via Munsell color space.
#' @export
mean_col <- function(...) {
	cols <- unlist(list(...))
	m <- grDevices::col2rgb(cols, alpha = TRUE) / 255.0
	# quadratic mean suggested at https://stackoverflow.com/a/29576746
	v <- apply(m, 1, quadratic_mean)
	grDevices::rgb(v[1], v[2], v[3], v[4])
}

quadratic_mean <- function(x) sqrt(mean(x^2))

#' Mix colors via Munsell color space
#'
#' `mix_col()` simulates subtractive (pigment) color mixing by converting
#' input colors to Munsell notation via [aqp::col2Munsell()], mixing them with
#' [aqp::mixMunsell()], and converting the result back to an R color string
#' via [aqp::parseMunsell()].
#'
#' @param ... Colors to mix.  Can be individual color strings or character vectors;
#'   all are combined into a single vector.
#' @param w A numeric vector of weights or proportions the same length as the combined color vector.
#'   Defaults to equal weights.
#' @param mixingMethod Mixing method passed to [aqp::mixMunsell()].
#' @return A single R color string in hex notation.
#' @examples
#' should_run_ex <- requireNamespace("aqp", quietly = TRUE) &&
#'     (interactive() || identical(Sys.getenv("IN_PKGDOWN"), "true"))
#' if (should_run_ex) {
#'   mix_col(c("red", "blue"))
#' }
#' if (should_run_ex) {
#'   mix_col("red", "yellow", "blue", w = c(2, 1, 1))
#' }
#' @seealso [mean_col()] for a simpler quadratic-mean RGB approach (no extra packages required).
#' @export
mix_col <- function(..., w = NULL, mixingMethod = "adaptive") {
	assert_suggested("aqp", fn = "mix_col")
	cols <- unlist(list(...))
	w <- w %||% rep.int(1, length(cols))
	df <- aqp::col2Munsell(cols)
	s <- aqp::formatMunsell(df$hue, df$value, df$chroma)
	mixed <- suppressMessages(aqp::mixMunsell(s, w = w, mixingMethod = mixingMethod))
	aqp::parseMunsell(mixed$munsell[1L])
}

test_that("`update_alpha()` works as expected", {
	expect_equal(update_alpha("blue", 0.5), "#0000FF80")
	expect_equal(update_alpha(list(c("blue", "red")), NA), c("#0000FFFF", "#FF0000FF"))
	expect_equal(update_alpha("#00FF0080", NA), "#00FF0080")
	expect_equal(update_alpha("#FF000080", 0.25), "#FF000040")

	expect_error(update_alpha(list("blue", "red"), NA))
	skip_if_not(getRversion() >= "4.1.0")
	expect_equal(
		update_pattern_alpha("red", 0.5, name = "nonrandom"),
		pattern(rectGrob(name = "nonrandom"), gp = gpar(fill = "#FF000080"))
	)
	expect_equal(
		update_pattern_alpha(linearGradient(c("black", "white")), 0.5),
		linearGradient(c("#00000080", "#FFFFFF80"))
	)
})

test_that("`mean_col()` works as expected", {
	expect_equal(mean_col("black", "white"), "#B4B4B4FF")
	expect_equal(mean_col(c("black", "white")), "#B4B4B4FF")
	expect_equal(mean_col("red", "blue"), "#B400B4FF")
})

test_that("`get_params()` works as expected", {
	params <- get_params()
	expect_equal(params$pattern_colour, "grey20")
	expect_equal(params$pattern_fill, "grey80")
	expect_equal(params$pattern_angle, 30)
	expect_equal(params$pattern_density, 0.2)
	expect_equal(params$pattern_spacing, 0.05)
	expect_equal(params$pattern_xoffset, 0)
	expect_equal(params$pattern_yoffset, 0)
	expect_equal(params$pattern_alpha, NA_real_)
	expect_equal(params$pattern_linetype, 1)
	expect_equal(params$pattern_linewidth, 1)

	params <- get_params(alpha = 0.5, spacing = 0.1)
	expect_equal(params$pattern_alpha, 0.5)
	expect_equal(params$pattern_spacing, 0.1)

	gp <- gpar(col = "blue", lty = 2)
	params <- get_params(gp = gp)
	expect_equal(params$pattern_colour, "blue")
	expect_equal(params$pattern_linetype, 2)
})

test_that("`star_scale()` works as expected", {
	# |8/3| star has internal angle 45 degrees and external angle 90 degrees
	scale <- star_scale(8, 45)
	scale2 <- star_scale(8, 90, external = TRUE)
	expect_equal(scale, scale2)
	expect_equal(star_angle(8, scale), 45)
	expect_equal(star_angle(8, scale, external = TRUE), 90)
	expect_equal(star_angle(2, star_scale(2, 30)), 30)
	expect_equal(star_angle(2, star_scale(2, 210, TRUE), TRUE), 210)
})

test_that("`assert_patterns_unique()` works as expected", {
	expect_null(assert_patterns_unique(list(), list()))
	expect_null(assert_patterns_unique(list(custom1 = 2), list(custom2 = 2)))
	expect_error(
		assert_patterns_unique(list(custom = 2, custom = 3), list()),
		'There are multiple custom "geometry" patterns named "custom"'
	)
	expect_error(
		assert_patterns_unique(list(), list(custom = 2, custom = 3)),
		'There are multiple custom "array" patterns named "custom"'
	)
	expect_error(
		assert_patterns_unique(list(custom = 2), list(custom = 2)),
		'There is a custom "geometry" pattern and custom "array" pattern both named "custom"'
	)
	expect_error(
		assert_patterns_unique(list(circle = 2), list()),
		'There is a custom "geometry" pattern and builtin \\{gridpattern\\} pattern both named "circle"'
	)
	expect_error(
		assert_patterns_unique(list(), list(image = 2)),
		'There is a custom "array" pattern and builtin \\{gridpattern\\} pattern both named "image"'
	)
})

test_that("`mix_col()` works as expected", {
	skip_on_cran()
	skip_if_not_installed("aqp")
	expect_equal(mix_col(c("red", "blue")), "#C2008FFF")
	expect_equal(mix_col(c("yellow", "green")), "#ACFF00FF")
	expect_equal(mix_col(c("red", "yellow", "blue"), w = c(2, 1, 1)), "#F46666FF")
})

test_that("`names_hatch()` works as expected", {
	# default is combinatorial, returns color equivalents
	nms <- names_hatch()
	expect_type(nms, "character")
	expect_true(all(!duplicated(nms)))
	expect_true(all(c("red", "blue", "green", "purple", "yellow", "black", "white") %in% nms))
	expect_true(all(c("magenta", "teal", "violet", "orange", "lime green") %in% nms))
	expect_true(all(c("pink", "grey", "mint green", "light blue", "lavender") %in% nms))

	# fox-davies returns heraldic names
	fd <- names_hatch("fox-davies")
	expect_true(all(!duplicated(fd)))
	expect_true(all(c("gules", "azure", "vert", "purpure", "sable", "argent", "or") %in% fd))
	expect_true(all(
		c(
			"eisenfarbe",
			"brunatre",
			"sanguine",
			"tenne",
			"carnation",
			"cendree",
			"orange",
			"bleu celeste",
			"proper"
		) %in%
			fd
	))

	# goodman: has new tinctures, lacks eisenfarbe and proper
	gd <- names_hatch("goodman")
	expect_true(all(!duplicated(gd)))
	expect_true(all(c("murrey", "steel", "copper", "bronze", "lead") %in% gd))
	expect_false("eisenfarbe" %in% gd)
	expect_false("proper" %in% gd)

	# unicode returns color equivalents matching emoji heart colors
	un <- names_hatch("unicode")
	expect_true(all(!duplicated(un)))
	expect_true(all(
		c(
			"red",
			"blue",
			"green",
			"yellow",
			"purple",
			"black",
			"white",
			"brown",
			"orange",
			"light blue",
			"grey",
			"pink"
		) %in%
			un
	))

	# accent substitutes accented spellings
	fd_acc <- names_hatch("fox-davies", accent = TRUE)
	expect_true("tenné" %in% fd_acc)
	expect_true("brunâtre" %in% fd_acc)
	expect_true("cendrée" %in% fd_acc)
	expect_true("bleu céleste" %in% fd_acc)
	expect_false("tenne" %in% fd_acc)

	# subtype matching is case-insensitive and ignores hyphens
	expect_equal(names_hatch("Fox-Davies"), names_hatch("fox-davies"))
	expect_equal(names_hatch("UNICODE"), names_hatch("unicode"))
	expect_equal(names_hatch("Goodman"), names_hatch("goodman"))
})

test_that("`grid.pattern_hatch()` warns on unsupported type/subtype combinations", {
	x <- c(0, 0, 1, 1)
	y <- c(1, 0, 0, 1)
	render <- function(expr) {
		f <- tempfile(fileext = ".pdf")
		on.exit(unlink(f))
		prev_dev <- dev.cur()
		pdf(f)
		on.exit(
			{
				suppressWarnings(dev.off())
				if (prev_dev > 1L) dev.set(prev_dev)
			},
			add = TRUE
		)
		force(expr)
	}
	# unknown type errors
	expect_error(
		render(grid.pattern_hatch(x, y, type = "notacolor")),
		"Unknown hatching type"
	)
	# type valid globally but not in this subtype errors
	expect_error(
		render(grid.pattern_hatch(x, y, type = "violet", subtype = "fox-davies")),
		"not supported by the 'fox-davies' subtype"
	)
	expect_error(
		render(grid.pattern_hatch(x, y, type = "proper", subtype = "goodman")),
		"not supported by the 'goodman' subtype"
	)
})

test_that("`assert_suggested()` works as expected", {
	expect_error(
		assert_suggested("doesnotexist", pattern = "blueberry"),
		"The suggested package \\{doesnotexist\\} must be installed in order to use the \"blueberry\" pattern"
	)
	expect_error(
		assert_suggested("doesnotexist", fn = "blueberry"),
		"The suggested package \\{doesnotexist\\} must be installed in order to use `blueberry\\(\\)`"
	)
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
can_render <- capabilities("png") || guess_has_R4.1_features("masks")
can_run_aqp <- requireNamespace("aqp", quietly = TRUE) &&
    (interactive() || identical(Sys.getenv("IN_PKGDOWN"), "true") || identical(Sys.getenv("NOT_CRAN"), "true"))

## ----setup--------------------------------------------------------------------
library("grid")
library("gridpattern")

## ----names--------------------------------------------------------------------
names_hatch()
names_hatch("fox-davies")
names_hatch("goodman")
names_hatch("unicode")

## ----combinatorial-table, fig.alt = "Color table showing the Combinatorial Petra Sancta tinctures arranged by achromatic, primaries, secondaries, and notable combinations", fig.width = 7, fig.height = 8.5, eval = can_render && can_run_aqp, echo = FALSE----
# # Munsell primary colors
# p_col <- c(
#     argent  = "#FFFFFF",
#     sable   = "#000000",
#     gules   = "#C83030", # 5R 4/14
#     # or      = "#E8C840", # 5Y 8/12
#     or      = "#D4B828", # 5Y 7/12
#     azure   = "#0072B0", # 5B 4/10
#     vert    = "#008060", # 5G 5/10
#     # purpure = "#7B4090" # 5P 4/10
#     purpure = "#9050C0" # 5P 6/12
# )
# # p_col <- c(
# # 	argent  = aqp::parseMunsell("N 9.5/"),
# # 	azure   = aqp::parseMunsell("5B 4/10"),
# # 	gules   = aqp::parseMunsell("5R 4/14"),
# # 	or      = aqp::parseMunsell("5Y 7/12"),
# # 	sable   = aqp::parseMunsell("N 1/"),
# #     purpure = aqp::parseMunsell("5P 6/12"),
# #     vert    = aqp::parseMunsell("5G 5/10")
# # )
# 
# # Five Munsell secondary hues via subtractive mixing
# s_col <- c(
#     orange   = mix_col(c(p_col["gules"],   p_col["or"])),       # YR: red + yellow
#     lime     = mix_col(c(p_col["or"],       p_col["vert"])),     # GY: yellow + green
#     teal     = mix_col(c(p_col["azure"],    p_col["vert"])),     # BG: blue + green
#     violet   = mix_col(c(p_col["azure"],    p_col["purpure"])),  # PB: blue + purple
#     sanguine = mix_col(c(p_col["gules"],    p_col["purpure"]))   # RP: red + purple
# )
# 
# # Notable combination colors
# w_col <- c(
#     carnation      = mix_col(c(p_col["argent"],  p_col["gules"])),
#     cendree        = mix_col(c(p_col["argent"],  p_col["sable"])),
#     mint           = mix_col(c(p_col["argent"],  p_col["vert"])),
#     `bleu celeste` = mix_col(c(p_col["argent"],  p_col["azure"])),
#     lavender       = mix_col(c(p_col["argent"],  p_col["purpure"]))
# )
# o_col <- c(
#     tenne          = mix_col(c(p_col["gules"],   p_col["vert"])),
#     slate          = mix_col(c(p_col["purpure"], p_col["vert"])),
#     olive          = mix_col(c(p_col["or"],       p_col["sable"])),
#     rose           = mix_col(c(p_col["or"],       p_col["purpure"])),
#     brunatre       = mix_col(c(p_col["azure"],    p_col["gules"],   p_col["vert"]))
# )
# 
# groups <- list(
#     list(
#         label     = "Achromatic",
#         tinctures = c("argent", "sable"),
#         cols      = p_col[c("argent", "sable")],
#         names     = c("white (W)", "black (K)")
#     ),
#     list(
#         label     = "Munsell Primary Hues",
#         tinctures = c("gules", "or", "vert", "azure", "purpure"),
#         cols      = p_col[c("gules", "or", "vert", "azure", "purpure")],
#         names     = c("red (R)", "yellow (Y)", "green (G)", "blue (B)", "purple (P)")
#     ),
#     list(
#         label     = "Munsell Secondary Hues",
#         tinctures = c("orange", "lime", "teal", "violet", "sanguine"),
#         cols      = s_col,
#         names     = c("orange (R+Y)", "lime (Y+G)", "teal (G+B)", "violet (B+P)", "magenta (P+R)")
#     ),
#     list(
#         label     = "Combinations with White",
#         tinctures = c("carnation", "cendree", "mint", "bleu celeste", "lavender"),
#         cols      = w_col,
#         names     = c("pink (R+W)", "grey (K+W)", "mint (G+W)", "light blue (B+W)", "lavender (P+W)")
#     ),
#     list(
#         label     = "Other Combinations*",
#         tinctures = c("tenne", "olive", "slate", "brunatre", "rose"),
#         cols      = o_col[c("tenne", "olive", "slate", "brunatre", "rose")],
#         names     = c("brown (R+G)", "olive (Y+K)", "slate (G+P)", "umbre (B+R+G)", "rose (Y+P)")
#     )
# )
# 
# rx <- c(0, 0, 1, 1)
# ry <- c(1, 0, 0, 1)
# ncols_fig <- 5L
# 
# row_heights <- unlist(lapply(groups, function(g) {
#     n_sr <- ceiling(length(g$tinctures) / ncols_fig)
#     c(0.28, rep(1, n_sr))
# }))
# 
# grid.newpage()
# grid.rect(gp = gpar(fill = "white", col = NA))
# pushViewport(viewport(width = 0.97, height = 0.97))
# grid.text(
#     "Combinatorial Petra Sancta",
#     y = unit(1, "npc") - unit(0.25, "cm"),
#     just = "top",
#     gp = gpar(fontsize = 31, fontface = "bold")
# )
# 
# # Upper-right rules legend
# pushViewport(viewport(
#     x = unit(1, "npc") - unit(0.2, "cm"),
# 	y = unit(1, "npc") - unit(1.3, "cm"),
#     just = c("right", "top"),
#     width = unit(10.0, "cm"), height = unit(3.1, "cm")
# ))
# grid.rect(gp = gpar(fill = "grey98", col = "grey60", lwd = 0.8))
# grid.text("Combination rules:", x = 0.01, y = 0.91, just = c("left", "top"),
#           gp = gpar(fontsize = 14, fontface = "bold"))
# legend_rules <- c(
#     "1. Dashed lines — combined with white",
#     "2. Dot-dash lines — combined with yellow",
#     "3. Crossed solid lines — mixed colors (if not black)"
# )
# for (i in seq_along(legend_rules)) {
#     grid.text(legend_rules[i], x = 0.01, y = 0.66 - (i - 1L) * 0.25,
#               just = c("left", "top"), gp = gpar(fontsize = 11))
# }
# popViewport()
# 
# pushViewport(viewport(
#     y = 0.49, height = 0.90,
#     layout = grid.layout(length(row_heights), ncols_fig, heights = unit(row_heights, "null"))
# ))
# 
# layout_row <- 1L
# for (g in groups) {
#     pushViewport(viewport(layout.pos.row = layout_row, layout.pos.col = 1:ncols_fig))
#     grid.text(g$label, x = 0.01, just = "left",
#               gp = gpar(fontsize = 18, fontface = "bold", col = "black"))
#     popViewport()
#     layout_row <- layout_row + 1L
# 
#     n_sr <- ceiling(length(g$tinctures) / ncols_fig)
#     for (sr in seq_len(n_sr)) {
#         idx_from <- (sr - 1L) * ncols_fig + 1L
#         idx_to   <- min(sr * ncols_fig, length(g$tinctures))
#         for (ci in idx_from:idx_to) {
#             t     <- g$tinctures[ci]
#             col_i <- (ci - 1L) %% ncols_fig + 1L
#             if (is.na(t)) next
#             col   <- unname(g$cols[ci])
#             nm    <- g$names[ci]
#             display_col <- if (t == "argent") "grey55" else col
# 
#             pushViewport(viewport(layout.pos.row = layout_row, layout.pos.col = col_i))
#             pushViewport(viewport(y = 0.58, width = 0.90, height = 0.72,
#                                   layout = grid.layout(1, 2)))
#             pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
#             grid.rect(gp = gpar(fill = col, col = display_col, lwd = 1.5))
#             popViewport()
#             pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
#             grid.pattern_hatch(rx, ry, type = t, color = display_col,
#                                spacing = 0.18, linewidth = 0.8)
#             grid.rect(gp = gpar(fill = NA, col = display_col, lwd = 1.5))
#             popViewport()
#             popViewport()
#             grid.text(nm, y = unit(0.105, "npc"), gp = gpar(fontsize = 9, col = "black"))
#             popViewport()
#         }
#         layout_row <- layout_row + 1L
#     }
# }
# 
# popViewport()
# 
# # Footnote
# grid.text(
#     "* Display colors are sensitive to the exact primary pigments chosen;\n  results are roughly consistent for saturated heraldic primaries with Munsell pigment mixing.",
#     x = 0.01, y = 0.004, just = c("left", "bottom"),
#     gp = gpar(fontsize = 10, col = "black", fontface = "italic")
# )
# 
# popViewport()

## ----fox-davies-shields, fig.alt = "Heraldic shields showing the Fox-Davies hatching tinctures", fig.width = 7, fig.height = 6.0, eval = can_render, echo = FALSE----
# Approximate display color for each tincture
tincture_col <- c(
    argent         = "grey40",
    azure          = "#003399",
    `bleu celeste` = "#4499CC",
    brunatre       = "#7B3A10",
    carnation      = "#CC6688",
    cendree        = "#708090",
    gules          = "#CC0000",
    eisenfarbe     = "#708090",
    proper         = "#228B22",
    or             = "#DAA520",
    orange         = "#EE7700",
    purpure        = "#660099",
    sable          = "#111111",
    sanguine       = "#880000",
    tenne          = "#BB6600",
    vert           = "#006400"
)
color_equiv <- c(
    argent         = "white/silver",
    azure          = "blue",
    `bleu celeste` = "light blue",
    brunatre       = "(earth) brown",
    carnation      = "carnation",
    cendree        = "ash grey",
    gules          = "red",
    eisenfarbe     = "iron grey",
    proper         = "color of nature",
    or             = "yellow/gold",
    orange         = "orange",
    purpure        = "purple",
    sable          = "black",
    sanguine       = "blood red",
    tenne          = "(tawny) brown",
    vert           = "green"
)

# Heater shield polygon (normalised to [0,1] x [0,1])
sx <- c(0.0, 0.0, 0.5, 1.0, 1.0)
sy <- c(1.0, 0.35, 0.0, 0.35, 1.0)

tinctures <- names_hatch("fox-davies")
tincture_labels <- names_hatch("fox-davies", accent = TRUE)
n <- length(tinctures)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.97, height = 0.97))
grid.text(
    "Heraldic Hatching (Petra Sancta + German Heraldry Extensions)",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(y = 0.47, height = 0.90, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    t <- tinctures[i]
    col <- tincture_col[t]
    row_i <- ((i - 1L) %/% ncols) + 1L
    col_i <- ((i - 1L) %% ncols) + 1L

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(y = 0.60, width = 0.78, height = 0.70))

    grid.polygon(sx, sy, gp = gpar(fill = "white", col = NA))
    grid.pattern_hatch(sx, sy, type = t, subtype = "fox-davies", color = col, spacing = 0.12, linewidth = 1.0)
    grid.polygon(sx, sy, gp = gpar(fill = NA, col = col, lwd = 1.5))

    popViewport()

    grid.text(tincture_labels[i], y = unit(0.20, "npc"), gp = gpar(fontsize = 8.5, col = "grey20"))
    grid.text(color_equiv[t], y = unit(0.06, "npc"), gp = gpar(fontsize = 7.5, col = col))

    popViewport()
}

popViewport()
popViewport()

## ----goodman-shields, fig.alt = "Heraldic shields showing Goodman tinctures that differ from Fox-Davies", fig.width = 7, fig.height = 4.0, eval = can_render, echo = FALSE----
tincture_col <- c(
    sanguine = "#880000",
    murrey   = "#990055",
    steel    = "#708090",
    copper   = "#B87333"
)

sx <- c(0.0, 0.0, 0.5, 1.0, 1.0)
sy <- c(1.0, 0.35, 0.0, 0.35, 1.0)

tinctures <- names(tincture_col)
n <- length(tinctures)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.97, height = 0.97))
grid.text(
    "Goodman — New and Different Tinctures",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(y = 0.44, height = 0.85, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    t <- tinctures[i]
    col <- tincture_col[t]
    row_i <- ((i - 1L) %/% ncols) + 1L
    col_i <- ((i - 1L) %% ncols) + 1L

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(y = 0.60, width = 0.78, height = 0.70))

    grid.polygon(sx, sy, gp = gpar(fill = "white", col = NA))
    grid.pattern_hatch(sx, sy, type = t, subtype = "goodman", color = col, spacing = 0.12, linewidth = 0.8)
    grid.polygon(sx, sy, gp = gpar(fill = NA, col = col, lwd = 1.5))

    popViewport()

    grid.text(t, y = unit(0.20, "npc"), gp = gpar(fontsize = 18, col = "black"))
    popViewport()
}

popViewport()
popViewport()

## ----unicode-hearts, fig.alt = "Twelve Unicode colored hearts rendered with hatching patterns", fig.width = 7, fig.height = 6, eval = requireNamespace("Unicode", quietly = TRUE) && can_render, echo = FALSE----
library("Unicode")

# The 12 Unicode colored hearts in codepoint order
heart_codepoints <- c(
    red          = 0x2764L, # HEAVY BLACK HEART (displays as red via emoji VS)
    blue         = 0x1F499L,
    green        = 0x1F49AL,
    yellow       = 0x1F49BL,
    purple       = 0x1F49CL,
    black        = 0x1F5A4L,
    white        = 0x1F90DL,
    brown        = 0x1F90EL,
    orange       = 0x1F9E1L,
    `light blue` = 0x1FA75L,
    grey         = 0x1FA76L,
    pink         = 0x1FA77L
)

# Approximate display colors
heart_col <- c(
    red          = "#CC0000",
    blue         = "#0055CC",
    green        = "#006400",
    yellow       = "#CCAA00",
    purple       = "#6600AA",
    black        = "#111111",
    white        = "#999999", # grey stroke so argent pattern is visible
    brown        = "#7B3A10",
    orange       = "#FF8000",
    `light blue` = "#4499CC",
    grey         = "#666666",
    pink         = "#DD4488"
)

uchars <- as.u_char(as.integer(heart_codepoints))
labels <- u_char_name(uchars)
heart_shape <- "♥" # U+2665 BLACK HEART SUIT — uniform shape template

n <- length(heart_codepoints)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.95, height = 0.95))
grid.text(
    "Unicode Colored Hearts with Hatching",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 22, fontface = "bold")
)
pushViewport(viewport(y = 0.48, height = 0.90, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    col_i <- ((i - 1L) %% ncols) + 1L
    row_i <- ((i - 1L) %/% ncols) + 1L
    col <- heart_col[i]

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(width = 0.85, height = 0.85))

    pfill <- patternFill(
        "hatch",
        type      = names(heart_codepoints)[i],
        subtype   = "unicode",
        color    = col,
        spacing   = 0.14,
        linewidth = 0.8
    )
    grid.draw(
        fillStrokeGrob(
            textGrob(heart_shape, gp = gpar(fontsize = 84)),
            gp = gpar(fill = pfill, col = col)
        )
    )

    grid.text(labels[i], y = unit(0.12, "npc"),
              gp = gpar(fontsize = 12, col = "black"))
    grid.text(sprintf("U+%04X", heart_codepoints[i]), y = unit(0.00, "npc"),
              gp = gpar(fontsize = 10, col = "black"))

    popViewport()
    popViewport()
}

popViewport()
popViewport()

## ----okabe-ito, fig.alt = "Table of Okabe-Ito palette colors paired with heraldic hatching patterns", fig.width = 6, fig.height = 6, eval = can_render----
oi_names <- c(
    "black", "orange", "sky blue", "bluish green",
    "yellow", "blue", "vermilion", "reddish purple", "white"
)
oi_hex <- c(
    "#000000", "#E69F00", "#56B4E9", "#009E73",
    "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#FFFFFF"
)
oi_hatch <- c(
    NA, "orange", "bleu celeste", "vert",
    "or", "azure", "gules", "purpure", NA
)
sx <- c(0, 0, 1, 1)
sy <- c(1, 0, 0, 1)
n <- length(oi_names)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.90, height = 0.94))
grid.text(
    "Okabe-Ito Palette with Heraldic Hatching",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(
    y = 0.46, height = 0.88,
    layout = grid.layout(
        n, 3,
        widths = unit(c(3, 2.5, 4), "null"),
        heights = unit(rep(1, n), "null")
    )
))

for (i in seq_len(n)) {
    grid.text(oi_names[i], x = 0.90, just = "right",
              gp = gpar(fontsize = 12, col = "black"),
	          vp = viewport(layout.pos.row = i, layout.pos.col = 1))

    grid.text(oi_hex[i],
              gp = gpar(fontsize = 12, fontfamily = "mono", col = "black"),
	          vp = viewport(layout.pos.row = i, layout.pos.col = 2))

    pushViewport(viewport(layout.pos.row = i, layout.pos.col = 3))
    grid.rect(gp = gpar(fill = oi_hex[i], col = "black", lwd = 3.0))
    if (!is.na(oi_hatch[i])) {
        grid.pattern_hatch(sx, sy, type = oi_hatch[i],
                           colour = "black", spacing = 0.18, linewidth = 0.8)
    }
    popViewport()
}

popViewport()
popViewport()

#' Sanity check an argument value
#'
#' This is a milder, more explicit version of \code{match.arg} which gives a warning
#' and returns a good default.  match.arg() would throw errors which makes
#' geom/pattern development a bit trickier.
#'
#' @param x argument value
#' @param options valid options this value could take. If this is not NULL, then
#'        x is checked to be a member of this set.  Default: NULL.
#' @param default the default value to use if the argument fails the checks.
#'        If default is not NULL, then use it as the fallback value.
#'        If default is NULL and options is not NULL, then use the first value
#'        in \code{options}.
#' @param type NULL, 'num' or 'char'
#' @param prefix prefix of warning message.
#'
#' @return original \code{x} value if there are no issues, otherwise the \code{default}
#'         if given, otherwise the first element in \code{options}.
#' @noRd
check_default <- function(x, options = NULL, default = NULL, type = NULL, prefix = "") {
	stopifnot(is.null(options) || is.atomic(options))

	default <- default %||% (options[1])
	if (is.null(default) || length(default) != 1) {
		abort("check_default(): Must specify 'default' or 'options'")
	}

	if (length(x) != 1) {
		res <- default
	} else if (!is.null(options) && !x %in% options) {
		res <- default
	} else {
		res <- x
	}

	if (!is.null(type)) {
		res <- switch(
			type,
			numeric = ,
			number = ,
			float = ,
			num = ifelse(is.numeric(res), res, default),
			character = ,
			chr = ,
			char = ifelse(is.character(res), res, default),
			{
				abort(paste0("check_default(): Don't know how to check for type: ", type))
			}
		)
	}

	res
}

#' abind clone for adding a matrix to an array
#'
#' A very cut-down version of \code{abind::abind()} that will only stack a
#' matrix onto an array.
#'
#' @param arr,mat array and matrix to be stacked
#'
#' @return new array with matrix added as a new plane at the end of the array
#' @noRd
my_abind <- function(arr, mat) {
	stopifnot(is.array(arr))
	stopifnot(is.matrix(mat))
	if (!identical(utils::head(dim(arr), -1), dim(mat))) {
		abort(glue("Dimension missmatch. Array: {deparse(dim(arr))} Matrix: {deparse(dim(mat))}"))
	}

	new_dim <- dim(arr)
	new_dim[3] <- new_dim[3] + 1

	array(c(arr, mat), dim = new_dim)
}

#' Generate an RGBA array and create a raster grob to fill the region
#'
#' the boundary of the shaded region will be used to generate a alpha mask
#' so that the array conforms to the shape.
#'
#' @inheritParams create_pattern_none
#' @param array_fn Array function to use
#'
#' @return rasterGrob
#'
#' @noRd
create_pattern_array <- function(
	params,
	boundary_df,
	aspect_ratio,
	legend,
	array_fn = create_magick_pattern_as_array
) {
	if (anyNA(boundary_df$x) || anyNA(boundary_df$y)) {
		return(grid::nullGrob())
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# What is the size of the bounding box of the boundary for this pattern?
	# Calculate the centre (x,y) and (width,height)
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	npc_xrange <- range(boundary_df$x, na.rm = TRUE)
	npc_yrange <- range(boundary_df$y, na.rm = TRUE)
	npc_width <- abs(diff(npc_xrange))
	npc_height <- abs(diff(npc_yrange))
	npc_x <- mean(npc_xrange)
	npc_y <- mean(npc_yrange)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Calculate the dimensions of the bounding box and use the integer
	# values of these to define the image dimensions
	# res of 72 DPI is the imagemagick default
	# Ideally could detect resolution of graphics device and use that as default...
	# NB. large pixel sizes can cause errors with getting 'placeholder' images
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	res <- params$pattern_res # defaults to 72 DPI
	in_vp_width <- grid::convertWidth(unit(1, 'npc'), 'inches', valueOnly = TRUE)
	in_width <- npc_width * in_vp_width
	in_height <- npc_height * in_vp_width / aspect_ratio
	arr_width <- as.integer(res * in_width)
	arr_height <- as.integer(res * in_height)
	if (arr_width == 0L || arr_height == 0L) {
		return(nullGrob(name = params$pattern))
	}

	# Override type for better looking legend when tiling
	if (legend) {
		if (params$pattern_type %in% c('tile', 'none')) {
			params$pattern_type <- 'fit'
		}
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# The image and the boundary_df bbox are coincident.  To mask the image,
	# scale the boundary_df to encompass the full 'npc' range from 0 to 1.
	# An alpha mask will be created to encompass the whole of the image, and
	# the resulting image will be placed at the bbox location.
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	scaled_boundary_df <- boundary_df
	scaled_boundary_df$x <- (scaled_boundary_df$x - npc_xrange[1]) / npc_width
	scaled_boundary_df$y <- (scaled_boundary_df$y - npc_yrange[1]) / npc_height

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# check for issues e.g. Zero area regions
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (any(is.nan(scaled_boundary_df$x)) || any(is.nan(scaled_boundary_df$y))) {
		return(grid::nullGrob())
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Fetch an image of the required dimensions.
	# Create a mask of the required dimensions from the scaled boundary_df
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	rgba_arr <- array_fn(arr_width, arr_height, params, legend)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Sanity check the array conforms to what we want.  This is especially
	# important as we're allowing users to generate arrays for patterns
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (
		!(is.array(rgba_arr) &&
			is.numeric(rgba_arr) &&
			length(dim(rgba_arr) == 3) &&
			dim(rgba_arr)[3] == 4 &&
			dim(rgba_arr)[1] == arr_height &&
			dim(rgba_arr)[2] == arr_width &&
			!anyNA(rgba_arr) &&
			min(rgba_arr) >= 0 &&
			max(rgba_arr) <= 1)
	) {
		warn(glue(
			"create_pattern_array(): Expecting a numeric RGBA array with dim = c({arr_height}, {arr_width}, 4) ",
			"but instead got a {deparse(class(rgba_arr))} ",
			"of type {typeof(rgba_arr)} with dimensions {deparse(dim(rgba_arr))}"
		))

		rgba_arr <- array(c(0, 1), dim = c(arr_height, arr_width, 4))
	}

	boundary_mask <- convert_polygon_df_to_alpha_channel(
		scaled_boundary_df,
		width = arr_width,
		height = arr_height
	)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Stack the current image array with an alpha channel.
	# Using a custom version of `abind::abind()` so I could avoid having it
	# as another package dependency
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (is.na(params$pattern_alpha)) {
		alpha <- 1
	} else {
		alpha <- params$pattern_alpha
	}
	rgba_arr[,, 4] <- rgba_arr[,, 4] * boundary_mask * alpha

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a rasterGrob image at the location of the 'boundary_df' bounding box.
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	raster_grob <- rasterGrob(
		rgba_arr,
		x = npc_x,
		y = npc_y,
		width = npc_width,
		height = npc_height,
		name = params$pattern
	)
	raster_grob
}

# We use `<<-` below to modify the package's namespace
# in order to prevent a build time dependency on `memoise`
# as recommended in <http://memoise.r-lib.org/reference/memoise.html#details>.
# We don't modify the global environment.
# See <https://github.com/r-lib/memoise/issues/76> for further details.

# Define function at build time
img_read_memoised <- img_read

# Modify function at load time
.onLoad <- function(libname, pkgname) {
	img_read_memoised <<- memoise::memoise(img_read)
}

#' @import grid
#' @importFrom glue glue
#' @importFrom grDevices col2rgb dev.capture dev.capabilities dev.off png rgb
#' @importFrom rlang %||% abort inform warn
#' @importFrom utils hasName head packageVersion tail
NULL

#' Line patterned grobs
#'
#' `grid.pattern_line()` draws a line pattern onto the graphic device.
#' Unlike [grid.pattern_stripe()] which fills bands with solid colour,
#' this pattern draws stroked lines using the device's native line rendering,
#' enabling all of R's built-in `linetype` values (including `"dotdash"`,
#' `"twodash"`, and custom line types specified as hex strings per `?par`).
#'
#' @inheritParams grid.pattern_circle
#' @inheritParams alphaMaskGrob
#' @param lineend Line end style, one of `"round"` (default), `"butt"`, or `"square"`.
#' @param stagger If `TRUE`, alternate lines are shifted by half the dash period so that
#'   dashes of adjacent lines interleave.
#'   Computed from `linetype` and `linewidth` per `?par`.  Default `FALSE`.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   grid::grid.newpage()
#'   grid.pattern_line(x_hex, y_hex, colour = "black", linetype = "dotdash",
#'                     angle = 45, spacing = 0.1, stagger = TRUE)
#' }
#'
#' # more intricate dashed lines are possible with hex strings
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   grid::grid.newpage()
#'   grid.pattern_line(x_hex, y_hex, gp = grid::gpar(col = "darkred", lty = "23632E"))
#' }
#' @seealso [grid.pattern_stripe()] for filled bands, [grid.pattern_wave()] for wavy bands, [grid.pattern_crosshatch()] for two sets of lines.
#' @export
grid.pattern_line <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	angle = 30,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	lineend = gp$lineend %||% "round",
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	stagger = FALSE,
	use_R4.1_masks = getOption(
		"ggpattern_use_R4.1_masks",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"line",
		x,
		y,
		id,
		colour = colour,
		angle = angle,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		lineend = lineend,
		stagger = stagger,
		use_R4.1_masks = use_R4.1_masks,
		png_device = png_device,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

# blank or solid lines return NULL
lty_pattern_str <- function(lty) {
	if (is.numeric(lty)) {
		lty <- c("blank", "solid", "dashed", "dotted", "dotdash", "longdash", "twodash")[lty + 1L]
	}
	switch(
		lty,
		blank = ,
		solid = NULL,
		dashed = "44",
		dotted = "13",
		dotdash = "1343",
		longdash = "73",
		twodash = "2262",
		lty
	)
}

# blank or solid lines return 0L
lty_period_sum <- function(lty) {
	pat <- lty_pattern_str(lty)
	if (is.null(pat)) {
		return(0L)
	}
	sum(strtoi(strsplit(pat, "")[[1L]], base = 16L))
}

# Expand one row of a staggered pattern into explicit solid on-segments.
# phase_offset_lwd: phase in lwd units (may be fractional, e.g. 5.5 for dotdash half-period).
# Returns list(x0, x1, y) — equal-length vectors.
expand_lty_row <- function(lty, lwd, x_min, x_max, y_row, phase_offset_lwd) {
	pat <- lty_pattern_str(lty)
	if (is.null(pat)) {
		if (lty_period_sum(lty) == 0L) {
			return(list(x0 = x_min, x1 = x_max, y = y_row))
		}
		return(list(x0 = numeric(0L), x1 = numeric(0L), y = numeric(0L)))
	}
	units_lwd <- strtoi(strsplit(pat, "")[[1L]], base = 16L)
	n <- length(units_lwd)
	period_lwd <- sum(units_lwd)
	phase <- phase_offset_lwd %% period_lwd
	cumul_lwd <- c(0, cumsum(units_lwd))
	seg_idx <- findInterval(phase, cumul_lwd[-length(cumul_lwd)])
	offset_in_seg <- phase - cumul_lwd[seg_idx]
	x0s <- numeric(0L)
	x1s <- numeric(0L)
	x_cur <- x_min
	first <- TRUE
	i <- seg_idx
	while (x_cur < x_max) {
		rem_lwd <- if (first) {
			first <- FALSE
			units_lwd[i] - offset_in_seg
		} else {
			units_lwd[i]
		}
		if (rem_lwd > 0) {
			x_end <- min(x_cur + rem_lwd * lwd, x_max)
			if (i %% 2L == 1L) {
				x0s <- c(x0s, x_cur)
				x1s <- c(x1s, x_end)
			}
			x_cur <- x_end
		}
		i <- i %% n + 1L
	}
	list(x0 = x0s, x1 = x1s, y = rep(y_row, length(x0s)))
}

# Build the segments grob for one set of lines (angle/lty/stagger already set in params).
# params must already be unit-converted (via convert_params_units).
# vpm must be from get_vp_measurements(default.units).
create_line_maskee <- function(params, vpm, default.units = "bigpts") {
	grid_xy <- get_xy_grid(params, vpm)

	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype
	lineend <- params$pattern_lineend
	gp <- gpar(col = col, lwd = lwd, lty = lty, lineend = lineend)

	x0 <- rep(grid_xy$x_min, length(grid_xy$y))
	x1 <- rep(grid_xy$x_max, length(grid_xy$y))
	seg_y <- grid_xy$y

	# Stagger alternating lines by half the dash period so that marks of
	# adjacent lines interleave (heraldic convention).  Explicit solid
	# sub-segments are used so the phase is device-independent (relying on a
	# segment start outside the viewport breaks dash phase on some devices).
	if (isTRUE(params$pattern_stagger)) {
		half_period_lwd <- lty_period_sum(lty) / 2
		if (half_period_lwd > 0) {
			segs <- lapply(seq_along(grid_xy$y), function(i) {
				phase <- if (i %% 2L == 0L) half_period_lwd else 0
				expand_lty_row(lty, lwd, grid_xy$x_min, grid_xy$x_max, grid_xy$y[i], phase)
			})
			x0_list <- lapply(segs, `[[`, "x0")
			x0 <- unlist(x0_list, use.names = FALSE)
			x1 <- unlist(lapply(segs, `[[`, "x1"), use.names = FALSE)
			seg_y <- unlist(lapply(segs, `[[`, "y"), use.names = FALSE)
			# Expand per-line gp values to per-sub-segment so recycling
			# matches the non-stagger path (one gp value per line, not per dash).
			n_sub <- lengths(x0_list)
			line_idx <- rep(seq_along(grid_xy$y), n_sub)
			recycle_to_lines <- function(v) v[(line_idx - 1L) %% length(v) + 1L]
			gp <- gpar(
				col = recycle_to_lines(col),
				lwd = recycle_to_lines(lwd),
				lineend = recycle_to_lines(lineend),
				lty = "solid"
			)
		}
	}

	xy0 <- rotate_xy(x0, seg_y, params$pattern_angle, vpm$x, vpm$y)
	xy1 <- rotate_xy(x1, seg_y, params$pattern_angle, vpm$x, vpm$y)

	segmentsGrob(xy0$x, xy0$y, xy1$x, xy1$y, default.units = default.units, gp = gp)
}

create_pattern_line <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	maskee <- create_line_maskee(params, vpm, default.units)
	masker <- convert_polygon_df_to_polygon_grob(
		boundary_df,
		default.units = default.units,
		gp = gpar(fill = "white", col = NA, lwd = 0)
	)
	alphaMaskGrob(
		maskee,
		masker,
		use_R4.1_masks = params$pattern_use_R4.1_masks,
		png_device = params$pattern_png_device,
		res = params$pattern_res,
		name = "line"
	)
}

assert_suggested <- function(package, pattern = NULL, fn = NULL) {
	if (!requireNamespace(package, quietly = TRUE)) {
		context <- if (!is.null(pattern)) {
			glue('in order to use the "{pattern}" pattern.')
		} else {
			glue("in order to use `{fn}()`.")
		}
		abort(c(
			glue("The suggested package {{{package}}} must be installed {context}"),
			i = glue('Install with the command `install.packages("{package}")`')
		))
	}
}

# base R's Cairo/Quartz devices as well as {ragg} / {svglite} / {vdiffr} devices
# should support Unicode without complaint
# Notably `pdf()` is a device that does not...
# Any other devices to add?
device_supports_unicode <- function() {
	device <- names(grDevices::dev.cur())
	unicode_devices <- c(
		"agg_capture",
		"agg_jpeg",
		"agg_ppm",
		"agg_png",
		"agg_record",
		"agg_tiff", # {ragg}
		"devSVG",
		"devSVG_vdiffr", # {svglite} / {vdiffr}
		"quartz",
		"quartz_off_screen", # Quartz
		"cairo_pdf",
		"cairo_ps",
		"svg",
		"X11cairo"
	) # Cairo
	if (
		any(vapply(
			unicode_devices,
			function(x) grepl(paste0("^", x), device),
			FUN.VALUE = logical(1L)
		))
	) {
		TRUE
	} else if (device %in% c("bmp", "jpeg", "png", "tiff")) {
		# on unix non-"cairo" type have different device names from "cairo" type
		# but on Windows can't distinguish between `type = "windows"` or `type = "cairo"`
		# Windows device doesn't support new patterns feature
		if (getRversion() >= "4.2.0") {
			"LinearGradient" %in% grDevices::dev.capabilities()$patterns
		} else {
			.Platform$OS.type == "unix"
		}
	} else {
		FALSE
	}
}

# returns list of pattern parameters using defaults if necessary
get_params <- function(..., pattern = "none", prefix = "pattern_", gp = gpar()) {
	l <- list(...)
	if (length(l)) {
		names(l) <- paste0(prefix, names(l))
	}
	l$pattern <- pattern

	# possibly get from gpar()
	l$pattern_alpha <- l$pattern_alpha %||% gp$alpha %||% NA_real_
	l$pattern_colour <- l$pattern_colour %||% l$pattern_color %||% gp$col %||% "grey20"
	l$pattern_fill <- l[["pattern_fill"]] %||% gp$fill %||% "grey80"
	l$pattern_lineend <- l$pattern_lineend %||% gp$lineend %||% "round"
	l$pattern_linetype <- l$pattern_linetype %||% gp$lty %||% 1
	l$pattern_linewidth <- l$pattern_linewidth %||% l$pattern_size %||% gp$lwd %||% 1
	if (pattern == "text") {
		l$pattern_size <- l$pattern_size %||% gp$fontsize %||% 12
	} else {
		l$pattern_size <- l$pattern_size %||% gp$lwd %||% 1
	}
	l$pattern_fontfamily <- l$pattern_fontfamily %||% gp$fontfamily %||% "sans"
	l$pattern_fontface <- l$pattern_fontface %||% gp$fontface %||% "plain"

	# never get from gpar()
	l$pattern_angle <- l$pattern_angle %||% 30
	l$pattern_aspect_ratio <- l$pattern_aspect_ratio %||% NA_real_
	l$pattern_density <- l$pattern_density %||% 0.2
	l$pattern_filename <- l$pattern_filename %||% ""
	l$pattern_fill2 <- l$pattern_fill2 %||%
		switch(pattern, crosshatch = l$pattern_fill, "#4169E1")
	l$pattern_filter <- l$pattern_filter %||%
		switch(pattern, magick = "box", "lanczos")
	l$pattern_grid <- l$pattern_grid %||% "square"
	l$pattern_key_scale_factor <- l$pattern_key_scale_factor %||% 1
	l$pattern_orientation <- l$pattern_orientation %||% "vertical"
	l$pattern_rot <- l$pattern_rot %||% 0
	l$pattern_shape <- l$pattern_shape %||%
		switch(pattern, regular_polygon = "convex4", 1)
	l$pattern_scale <- l$pattern_scale %||%
		switch(pattern, regular_polygon = 0.5, 1)
	l$pattern_spacing <- l$pattern_spacing %||% 0.05
	# l$pattern_subtype <- l$pattern_subtype
	l$pattern_type <- l$pattern_type %||% default_pattern_type(pattern)
	if (is.na(l$pattern_type)) {
		l$pattern_type <- default_pattern_type(pattern)
	}
	l$pattern_units <- l$pattern_units %||% "snpc"
	l$pattern_reverse <- l$pattern_reverse %||% FALSE
	l$pattern_stagger <- l$pattern_stagger %||% FALSE
	l$pattern_xoffset <- l$pattern_xoffset %||% 0
	l$pattern_yoffset <- l$pattern_yoffset %||% 0

	l$pattern_gravity <- l$pattern_gravity %||%
		switch(l$pattern_type, tile = "southwest", "center")
	if (is.na(l$pattern_gravity)) {
		l$pattern_gravity <- switch(l$pattern_type, tile = "southwest", "center")
	}

	l$pattern_res <- l$pattern_res %||% getOption("ggpattern_res", 72) # in PPI

	# Additional ambient defaults
	l$pattern_frequency <- l$pattern_frequency %||%
		switch(pattern, ambient = 0.01, rose = 0.1, 1 / l$pattern_spacing)
	l$pattern_interpolator <- l$pattern_interpolator %||% "quintic" # perlin, simplex, value
	l$pattern_fractal <- l$pattern_fractal %||%
		switch(l$pattern_type, worley = "none", "fbm")
	l$pattern_pertubation <- l$pattern_pertubation %||% "none" # all
	l$pattern_octaves <- l$pattern_octaves %||% 3 # all but white
	l$pattern_lacunarity <- l$pattern_lacunarity %||% 2 # all but white
	l$pattern_gain <- l$pattern_gain %||% 0.5 # all but white
	l$pattern_amplitude <- l$pattern_amplitude %||%
		switch(pattern, wave = 0.5 * l$pattern_spacing, 1) # all
	l$pattern_value <- l$pattern_value %||% "cell"
	l$pattern_distance_ind <- l$pattern_distance_ind %||% c(1, 2)
	l$pattern_jitter <- l$pattern_jitter %||% 0.45

	l
}

get_R4.1_params <- function(l) {
	# R 4.1 features
	l$pattern_use_R4.1_clipping <- l$pattern_use_R4.1_clipping %||%
		getOption("ggpattern_use_R4.1_clipping") %||%
		getOption("ggpattern_use_R4.1_features") %||%
		guess_has_R4.1_features("clippingPaths")
	l$pattern_use_R4.1_gradients <- l$pattern_use_R4.1_gradients %||%
		getOption("ggpattern_use_R4.1_gradients") %||%
		getOption("ggpattern_use_R4.1_features") %||%
		guess_has_R4.1_features("gradients")
	l$pattern_use_R4.1_masks <- l$pattern_use_R4.1_masks %||%
		getOption("ggpattern_use_R4.1_masks") %||%
		getOption("ggpattern_use_R4.1_features") %||%
		guess_has_R4.1_features("masks")
	l$pattern_use_R4.1_patterns <- l$pattern_use_R4.1_patterns %||%
		getOption("ggpattern_use_R4.1_patterns") %||%
		getOption("ggpattern_use_R4.1_features") %||%
		guess_has_R4.1_features("patterns")
	l
}

convert_params_units <- function(params, units = "bigpts") {
	p_units <- params$pattern_units
	params$pattern_amplitude <- convertX(
		unit(params$pattern_amplitude, p_units),
		units,
		valueOnly = TRUE
	)
	params$pattern_spacing <- convertX(
		unit(params$pattern_spacing, p_units),
		units,
		valueOnly = TRUE
	)
	params$pattern_xoffset <- convertX(
		unit(params$pattern_xoffset, p_units),
		units,
		valueOnly = TRUE
	)
	params$pattern_yoffset <- convertX(
		unit(params$pattern_yoffset, p_units),
		units,
		valueOnly = TRUE
	)
	params$pattern_wavelength <- convertX(
		unit(1 / params$pattern_frequency, p_units),
		units,
		valueOnly = TRUE
	)
	params
}

default_pattern_type <- function(pattern) {
	switch(
		pattern,
		ambient = "simplex",
		aRtsy = "strokes",
		hatch = "gules",
		image = "fit",
		placeholder = "bear",
		polygon_tiling = "square",
		magick = "hexagons",
		wave = "indented",
		weave = "plain",
		NA_character_
	)
}

#' Plasma patterned grobs
#'
#' `grid.pattern_plasma()` draws a plasma pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_gradient
#' @param scale Extra scaling
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'   if (requireNamespace("magick")) {
#'     x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     grid.pattern_plasma(x_hex, y_hex, fill = "green")
#'   }
#' @seealso [grid.pattern_ambient()] provides a noise pattern using the `ambient` package.
#'          Pseudorandom seeds for the plasma pattern may be set via [magick::magick_set_seed()].
#' @export
grid.pattern_plasma <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	fill = gp$fill %||% "grey80",
	scale = 1,
	alpha = gp$alpha %||% NA_real_,
	aspect_ratio = 1,
	key_scale_factor = 1,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"plasma",
		x,
		y,
		id,
		fill = fill,
		scale = scale,
		alpha = alpha,
		aspect_ratio = aspect_ratio,
		key_scale_factor = key_scale_factor,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' Read a user specified filename as an image
#'
#' @inheritParams create_gradient_as_array
#'
#' @return array
#' @noRd
create_magick_plasma_as_array <- function(width, height, params, legend) {
	assert_suggested("magick", "plasma")

	colour <- as.character(params$pattern_fill)

	img <- create_magick_plasma_img(
		width = width,
		height = height,
		colour = colour
	)

	convert_img_to_array(img)
}

#' Create plasma using imagemagick
#'
#' Ref: \url{https://www.imagemagick.org/Usage/canvas/}
#'
#' @param width,height image dimensions
#' @param colour colour
#'
#' @noRd
create_magick_plasma_img <- function(width = 100, height = 100, colour) {
	colour <- convert_r_colour_to_magick_colour(colour)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a pattern image of the required size
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	pseudo <- "plasma:"
	img <- magick::image_blank(width, height, pseudo_image = pseudo)
	img <- magick::image_convert(img, colorspace = 'gray', depth = 8)
	img <- magick::image_blur(img, radius = 2)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Make the white transparent
	# Colorize the black pixels into the desired colour
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	img <- magick::image_transparent(img, 'white')
	img <- magick::image_colorize(img, opacity = 50, colour)

	img
}

#' Polygon tiling patterned grobs
#'
#' `grid.pattern_polygon_tiling()` draws a specified polygon tiling pattern onto the graphic device.
#'  `names_polygon_tiling` lists all supported types.
#'
#' `grid.pattern_polygon_tiling()` supports 1, 2, or 3 `fill` colors with the first colors (weakly)
#'  covering a larger area.  Size of the pattern is controlled by `spacing`.
#'  We support the following polygon tiling `type`s:
#'
#' \describe{
#' \item{`elongated_triangular`}{Creates an elongated triangular tiling made of squares and triangles.}
#' \item{`herringbone`}{Creates a herringbone tiling made of rectangles.}
#' \item{`hexagonal`}{Creates a hexagonal tiling made of hexagons.}
#' \item{`pythagorean`}{Creates a Pythagorean tiling made of squares of two different sizes.}
#' \item{`rhombille`}{Creates a rhombille tiling made of rhombi.}
#' \item{`rhombitrihexagonal`}{Creates a rhombitrihexagonal tiling made out of
#'                            dodecagons, hexagons, and squares.}
#' \item{`snub_square`}{Creates a snub square tiling made of squares and triangles.}
#' \item{`snub_trihexagonal`}{Creates a snub trihexagonal tiling made of hexagons and triangles.}
#' \item{`square`}{Creates a square tiling made of squares.}
#' \item{`tetrakis_square`}{Creates a tetrakis square tiling made of isosceles right triangles.}
#' \item{`triangular`}{Creates a triangular tiling made of equilateral triangles.}
#' \item{`trihexagonal`}{Creates a trihexagonal tiling made of hexagons and triangles.}
#' \item{`truncated_square`}{Creates a truncated square tiling made of octagons and squares.}
#' \item{`truncated_hexagonal`}{Creates a truncated hexagonal tiling made of dodecagons and triangles.}
#' \item{`truncated_trihexagonal`}{Creates a truncated trihexagonal tiling made of hexagons, squares, and triangles.}
#' \item{`2*.2**.2*.2**`}{Creates a polygon tiling made of rhombi.}
#' \item{`2**.3**.12*`}{Creates a polygon tiling made of rhombi, triangles, and twelve-pointed stars.}
#' \item{`3.3.3.3**`}{Creates a polygon tiling made of triangles.}
#' \item{`3.3*.3.3**`}{Creates a regular (star) polygon tiling made of triangles and three-pointed stars.}
#' \item{`3.3.3.12*.3.3.12*`}{Creates a regular (star) polygon tiling made of triangles and twelve-pointed stars.}
#' \item{`3.3.8*.3.4.3.8*`}{Creates a regular (star) polygon tiling made of triangles, squares, and eight-pointed stars.}
#' \item{`3.3.8*.4**.8*`}{Creates a regular (star) polygon tiling made of triangles, four-pointed stars, and eight-pointed stars.}
#' \item{`3.4.6.3.12*`}{Creates a regular (star) polygon tiling made of triangles, squares, hexagons, and twelve-pointed stars.}
#' \item{`3.4.8.3.8*`}{Creates a regular (star) polygon tiling made of triangles, squares, octagons, and eight-pointed stars.}
#' \item{`3.6*.6**`}{Creates a regular (star) polygon tiling made of triangles and six-pointed stars.}
#' \item{`4.2*.4.2**`}{Creates a polygon tiling made of squares and rhombi.}
#' \item{`4.4*.4**`}{Creates a regular (star) polygon tiling made of squares and four-pointed stars.}
#' \item{`4.6.4*.6`}{Creates a regular (star) polygon tiling made of squares, hexagons, and four-pointed stars.}
#' \item{`4.6*.4.6*.4.6*`}{Creates a regular (star) polygon tiling made of squares and six-pointed stars.}
#' \item{`4.8*.4**.8*`}{Creates a polygon tiling of squares and eight-pointed stars.}
#' \item{`6.6*.6.6*`}{Creates a regular (star) polygon tiling made of hexagons and six-pointed stars.}
#' \item{`8.4*.8.4*`}{Creates a regular (star) polygon tiling made of octagons and four-pointed stars.}
#' \item{`9.3.9.3*`}{Creates a regular (star) polygon tiling made of triangles, nonagons, and three-pointed stars.}
#' \item{`12.3*.12.3*`}{Creates a regular (star) polygon tiling made of dodecagons and three-pointed stars.}
#' \item{`12.12.4*`}{Creates a regular (star) polygon tiling made of dodecagons and four-pointed stars.}
#' \item{`18.18.3*`}{Creates a regular (star) polygon tiling made of eighteen-sided polygons and three-pointed stars.}
#' }
#'
#' @inheritParams grid.pattern_circle
#' @param alpha Alpha (between 0 and 1) or `NA` (default, preserves colors' alpha value).
#'              Not supported for all polygon tiling `type`.
#' @param type Name of polygon tiling to draw.  See Details.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @seealso The tiling vignette `vignette("tiling", package = "gridpattern")` for more
#'          information about these tilings as well as more
#'          examples of polygon tiling using the [grid.pattern_regular_polygon()] function.
#' @examples
#' print(names_polygon_tiling)
#'
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' gp1 <- grid::gpar(fill = "yellow", col = "black")
#' gp2 <- grid::gpar(fill = c("yellow", "red"), col = "black")
#' gp3 <- grid::gpar(fill = c("yellow", "red", "blue"), col = "black")
#'
#' grid.pattern_polygon_tiling(x_hex, y_hex, type = "herringbone", gp = gp1)
#'
#' grid::grid.newpage()
#' grid.pattern_polygon_tiling(x_hex, y_hex, type = "hexagonal",
#'                             spacing = 0.2, gp = gp3)
#'
#' grid::grid.newpage()
#' grid.pattern_polygon_tiling(x_hex, y_hex, type = "pythagorean",
#'                             spacing = 0.2, gp = gp2)
#'
#' grid::grid.newpage()
#' grid.pattern_polygon_tiling(x_hex, y_hex, type = "snub_trihexagonal",
#'                             spacing = 0.2, gp = gp3)
#'
#' grid::grid.newpage()
#' grid.pattern_polygon_tiling(x_hex, y_hex, type = "rhombille",
#'                               spacing = 0.2, gp = gp3)
#' @export
grid.pattern_polygon_tiling <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	type = "square",
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"polygon_tiling",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		type = type,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' @rdname grid.pattern_polygon_tiling
#' @export
names_polygon_tiling <- c(
	"elongated_triangular",
	"herringbone",
	"hexagonal",
	"pythagorean",
	"rhombitrihexagonal",
	"rhombille",
	"snub_square",
	"snub_trihexagonal",
	"square",
	"tetrakis_square",
	"triangular",
	"trihexagonal",
	"truncated_square",
	"truncated_hexagonal",
	"truncated_trihexagonal",
	"2*.2**.2*.2**",
	"2**.3**.12*",
	"3.3.3.3**",
	"3.3*.3.3**",
	"3.3.3.12*.3.3.12*",
	"3.3.8*.3.4.3.8*",
	"3.3.8*.4**.8*",
	"3.4.6.3.12*",
	"3.4.8.3.8*",
	"3.6*.6**",
	"4.2*.4.2**",
	"4.4*.4**",
	"4.6.4*.6",
	"4.6*.4.6*.4.6*",
	"4.8*.4**.8*",
	"6.6*.6.6*",
	"8.4*.8.4*",
	"9.3.9.3*",
	"12.3*.12.3*",
	"12.12.4*",
	"18.18.3*"
)

create_pattern_polygon_tiling <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	type <- tolower(params$pattern_type)

	xyi <- boundary_df

	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth
	lty <- params$pattern_linetype
	stopifnot(length(fill) < 4L, max(lengths(list(col, lwd, lty))) == 1L)
	gp <- gpar(fill = fill, col = col, lwd = lwd, lty = lty)

	angle <- params$pattern_angle
	spacing <- params$pattern_spacing
	units <- params$pattern_units

	fn <- switch(
		type,
		elongated_triangular = create_el_tri_tiling,
		herringbone = create_herringbone_tiling,
		hexagonal = create_hexagonal_tiling,
		pythagorean = create_pythagorean_tiling,
		snub_square = create_snub_square_tiling,
		snub_trihexagonal = create_snub_trihex_tiling,
		square = create_square_tiling,
		rhombille = create_rhombille_tiling,
		rhombitrihexagonal = create_rhombitrihexagonal_tiling,
		tetrakis_square = create_tetrakis_tiling,
		triangular = create_triangular_tiling,
		trihexagonal = create_trihexagonal_tiling,
		truncated_hexagonal = create_trunc_hex_tiling,
		truncated_square = create_trunc_square_tiling,
		truncated_trihexagonal = create_trunc_trihex_tiling,
		`2*.2**.2*.2**` = create_2_53.2__233.2_53.2__233_tiling,
		`2**.3**.12*` = create_2__.3__.12__tiling,
		`3.3.3.3**` = create_3.3.3.3___tiling,
		`3.3*.3.3**` = create_3.3_30.3.3_30_tiling,
		`3.3.3.12*.3.3.12*` = create_3.3.3.12_30.3.3.12_30_tiling,
		`3.4.6.3.12*` = create_3.4.6.3.12_30_tiling,
		`3.3.8*.3.4.3.8*` = create_3.3.8_15.3.4.3.8_15_tiling,
		`3.3.8*.4**.8*` = create_3.3.8_15.4__60.8_15_tiling,
		`3.4.8.3.8*` = create_3.4.8.3.8_15_tiling,
		`3.6*.6**` = create_3.6_30.6__30_tiling,
		`4.2*.4.2**` = create_4.2_60.4.2__240_tiling,
		`4.4*.4**` = create_4.4_30.4__30_tiling,
		`4.6.4*.6` = create_4.6.4_30.6_tiling,
		`4.6*.4.6*.4.6*` = create_4.6_30.4.6_30.4.6_30_tiling,
		`4.8*.4**.8*` = create_4.8_.4__.8__tiling,
		`6.6*.6.6*` = create_6.6_60.6.6_60_tiling,
		`8.4*.8.4*` = create_8.4_45.8.4_45_tiling,
		`9.3.9.3*` = create_9.3.9.3_40_tiling,
		`12.3*.12.3*` = create_12.3_30.12.3_30_tiling,
		`12.12.4*` = create_12.12.4_60_tiling,
		`18.18.3*` = create_18.18.3__tiling,
		abort(paste("Don't know how to do tiling", type))
	)
	gTree(children = fn(xyi, gp, spacing, units, angle), name = "polygon_tiling")
}

create_3.3.3.12_30.3.3.12_30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_dod <- gp_bg <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp_star$fill <- gp$fill[1L]
		gp_dod$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[3]
		gp_dod$fill <- gp$fill[2]
		gp_star$fill <- gp$fill[1]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	dodecagons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		grid = "hex_circle",
		shape = "convex12",
		density = 1.034,
		rot = 15,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dod,
		name = "dodecagons"
	)
	scale <- star_scale(12, 60, external = TRUE)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		grid = "hex_circle",
		shape = "star12",
		density = 1.034,
		rot = 15,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, dodecagons, stars)
}

create_3.3.8_15.4__60.8_15_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star8 <- gp_star4 <- gp_sq <- gp
	if (n_col == 2L) {
		gp_sq$fill <- gp_star4$fill <- gp$fill[1L]
		gp_star8$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_sq$fill <- gp$fill[2]
		gp_star4$fill <- gp$fill[1]
		gp_star8$fill <- gp$fill[3]
	}
	sq <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "square",
		density = 1,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_sq,
		name = "background_color"
	)
	scale <- star_scale(4, 60)
	star4 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star4",
		density = 1.14,
		rot = 45,
		spacing = spacing,
		units = units,
		xoffset = spacing / 2,
		yoffset = spacing / 2,
		angle = angle,
		gp = gp_star4,
		name = "star4"
	)
	scale <- star_scale(8, 15)
	star8 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star8",
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star8,
		scale = scale,
		name = "star8"
	)
	gList(sq, star4, star8)
}
create_3.3.8_15.3.4.3.8_15_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_oct <- gp_bg <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp_star$fill <- gp$fill[1L]
		gp_oct$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[3]
		gp_oct$fill <- gp$fill[1]
		gp_star$fill <- gp$fill[2]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	octagons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex8",
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_oct,
		name = "octagons"
	)
	scale <- star_scale(8, 60, external = TRUE)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star8",
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, octagons, stars)
}

create_3.4.8.3.8_15_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_oct <- gp_bg <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp_star$fill <- gp$fill[2L]
		gp_oct$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[2]
		gp_oct$fill <- gp$fill[c(1, 3)]
		gp_star$fill <- gp$fill[2]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	octagons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex8",
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_oct,
		name = "octagons"
	)
	scale <- star_scale(8, 60, external = TRUE)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = c("null", "star8"),
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, octagons, stars)
}
create_4.2_60.4.2__240_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_rh <- gp_sq1 <- gp_sq2 <- gp
	if (n_col == 2L) {
		gp_sq1$fill <- gp_sq2$fill <- gp$fill[1L]
		gp_rh$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_sq2$fill <- gp$fill[3]
		gp_sq1$fill <- gp$fill[2]
		gp_rh$fill <- gp$fill[1]
	}
	dens_sq <- 0.73
	squares.1 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex4",
		density = dens_sq,
		rot = 60,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_sq1,
		name = "squares.1"
	)
	squares.2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex4",
		density = dens_sq,
		rot = -60,
		xoffset = 0.5 * spacing,
		yoffset = 0.5 * spacing,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_sq2,
		name = "squares.2"
	)
	scale <- star_scale(2, 60)
	dens_rh <- 0.88
	rhombi.1 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = dens_rh,
		rot = 45,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh,
		xoffset = 0.5 * spacing,
		scale = scale,
		name = "rhombi.1"
	)
	rhombi.2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = dens_rh,
		rot = -45,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh,
		yoffset = 0.5 * spacing,
		scale = scale,
		name = "rhombi.2"
	)
	gList(squares.1, squares.2, rhombi.1, rhombi.2)
}
create_2_53.2__233.2_53.2__233_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_rh1 <- gp_rh2 <- gp_rh3 <- gp_rh4 <- gp
	if (n_col == 2L) {
		gp_rh1$fill <- gp_rh2$fill <- gp$fill[1L]
		gp_rh3$fill <- gp_rh4$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_rh1$fill <- gp$fill[2:3]
		gp_rh2$fill <- gp$fill[3:2]
		gp_rh3$fill <- gp$fill[1]
		gp_rh4$fill <- gp$fill[1]
	}
	rhombi.1 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = 1,
		scale = 0.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh1,
		name = "rhombi.1",
		type = "horizontal"
	)
	rhombi.2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = 1,
		scale = 0.5,
		xoffset = 0.5 * spacing,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh2,
		name = "rhombi.2",
		type = "horizontal"
	)
	rhombi.3 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = 1,
		scale = 0.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh3,
		xoffset = 0.25 * spacing,
		yoffset = 0.5 * spacing,
		scale = scale,
		name = "rhombi.3"
	)
	rhombi.4 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star2",
		density = 1,
		scale = 0.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_rh4,
		xoffset = 0.75 * spacing,
		yoffset = 0.5 * spacing,
		scale = scale,
		name = "rhombi.4"
	)
	gList(rhombi.1, rhombi.2, rhombi.3, rhombi.4)
}

create_4.6_30.4.6_30.4.6_30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_bg <- gp
	gp_bg$fill <- gp$fill[1L]
	if (n_col > 1L) {
		gp_star$fill <- rev(gp$fill[-1L])
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(6, 30)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star6",
		density = 1,
		grid = "hex",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, stars)
}
create_8.4_45.8.4_45_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_oct <- gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	if (n_col == 2L) {
		gp_oct$fill <- gp$fill[-n_col]
	} else if (n_col == 3L) {
		gp_oct$fill <- rep(gp$fill[-n_col], each = 2)
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	subtype <- if (n_col > 2) "2134" else NULL
	octs <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = c("convex8", "null"),
		density = 1.41,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_oct,
		type = "square_tiling",
		subtype = subtype,
		scale = scale,
		name = "octagons"
	)
	gList(bg, octs)
}

create_12.3_30.12.3_30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_dod <- gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	if (n_col > 1L) {
		gp_dod$fill <- gp$fill[-n_col]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	dodecagons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex12",
		density = 1,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dod,
		scale = scale,
		name = "dodecagons"
	)
	gList(bg, dodecagons)
}
create_12.12.4_60_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_dod <- gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	if (n_col > 1L) {
		gp_dod$fill <- gp$fill[-n_col]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	dodecagons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex12",
		density = 1.035,
		rot = 15,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dod,
		scale = scale,
		name = "dodecagons"
	)
	gList(bg, dodecagons)
}
create_4.8_.4__.8__tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_dod <- gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	if (n_col > 1L) {
		gp_dod$fill <- gp$fill[1:2]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(8, 90, external = TRUE)
	polygons <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = c("star8", "convex4"),
		density = c(1.53, 1.2),
		rot = c(22.5, 0),
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dod,
		scale = scale,
		name = "polygons"
	)
	gList(bg, polygons)
}
create_18.18.3__tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_dod <- gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	if (n_col > 1L) {
		gp_dod$fill <- rev(gp$fill[-n_col])
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	eighteen <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex18",
		density = 1.014,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dod,
		scale = scale,
		name = "eighteen-sided"
	)
	gList(bg, eighteen)
}
create_3.6_30.6__30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_bg <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp$fill[2L]
		gp_star$fill <- rev(gp$fill[1L])
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[2L]
		gp_star$fill <- gp$fill[c(3, 1)]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(6, 30)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star6",
		density = 1.30,
		rot = -22.0,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		grid = "hex",
		name = "stars"
	)
	gList(bg, stars)
}
create_4.4_30.4__30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_bg <- gp
	if (n_col == 2L) {
		gp_star$fill <- gp$fill[1]
		gp_bg$fill <- gp$fill[2]
	} else if (n_col == 3L) {
		gp_star$fill <- gp$fill[2:3]
		gp_bg$fill <- gp$fill[1]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(4, 30)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star4",
		density = 1.60,
		rot = -9.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, stars)
}

create_3.3_30.3.3_30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_bg <- gp
	if (n_col == 2L) {
		gp_star$fill <- gp$fill[1L]
		gp_bg$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_star$fill <- gp$fill[c(3L, 1L)]
		gp_bg$fill <- gp$fill[2L]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(3, 30)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star3",
		density = 1.57,
		grid = "hex_circle",
		rot = 30,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, stars)
}
create_3.3.3.3___tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_tri <- gp_bg <- gp
	if (n_col == 2L) {
		gp_tri$fill <- gp$fill[1L]
		gp_bg$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_tri$fill <- gp$fill[c(3L, 1L)]
		gp_bg$fill <- gp$fill[2L]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	triangles <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex3",
		density = 1.33,
		grid = "hex_circle",
		rot = 30,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_tri,
		scale = scale,
		name = "triangles"
	)
	gList(bg, triangles)
}
create_6.6_60.6.6_60_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_star <- gp_bg <- gp
	if (n_col == 2L) {
		gp_star$fill <- gp$fill[1L]
		gp_bg$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_star$fill <- gp$fill[c(3L, 1L)]
		gp_bg$fill <- gp$fill[2L]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	scale <- star_scale(6, 120, external = TRUE)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star6",
		density = 1,
		grid = "hex_circle",
		rot = 30,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_star,
		scale = scale,
		name = "stars"
	)
	gList(bg, stars)
}

create_herringbone_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	fill2 <- gp$fill[1L]
	if (n_col > 1L) {
		gp$fill <- gp$fill[-1L]
	}
	grob <- patternGrob(
		"weave",
		xyi$x,
		xyi$y,
		xyi$id,
		type = "twill",
		subtype = "2/2(1)",
		density = 1,
		fill2 = fill2,
		spacing = spacing,
		units = units,
		angle = angle + 45,
		gp = gp,
		name = "rectangles"
	)
	gList(grob)
}

create_hexagonal_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	if (n_col == 2L) {
		gp$fill <- rev(gp$fill)
	}
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex6",
		density = 1,
		grid = "hex",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "hexagons"
	)
	gList(grob)
}

create_el_tri_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp$fill[2L]
		gp$fill <- rep(c(gp$fill[1L], gp$fill[2L]), each = 2)
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[1L]
		gp$fill <- c(gp$fill[2L], gp$fill[3L], gp$fill[1L], gp$fill[1L])
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = rep(c("convex4", "convex3"), each = 2),
		density = rep(c(1.41, 1.15), each = 2),
		grid = "elongated_triangle",
		type = "square_tiling",
		subtype = "3412",
		rot = rep(c(45, 0), each = 2),
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "squares_and_triangles"
	)
	gList(bg, grob)
}

create_pythagorean_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[2:1]
	}
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex4",
		density = 1.222,
		rot = 15,
		spacing = spacing,
		units = units,
		angle = angle - 60,
		gp = gp,
		name = "squares_larger"
	)
	gList(bg, grob)
}

create_rhombitrihexagonal_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_sq <- gp_bg
	gp_dd <- gp_bg
	gp_bg$fill <- gp$fill[n_col]
	if (n_col == 2L) {
		gp_sq$fill <- gp$fill[1L]
		gp_dd$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[2L]
		gp_sq$fill <- gp$fill[3L]
		gp_dd$fill <- gp$fill[1L]
	}
	# hexagons
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	# squares
	stripe1 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.1"
	)
	stripe2 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle + 60,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.2"
	)
	stripe3 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle - 60,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.3"
	)
	# dodecagons
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex12",
		density = 0.82,
		rot = 15,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dd,
		name = "dodecagons"
	)
	gList(bg, stripe1, stripe2, stripe3, grob)
}

create_3.4.6.3.12_30_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp_sq <- gp_dd <- gp
	if (n_col == 2L) {
		gp_bg$fill <- gp$fill[1L]
		gp_sq$fill <- gp_dd$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp_bg$fill <- gp$fill[1L]
		gp_dd$fill <- gp$fill[2L]
		gp_sq$fill <- gp$fill[3L]
	}
	# hexagons
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	# squares
	stripe1 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.1"
	)
	stripe2 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle + 60,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.2"
	)
	stripe3 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle - 60,
		grid = "hex_circle",
		density = 0.25,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.3"
	)
	# triangles
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex12",
		density = 0.82,
		rot = 15,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_dd,
		name = "dodecagons"
	)
	# twelve-pointed stars
	scale <- star_scale(12, 30)
	stars <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star12",
		density = 0.82,
		rot = 15,
		grid = "hex_circle",
		scale = scale,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_bg,
		name = "stars"
	)
	gList(bg, stripe1, stripe2, stripe3, grob, stars)
}

create_snub_square_tiling <- function(xyi, gp, spacing, units, angle) {
	scale_star <- star_scale(4, 90 + 60, external = TRUE)
	n_col <- length(gp$fill)
	gp_sq <- gp_sq2 <- gp_tri <- gp
	if (n_col == 2) {
		gp_sq2$fill <- gp_sq$fill <- gp$fill[1L]
		gp_tri$fill <- gp$fill[2L]
	} else if (n_col == 3) {
		gp_tri$fill <- gp$fill[1L]
		gp_sq$fill <- gp$fill[2L]
		gp_sq2$fill <- gp$fill[3L]
	}
	sq1 <- polygonGrob(xyi$x, xyi$y, xyi$id, default.units = "npc", gp = gp_sq, name = "squares.1")
	tri <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star4",
		scale = scale_star,
		angle = angle,
		rot = 15,
		spacing = spacing,
		units = units,
		density = 1.41,
		gp = gp_tri,
		name = "triangles"
	)
	sq2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex4",
		scale = scale_star,
		angle = angle,
		rot = 60,
		spacing = spacing,
		units = units,
		density = scale_star * 1.41,
		gp = gp_sq2,
		name = "squares.2"
	)
	gList(sq1, tri, sq2)
}

create_snub_trihex_tiling <- function(xyi, gp, spacing, units, angle) {
	scale_star <- star_scale(6, 60 + 60, external = TRUE)
	n_col <- length(gp$fill)
	gp_tri1 <- gp_tri2 <- gp_hex <- gp
	if (n_col == 2) {
		gp_tri1$fill <- gp_tri2$fill <- gp$fill[1L]
		gp_hex$fill <- gp$fill[2L]
	} else if (n_col == 3) {
		gp_tri2$fill <- gp$fill[1L]
		gp_hex$fill <- gp$fill[2L]
		gp_tri1$fill <- gp$fill[3L]
	}
	tri1 <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_tri1,
		name = "triangles.1"
	)
	tri2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star6",
		scale = scale_star,
		grid = "hex_circle",
		angle = angle,
		rot = 19,
		spacing = spacing,
		units = units,
		density = 1.305,
		gp = gp_tri2,
		name = "triangles.2"
	)
	hex <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex6",
		grid = "hex_circle",
		angle = angle,
		rot = 49,
		spacing = spacing,
		units = units,
		density = scale_star * 1.305,
		gp = gp_hex,
		name = "hexagons"
	)
	gList(tri1, tri2, hex)
}

create_square_tiling <- function(xyi, gp, spacing, units, angle) {
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "square",
		density = 1,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "squares"
	)
	gList(grob)
}
create_rhombille_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_rh1 <- gp_rh2 <- gp_rh3 <- gp
	if (n_col == 2) {
		gp_rh1$fill <- gp_rh2$fill <- gp$fill[1L]
		gp_rh3$fill <- gp$fill[2L]
	} else if (n_col == 3) {
		gp_rh1$fill <- gp$fill[1L]
		gp_rh2$fill <- gp$fill[2L]
		gp_rh3$fill <- gp$fill[3L]
	}
	rh1 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "rhombille_rhombus",
		grid = "hex",
		angle = angle,
		rot = -120,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_rh1,
		name = "rhombi.1"
	)
	rh2 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "rhombille_rhombus",
		grid = "hex",
		angle = angle,
		rot = 120,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_rh2,
		name = "rhombi.2"
	)
	rh3 <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "rhombille_rhombus",
		grid = "hex",
		angle = angle,
		rot = 0,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_rh3,
		name = "rhombi.3"
	)
	gList(rh1, rh2, rh3)
}
create_9.3.9.3_40_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_non <- gp_tri <- gp_star3 <- gp
	if (n_col == 2) {
		gp_non$fill <- gp$fill[1L]
		gp_star3$fill <- gp_tri$fill <- gp$fill[2L]
	} else if (n_col == 3) {
		gp_non$fill <- gp$fill[1L]
		gp_star3$fill <- gp$fill[2L]
		gp_tri$fill <- gp$fill[3L]
	}
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_star3,
		name = "background_color"
	)
	stripe <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		grid = "hex_circle",
		yoffset = 0.24 * spacing,
		angle = angle,
		spacing = spacing,
		units = units,
		density = 0.35,
		gp = gp_tri,
		name = "stripes"
	)
	non <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex9",
		grid = "hex_circle",
		angle = angle,
		rot = 0,
		spacing = spacing,
		units = units,
		density = 1.01,
		gp = gp_non,
		name = "nonagons"
	)
	gList(bg, stripe, non)
}
create_tetrakis_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_tri1 <- gp_tri2 <- gp_tri3 <- gp
	if (n_col == 2) {
		gp_tri1$fill <- gp$fill[1L]
		gp_tri2$fill <- gp_tri3$fill <- gp$fill[2L]
	} else if (n_col == 3) {
		gp_tri1$fill <- gp$fill[1L]
		gp_tri2$fill <- gp$fill[2L]
		gp_tri3$fill <- gp$fill[3L]
	}
	tri1.a <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_left",
		angle = angle,
		rot = 0,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri1,
		name = "triangles.1.a"
	)
	tri1.b <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_left",
		angle = angle,
		rot = 90,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri1,
		name = "triangles.1.b"
	)
	tri1.c <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_left",
		angle = angle,
		rot = 180,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri1,
		name = "triangles.1.c"
	)
	tri1.d <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_left",
		angle = angle,
		rot = 270,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri1,
		name = "triangles.1.d"
	)
	tri2.a <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_right",
		angle = angle,
		rot = 0,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri2,
		name = "triangles.2.a"
	)
	tri2.b <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_right",
		angle = angle,
		rot = 180,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri2,
		name = "triangles.2.b"
	)
	tri3.a <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_right",
		angle = angle,
		rot = 90,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri3,
		name = "triangles.3.a"
	)
	tri3.b <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "tetrakis_right",
		angle = angle,
		rot = 270,
		spacing = spacing,
		units = units,
		density = 1,
		gp = gp_tri3,
		name = "triangles.3.b"
	)
	gList(tri1.a, tri1.b, tri1.c, tri1.d, tri2.a, tri2.b, tri3.a, tri3.b)
}

create_triangular_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[1]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[3:2]
	}
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex3",
		density = 1,
		grid = "hex",
		rot = 180,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "triangles"
	)
	gList(bg, grob)
}

create_2__.3__.12__tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[1]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col > 1L) {
		gp$fill <- rev(gp$fill[-1L])
	}
	scale <- star_scale(12, 60, external = TRUE)
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "star12",
		density = 1.034,
		rot = 15,
		grid = "hex_circle",
		scale = scale,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "dodecagons"
	)
	gList(bg, grob)
}
create_trunc_hex_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[2:1]
	}

	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex12",
		density = 1.034,
		rot = 15,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "dodecagons"
	)
	gList(bg, grob)
}

create_trunc_square_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[1:2]
	}

	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex8",
		density = 1.082,
		rot = 22.5,
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "octagons"
	)
	gList(bg, grob)
}

create_4.6.4_30.6_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[1L]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[2L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[2:3]
	}

	scale <- star_scale(4, 120, external = TRUE)
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = c("star4", "convex4"),
		density = c(1.2, 0.8),
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "stars_and_squares",
		scale = scale
	)
	gList(bg, grob)
}

create_trunc_trihex_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_sq <- gp_bg
	gp_hx <- gp_bg
	gp_bg$fill <- gp$fill[n_col]
	if (n_col == 2L) {
		gp_bg$fill <- gp$fill[1L]
		gp_sq$fill <- gp$fill[2L]
		gp_hx$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp_sq$fill <- gp$fill[2L]
		gp_hx$fill <- gp$fill[1L]
	}
	# triangles
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	# squares
	stripe1 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle,
		grid = "hex_circle",
		density = 0.42,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.1"
	)
	stripe2 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle + 60,
		grid = "hex_circle",
		density = 0.42,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.2"
	)
	stripe3 <- patternGrob(
		"stripe",
		xyi$x,
		xyi$y,
		xyi$id,
		angle = angle - 60,
		grid = "hex_circle",
		density = 0.42,
		spacing = spacing,
		units = units,
		gp = gp_sq,
		name = "square_stripes.3"
	)
	# hexagons
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex6",
		density = 0.75,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp_hx,
		name = "hexagons"
	)
	gList(bg, stripe1, stripe2, stripe3, grob)
}

create_trihexagonal_tiling <- function(xyi, gp, spacing, units, angle) {
	n_col <- length(gp$fill)
	gp_bg <- gp
	gp_bg$fill <- gp$fill[n_col]
	bg <- polygonGrob(
		xyi$x,
		xyi$y,
		xyi$id,
		default.units = "npc",
		gp = gp_bg,
		name = "background_color"
	)
	if (n_col == 2L) {
		gp$fill <- gp$fill[1L]
	} else if (n_col == 3L) {
		gp$fill <- gp$fill[2:1]
	}
	grob <- patternGrob(
		"regular_polygon",
		xyi$x,
		xyi$y,
		xyi$id,
		shape = "convex6",
		density = 1,
		rot = 30,
		grid = "hex_circle",
		spacing = spacing,
		units = units,
		angle = angle,
		gp = gp,
		name = "hexagons"
	)
	gList(bg, grob)
}

context("tiling")
test_that("tiling patterns work as expected", {
	skip_if_not_installed("vdiffr")
	skip_on_ci()
	library("vdiffr")

	x <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	y <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	g.ppt <- function(type, ...) {
		grid.pattern_polygon_tiling(x, y, angle = 0, type = type, spacing = 0.15, ...)
	}
	gp1 <- gpar(fill = "yellow", col = "black")
	gp2 <- gpar(fill = c("yellow", "red"), col = "black")
	gp3 <- gpar(fill = c("yellow", "red", "blue"), col = "black")

	expect_doppelganger("elongated_triangular", function() {
		g.ppt("elongated_triangular", gp = gp2)
	})
	expect_doppelganger("herringbone", function() {
		g.ppt("herringbone", gp = gp2)
	})
	expect_doppelganger("hexagonal_tiling", function() {
		g.ppt("hexagonal", gp = gp2)
	})
	expect_doppelganger("pythagorean", function() {
		g.ppt("pythagorean", gp = gp2)
	})
	expect_doppelganger("rhombitrihexagonal_tiling", function() {
		g.ppt("rhombitrihexagonal", gp = gp3)
	})
	expect_doppelganger("rhombille", function() {
		g.ppt("rhombille", gp = gp3)
	})
	expect_doppelganger("snub_square_tiling", function() {
		g.ppt("snub_square", gp = gp3)
	})
	expect_doppelganger("snub_trihex_tiling", function() {
		g.ppt("snub_trihexagonal", gp = gp3)
	})
	expect_doppelganger("square_tiling", function() {
		g.ppt("square", gp = gp2)
	})
	expect_doppelganger("tetrakis_square", function() {
		g.ppt("tetrakis_square", gp = gp3)
	})
	expect_doppelganger("triangular_tiling", function() {
		g.ppt("triangular", gp = gp2)
	})
	expect_doppelganger("trihexagonal_tiling", function() {
		g.ppt("trihexagonal", gp = gp2)
	})
	expect_doppelganger("trunc_hex_tiling", function() {
		g.ppt("truncated_hexagonal", gp = gp3)
	})
	expect_doppelganger("trunc_trihex_tiling", function() {
		g.ppt("truncated_trihexagonal", gp = gp2)
	})
	expect_doppelganger("trunc_square_tiling", function() {
		g.ppt("truncated_square", gp = gp3)
	})
	expect_doppelganger("2*.2**.2*.2**", function() {
		g.ppt("2*.2**.2*.2**", gp = gp3)
	})
	expect_doppelganger("2**.3**.12*", function() {
		g.ppt("2**.3**.12*", gp = gp3)
	})
	expect_doppelganger("3.3.3.3_alt", function() {
		g.ppt("3.3.3.3**", gp = gp2)
	})
	expect_doppelganger("3.3*.3.3**", function() {
		g.ppt("3.3*.3.3**", gp = gp2)
	})
	expect_doppelganger("3.3.3.12*.3.3.12*", function() {
		g.ppt("3.3.3.12*.3.3.12*", gp = gp3)
	})
	expect_doppelganger("3.3.8*.3.4.3.8*", function() {
		g.ppt("3.3.8*.3.4.3.8*", gp = gp3)
	})
	expect_doppelganger("3.3.8*.4**.8*", function() {
		g.ppt("3.3.8*.4**.8*", gp = gp3)
	})
	expect_doppelganger("3.4.6.3.12*", function() {
		g.ppt("3.4.6.3.12*", gp = gp3)
	})
	expect_doppelganger("3.4.8.3.8*", function() {
		g.ppt("3.4.8.3.8*", gp = gp3)
	})
	expect_doppelganger("3.6*.6**", function() {
		g.ppt("3.6*.6**", gp = gp3)
	})
	expect_doppelganger("4.2*.4.2**", function() {
		g.ppt("4.2*.4.2**", gp = gp3)
	})
	expect_doppelganger("4.4*.4**", function() {
		g.ppt("4.4*.4**", gp = gp2)
	})
	expect_doppelganger("4.6.4*.6", function() {
		g.ppt("4.6.4*.6", gp = gp3)
	})
	expect_doppelganger("4.6*.4.6*.4.6*", function() {
		g.ppt("4.6*.4.6*.4.6*", gp = gp2)
	})
	expect_doppelganger("4.8*.4**.8*", function() {
		g.ppt("4.8*.4**.8*", gp = gp2)
	})
	expect_doppelganger("6.6*.6.6*", function() {
		g.ppt("6.6*.6.6*", gp = gp2)
	})
	expect_doppelganger("8.4*.8.4*", function() {
		g.ppt("8.4*.8.4*", gp = gp2)
	})
	expect_doppelganger("9.3.9.3*", function() {
		g.ppt("9.3.9.3*", gp = gp3)
	})
	expect_doppelganger("12.3*.12.3*", function() {
		g.ppt("12.3*.12.3*", gp = gp2)
	})
	expect_doppelganger("12.12.4*", function() {
		g.ppt("12.12.4*", gp = gp2)
	})
	expect_doppelganger("18.18.3*", function() {
		g.ppt("18.18.3*", gp = gp2)
	})
})

#' Rose curve patterned grobs
#'
#' `grid.pattern_rose()` draws a rose curve pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_circle
#' @inheritParams alphaMaskGrob
#' @param rot Angle to rotate rose (degrees, counter-clockwise).
#' @param frequency The \dQuote{angular frequency} parameter of the rose pattern.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @seealso See \url{https://en.wikipedia.org/wiki/Rose_(mathematics)} for more information.
#' @examples
#'   if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'     x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     gp <- grid::gpar(fill = c("blue", "red", "yellow", "green"), col = "black")
#'     grid.pattern_rose(x_hex, y_hex,
#'                       spacing = 0.15, density = 0.5, angle = 0,
#'                       frequency = 1:4, gp = gp)
#'   }
#'   if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'     grid::grid.newpage()
#'     grid.pattern_rose(x_hex, y_hex,
#'                       spacing = 0.15, density = 0.5, angle = 0,
#'                       frequency = 1/1:4, gp = gp)
#'   }
#'   if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'     grid::grid.newpage()
#'     grid.pattern_rose(x_hex, y_hex,
#'                       spacing = 0.18, density = 0.5, angle = 0,
#'                       frequency = c(3/2, 7/3, 5/4, 3/7), gp = gp)
#'   }
#' @export
grid.pattern_rose <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	frequency = 0.1,
	grid = "square",
	type = NULL,
	subtype = NULL,
	rot = 0,
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	use_R4.1_masks = getOption(
		"ggpattern_use_R4.1_masks",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"rose",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		scale = scale,
		frequency = frequency,
		grid = grid,
		type = type,
		subtype = subtype,
		rot = rot,
		use_R4.1_masks = use_R4.1_masks,
		png_device = png_device,
		res = res,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_rose <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	spacing <- params$pattern_spacing
	grid <- params$pattern_grid

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm)

	# construct grobs using subsets if certain inputs are vectorized
	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype

	density <- params$pattern_density
	rot <- params$pattern_rot
	frequency <- params$pattern_frequency

	n_par <- max(lengths(list(fill, col, lwd, lty, density, rot, frequency)))

	fill <- rep_len_fill(fill, n_par)
	col <- rep_len(col, n_par)
	lwd <- rep_len(lwd, n_par)
	lty <- rep_len(lty, n_par)
	density <- rep_len(density, n_par)
	rot <- rep_len(rot, n_par)
	frequency <- rep_len(frequency, n_par)

	density_max <- max(density)

	# compute regular polygon relative coordinates which we will center on points
	radius_mult <- switch(grid, hex = 0.578, 0.5)
	radius_max <- radius_mult * spacing * density_max

	# compute pattern matrix of graphical elements (e.g. fill colors)
	if (is.null(params$pattern_type) || is.na(params$pattern_type)) {
		params$pattern_type <- switch(grid, square = "square", "hex")
	}
	m_pat <- get_pattern_matrix(params$pattern_type, params$pattern_subtype, grid_xy, n_par)

	gl <- gList()
	for (i_par in seq(n_par)) {
		radius_outer <- radius_mult * spacing * density[i_par]
		xy_rose <- get_xy_rose(frequency[i_par], params, radius_outer, rot[i_par])
		xy_par <- get_xy_par(grid_xy, i_par, m_pat, grid, spacing)
		if (length(xy_par$x) == 0) {
			next
		}

		# rotate by 'angle'
		xy_par <- rotate_xy(xy_par$x, xy_par$y, params$pattern_angle, vpm$x, vpm$y)

		gp <- gpar(fill = fill[[i_par]], col = col[i_par], lwd = lwd[i_par], lty = lty[i_par])

		name <- paste0("rose.", i_par)
		grob <- points_to_rose_grob(xy_par, xy_rose, gp, default.units, name)
		gl <- append_gList(gl, grob)
	}
	maskee <- gTree(children = gl)
	masker <- convert_polygon_df_to_polygon_grob(
		boundary_df,
		default.units = "bigpts",
		gp = gpar(fill = "white", col = NA, lwd = 0)
	)
	alphaMaskGrob(
		maskee,
		masker,
		use_R4.1_masks = params$pattern_use_R4.1_masks,
		png_device = params$pattern_png_device,
		res = params$pattern_res,
		name = "rose"
	)
}

get_xy_rose <- function(frequency, params, radius_outer, rot) {
	theta <- to_radians(seq.int(from = 0, to = 12 * 360, by = 3))
	x <- radius_outer * cos(frequency * theta) * cos(theta)
	y <- radius_outer * cos(frequency * theta) * sin(theta)
	rose_angle <- rot + params$pattern_angle
	rotate_xy(x, y, rose_angle, 0, 0)
}

points_to_rose_grob <- function(xy_par, xy_rose, gp, default.units, name) {
	points_mat <- as.data.frame(xy_par)
	df_polygon <- as.data.frame(xy_rose)
	l_xy <- lapply(seq(nrow(points_mat)), function(i_r) {
		x0 <- points_mat[i_r, 1]
		y0 <- points_mat[i_r, 2]
		df <- df_polygon
		df$x <- df$x + x0
		df$y <- df$y + y0
		df
	})
	df <- do.call(rbind, l_xy)
	if (is.null(df)) {
		nullGrob()
	} else {
		df$id <- rep(seq(nrow(points_mat)), each = nrow(df_polygon))
		pathGrob(
			x = df$x,
			y = df$y,
			id = df$id,
			default.units = default.units,
			gp = gp,
			name = name
		)
	}
}

# radians <-> degrees
to_radians <- function(t) pi * t / 180
to_degrees <- function(t) 180 * t / pi

# polar coordinates <-> cartesian coordinates
to_x <- function(t, r) {
	r * cos(to_radians(t))
}
to_y <- function(t, r) {
	r * sin(to_radians(t))
}
to_theta <- function(x, y) {
	to_degrees(atan2(y, x))
}
to_radius <- function(x, y) {
	sqrt(x^2 + y^2)
}

# rotate (x,y) `t` degrees centered around (x0, y0)
rotate_xy <- function(x, y, theta = 0, x0 = NULL, y0 = NULL) {
	x0 <- x0 %||% mean(x)
	y0 <- y0 %||% mean(y)
	xc <- x - x0
	yc <- y - y0
	theta <- to_theta(xc, yc) + theta
	radius <- to_radius(xc, yc)
	x1 <- to_x(theta, radius) + x0
	y1 <- to_y(theta, radius) + y0
	list(x = x1, y = y1)
}

# rotate (x,y) by `angle` and return a closed polygon ring (list of one matrix) for sf::st_multipolygon()
polygon_ring <- function(x, y, angle, vpm) {
	xy <- rotate_xy(x, y, angle, vpm$x, vpm$y)
	m <- as.matrix(as.data.frame(xy))
	list(rbind(m, m[1, ]))
}

# (x,y) coordinates of convex regular polygon centered at (0, 0)
convex_xy <- function(n_vertices, theta = 90, radius_outer = 0.5) {
	t <- theta + seq(0, 360, length.out = n_vertices + 1)
	x <- to_x(t, radius_outer)
	y <- to_y(t, radius_outer)
	list(x = head(x, -1), y = head(y, -1))
}

# (x,y)coordinates of rhombus quadrilateral
rhombus_xy <- function(theta = 90, radius_outer = 0.5) {
	t <- theta + c(0, -60, 0, 60)
	r <- c(0, rep(radius_outer, 3))
	x <- to_x(t, r)
	y <- to_y(t, r)
	list(x = x, y = y)
}

# (x,y) coordinates of "left" Tetrakis triangle
tetrakis_left_xy <- function(theta = 90, radius_outer = 0.5) {
	t <- c(90, 135)
	r <- c(1e-6, radius_outer)

	x <- to_x(t, r)
	y <- to_y(t, r)
	x <- c(x, x[1])
	y <- c(y, y[2])
	rotate_xy(x, y, theta - 135, 0, 0)
}

# (x,y) coordinates of "right" Tetrakis triangle
tetrakis_right_xy <- function(theta = 90, radius_outer = 0.5) {
	t <- c(90, 45)
	r <- c(1e-6, radius_outer)

	x <- to_x(t, r)
	y <- to_y(t, r)
	x <- c(x, x[1])
	y <- c(y, y[2])
	rotate_xy(x, y, theta - 135, 0, 0)
}

# (x,y) coordinates of concave (star) regular polygon centered at (0, 0)
concave_xy <- function(
	n_vertices,
	theta = 90,
	radius_outer = 0.5,
	radius_inner = 0.5 * radius_outer
) {
	t_outer <- theta + seq(0, 360, length.out = n_vertices + 1)
	n_degrees <- 360 / n_vertices / 2
	t_inner <- theta + seq(n_degrees, 360 - n_degrees, length.out = n_vertices)
	x_outer <- to_x(t_outer, radius_outer)
	x_inner <- to_x(t_inner, radius_inner)
	y_outer <- to_y(t_outer, radius_outer)
	y_inner <- to_y(t_inner, radius_inner)
	x <- splice(x_outer, x_inner)
	y <- splice(y_outer, y_inner)
	list(x = head(x, -1), y = head(y, -1))
}

splice <- function(x0, x1) {
	vec <- as.numeric()
	for (ii in seq_along(x1)) {
		vec <- append(vec, x0[ii])
		vec <- append(vec, x1[ii])
	}
	append(vec, x0[ii + 1])
}

get_n_vertices <- function(shape) {
	as.numeric(gsub("convex|concave|star", "", shape))
}

# returns numeric(0) if 'from' greater than 'to'
seq_robust <- function(from = 1, to = 1, by = ((to - from) / (length.out - 1)), length.out = NULL) {
	if (from > to) {
		numeric(0)
	} else {
		if (is.null(length.out)) {
			seq(from, to, by)
		} else {
			seq(from, to, by, length.out)
		}
	}
}

# cycle_elements(1:5, -2) = c(4, 5, 1, 2, 3)
# cycle_elements(1:5, -1) = c(5, 1, 2, 3, 4)
# cycle_elements(1:5,  0) = c(1, 2, 3, 4, 5)
# cycle_elements(1:5,  1) = c(2, 3, 4, 5, 1)
# cycle_elements(1:5,  2) = c(3, 4, 5, 1, 2)
cycle_elements <- function(x, n = 1) {
	l <- length(x)
	if (l < 2 || n == l || n == 0 || n == -l) {
		return(x)
	}
	if (n > 0) {
		if (n < l) {
			c(x[(n + 1):l], x[1:n])
		} else {
			cycle_elements(x, n - l)
		}
	} else {
		if (-l < n) {
			c(x[(l + n + 1):l], x[1:(l + n)])
		} else {
			cycle_elements(x, n + l)
		}
	}
}

nigh <- function(x, y) abs(x - y) < .Machine$double.eps^0.5

#' Weave pattern matrix
#'
#' `pattern_weave()` returns a logical matrix indicating where the warp lines should
#'  be "up" for a specified weave pattern type and subtype.
#' `names_weave` is a character vector listing supported weave pattern types.
#'
#' Here is a list of the various weave `type`s supported:
#'
#' \describe{
#' \item{basket}{A simple criss-cross pattern using two threads at a time.
#'              Same as the "matt_irregular" weave but with a default `subtype` of `2L`.}
#' \item{matt}{A simple criss-cross pattern using 3 (or more) threads at a time.
#'             Same as the "matt_irregular" weave but with a default `subtype` of `3L`.}
#' \item{matt_irregular}{A generalization of the "plain" weave.
#'                       A character `subtype` `"U/D(L+R)"` is a standard matt weave specification:
#'                       `U` indicates number warp up, `D` indicates number warp down,
#'                       `L` indicates number of warp up in repeat, and
#'                       `R` indicates number of warp down in repeat.
#'                       An integer `subtype` `N` will be interpreted as a `"N/N(N+N)"` irregular matt weave.
#'                       A character `subtype` `"U/D"` will be interpreted as a `"U/D(U+D)"` irregular matt weave.
#'                       Has a default `subtype` of `"3/2(4+2)"`.}
#' \item{plain}{A simple criss-cross pattern.
#'              Same as the "matt_irregular" weave but with a default `subtype` of `1L`.}
#' \item{rib_warp}{A plain weave variation that emphasizes vertical lines.
#'                 An integer `subtype` `N` will be interpreted as a "matt_irregular" `"N/N(1+1)"` weave.
#'                 A character `subtype` `"U/D"` will be interpreted as a "matt_irregular" `"U/D(1+1)"` weave.
#'                 Default `subtype` of `2L`.}
#' \item{satin}{A "regular" satin weave is a special type of the elongated twill weave
#'              with a move number carefully chosen so no twill line is distinguishable.
#'              Same as the "twill_elongated" weave but with a default `subtype` of `5L`.}
#' \item{twill}{A simple diagonal pattern.
#'              Same as the "twill_elongated" weave but with a default `subtype` of `"2/1"`.}
#' \item{twill_elongated}{A generalization of the "twill" weave.
#'                        A character `subtype` `"U/D(M)"` is a standard twill weave specification:
#'                        `U` indicates number warp up, `D` indicates number warp down,
#'                        and `M` indicates the "move" number.
#'                        A character `subtype` `"U/D"` will be interpreted as a `"U/D(1)"` elongated twill weave.
#'                        An integer `subtype` `N` will provide a `"{N-1}/1(1)"` elongated twill weave
#'                        if `N` is less than 5, 6, or greater than 14 otherwise it will
#'                        provide a `"{N-1}/1(M)"` weave where `M` is the largest
#'                        possible regular "satin" move number.
#'                        Default `subtype` of `"4/3(2)"`.}
#' \item{twill_herringbone}{Adds a (vertical) "herringbone" effect to
#'                          the specified "twill_elongated" weave.
#'                          Default `subtype` of `"4/3(2)"`.}
#' \item{twill_zigzag}{Adds a (vertical) "zig-zag" effect to the specified "twill_elongated" weave.
#'                     Default `subtype` of `"4/3(2)"`.}
#' }
#' For both "matt" and "twill" weaves the `U/D` part of the subtype can be further extended
#' to `U1/D1*U2/D2`, `U1/D1*U2/D2*U3/D3`, etc.
#' For the "matt" weave the "(L+R)" part of the subtype can be further extended
#' to `(L1+R1+L2+R2)`, `(L1+R1+L2+R2+L3+R3)`, etc.
#'
#' @param type Type of weave.  See Details.
#' @param subtype Subtype of weave.  See Details.
#' @param nrow Number of rows (length of warp).
#' @param ncol Number of columns (length of weft).
#' @return A matrix of logical values indicating where the "warp"
#'         is "up" (if `TRUE`) or "down" (if `FALSE`).
#'         Indices `[1,1]` of the matrix corresponds to the bottom-left of the weave
#'         while indices `[1,ncol]` corresponds to the bottom-right of the weave.
#'         This matrix has a "pattern_weave" subclass which supports a special `print()` method.
#' @examples
#'  # supported weave names
#'  print(names_weave)
#'
#'  plain <- pattern_weave("plain", nrow = 7, ncol = 9)
#'  print(plain)
#'
#'  matt_irregular <- pattern_weave("matt_irregular", nrow = 9, ncol = 11)
#'  print(matt_irregular)
#'
#'  satin <- pattern_weave("satin", nrow = 9, ncol = 11)
#'  print(satin)
#'
#'  twill <- pattern_weave("twill", nrow = 9, ncol = 11)
#'  print(twill)
#'
#'  twill_zigzag <- pattern_weave("twill_zigzag", nrow = 18, ncol = 11)
#'  print(twill_zigzag)
#'
#' @seealso [grid.pattern_weave()] for drawing weaves onto a graphics device.
#'          See \url{https://textilestudycenter.com/derivatives-of-plain-weave/}
#'          for further information on the "matt" family of weaves,
#'          \url{https://textilelearner.net/twill-weave-features-classification-derivatives-and-uses/}
#'          for further information on the "twill" family of weaves, and
#'          \url{https://texwiz101.blogspot.com/2012/03/features-and-classification-of-satin.html}
#'          for further information on "satin" weaves.
#' @export
pattern_weave <- function(type = "plain", subtype = NULL, nrow = 5L, ncol = 5L) {
	spec <- get_weave_spec(type, subtype)

	if (is_twill(type)) {
		m <- pattern_weave_twill(spec, nrow, ncol)
	} else {
		m <- pattern_weave_matt(spec, nrow, ncol)
	}
	class(m) <- c("pattern_weave", "matrix", "array")
	m
}

# rep_each(1:2, 1:4) -> c(1, 2,2, 1,1,1, 2,2,2,2)
rep_each <- function(x, each) {
	n <- max(lengths(list(x, each)))
	x <- rep_len(x, n)
	each <- rep_len(each, n)
	unlist(lapply(seq.int(n), function(i) rep_len(x[i], each[i])))
}

pattern_weave_matt <- function(spec, nrow, ncol) {
	m <- matrix(FALSE, nrow = nrow, ncol = ncol)
	v <- rep_each(c(TRUE, FALSE), spec$up_down)
	v <- rep_len(v, nrow)
	not_v <- !v
	repeat_up_down <- rep_each(c(TRUE, FALSE), spec$up_down_reps)
	repeat_up_down <- rep_len(repeat_up_down, ncol)
	for (j in seq.int(ncol)) {
		if (repeat_up_down[j]) {
			m[, j] <- v
		} else {
			m[, j] <- not_v
		}
	}
	m
}

pattern_weave_twill <- function(spec, nrow, ncol) {
	m <- matrix(FALSE, nrow = nrow, ncol = ncol)
	skip <- 0L
	v0 <- rep_each(c(TRUE, FALSE), spec$up_down)
	for (j in seq_len(ncol)) {
		v <- cycle_elements(v0, -skip)
		if (spec$zigzag) {
			v <- add_zigzag(v)
		}
		if (spec$herringbone) {
			v <- add_herringbone(v)
		}
		v <- rep_len(v, nrow)
		m[, j] <- v
		skip <- skip + spec$move
	}
	m
}

#' @export
print.pattern_weave <- function(x, ...) {
	indices_x <- which(x)
	indices_o <- which(!x)
	x[indices_x] <- "X"
	x[indices_o] <- " "
	cat("/", rep_len("-", ncol(x)), "\\", "\n")
	for (i in rev(seq_len(nrow(x)))) {
		cat("|", x[i, ], "|", "\n")
	}
	cat("\\", rep_len("-", ncol(x)), "/", "\n")
	invisible(NULL)
}

#' @rdname pattern_weave
#' @export
names_weave <- c(
	"basket",
	"matt",
	"matt_irregular",
	"plain",
	"rib_warp",
	"satin",
	"twill",
	"twill_elongated",
	"twill_herringbone",
	"twill_zigzag"
)

get_weave_spec <- function(type = "plain", subtype = NULL) {
	if (!is.null(subtype) && is.na(subtype)) {
		subtype <- NULL
	}
	switch(
		type,
		# cases of irregular matt weave
		basket = get_weave_spec_matt(subtype %||% 2L),
		plain = get_weave_spec_matt(subtype %||% 1L),
		matt = get_weave_spec_matt(subtype %||% 3L),
		matt_irregular = get_weave_spec_matt(subtype %||% "3/2(4+2)"),
		rib_warp = get_weave_spec_matt(subtype %||% 2L, warp = TRUE),

		# cases of elongated twill weave
		satin = get_weave_spec_twill(subtype %||% 5L),
		twill = get_weave_spec_twill(subtype %||% "2/1"),
		twill_elongated = get_weave_spec_twill(subtype %||% "4/3(2)"),
		twill_zigzag = get_weave_spec_twill(subtype %||% "4/3(2)", zigzag = TRUE),
		twill_herringbone = get_weave_spec_twill(subtype %||% "4/3(2)", herringbone = TRUE),

		abort(paste("Don't know weave type", type))
	)
}

is_twill <- function(type) grepl("^twill|^satin", type)

# elongated twill U/D(M)
# U = number warp up, D = number warp down, M = move number
get_weave_spec_twill <- function(subtype = "2/1(2)", zigzag = FALSE, herringbone = FALSE) {
	if (is_integer(subtype)) {
		subtype <- n_to_twill(subtype)
	}
	if (is_ud(subtype)) {
		subtype <- ud_to_twill(subtype)
	}

	up_down <- get_ud(subtype)
	move <- get_extra(subtype)
	list(up_down = up_down, move = move, herringbone = herringbone, zigzag = zigzag)
}

# irregular matt U/D(L+R)
# U = number warp up, D = number warp down,
# L = number of warp up in repeat,  R = number of warp down in repeat
get_weave_spec_matt <- function(subtype = "3/3(4+2)", warp = FALSE) {
	if (is_integer(subtype)) {
		subtype <- n_to_matt(subtype, warp = warp)
	}
	if (is_ud(subtype)) {
		subtype <- ud_to_matt(subtype, warp = warp)
	}

	up_down <- get_ud(subtype)
	reps <- get_extra(subtype)
	list(up_down = up_down, up_down_reps = reps)
}

n_to_matt <- function(n = 1L, warp = FALSE) {
	n <- as.integer(n)
	if (warp) {
		glue("{n}/{n}(1+1)")
	} else {
		glue("{n}/{n}({n}+{n})")
	}
}
ud_to_matt <- function(ud = "2/1", warp = FALSE) {
	v_ud <- get_ud(ud)
	if (warp) {
		glue("{ud}(1+1)")
	} else {
		glue("{ud}({paste(v_ud, collapse='+')})")
	}
}

# satin is special case of twill elongated
# legal satin move is not one, repeat number, repeat number minus one, (multiple of) a factor of repeat number
# legal: 5:2 | 7:2,3 | 8:3 | 9:2,4 | 10:3 | 11:2,3,4,5 | 12:5 | 13: 2,3,4,5,6 | 14:3,5
n_to_twill <- function(n = 1L) {
	n <- as.integer(n)
	if (n < 5L) {
		move <- 1L
	} else {
		move <- switch(
			as.character(n),
			"5" = 2L,
			"6" = 1L, # no legal satin move
			"7" = 3L,
			"8" = 3L,
			"9" = 4L,
			"10" = 3L,
			"11" = 5L,
			"12" = 5L,
			"13" = 6L,
			"14" = 5L,
			n - 1L
		)
	}
	glue("{n-1}/1({move})")
}

ud_to_twill <- function(ud = "2/1") {
	glue("{ud}(1)")
}

is_integer <- function(s) is.integer(s) || grepl("^[[:digit:]]+$", s)

# "5/1" -> TRUE ; "5" -> FALSE ; "5/1(3)" -> FALSE ; "5/1(4+2)" -> FALSE
is_ud <- function(ud) grepl("^[[:digit:]]+/[[:digit:]]+(\\*[[:digit:]]+/[[:digit:]]+)*$", ud)

# "5/1(4+2)" -> c(5L, 1L) or "5/1(3)" -> c(5L, 1L)
get_ud <- function(ude) {
	ud <- as.integer(strsplit(gsub("\\(.*", "", ude), "/|\\*")[[1]])
	stopifnot(length(ud) %% 2L == 0L)
	ud
}

# "5/1(4+2)" -> c(4L, 2L) or "5/1(3)" -> c(3L)
get_extra <- function(ude) {
	as.integer(strsplit(gsub("[[:digit:]/*]+\\((.*)\\)", "\\1", ude), "\\+")[[1]])
}

add_zigzag <- function(x) {
	n <- length(x)
	c(x, rev(x[-n]), x[n])
}

add_herringbone <- function(x) c(x, !rev(x))

test_that("weaves work as expected", {
	pww <- function(...) print.pattern_weave(pattern_weave(...))

	expect_error(pattern_weave("foobar"), "Don't know weave type foobar")

	# irregular mat
	verify_output("../text_diagrams/plain.txt", pww("plain", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/basket.txt", pww("basket", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/matt.txt", pww("matt", 3, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/matt_21.txt", pww("matt", "2/1", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/matt3221.txt", pww("matt", "3/2*2/1", nrow = 15, ncol = 9))
	verify_output(
		"../text_diagrams/matt_irregular.txt",
		pww("matt_irregular", "3/2(4+2)", nrow = 7, ncol = 9)
	)
	verify_output("../text_diagrams/rib_warp", pww("rib_warp", "2/1", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/rib_warp.txt", pww("rib_warp", "2", nrow = 7, ncol = 9))

	# elongated twill
	verify_output("../text_diagrams/satin_5.txt", pww("satin", "5", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/twill_3.txt", pww("twill", 3L, nrow = 7, ncol = 9))
	verify_output(
		"../text_diagrams/twill_212.txt",
		pww("twill_elongated", "2/1(2)", nrow = 7, ncol = 9)
	)
	verify_output("../text_diagrams/twill_22.txt", pww("twill", "2/2", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/twill_13.txt", pww("twill", "1/3", nrow = 7, ncol = 9))
	verify_output(
		"../text_diagrams/twill_22_zigzag.txt",
		pww("twill_zigzag", "2/2", nrow = 15, ncol = 9)
	)
	verify_output(
		"../text_diagrams/twill_13_herringbone.txt",
		pww("twill_herringbone", "1/3", nrow = 15, ncol = 9)
	)
	verify_output("../text_diagrams/twill3221.txt", pww("twill", "3/2*2/1", nrow = 15, ncol = 9))
})

#' Circle patterned grobs
#'
#' `grid.pattern_circle()` draws a circle pattern onto the graphic device.
#'
#' @inheritParams grid.pattern
#' @param ... Currently ignored.
#' @param colour Stroke colour(s).
#' @param fill Fill colour(s) or [grid::pattern()] / gradient object(s).
#' @param angle Rotation angle in degrees.
#' @param density Approx. fraction of area the pattern fills.
#' @param spacing Spacing between repetitions of pattern (in `units` units).
#' @param xoffset Shift pattern along x axis (in `units` units).
#' @param yoffset Shift pattern along y axis (in `units` units).
#' @param units [grid::unit()] units for `spacing`, `xoffset`, and `yoffset` parameters.
#' @param alpha Alpha (between 0 and 1) or `NA` (default, preserves colors' alpha value).
#' @param linetype Stroke linetype.
#' @param linewidth Stroke linewidth.
#' @param size For backwards compatibility can be used to set `linewidth`.
#' @param grid Adjusts placement and density of certain graphical elements.
#'             `"square"` (default) is a square grid.
#'             `"hex"` is a hexagonal grid suitable for hexagonal and triangular tiling.
#'             `"hex_circle"` is a hexagonal grid suitable for circle packing.
#'             `"elongated_triangle"` is a grid used for the "elongated triangle" tiling.
#' @param type Adjusts the repeating of certain aesthetics such as color.
#'             Can use any type in `names_hex`, `names_square`, or `names_weave`.
#'             See for [pattern_hex()], [pattern_square()], and [pattern_weave()] for
#'             more information about supported `type` arguments.
#' @param subtype See for [pattern_hex()], [pattern_square()], and [pattern_weave()] for
#'             more information about supported `subtype` arguments.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_circle(x_hex, y_hex, fill = c("blue", "yellow"), density = 0.5)
#' grid::grid.newpage()
#' grid.pattern_circle(x_hex, y_hex, density = 0.8, grid = "hex_circle",
#'                     gp = grid::gpar(fill = c("blue", "yellow", "red")))
#' grid::grid.newpage()
#' grid.pattern_circle(x_hex, y_hex, density = 1.2, grid = "hex_circle",
#'                     gp = grid::gpar(fill = c("blue", "yellow", "red")))
#' # using a "twill_zigzag" 'weave' pattern
#' grid::grid.newpage()
#' grid.pattern_circle(x_hex, y_hex, fill = "blue", density = 0.5, type = "twill_zigzag")
#' @seealso
#'   See [grid.pattern_regular_polygon()] for a more general case of this pattern.
#' @export
grid.pattern_circle <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	grid = "square",
	type = NULL,
	subtype = NULL,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"circle",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		grid = grid,
		type = type,
		subtype = subtype,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_circle_via_sf <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	params$pattern_shape <- "circle"
	grob <- create_pattern_regular_polygon_via_sf(
		params,
		boundary_df,
		aspect_ratio,
		legend = legend
	)
	grob <- editGrob(grob, name = "circle")
	grob
}

#' Gradient patterned grobs
#'
#' `grid.pattern_gradient()` draws a gradient pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_circle
#' @param fill Colour.
#' @param fill2 Second colour.
#' @param orientation vertical, horizontal, or radial.
#' @param aspect_ratio Override aspect ratio.
#' @param use_R4.1_gradients Whether to use the gradient feature introduced in R v4.1
#'                           or use a `rasterGrob` approximation.
#'                           Note not all graphic devices support the grid gradient feature.
#' @param key_scale_factor Additional scale factor for legend.
#' @param res Assumed resolution (in pixels per graphic device inch) to use when creating array pattern.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'  if (requireNamespace("magick") && capabilities("png")) {
#'    x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'    y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'    grid.pattern_gradient(x_hex, y_hex, fill = "green")
#'  }
#'  if (requireNamespace("magick") && capabilities("png")) {
#'    grid::grid.newpage()
#'    grid.pattern_gradient(x_hex, y_hex, fill = "green", orientation = "radial")
#'  }
#' @export
grid.pattern_gradient <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	fill = gp$fill %||% "grey80",
	fill2 = "#4169E1",
	orientation = "vertical",
	alpha = gp$alpha %||% NA_real_,
	use_R4.1_gradients = getOption(
		"ggpattern_use_R4.1_gradients",
		getOption("ggpattern_use_R4.1_features")
	),
	aspect_ratio = 1,
	key_scale_factor = 1,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"gradient",
		x,
		y,
		id,
		fill = fill,
		fill2 = fill2,
		orientation = orientation,
		alpha = alpha,
		use_R4.1_gradients = use_R4.1_gradients,
		aspect_ratio = aspect_ratio,
		key_scale_factor = key_scale_factor,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' create a gradient image as an array
#'
#' @param width,height image dimensions
#' @param colour1,colour2 gradient colours
#' @param orientation vertical, horizontal or radial
#'
#' @return magick image
#'
#' @noRd
create_gradient_img <- function(
	width = 100,
	height = 100,
	colour1 = 'red',
	colour2 = 'blue',
	orientation = 'vertical'
) {
	colour1 <- convert_r_colour_to_magick_colour(colour1)
	colour2 <- convert_r_colour_to_magick_colour(colour2)
	colour_spec <- paste0(colour2, "-", colour1)

	if (orientation == 'radial') {
		colour_spec <- paste0(colour1, "-", colour2)
		pseudo <- paste0('radial-gradient:', colour_spec)
		img <- magick::image_blank(width, height, pseudo_image = pseudo)
	} else if (orientation == 'vertical') {
		pseudo <- paste0('gradient:', colour_spec)
		img <- magick::image_blank(width, height, pseudo_image = pseudo)
	} else if (orientation == 'horizontal') {
		pseudo <- paste0('gradient:', colour_spec)
		img <- magick::image_blank(height, width, pseudo_image = pseudo)
		img <- magick::image_rotate(img, 90)
	} else {
		abort(paste0("create_gradient_img() - Orientation not supported: ", orientation))
	}

	img
}

create_pattern_gradient <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	if (params$pattern_use_R4.1_gradients) {
		create_gradient_as_geometry(params, boundary_df, aspect_ratio, legend)
	} else {
		create_pattern_array(params, boundary_df, aspect_ratio, legend, create_gradient_as_array)
	}
}

create_gradient_as_geometry <- function(params, boundary_df, aspect_ratio, legend) {
	orientation <- check_default(
		params$pattern_orientation,
		options = c('vertical', 'horizontal', 'radial')
	)
	colour1 <- params$pattern_fill
	colour2 <- params$pattern_fill2

	x_min <- min(boundary_df$x)
	x_max <- max(boundary_df$x)
	y_min <- min(boundary_df$y)
	y_max <- max(boundary_df$y)
	x_range <- convertX(unit(x_max - x_min, "npc"), "in", valueOnly = TRUE)
	y_range <- convertY(unit(y_max - y_min, "npc"), "in", valueOnly = TRUE)
	if (x_range == 0 || y_range == 0) {
		return(nullGrob())
	}

	if (x_range > y_range) {
		r2 <- 0.5 * x_range / y_range
	} else {
		r2 <- 0.5 * y_range / x_range
	}

	gradient <- switch(
		orientation,
		horizontal = linearGradient(
			c(colour1, colour2),
			extend = "pad",
			x1 = 0,
			y1 = 0.5,
			x2 = 1,
			y2 = 0.5
		),
		radial = radialGradient(
			c(colour1, colour2),
			extend = "pad",
			cx1 = 0.5,
			cy1 = 0.5,
			cx2 = 0.5,
			cy2 = 0.5,
			r1 = 0,
			r2 = r2
		),
		vertical = linearGradient(
			c(colour1, colour2),
			extend = "pad",
			x1 = 0.5,
			y1 = 0,
			x2 = 0.5,
			y2 = 1
		)
	)
	gp <- gpar(col = NA, fill = gradient)

	convert_polygon_df_to_polygon_grob(boundary_df, gp = gp)
}

#' A shim to go between the main pattern function for an image, and the
#' specific pattern functon for an image.
#'
#' This function really just unwraps the 'params' into better arguments and
#' passes them to the actual img/array creation function
#'
#' @param width,height image dimensions
#' @param params geom parameters from ggplot
#' @param legend logical. TRUE if call comes during legend creation, otherwise FALSE.
#'
#' @return RGBA array
#'
#' @noRd
create_gradient_as_array <- function(width, height, params, legend) {
	assert_suggested("magick", "gradient")

	orientation <- check_default(
		params$pattern_orientation,
		options = c('vertical', 'horizontal', 'radial')
	)
	colour1 <- params$pattern_fill
	colour2 <- params$pattern_fill2

	img <- create_gradient_img(
		width = width,
		height = height,
		colour1 = colour1,
		colour2 = colour2,
		orientation = orientation
	)

	convert_img_to_array(img)
}

#' Ambient patterned grobs
#'
#' `grid.pattern_ambient()` draws noise patterns onto the graphic device powered by the `ambient` package.
#'
#' @param type Either cubic, perlin, simplex, value, white, or worley
#' @inheritParams grid.pattern_gradient
#' @inheritParams ambient::noise_simplex
#' @inheritParams ambient::noise_worley
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'  if (requireNamespace("ambient", quietly = TRUE)) {
#'    x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'    y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'    grid.pattern_ambient(x_hex, y_hex, fill = "green", fill2 = "blue")
#'  }
#'  if (requireNamespace("ambient")) {
#'    grid::grid.newpage()
#'    grid.pattern_ambient(x_hex, y_hex, fill = "green", fill2 = "blue", type = "cubic")
#'  }
#' @seealso For more information about the noise types please see the relevant `ambient` documentation:
#'   [ambient::noise_cubic()], [ambient::noise_perlin()], [ambient::noise_simplex()],
#'   [ambient::noise_value()], [ambient::noise_white()], and [ambient::noise_worley()].
#'  [grid.pattern_plasma()] provides an alternative noise pattern that depends on `magick`.
#' @export
grid.pattern_ambient <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	type = "simplex",
	fill = gp$fill %||% "grey80",
	fill2 = "#4169E1",
	frequency = 0.01,
	interpolator = "quintic",
	fractal = switch(type, worley = "none", "fbm"),
	octaves = 3,
	lacunarity = 2,
	gain = 0.5,
	pertubation = "none",
	pertubation_amplitude = 1,
	value = "cell",
	distance_ind = c(1, 2),
	jitter = 0.45,
	res = getOption("ggpattern_res", 72),
	alpha = NA_real_,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"ambient",
		x,
		y,
		id,
		type = type,
		fill = fill,
		fill2 = fill2,
		frequency = frequency,
		interpolator = interpolator,
		fractal = fractal,
		octaves = octaves,
		lacunarity = lacunarity,
		gain = gain,
		pertubation = pertubation,
		pertubation_amplitude = pertubation_amplitude,
		value = value,
		distance_ind = distance_ind,
		jitter = jitter,
		res = res,
		alpha = alpha,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' Create an array of noise using the 'ambient' package
#'
#' @param width,height area dimensions
#' @param params aesthetic parameters passed from the geom e.g. 'pattern_fill',
#'        'pattern_frequency' etc.
#' @param legend logical. If the request to create a pattern comes during
#'        creation of the legend, then this is TRUE, otherwise FALSE
#'
#' @return an RGBA numeric array with dimensions (`height`, `width`, 4)
#' @noRd
create_pattern_ambient <- function(width, height, params, legend) {
	assert_suggested("ambient", "ambient")

	colour1 <- as.character(params$pattern_fill)
	colour2 <- as.character(params$pattern_fill2)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a ramp function from these 2 colours
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	ramp_func <- grDevices::colorRamp(c(colour1, colour2), alpha = TRUE)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a noise matrix of the requested dimensions using 'ambient'.
	# The contents are normalised to all be in the range [0,1]
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	fn <- ambient_fn(params)
	noise_matrix <- fn(dim = c(height, width))
	noise_matrix <- ambient::normalise(noise_matrix)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Use each value in the noise matrix to lookup a colour using the
	# colour ramp function, then ensure the results are an RGBA array of the
	# correct dimensions.
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	colour_matrix <- ramp_func(noise_matrix) / 255
	noise_array <- array(colour_matrix, dim = c(height, width, 4))

	noise_array
}

ambient_fn <- function(params) {
	type <- params$pattern_type
	args <- list()
	args$frequency <- params$pattern_frequency
	args$pertubation <- params$pattern_pertubation
	args$pertubation_amplitude <- params$pattern_pertubation_amplitude
	if (type %in% c("perlin", "simplex", "value")) {
		args$interpolator <- params$pattern_interpolator
	}
	if (type != "white") {
		args$fractal <- params$pattern_fractal
		args$octaves <- params$pattern_octaves
		args$lacunarity <- params$pattern_lacunarity
		args$gain <- params$pattern_gain
	}
	if (type == "worley") {
		args$value <- params$pattern_value
		args$distance_ind <- params$pattern_distance_ind
		args$jitter <- params$pattern_jitter
	}
	function(dim) {
		args$dim <- dim
		fn <- switch(
			type,
			cubic = ambient::noise_cubic,
			perlin = ambient::noise_perlin,
			simplex = ambient::noise_simplex,
			value = ambient::noise_value,
			white = ambient::noise_white,
			worley = ambient::noise_worley,
			abort(glue("Don't know ambient type {type}"))
		)
		do.call(fn, args)
	}
}

#' Create grob objects for the pattern elements within a boundary
#'
#' @param params params/coords for a single element. named list or single row data.frame
#' @param boundary_df mask for the pattern rendering
#' @param aspect_ratio a aspect ratio of the plotting area.
#' @param legend is the pattern being created in the legend? default FALSE.
#'  Use this flag if you want different pattern drawing behviour for the legend.
#'
#' @return grid grob objects.
#' @noRd
create_pattern_aRtsy <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	assert_suggested("aRtsy", "aRtsy")
	requireNamespace("aRtsy", quietly = TRUE)
	stopifnot(guess_has_R4.1_features("patterns"))
	alpha <- ifelse(is.na(params$pattern_alpha), 1, params$pattern_alpha)
	colors <- update_alpha(params$pattern_fill, alpha)
	fn_name <- paste0("canvas_", params$pattern_type)
	fn <- utils::getFromNamespace(fn_name, "aRtsy")
	args <- list()
	nformals <- names(formals(fn))
	if ("color" %in% nformals) {
		# e.g. `canvas_maze()`
		args$color <- colors
	}
	if ("colors" %in% nformals) {
		# e.g. most canvas functions
		args$colors <- colors
	}
	pat <- ggplot2pat(do.call(fn, args))
	gp <- grid::gpar(col = NA_character_, fill = pat)
	convert_polygon_df_to_polygon_grob(boundary_df, gp = gp)
}

#' Grobs with patterns powered by the aRtsy package
#'
#' `grid.pattern_aRtsy()` draws patterns powered by the `aRtsy` package.
#' `names_aRtsy()` returns character vector of supported types.
#'
#' @param type Name of pattern.
#' @inheritParams grid.pattern_circle
#' @param fill Passed to the underlying `aRtsy` function's `color` / `colors` argument.
#' @param ... Currently ignored
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' if (requireNamespace("aRtsy", quietly = TRUE)) {
#'   print(names_aRtsy())
#' }
#'
#' \donttest{# Make take more than 5 seconds on CRAN servers
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' if (requireNamespace("aRtsy", quietly = TRUE) &&
#'     guess_has_R4.1_features("patterns")) {
#'   grid::grid.newpage()
#'   grid.pattern_aRtsy(x_hex, y_hex, type = "forest",
#'                      fill = c("black", "white", "grey"))
#' }
#' }
#' @seealso <https://koenderks.github.io/aRtsy/> for more information about the `aRtsy` package.
#' @export
grid.pattern_aRtsy <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	type = "strokes",
	fill = gp$fill %||% "grey80",
	alpha = gp$alpha %||% NA_real_,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"aRtsy",
		x,
		y,
		id,
		type = type,
		fill = fill,
		alpha = alpha,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' @rdname grid.pattern_aRtsy
#' @export
names_aRtsy <- function() {
	assert_suggested("aRtsy", "aRtsy")
	requireNamespace("aRtsy", quietly = TRUE)
	fns <- grep("^canvas", getNamespaceExports("aRtsy"), value = TRUE)
	gsub("^canvas_", "", fns)
}

test_that("aRtsy patterns works as expected", {
	skip_on_ci()
	skip_on_cran()
	skip_if_not_installed("aRtsy")
	skip_if_not(getRversion() >= "4.3.0")
	skip_if_not(isTRUE(all(capabilities(c("cairo", "png")))))

	f <- tempfile(fileext = ".png")
	png(f, type = "cairo")
	grid.pattern_aRtsy(type = "maze")
	dev.off()
	expect_true(file.size(f) > 0)
	unlink(f)

	f <- tempfile(fileext = ".png")
	png(f, type = "cairo")
	grid.pattern_aRtsy(type = "strokes")
	dev.off()
	expect_true(file.size(f) > 0)
	unlink(f)

	expect_true(length(names_aRtsy()) >= 34L)
})

#' Plotting character patterned grobs
#'
#' `grid.pattern_pch()` draws a plotting character pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_regular_polygon
#' @param shape An integer from `0` to `25` or `NA`.
#'              See [graphics::points()] for more details.
#'              Note we only support these shapes and do not
#'              support arbitrary ASCII / Unicode characters.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @seealso [grid.pattern_regular_polygon()] which is used to implement this pattern.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' gp <- grid::gpar(col = "black", fill = "lightblue")
#'
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   # pch 0-6 are simple shapes with no fill
#'   grid.pattern_pch(x_hex, y_hex, shape = 0:6, gp = gp,
#'                    spacing = 0.1, density = 0.4, angle = 0)
#' }
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   # pch 7-14 are compound shapes with no fill
#'   grid::grid.newpage()
#'   grid.pattern_pch(x_hex, y_hex, shape = 7:14, gp = gp,
#'                    spacing = 0.1, density = 0.4, angle = 0)
#' }
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   # pch 15-20 are filled with 'col'
#'   grid::grid.newpage()
#'   grid.pattern_pch(x_hex, y_hex, shape = 15:20, gp = gp,
#'                    spacing = 0.1, density = 0.4, angle = 0)
#' }
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   # pch 21-25 are filled with 'fill'
#'   grid::grid.newpage()
#'   grid.pattern_pch(x_hex, y_hex, shape = 21:25, gp = gp,
#'                    spacing = 0.1, density = 0.4, angle = 0)
#' }
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   # using a 'basket' weave `type` with two shapes
#'   grid::grid.newpage()
#'   grid.pattern_pch(x_hex, y_hex, shape = c(1,4), gp = gp,
#'                    type = "basket",
#'                    spacing = 0.1, density = 0.4, angle = 0)
#' }
#' @export
grid.pattern_pch <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	scale = 0.5,
	shape = 1L,
	grid = "square",
	type = NULL,
	subtype = NULL,
	rot = 0,
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"pch",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		scale = scale,
		shape = shape,
		grid = grid,
		type = type,
		subtype = subtype,
		rot = rot,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

# each pch will be represented by two regular polygons (although one may be "null")
create_pattern_pch <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	# vectorize fill, col, lwd, lty, density, rot, and shape
	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth
	lty <- params$pattern_linetype
	params$pattern_alpha <- NA_real_

	density <- params$pattern_density
	rot <- params$pattern_rot
	shape <- params$pattern_shape

	n_par <- max(lengths(list(fill, col, lwd, lty, density, rot, shape)))

	fill <- rep_len_fill(fill, n_par)
	col <- rep_len(col, n_par)
	lwd <- rep_len(lwd, n_par)
	lty <- rep_len(lty, n_par)
	density <- rep_len(density, n_par)
	rot <- rep_len(rot, n_par)
	shape <- rep_len(shape, n_par)

	# setup bottom and top regular polygons
	pint <- as.integer(shape)
	if (!all(is.na(pint))) {
		stopifnot(any(na_omit(pint) >= 0), any(na_omit(pint) <= 25))
	}
	pch <- ifelse(is.na(pint), "26", as.character(pint))
	pint <- ifelse(is.na(pint), 26L, pint)

	density1 <- ifelse(pint == 4L, 1.414 * density, density)
	density1 <- ifelse(pint == 20L, 2 / 3 * density, density1)

	density2 <- ifelse(pint == 7L | pint == 13L, 1.414 * density, density)

	fill <- ifelse(pint < 21L, col, fill)
	fill <- ifelse(pint < 15L, NA_character_, fill)

	col <- ifelse(pint > 14L & pint < 19L, NA_character_, col)

	rot1 <- rot + sapply(pch, get_rot_base)
	rot2 <- rot + sapply(pch, get_rot_top)

	shape1 <- sapply(pch, get_shape_base)
	shape2 <- sapply(pch, get_shape_top)

	params$pattern_fill <- fill
	params$pattern_col <- col
	params$pattern_linewidth <- lwd
	params$pattern_linetype <- lty
	params$pattern_scale <- 0.001
	params_base <- params_top <- params

	# bottom regular polygon
	params_base$pattern_shape <- shape1
	params_base$pattern_rot <- rot1
	params_base$pattern_density <- density1
	grob_base <- create_pattern_regular_polygon_via_sf(
		params_base,
		boundary_df,
		aspect_ratio,
		legend
	)
	grob_base <- editGrob(grob_base, name = "pch_base")

	# top regular polygon
	params_top$pattern_shape <- shape2
	params_top$pattern_rot <- rot2
	params_top$pattern_density <- density2
	grob_top <- create_pattern_regular_polygon_via_sf(params_top, boundary_df, aspect_ratio, legend)
	grob_top <- editGrob(grob_top, name = "pch_top")

	gl <- gList(grob_base, grob_top)

	gTree(children = gl, name = "pch")
}

get_rot_base <- function(pch) {
	switch(pch, "4" = 45, "6" = 180, "25" = 180, 0)
}

get_rot_top <- function(pch) {
	switch(pch, "7" = 45, "11" = 180, "13" = 45, 0)
}

get_shape_base <- function(pch) {
	switch(
		pch,
		"0" = "square",
		"2" = "convex3",
		"3" = "star4",
		"4" = "star4",
		"5" = "convex4",
		"6" = "convex3",
		"7" = "square",
		"9" = "convex4",
		"8" = "star8",
		"11" = "convex3",
		"12" = "square",
		"14" = "square",
		"15" = "square",
		"17" = "convex3",
		"18" = "convex4",
		"22" = "square",
		"23" = "convex4",
		"24" = "convex3",
		"25" = "convex3",
		"26" = "null",
		"circle"
	)
}

get_shape_top <- function(pch) {
	switch(
		pch,
		"7" = "star4",
		"9" = "star4",
		"10" = "star4",
		"11" = "convex3",
		"12" = "star4",
		"13" = "star4",
		"14" = "convex3",
		"null"
	)
}

na_omit <- function(x) Filter(Negate(is.na), x)

context("pch")
test_that("pch patterns work as expected", {
	skip_if_not_installed("vdiffr")
	skip_on_ci()
	library("vdiffr")

	x <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	y <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	gp <- gpar(fill = "yellow", col = "black")
	g.pp <- function(shape, ...) {
		grid.pattern_pch(x, y, shape = shape, angle = 0, spacing = 0.15, gp = gp)
	}

	expect_doppelganger("simple", function() g.pp(0:6))
	expect_doppelganger("compound", function() g.pp(7:14))
	expect_doppelganger("col_fill", function() g.pp(15:20))
	expect_doppelganger("fill_fill", function() g.pp(21:25))
})

# Added to ggplot2 by Thomas Lin Pedersen
# Fast data.frame constructor and indexing
# No checking, recycling etc. unless asked for
new_data_frame <- function(x = list(), n = NULL) {
	if (length(x) != 0 && is.null(names(x))) {
		abort("Elements must be named")
	}
	lengths <- vapply(x, length, integer(1))
	if (is.null(n)) {
		n <- if (length(x) == 0 || min(lengths) == 0) 0 else max(lengths)
	}
	for (i in seq_along(x)) {
		if (lengths[i] == n) {
			next
		}
		if (lengths[i] != 1) {
			abort("Elements must equal the number of rows or 1")
		}
		x[[i]] <- rep(x[[i]], n)
	}

	class(x) <- "data.frame"

	attr(x, "row.names") <- .set_row_names(n)
	x
}
data_frame <- function(...) {
	new_data_frame(list(...))
}
.pt <- 2.845276 # ggplot2 constant

ggplot2pat <- function(gg) {
	stopifnot(
		getRversion() >= "4.1.0",
		requireNamespace("ggplot2", quietly = TRUE),
		requireNamespace("gtable", quietly = TRUE)
	)
	gg <- suppressMessages(
		gg +
			ggplot2::scale_x_continuous(expand = c(0, 0)) +
			ggplot2::scale_y_continuous(expand = c(0, 0))
	)
	grob <- gtable::gtable_filter(ggplot2::ggplotGrob(gg), "panel")
	pat <- grid::pattern(grob)
	pat
}

#' Image patterned grobs
#'
#' `grid.pattern_image()` draws an image pattern onto the graphic device.
#'
#' Here is a description of the `type` arguments:
#' \describe{
#'   \item{expand}{Scale the image beyond the bounding box and crop it such that
#'                 the image fully covers the width and the height of the region.}
#'   \item{fit}{Scale the image such that either the width or the height of the image fits in the bounding box.
#'              Affected by `gravity`}
#'   \item{none}{Position a single image in the region without attempting to scale to the bounding box size.
#'               Affected by `scale` and `gravity`.}
#'   \item{squish}{Distort the image to cover the bounding box of the region.}
#'   \item{tile}{Repeat the image to cover the bounding box.  Affected by `tile`.}
#'  }
#'
#' @inheritParams grid.pattern_plasma
#' @param filename Image of filename or URL
#' @param type Image scaling type
#' @param gravity Position of image within area.  `magick::gravity_types()` returns a vector of supported values.
#' @param filter Filter to use when scaling. `magick::filter_types()` returns a vector of supported values.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'   \donttest{# May emit a "CPU time > 2.5 times elapsed time" NOTE in a CRAN check
#'   if (requireNamespace("magick")) {
#'     x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     logo_filename <- system.file("img", "Rlogo.png" , package = "png")
#'     grid.pattern_image(x_hex, y_hex, filename = logo_filename, type = "fit")
#'   }
#'   if (requireNamespace("magick")) {
#'     # "tile" `type` image pattern depends on `magick` functionality
#'     # which is not reliable across platforms
#'     grid::grid.newpage()
#'     try(grid.pattern_image(x_hex, y_hex, filename = logo_filename,
#'                            type = "tile"))
#'   }
#'   }
#' @seealso [grid.pattern_placeholder()] is an image pattern that uses images
#'          downloaded from the internet.
#'          [reset_image_cache()] resets the image cache used by `grid.pattern_image()`
#'          and [grid.pattern_placeholder()].
#' @export
grid.pattern_image <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	filename = "",
	type = "fit",
	scale = 1,
	gravity = switch(type, tile = "southwest", "center"),
	filter = "lanczos",
	alpha = gp$alpha %||% NA_real_,
	aspect_ratio = 1,
	key_scale_factor = 1,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"image",
		x,
		y,
		id,
		filename = filename,
		type = type,
		scale = scale,
		gravity = gravity,
		filter = filter,
		alpha = alpha,
		aspect_ratio = aspect_ratio,
		key_scale_factor = key_scale_factor,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' Read a user specified filename/URL as an image
#'
#' @inheritParams create_gradient_as_array
#'
#' @return array
#'
#' @noRd
img_read_as_array_wrapper <- function(width, height, params, legend) {
	assert_suggested("magick", "image")

	filename <- as.character(params$pattern_filename)

	fill_type <- tolower(as.character(params$pattern_type))
	fill_type <- check_default(fill_type, options = fill_types)

	gravity <- tolower(as.character(params$pattern_gravity))
	gravity <- check_default(gravity, tolower(magick::gravity_types()), 'center')

	filter <- tolower(as.character(params$pattern_filter))
	filter <- check_default(filter, tolower(magick::filter_types()), 'lanczos')

	scale <- params$pattern_scale
	scale <- check_default(scale, default = 1, type = 'numeric')

	img_read_as_array(
		filename = filename,
		width = width,
		height = height,
		fill_type = fill_type,
		gravity = gravity,
		filter = filter,
		scale = scale
	)
}

# The code for this function is in `standalone-guess_has_R4.1_features.R`

#' Guess whether "active" graphics device supports
#' the grid graphics features introduced in R v4.1.
#'
#' `guess_has_R4.1_features()` guesses whether "active" graphics device supports
#' the grid graphics features introduced in R v4.1.  If it guesses it does
#' it returns `TRUE` else `FALSE`.
#'
#' @section Usage in other packages:
#'
#' To avoid taking a dependency on `gridpattern` you may copy the source of `guess_has_R4.1_features()`
#' into your own package under the permissive MIT No Attribution (MIT-0) license.  Either use
#' `usethis::use_standalone("trevorld/gridpattern", "standalone-guess_has_R4.1_features.R")`
#' or copy the file `standalone-guess_has_R4.1_features.R` into your `R` directory and
#' add `grDevices` and `utils` to the `Imports` of your `DESCRIPTION` file.
#'
#' @param features Character vector of features to guess support for.
#'                 Will return `TRUE` only if guesses support for all requested features.\describe{
#'                 \item{"clippingPaths"}{Supports clipping path feature}
#'                 \item{"gradients"}{Supports (both linear and radial) gradient feature}
#'                 \item{"masks"}{Supports (alpha) mask feature}
#'                 \item{"patterns"}{Supports (tiling) pattern feature}
#'                 }
#' @seealso \url{https://www.stat.auckland.ac.nz/~paul/Reports/GraphicsEngine/definitions/definitions.html} for more info about the new grid graphics
#'         features introduced in R v4.1.
#' @return `TRUE` if we guess all `features` are supported else `FALSE`
#' @examples
#'   # If R version (weakly) greater than 4.1 should be TRUE
#'   pdf(tempfile(fileext = ".pdf"))
#'   print(guess_has_R4.1_features())
#'   invisible(dev.off())
#'
#'   # Should be FALSE
#'   postscript(tempfile(fileext = ".ps"))
#'   print(guess_has_R4.1_features())
#'   invisible(dev.off())
#'
#' @export
guess_has_R4.1_features <- guess_has_R4.1_features

#' Fetch a given path or URL as a magick image
#'
#' @param filename filename or URL
#'
#' @return magick image
#' @noRd
img_read <- function(filename) {
	if (identical(filename, '')) {
		return(magick::image_blank(100, 100, color = 'none'))
	}
	if (is.null(filename) || length(filename) == 0 || is.na(filename) || filename == '') {
		abort(paste0("bad filename: ", deparse(filename)))
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Fetch the URL as an image
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	img <- tryCatch(
		{
			magick::image_read(filename)
		},
		error = function(cond) {
			msg <- c(glue("couldn't read {shQuote(filename)}"), i = cond$message)
			abort(msg)
		}
	)
	img
}

#' Fetch a given path or URL as a 3D RGB array of values
#'
#' @param filename filename or URL
#' @param width,height if specified scale the image to these dimensions before
#'        converting to an array
#' @param fill_type how to fill the image area. 'none', 'fit',
#'        'squish', 'expand', 'tile'
#' @param gravity imagemagick gravity option on how to position an image during
#'                a resize/fill
#' @param scale scale image prior to tiling. Only for fill_type == 'tile'
#' @param filter filter for scaling. default: lanczos. only for fill_type = 'tile'
#'
#' @return 3D array. If the image was pure gray, then it will be promoted to
#'         be an unsaturated RGB image.
#'
#' @noRd
img_read_as_array <- function(
	filename,
	width = NULL,
	height = NULL,
	fill_type = 'squish',
	gravity = 'Center',
	scale = 1,
	filter = 'lanczos'
) {
	img <- img_read_memoised(filename)

	if (is.null(img)) {
		abort(glue("couldn't read {shQuote(filename)}"))
	}

	img <- fill_area_with_img(
		img,
		width,
		height,
		type = fill_type,
		gravity = gravity,
		filter = filter,
		scale = scale
	)

	convert_img_to_array(img)
}

#' Magick patterned grobs
#'
#' `grid.pattern_magick()` draws a `imagemagick` pattern onto the graphic device.
#' `names_magick`, `names_magick_intensity`, and
#' `names_magick_stripe` are character vectors of supported `type` values
#' plus subsets for shaded intensity and stripes.
#'
#' @inheritParams grid.pattern_image
#' @param type Magick pattern types.  `names_magick`, `names_magick_intensity`, and
#'             `names_magick_stripe` are character vectors of supported `type` values
#'             plus subsets for shaded intensity and stripes.
#' @param fill Fill colour
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'   if (requireNamespace("magick")) {
#'     x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     grid.pattern_magick(x_hex, y_hex, type="octagons", fill="blue", scale=2)
#'   }
#'
#'   # supported magick pattern names
#'   print(names_magick)
#' @seealso The `imagemagick` documentation <http://www.imagemagick.org/script/formats.php> for more information.
#' @export
grid.pattern_magick <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	type = "hexagons",
	fill = "grey20",
	scale = 1,
	filter = "box",
	alpha = gp$alpha %||% NA_real_,
	aspect_ratio = 1,
	key_scale_factor = 1,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"magick",
		x,
		y,
		id,
		type = type,
		fill = fill,
		scale = scale,
		scale = scale,
		filter = filter,
		alpha = alpha,
		aspect_ratio = aspect_ratio,
		key_scale_factor = key_scale_factor,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

## Names of patterns available in image magick, plus subsets for shaded intensity and stripes
## See \url{http://www.imagemagick.org/script/formats.php} for more information.

#' @rdname grid.pattern_magick
#' @export
names_magick <- c(
	"bricks",
	"checkerboard",
	"circles",
	"crosshatch",
	"crosshatch30",
	"crosshatch45",
	"fishscales",
	"gray0",
	"gray5",
	"gray10",
	"gray15",
	"gray20",
	"gray25",
	"gray30",
	"gray35",
	"gray40",
	"gray45",
	"gray50",
	"gray55",
	"gray60",
	"gray65",
	"gray70",
	"gray75",
	"gray80",
	"gray85",
	"gray90",
	"gray95",
	"gray100",
	"hexagons",
	"horizontal",
	"horizontal2",
	"horizontal3",
	"horizontalsaw",
	"hs_bdiagonal",
	"hs_cross",
	"hs_diagcross",
	"hs_fdiagonal",
	"hs_horizontal",
	"hs_vertical",
	"left30",
	"left45",
	"leftshingle",
	"octagons",
	"right30",
	"right45",
	"rightshingle",
	"smallfishscales",
	"vertical",
	"vertical2",
	"vertical3",
	"verticalbricks",
	"verticalleftshingle",
	"verticalrightshingle",
	"verticalsaw"
)

#' @rdname grid.pattern_magick
#' @export
names_magick_intensity <- c(
	"gray0",
	"gray5",
	"gray10",
	"gray15",
	"gray20",
	"gray25",
	"gray30",
	"gray35",
	"gray40",
	"gray45",
	"gray50",
	"gray55",
	"gray60",
	"gray65",
	"gray70",
	"gray75",
	"gray80",
	"gray85",
	"gray90",
	"gray95",
	"gray100"
)


#' @rdname grid.pattern_magick
#' @export
names_magick_stripe <- c(
	"crosshatch",
	"crosshatch30",
	"crosshatch45",
	"horizontal",
	"horizontal2",
	"horizontal3",
	"hs_bdiagonal",
	"hs_cross",
	"hs_diagcross",
	"hs_fdiagonal",
	"hs_horizontal",
	"hs_vertical",
	"left30",
	"left45",
	"right30",
	"right45",
	"vertical",
	"vertical2",
	"vertical3"
)

#' Read a user specified filename as an image
#'
#' @inheritParams create_gradient_as_array
#'
#' @return array
#'
#' @noRd
create_magick_pattern_as_array <- function(width, height, params, legend) {
	assert_suggested("magick", "magick")

	if (legend) {
		params$pattern_scale <- params$pattern_scale * params$pattern_key_scale_factor
	}
	type <- check_default(
		as.character(params$pattern_type),
		options = names_magick,
		default = 'hexagons'
	)

	scale <- check_default(params$pattern_scale, default = 1, type = 'numeric')

	filter <- tolower(as.character(params$pattern_filter))
	filter <- check_default(filter, options = tolower(magick::filter_types()), default = 'box')

	colour <- as.character(params$pattern_fill)

	img <- create_magick_pattern_img_scaled(
		width = width,
		height = height,
		type = type,
		colour = colour,
		scale = scale,
		filter = filter
	)

	convert_img_to_array(img)
}

#' Create grob objects for the pattern elements within a boundary
#'
#' @param params params/coords for a single element. named list or single row data.frame
#' @param boundary_df mask for the pattern rendering
#' @param aspect_ratio a aspect ratio of the plotting area.
#' @param legend is the pattern being created in the legend? default FALSE.
#'  Use this flag if you want different pattern drawing behviour for the legend.
#'
#' @return grid grob objects.
#' @noRd
create_pattern_none <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	grid::nullGrob()
}

#' Grobs without any pattern
#'
#' `grid.pattern_none()` draws nothing onto the graphic device.
#'
#' @inheritParams grid.pattern
#' @param ... Currently ignored
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_none(x_hex, y_hex)
#' @seealso [grid::grid.null()]
#' @export
grid.pattern_none <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"none",
		x,
		y,
		id,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' Weave patterned grobs
#'
#' `grid.pattern_weave()` draws a weave pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_crosshatch
#' @param fill The fill colour for the horizontal "weft" lines.
#' @param fill2 The fill colour for the vertical "warp" lines.
#' @param type The weave type.  See [pattern_weave()] for more details.
#' @param subtype The weave subtype.  See [pattern_weave()] for more details.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' gp <- grid::gpar(colour = "black", fill = "lightblue", lwd=0.5)
#'
#' # Plain weave (default weave)
#' grid.pattern_weave(x_hex, y_hex, fill2 = "yellow",
#'                    gp = gp, spacing = 0.1, density = 0.3)
#'
#' # Irregular matt weave
#' grid::grid.newpage()
#' grid.pattern_weave(x_hex, y_hex,  type = "matt_irregular",
#'                    fill2 = "yellow", gp = gp, spacing = 0.1, density = 0.3)
#'
#' # Twill weave
#' grid::grid.newpage()
#' grid.pattern_weave(x_hex, y_hex, type = "twill",
#'                    fill2 = "yellow", gp = gp, spacing = 0.1, density = 0.3)
#'
#' # Zig-zag twill
#' grid::grid.newpage()
#' grid.pattern_weave(x_hex, y_hex, type = "twill_zigzag",
#'                    fill2 = "yellow", gp = gp, spacing = 0.05, density = 0.7)
#'
#' # Herringbone twill with density 1
#' grid::grid.newpage()
#' gp$col <- NA
#' grid.pattern_weave(x_hex, y_hex, type = "twill_herringbone",
#'                    fill2 = "yellow", gp = gp, spacing = 0.05, density = 1.0)
#' @seealso [pattern_weave()]
#' @export
grid.pattern_weave <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	fill2 = fill,
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	grid = "square",
	type = "plain",
	subtype = NA,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"weave",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		fill2 = fill2,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		grid = grid,
		type = type,
		subtype = subtype,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_weave_via_sf <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	# 'weft' 'yarns' will just be normal (horizontal) stripes
	grob_weft <- create_crosshatch_via_sf_helper(params, boundary_df, add_top_hatch = FALSE)
	grob_weft <- editGrob(grob_weft, name = "weft")
	# we'll compute 'covered' and 'uncovered' 'warp' rectangles to represent warp 'yarns'
	l <- create_warp_via_sf(params, boundary_df)
	grobTree(l$warp_covered, grob_weft, l$warp_uncovered, name = "weave")
}

create_warp_via_sf <- function(params, boundary_df) {
	if (abs(params$pattern_density - 1) < .Machine$double.eps^0.5) {
		params$pattern_density <- 1 - 1e-6
	}
	stopifnot(params$pattern_density <= 1)

	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm)

	fill <- update_alpha(params$pattern_fill2, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype
	gp <- gpar(col = col, fill = fill, lwd = lwd, lty = lty, lineend = 'square')

	m_weave <- pattern_weave(
		params$pattern_type,
		params$pattern_subtype,
		nrow = length(grid_xy$y),
		ncol = length(grid_xy$x)
	)

	# compute vertical stripes clipped to boundary
	boundary_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = 0)
	stripes_sf <- create_v_stripes_sf(params, grid_xy, vpm)
	clipped_stripes_sf <- sf::st_intersection(stripes_sf, boundary_sf)

	# compute warp squares covered by weft lines
	warp_covered_sf <- create_warp_covered_sf(params, grid_xy, vpm, m_weave)
	warp_covered_sf <- sf::st_buffer(warp_covered_sf, dist = 0)
	clipped_covered_sf <- sf::st_intersection(clipped_stripes_sf, warp_covered_sf)

	# warp rectangles not covered by weft lines is just stripes minus under squares
	buffered_covered_sf <- sf::st_buffer(clipped_covered_sf, vpm$length / 1e9)
	clipped_uncovered_sf <- sf::st_difference(clipped_stripes_sf, buffered_covered_sf)

	grob_uncovered <- sf_multipolygon_to_polygon_grob(
		clipped_uncovered_sf,
		gp,
		default.units,
		"warp_uncovered"
	)

	grob_covered <- sf_multipolygon_to_polygon_grob(
		clipped_covered_sf,
		gp,
		default.units,
		"warp_covered"
	)

	list(warp_uncovered = grob_uncovered, warp_covered = grob_covered)
}

create_warp_covered_sf <- function(params, grid_xy, vpm, m_weave) {
	halfwidth <- 0.5 * grid_xy$h_spacing * params$pattern_density
	# need list of lists each containing a five row matrix of rectangle vertices
	l_rects <- list()
	for (i in seq_len(nrow(m_weave))) {
		for (j in seq_len(ncol(m_weave))) {
			if (!m_weave[i, j]) {
				x0 <- grid_xy$x[j]
				y0 <- grid_xy$y[i]
				x <- x0 + c(-1, -1, 1, 1) * halfwidth
				y <- y0 + c(-1, 1, 1, -1) * halfwidth
				l_rects <- append(l_rects, list(polygon_ring(x, y, params$pattern_angle, vpm)))
			}
		}
	}
	sf::st_multipolygon(l_rects)
}

# ---
# repo: trevorld/gridpattern
# file: standalone-update_alpha.R
# last-updated: 2024-04-18
# license: https://spdx.org/licenses/MIT.html
# imports: [grDevices, grid, rlang]
# ---
#
# nocov start
#
# You may copy this source into your own R package
# by either using `usethis::use_standalone("trevorld/gridpattern", "standalone-update_alpha.R")`
# or simply copying this file into your `R` directory and adding `grDevices`, `grid`, and `rlang` to
# the `Imports` of your `DESCRIPTION` file.

# The MIT License (MIT)
# =====================
#
# Copyright © 2024 Trevor L. Davis
# Copyright © 2020 mikefc@coolbutuseless.com
# Copyright © 2023 ggplot2 authors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the “Software”), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

# Pattern utilities mainly added to ggplot2 by Teun van den Brand
# Tweaked by Trevor L. Davis to remove external dependencies
# and work better for {ggpattern} / {gridpattern} use cases.
update_alpha <- function(fill, alpha) {
	if (!is.list(fill)) {
		# Happy path of no patterns
		update_alpha_col(fill, alpha)
	} else if (is_pattern(fill) || any(vapply(fill, is_pattern, logical(1)))) {
		# Path with patterns
		update_pattern_alpha(fill, alpha)
	} else if (is.list(fill) && length(fill) == 1L && !any(vapply(fill, is_pattern, logical(1)))) {
		# List of length one of (possibly multiple) colours
		update_alpha_col(fill[[1L]], alpha)
	} else {
		# We are either dealing with faulty fill specification
		stop("`fill` must be a vector of colours or list of <GridPattern> objects.")
	}
}

# Similar to grid:::is.pattern
is_pattern <- function(x) {
	inherits(x, "GridPattern")
}

# replacement for `scales::alpha()` that only depends on {grDevices}
update_alpha_col <- function(colour, alpha = NA_real_) {
	n <- max(lengths(list(colour, alpha)))
	colour <- rep_len(colour, n)
	alpha <- rep_len(alpha, n)
	m <- grDevices::col2rgb(colour, alpha = TRUE) / 255.0
	m[4, ] <- ifelse(is.na(alpha), m[4, ], alpha)
	apply(m, 2, function(x) grDevices::rgb(x[1], x[2], x[3], x[4]))
}

#' Modify transparency for patterns
#'
#' This generic allows you to add your own methods for adding transparency to
#' pattern-like objects.
#'
#' @param x Object to be interpreted as pattern.
#' @param alpha A `numeric` vector between 0 and 1. If `NA`, alpha values
#'   are preserved.
#'
#' @return `x` with modified transparency
#' @noRd
update_pattern_alpha <- function(x, alpha, ...) {
	UseMethod("update_pattern_alpha")
}

#' @export
update_pattern_alpha.default <- function(x, alpha, ..., name = NULL) {
	if (!is.atomic(x)) {
		stop("Can't apply `update_pattern_alpha()` to this object.")
	}
	grid::pattern(grid::rectGrob(name = name), gp = grid::gpar(fill = update_alpha_col(x, alpha)))
}

#' @export
update_pattern_alpha.GridPattern <- function(x, alpha, ...) {
	x$colours <- update_alpha_col(x$colours, alpha[1])
	x
}

#' @export
update_pattern_alpha.GridTilingPattern <- function(x, alpha, ...) {
	if (all(is.na(alpha) | alpha == 1)) {
		return(x)
	}
	grob <- rlang::env_get(environment(x$f), "grob")
	gp <- grid::gpar(fill = update_alpha_col("white", alpha))
	mask <- grid::as.mask(grid::rectGrob(gp = gp))
	if (is.null(grob$vp)) {
		grob$vp <- grid::viewport(mask = mask)
	} else {
		grob$vp <- grid::editViewport(grob$vp, mask = mask)
	}
	new_env <- new.env(parent = environment(x$f))
	rlang::env_bind(new_env, grob = grob)
	environment(x$f) <- new_env
	x
}

#' @export
update_pattern_alpha.list <- function(x, alpha, ...) {
	Map(update_pattern_alpha, x = x, alpha = alpha)
}

# nocov end

#' Mask grob using another grob to specify the (alpha) mask
#'
#' `alphaMaskGrob()` masks a grob using another grob to specify the (alpha) mask.
#'
#' @param maskee Grob to be masked
#' @param masker Grob that defines masking region
#' @param use_R4.1_masks If `TRUE` use the grid mask feature introduced in R v4.1.0.
#'                       If `FALSE` do a `rasterGrob` approximation.
#'                       If `NULL` try to guess an appropriate choice.
#'                       Note not all graphic devices support the grid mask feature.
#' @param png_device \dQuote{png} graphics device to save intermediate raster data with if `use_R4.1_masks` is `FALSE`.
#'                   If `NULL` and suggested package `ragg` is available
#'                   and versions are high enough we directly capture masked raster via [ragg::agg_capture()].
#'                   Otherwise we will use `png_device`
#'                   (default [ragg::agg_png()] if available else [grDevices::png()]) and [png::readPNG()]
#'                   to manually compute a masked raster.
#' @param res Resolution of desired `rasterGrob` in pixels per inch if `use_R4.1_masks` is `FALSE`.
#' @return A `grid` grob
#' @inheritParams grid::polygonGrob
#' @examples
#'   \donttest{# May take more than 5 seconds on CRAN servers
#'   if (capabilities("png") && require("grid")) {
#'     maskee <- patternGrob("circle", gp = gpar(col = "black", fill = "yellow"),
#'                            spacing = 0.1, density = 0.5)
#'     angle <- seq(2 * pi / 4, by = 2 * pi / 6, length.out = 7)
#'     x_hex_outer <- 0.5 + 0.5 * cos(angle)
#'     y_hex_outer <- 0.5 + 0.5 * sin(angle)
#'     x_hex_inner <- 0.5 + 0.25 * cos(rev(angle))
#'     y_hex_inner <- 0.5 + 0.25 * sin(rev(angle))
#'     gp <- gpar(lwd = 0, col = NA, fill = "white")
#'     masker <- grid::pathGrob(x = c(x_hex_outer, x_hex_inner),
#'                              y = c(y_hex_outer, y_hex_inner),
#'                              id = rep(1:2, each = 7),
#'                              rule = "evenodd", gp = gp)
#'     masked <- alphaMaskGrob(maskee, masker, use_R4.1_masks = FALSE)
#'     grid.draw(masked)
#'   }
#'   if (capabilities("png") && require("grid")) {
#'     maskee_transparent <- rectGrob(gp = gpar(col = NA, fill = "blue"))
#'     gp <- gpar(lwd = 20, col = "black", fill = grDevices::rgb(0, 0, 0, 0.5))
#'     masker_transparent <- editGrob(masker, gp = gp)
#'     masked_transparent <- alphaMaskGrob(maskee_transparent,
#'                                         masker_transparent,
#'                                         use_R4.1_masks = FALSE)
#'     grid.newpage()
#'     grid.draw(masked_transparent)
#'   }
#'   }
#' @export
alphaMaskGrob <- function(
	maskee,
	masker,
	use_R4.1_masks = getOption(
		"ggpattern_use_R4.1_masks",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	name = NULL,
	gp = gpar(),
	vp = NULL
) {
	gTree(
		maskee = maskee,
		masker = masker,
		use_R4.1_masks = use_R4.1_masks,
		res = res,
		png_device = png_device,
		name = name,
		gp = gp,
		vp = vp,
		cl = "alpha_mask"
	)
}

# Avoid R CMD check WARNING on R 4.0 which lacks `mask` argument
vport <- function(...) viewport(...)

#' @export
makeContent.alpha_mask <- function(x) {
	current_dev <- grDevices::dev.cur()
	on.exit(grDevices::dev.set(current_dev))

	use_R4.1_masks <- x$use_R4.1_masks
	if (is.null(use_R4.1_masks)) {
		use_R4.1_masks <- guess_has_R4.1_features("masks")
	} else {
		use_R4.1_masks <- as.logical(use_R4.1_masks)
	}

	stopifnot(getRversion() >= '4.1.0' || !use_R4.1_masks)

	if (use_R4.1_masks) {
		grob <- grobTree(x$maskee, vp = vport(mask = x$masker), name = "alpha_mask")
	} else if (
		is.null(x$png_device) &&
			getRversion() >= '4.1.0' &&
			requireNamespace("ragg", quietly = TRUE) &&
			packageVersion("ragg") >= '1.2.0'
	) {
		grob <- gridpattern_mask_agg_capture(x$maskee, x$masker, x$res)
	} else {
		png_device <- x$png_device %||% default_png_device()
		if (device_supports_masks(png_device)) {
			grob <- gridpattern_mask_raster_straight(x$maskee, x$masker, x$res, png_device)
		} else {
			grob <- gridpattern_mask_raster_manual(x$maskee, x$masker, x$res, png_device)
		}
	}

	gl <- gList(grob)
	setChildren(x, gl)
}

device_supports_masks <- function(png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	png_file <- tempfile(fileext = ".png")
	on.exit(unlink(png_file), add = TRUE)
	png_device(png_file)
	value <- guess_has_R4.1_features("masks")
	dev.off()
	value
}

gridpattern_mask_agg_capture <- function(maskee, masker, res) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	ragg::agg_capture(height = height, width = width, res = res, bg = "transparent")
	grob <- alphaMaskGrob(maskee, masker, use_R4.1_masks = TRUE)
	grid.draw(grob)
	raster_masked <- dev.capture(native = FALSE)
	dev.off()
	grid::rasterGrob(raster_masked)
}

default_png_device <- function() {
	if (requireNamespace("ragg", quietly = TRUE)) {
		ragg::agg_png
	} else {
		stopifnot(capabilities("png"))
		grDevices::png
	}
}

gridpattern_mask_raster_straight <- function(maskee, masker, res, png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	png_masked <- tempfile(fileext = ".png")
	on.exit(unlink(png_masked), add = TRUE)
	png_device(png_masked, height = height, width = width, res = res, bg = "transparent")
	grob <- alphaMaskGrob(maskee, masker, use_R4.1_masks = TRUE)
	grid.draw(grob)
	dev.off()

	raster_masked <- png::readPNG(png_masked, native = FALSE)
	grid::rasterGrob(raster_masked)
}

gridpattern_mask_raster_manual <- function(maskee, masker, res, png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	png_maskee <- tempfile(fileext = ".png")
	on.exit(unlink(png_maskee), add = TRUE)
	png_device(png_maskee, height = height, width = width, res = res, bg = "transparent")
	grid.draw(maskee)
	dev.off()

	png_masker <- tempfile(fileext = ".png")
	on.exit(unlink(png_masker), add = TRUE)
	png_device(png_masker, height = height, width = width, res = res, bg = "transparent")
	grid.draw(masker)
	dev.off()

	raster_maskee <- png::readPNG(png_maskee, native = FALSE)
	raster_masker <- png::readPNG(png_masker, native = FALSE)

	stopifnot(
		length(dim(raster_maskee)) == 3L,
		length(dim(raster_masker)) == 3L,
		dim(raster_maskee)[3L] >= 3L,
		dim(raster_masker)[3L] >= 3L
	)
	if (dim(raster_maskee)[3L] < 4L) {
		raster_maskee <- add_alpha_channel(raster_maskee)
	}
	if (dim(raster_masker)[3L] < 4L) {
		raster_masker <- add_alpha_channel(raster_masker)
	}

	raster_masked <- raster_maskee
	raster_masked[,, 4L] <- raster_maskee[,, 4L] * raster_masker[,, 4L]

	rasterGrob(raster_masked, name = "alpha_mask")
}

add_alpha_channel <- function(a) {
	a_ <- array(NA, dim = c(dim(a)[1], dim(a)[2], 4L))
	a_[,, -4L] <- a
	a_[,, 4L] <- 1.
	a_
}

test_raster <- function(ref_png, fn, update = FALSE) {
	f <- file.path("../figs/array", ref_png)
	if (update) {
		my_png(f, fn)
	}
	ref <- magick::image_read(f)

	tmpfile <- tempfile(fileext = ".png")
	my_png(tmpfile, fn)
	image <- magick::image_read(tmpfile)
	unlink(tmpfile)

	diff <- magick::image_compare(image, ref, "AE")
	bool <- attr(diff, "distortion") < 0.01
	if (!bool) {
		grDevices::dev.new()
		grid::grid.text(ref_png, y = 0.95)
		grid::pushViewport(grid::viewport(x = 0.25, width = 0.5))
		grid::grid.text("ref", y = 0.9)
		grid::grid.raster(ref)
		grid::popViewport()
		grid::pushViewport(grid::viewport(x = 0.75, width = 0.5))
		grid::grid.text("new", y = 0.9)
		grid::grid.raster(image)
		grid::popViewport()
	}
	expect_true(bool)
}

my_png <- function(f, fn) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	grDevices::png(f, type = "cairo", width = 240, height = 240)
	fn()
	grDevices::dev.off()
}

test_that("array patterns works as expected", {
	skip_on_ci()
	skip_on_cran()
	skip_if_not(capabilities("cairo"))
	skip_if_not_installed("magick", "2.7.4")
	skip_if_not_installed("ragg")

	x <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	y <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	test_raster("gradient.png", function() {
		grid.pattern_gradient(
			x,
			y,
			fill = "blue",
			fill2 = "green",
			orientation = "radial",
			use_R4.1_gradients = FALSE
		)
	})
	test_raster("gradient_horizontal.png", function() {
		grid.pattern_gradient(
			x,
			y,
			fill = "blue",
			fill2 = "green",
			orientation = "horizontal",
			use_R4.1_gradients = FALSE
		)
	})
	logo_filename <- system.file("img", "Rlogo.png", package = "png")
	test_raster("image.png", function() {
		grid.pattern_image(x, y, filename = logo_filename, type = "fit")
	})
	test_raster("image_expand.png", function() {
		grid.pattern_image(x, y, filename = logo_filename, type = "expand")
	})
	test_raster("image_tile.png", function() {
		grid.pattern_image(x, y, filename = logo_filename, type = "tile", scale = -2)
	})
	test_raster("image_none.png", function() {
		grid.pattern_image(x, y, filename = logo_filename, type = "none", scale = -1)
	})
	test_raster("image_squish.png", function() {
		grid.pattern_image(x, y, filename = logo_filename, type = "squish")
	})
	test_raster("hatch_azure.png", function() {
		grid.pattern_hatch(x, y, type = "azure", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_gules.png", function() {
		grid.pattern_hatch(x, y, type = "gules", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_sable.png", function() {
		grid.pattern_hatch(x, y, type = "sable", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_sanguine.png", function() {
		grid.pattern_hatch(x, y, type = "sanguine", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_or.png", function() {
		grid.pattern_hatch(x, y, type = "or", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_cendree.png", function() {
		grid.pattern_hatch(x, y, type = "cendree", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_olive.png", function() {
		grid.pattern_hatch(x, y, type = "olive", colour = "black", spacing = 0.1)
	})
	test_raster("hatch_proper.png", function() {
		grid.pattern_hatch(
			x,
			y,
			type = "proper",
			subtype = "fox-davies",
			colour = "black",
			spacing = 0.1
		)
	})
	test_raster("hatch_steel.png", function() {
		grid.pattern_hatch(
			x,
			y,
			type = "steel",
			subtype = "goodman",
			colour = "black",
			spacing = 0.1
		)
	})
	test_raster("hatch_copper.png", function() {
		grid.pattern_hatch(
			x,
			y,
			type = "copper",
			subtype = "goodman",
			colour = "black",
			spacing = 0.1
		)
	})
	test_raster("hatch_unicode_orange.png", function() {
		grid.pattern_hatch(
			x,
			y,
			type = "orange",
			subtype = "unicode",
			colour = "black",
			spacing = 0.1
		)
	})
	test_raster("hatch_unicode_grey.png", function() {
		grid.pattern_hatch(
			x,
			y,
			type = "grey",
			subtype = "unicode",
			colour = "black",
			spacing = 0.1
		)
	})
	test_raster("line.png", function() {
		grid.pattern_line(x, y, colour = "black", angle = 0, spacing = 0.1)
	})
	test_raster("line_stagger.png", function() {
		grid.pattern_line(
			x,
			y,
			colour = "black",
			angle = 0,
			spacing = 0.1,
			linetype = "dashed",
			stagger = TRUE
		)
	})
	test_raster("magick.png", function() {
		grid.pattern_magick(x, y, type = "octagons", fill = "blue", scale = 2)
	})
	test_raster("placeholder.png", function() grid.pattern_placeholder(x, y, type = "bear"))

	test_raster("plasma_zero.png", function() {
		grid.pattern_plasma(x = c(0.5, 0.5, 0.5, 0.5), y = c(0, 1, 1, 0), fill = "green")
	})

	test_raster("plasma.png", function() {
		magick::magick_set_seed(42) # introduced in v2.7.4
		grid.pattern_plasma(x, y, fill = "green")
	})

	playing_card_symbols <- c("\u2660", "\u2665", "\u2666", "\u2663")
	test_raster("text.png", function() {
		grid.pattern_text(
			x,
			y,
			shape = playing_card_symbols,
			colour = c("black", "red", "red", "black"),
			use_R4.1_masks = TRUE,
			size = 18,
			spacing = 0.1,
			angle = 0
		)
	})

	gp <- gpar(fill = c("blue", "red", "yellow", "green"), col = "black")
	test_raster("rose.png", function() {
		grid.pattern_rose(
			x,
			y,
			spacing = 0.15,
			density = 0.5,
			angle = 0,
			use_R4.1_masks = NULL,
			frequency = 1:4,
			gp = gp
		)
	})

	create_pattern_simple <- function(width, height, params, legend) {
		choice <- params$pattern_type
		if (is.null(choice) || is.na(choice) || !is.character(choice)) {
			choice <- 'a'
		}
		values <- switch(
			choice,
			a = rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3),
			b = rep(c(1, 0, 0, 1, 0.5, 0.5, 1, 1, 0, 0, 0, 0, 0, 0.5), each = 7),
			c = rep(seq(0, 1, 0.05), each = 7),
			rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3)
		)
		simple_array <- array(values, dim = c(height, width, 4))

		simple_array
	}
	options(ggpattern_array_funcs = list(simple = create_pattern_simple))
	test_raster("simple.png", function() grid.pattern("simple", x, y, type = "b"))

	# clippingPathGrob()
	clippee <- patternGrob(
		"circle",
		gp = gpar(col = "black", fill = "yellow"),
		spacing = 0.1,
		density = 0.5
	)
	angle <- seq(2 * pi / 4, by = 2 * pi / 6, length.out = 7)
	x_hex_outer <- 0.5 + 0.5 * cos(angle)
	y_hex_outer <- 0.5 + 0.5 * sin(angle)
	x_hex_inner <- 0.5 + 0.25 * cos(rev(angle))
	y_hex_inner <- 0.5 + 0.25 * sin(rev(angle))
	clipper <- grid::pathGrob(
		x = c(x_hex_outer, x_hex_inner),
		y = c(y_hex_outer, y_hex_inner),
		id = rep(1:2, each = 7),
		rule = "evenodd"
	)

	clipped <- clippingPathGrob(
		clippee,
		clipper,
		use_R4.1_clipping = FALSE,
		png_device = grDevices::png
	)
	test_raster("clipGrob_cairo.png", function() grid.draw(clipped))

	clipped <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = NULL)
	test_raster("clipGrob_feature.png", function() grid.draw(clipped))

	png_device <- default_png_device()
	test_raster("clipGrob_manual.png", function() {
		clipped <- gridpattern_clip_raster_manual(clippee, clipper, 72, png_device)
		grid.draw(clipped)
	})

	clipped <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = FALSE)
	test_raster("clipGrob_ragg.png", function() grid.draw(clipped))

	# alphaMaskGrob()
	clippee2 <- rectGrob(gp = gpar(fill = "blue", col = NA))
	clipper2 <- editGrob(clipper, gp = gpar(col = NA, fill = "black"))
	clipper3 <- editGrob(clipper2, gp = gpar(col = "black", lwd = 20, fill = rgb(0, 0, 0, 0.5)))

	masked <- alphaMaskGrob(clippee2, clipper3, use_R4.1_masks = FALSE, png_device = grDevices::png)
	test_raster("alphaMaskGrob_cairo.png", function() {
		grid.draw(masked)
	})

	masked <- alphaMaskGrob(clippee2, clipper3, use_R4.1_masks = NULL)
	test_raster("alphaMaskGrob_feature.png", function() grid.draw(masked))

	test_raster("alphaMaskGrob_manual.png", function() {
		masked <- gridpattern_mask_raster_manual(clippee2, clipper3, 72, png_device)
		grid.draw(masked)
	})

	masked <- alphaMaskGrob(clippee, clipper2, use_R4.1_masks = FALSE)
	test_raster("alphaMaskGrob_ragg.png", function() grid.draw(masked))

	# ambient
	skip_if_not_installed("ambient")
	test_raster("ambient.png", function() {
		set.seed(42)
		grid.pattern_ambient(x, y, fill = "green", fill2 = "blue")
	})
	test_raster("ambient_worley.png", function() {
		set.seed(42)
		grid.pattern_ambient(x, y, type = "worley", fill = "green", fill2 = "blue")
	})
})

# The code for this function is in `standalone-update_alpha.R`

#' Update colour and/or pattern transparency
#'
#' `update_alpha()` modifies the transparency of colours and/or patterns.
#'
#' * This is a fork of pattern utilities mainly added to `{ggplot2}` by Teun van den Brand.
#' * `update_alpha()` does not depend on `{ggplot2}` or `{scales}`.
#' * Like [ggplot2::fill_alpha()] but unlike [scales::alpha()] it also attempts
#'   to set the transparency of `<GridPattern>` objects.
#' * Unlike [ggplot2::fill_alpha()] it will work on a list of length one
#'   containing a vector of color strings.
#'
#' @section Usage in other packages:
#'
#' To avoid taking a dependency on `gridpattern` you may copy the source of `update_alpha()`
#' into your own package under the permissive MIT license.  Either use
#' `usethis::use_standalone("trevorld/gridpattern", "standalone-update_alpha.R")`
#' or copy the file `update_alpha.R` into your `R` directory and
#' add `grDevices`, `grid`, and `rlang` to the `Imports` of your `DESCRIPTION` file.
#'
#' @param fill A fill colour given as a `character` or `integer` vector, or as a
#'   (list of) `<GridPattern>` object(s) and/or colour(s).
#' @param alpha A transparency value between 0 (transparent) and 1 (opaque),
#'   parallel to `fill`.
#' @return A `character` vector of colours or list of `<GridPattern>` objects.
#' @examples
#' # Typical color input
#' update_alpha("red", 0.5)
#'
#' # Pattern input
#' if (getRversion() >= "4.2" && requireNamespace("grid", quietly = TRUE)) {
#'   update_alpha(list(grid::linearGradient()), 0.5)
#' }
#' @export
update_alpha <- update_alpha

#' Hex pattern matrix
#'
#' `pattern_hex()` returns an integer matrix indicating where each
#'  color (or other graphical element) should be drawn on a (horizontal) hex grid
#'  for a specified hex pattern type and subtype.
#'  `names_hex` lists the currently supported hex `type`s.
#'
#' \describe{
#' \item{"hex"}{Attempts to use a uniform coloring if it exists.
#'              For subtype `1L`, `2L`, and `3L` we use the "hex1" pattern.
#'              For subtype `4L` we use the "hex2" pattern.
#'              For subtype `7L` we use the "hex3" pattern.
#'              Else a uniform coloring does not exist and we use the "hex_skew" pattern.}
#' \item{"hex1"}{Provides the 1-uniform colorings of a hexagonal tiling.  Only exists for `subtype` `1L`, `2L`, or `3L`.}
#' \item{"hex2"}{Provides the 2-uniform colorings of a hexagonal tiling.  Only exists for `subtype` `2L` or `4L`.}
#' \item{"hex3"}{Provides the 3-uniform colorings of a hexagonal tiling.  Only exists for `subtype` `2L` or `7L`.}
#' \item{"hex_skew"}{For the "hex_skew" `type` we cycle through `subtype` elements on the horizontal line and "main" diagonal line.
#'    For some `subtype` numbers this may lead to noticeable color repeats on the "skew" diagonal line.
#'    If `subtype` is strictly greater than `2L` then a hexagon should never touch another hexagon of the same color.}
#' }
#'
#' @param type Currently just supports "hex".
#' @param subtype An integer indicating number of colors (or other graphical elements).
#' @param nrow Number of rows (height).
#' @param ncol Number of columns (width).
#' @return A matrix of integer values indicating where the each color
#'         or other graphical elements should be drawn on a horizontal hex grid
#'         (i.e. hexagons are assumed to be pointy side up).
#'         Indices `[1,1]` of the matrix corresponds to the bottom-left of the grid
#'         while indices `[1,ncol]` corresponds to the bottom-right of the grid.
#'         The even rows are assumed to be on the **left** of the ones on the odd rows
#'         (for those in the same column in the matrix).
#'         This matrix has a "pattern_hex" subclass which supports a special `print()` method.
#' @examples
#'  # supported hex names
#'  print(names_hex)
#'
#'  # 1-uniform 3-color
#'  hex_3color <- pattern_hex("hex1", 3L, nrow = 7L, ncol = 9L)
#'  print(hex_3color)
#'
#'  # 2-uniform 4-color
#'  hex_4color <- pattern_hex("hex2", 4L, nrow = 7L, ncol = 9L)
#'  print(hex_4color)
#'
#' @seealso [grid.pattern_regular_polygon()] for drawing to a graphics device
#'           hexagons, triangles, circles, etc. in hexagon patterns.
#'          The tiling vignette features several examples of regular polygon tiling using
#'          this both the "hex" and "hex_circle" types `vignette("tiling", package = "gridpattern")`.
#'          For more information on uniform colorings of a hexagonal tiling see
#'          \url{https://en.wikipedia.org/wiki/Hexagonal_tiling#Uniform_colorings}.
#' @export
pattern_hex <- function(type = "hex", subtype = NULL, nrow = 5L, ncol = 5L) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 2L
	}
	stopifnot(is_integer(subtype))
	n <- as.integer(subtype)
	if (type == "hex") {
		if (n < 4L) {
			type <- "hex1"
		} else if (n == 4L) {
			type <- "hex2"
		} else if (n == 7L) {
			type <- "hex3"
		} else {
			type <- "hex_skew"
		}
	}
	m <- switch(
		type,
		hex_skew = pattern_hex_skew(n, nrow, ncol),
		hex1 = pattern_hex1(n, nrow, ncol),
		hex2 = pattern_hex2(n, nrow, ncol),
		hex3 = pattern_hex3(n, nrow, ncol),
		abort(paste("Don't know hex pattern type", type))
	)
	class(m) <- c("pattern_hex", "matrix", "array")
	m
}

pattern_hex_skew <- function(n = NULL, nrow = 5L, ncol = 5L) {
	m <- matrix(1L, nrow = nrow, ncol = ncol)
	if (n == 1L) {
		return(m)
	}
	s <- seq.int(n)
	skip <- 0L
	for (i in seq.int(nrow)) {
		step <- skip + i - 1L
		v <- rep_len(cycle_elements(s, step), ncol)
		m[i, ] <- v
		if (i %% 2 == 0) skip <- skip + 1L
	}
	m
}

pattern_hex1 <- function(n, nrow, ncol) {
	stopifnot(n < 4L)
	if (n == 2L) {
		m <- pattern_hex_skew(3L, nrow, ncol)
		indices <- which(m == 3L)
		m[indices] <- 2L
		m
	} else {
		pattern_hex_skew(n, nrow, ncol)
	}
}

pattern_hex2 <- function(n, nrow, ncol) {
	stopifnot(n == 2L || n == 4L)
	m <- matrix(2L, nrow = nrow, ncol = ncol)
	v1 <- rep_len(1:2, ncol)
	v3 <- rep_len(2:1, ncol)
	for (i in seq.int(nrow)) {
		im <- i %% 4L
		if (im == 1L) {
			m[i, ] <- v1
		} else if (im == 3L) {
			m[i, ] <- v3
		}
	}
	if (n == 4L) {
		v2 <- rep_len(3:4, ncol)
		v4 <- rep_len(4:3, ncol)
		for (i in seq.int(nrow)) {
			im <- i %% 4L
			if (im == 2L) {
				m[i, ] <- v2
			} else if (im == 0L) {
				m[i, ] <- v4
			}
		}
	}
	m
}

pattern_hex3 <- function(n, nrow, ncol) {
	stopifnot(n == 2L || n == 7L)
	m <- matrix(1L, nrow = nrow, ncol = ncol)
	if (n == 2L) {
		s <- c(1L, rep_len(2L, 6L))
	} else {
		s <- seq.int(7L)
	}
	skip <- 0L
	for (i in seq.int(nrow)) {
		step <- skip + 4 * (i - 1L)
		v <- rep_len(cycle_elements(s, step), ncol)
		m[i, ] <- v
		if (i %% 2 == 0) skip <- skip + 1L
	}
	m
}

#' @rdname pattern_hex
#' @export
names_hex <- c("hex", "hex1", "hex2", "hex3", "hex_skew")

#' @export
print.pattern_hex <- function(x, ...) {
	d <- dim(x)
	x <- matrix(int_to_char(x), nrow = d[1], ncol = d[2])
	cat("/", rep("-", 2 * ncol(x)), "\\", "\n", sep = "")
	for (i in rev(seq_len(nrow(x)))) {
		if (i %% 2 == 1) {
			cat("|", paste0(" ", x[i, ]), "|", "\n", sep = "")
		} else {
			cat("|", paste0(x[i, ], " "), "|", "\n", sep = "")
		}
	}
	cat("\\", rep("-", 2 * ncol(x)), "/", "\n", sep = "")
	invisible(NULL)
}

test_that("hex patterns work as expected", {
	phh <- function(...) print.pattern_hex(pattern_hex(...))

	verify_output("../text_diagrams/hex.txt", phh("hex_skew", 3L, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/hex1_1.txt", phh("hex", 1L, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/hex1_2.txt", phh("hex", NULL, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/hex2_2.txt", phh("hex2", 2L, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/hex2_4.txt", phh("hex", 4L, nrow = 9, ncol = 9))
	verify_output("../text_diagrams/hex3_2.txt", phh("hex3", 2L, nrow = 9, ncol = 9))
	verify_output("../text_diagrams/hex3_7.txt", phh("hex", 7L, nrow = 9, ncol = 9))
	verify_output("../text_diagrams/hex_skew_5.txt", phh("hex", 5L, nrow = 9, ncol = 9))
})

#' Create grob objects for the pattern elements within a boundary
#'
#' @param params params/coords for a single element. named list or single row data.frame
#' @param boundary_df mask for the pattern rendering
#' @param aspect_ratio a aspect ratio of the plotting area.
#' @param legend is the pattern being created in the legend? default FALSE.
#'  Use this flag if you want different pattern drawing behviour for the legend.
#'
#' @return grid grob objects.
#' @noRd
create_pattern_fill <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	alpha <- ifelse(is.na(params$pattern_alpha), 1, params$pattern_alpha)
	fill <- update_alpha(params$pattern_fill, alpha)
	gp <- grid::gpar(col = NA_character_, fill = fill)

	convert_polygon_df_to_polygon_grob(boundary_df, gp = gp)
}

#' Grobs with a simple fill pattern
#'
#' `grid.pattern_fill()` draws a simple fill pattern onto the graphics device.
#'
#' @inheritParams grid.pattern_circle
#' @param ... Currently ignored
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_fill(x_hex, y_hex, fill = "blue")
#'
#' if (guess_has_R4.1_features("patterns")) {
#'   grid::grid.newpage()
#'   stripe_fill <- patternFill("stripe", fill = c("red", "blue"))
#'   grid.pattern_fill(x_hex, y_hex, fill = stripe_fill)
#' }
#' @seealso [grid::grid.polygon()]
#' @export
grid.pattern_fill <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	fill = gp$fill %||% "grey80",
	alpha = gp$alpha %||% NA_real_,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"fill",
		x,
		y,
		id,
		fill = fill,
		alpha = alpha,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' @section Package options:
#' The following `gridpattern` options may be set globally via [base::options()]:
#'  \describe{
#'    \item{ggpattern_array_funcs}{Set custom \dQuote{array} pattern functions.}
#'    \item{ggpattern_geometry_funcs}{Set custom \dQuote{geometry} pattern functions.}
#'    \item{ggpattern_res}{Set custom raster image resolution (pixels per inch) for certain patterns.}
#'    \item{ggpattern_use_R4.1_clipping}{If `TRUE` use the grid clipping path feature introduced in R v4.1.0.
#'                          If `FALSE` do a `rasterGrob` approximation of the clipped pattern.
#'                          If `NULL` try to guess an appropriate choice.}
#'    \item{ggpattern_use_R4.1_features}{If `TRUE` sets the default for all the other
#'                          `ggpattern_use_R4.1_*` options arguments to `TRUE`.
#'                          If `FALSE` sets them to `FALSE`.}
#'    \item{ggpattern_use_R4.1_gradients}{If `TRUE` use the grid gradient feature introduced in R v4.1.0.
#'                          If `FALSE` do a `rasterGrob` approximation of the gradient pattern.
#'                          If `NULL` try to guess an appropriate choice.}
#'    \item{ggpattern_use_R4.1_masks}{If `TRUE` use the grid mask feature introduced in R v4.1.0.
#'                          If `FALSE` do a `rasterGrob` approximation of the masked pattern.
#'                          If `NULL` try to guess an appropriate choice.}
#'    \item{ggpattern_use_R4.1_patterns}{If `TRUE` use the grid pattern feature introduced in R v4.1.0.
#'                          Currently only used by a couple of examples.}
#'  }
#'  Note to use the R v4.1.0 features one needs R be (at least) version 4.1 and not all graphic devices
#'  support any/all these features.  See \url{https://www.stat.auckland.ac.nz/~paul/Reports/GraphicsEngine/definitions/definitions.html} for more information on these features.
"_PACKAGE"

#' Create one of imagemagick's internal patterns
#'
#' These are all 2-colour pixel patterns - the 'white' part will be made transparent.
#'
#' @param width,height image dimensions
#' @param type name of the imagemagick pattern. See \url{http://www.imagemagick.org/script/formats.php}
#'        for more information. See \link{names_magick} for a list of all
#'        supported imagemagick patterns.
#' @param colour colour used to draw the pattern
#'
#' @noRd
create_magick_pattern_img <- function(
	width = 100,
	height = 100,
	type = 'hexagons',
	colour = 'black'
) {
	type <- check_default(tolower(type), names_magick, default = 'checkerboard')
	colour <- convert_r_colour_to_magick_colour(colour)

	if (width == 0 || height == 0) {
		warn("create_magick_pattern_img(): zero size")
		return(magick::image_blank(10, 10))
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a pattern image of the required size
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	pseudo <- paste0("pattern:", type)
	img <- magick::image_blank(width, height, pseudo_image = pseudo)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# The checkerboard is the only(?) pattern which isn't pure black and white.
	# for the sake of consistency it will be thresholded from its original
	# two-level gray colours into pure black and white.
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (type %in% 'checkerboard') {
		img <- magick::image_threshold(img, type = 'black')
		img <- magick::image_threshold(img, type = 'white')
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Make the white transparent
	# Colourie the black pixels into the desired colour
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	img <- magick::image_transparent(img, 'white')
	img <- magick::image_colorize(img, opacity = 100, colour)

	img
}

#' Create a scaled version one of imagemagick's internal patterns
#'
#' These are all 2-colour pixel patterns - the 'white' part will be made transparent.
#'
#' @inheritParams create_magick_pattern_img
#' @param scale pattern scaling factor defualt: 1
#' @param filter filter to apply when sacling pattern.  default: 'box' for crisp
#'        edges to the pixel.  See \code{magick::filter_types()} for a full list
#'        of filters.
#'
#' @noRd
create_magick_pattern_img_scaled <- function(
	width = 100,
	height = 100,
	type = 'hexagons',
	colour = 'black',
	scale = 1,
	filter = 'box'
) {
	scale <- check_default(scale, default = 1, type = 'numeric')

	if (scale < 0.01 || scale > 100) {
		scale <- 1
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Create a scaled version of the pattern
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	new_width <- width / scale
	new_height <- height / scale

	img <- create_magick_pattern_img(new_width, new_height, type = type, colour = colour)

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Which we'll scale back down to the original size
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	img <- fill_area_with_img_squish(img, width, height, filter = filter)

	img
}

#' Heraldic color hatching patterned grobs
#'
#' `grid.pattern_hatch()` draws a heraldic color hatching patterns onto the graphic device.
#' `names_hatch()` returns a character vector of supported `type` values.
#'
#' @inheritParams grid.pattern_line
#' @param type A tincture or color name.  `names_hatch()` lists supported values.
#'   Both traditional tincture names (e.g. `"gules"`) and modern color equivalents
#'   (e.g. `"red"`) are accepted.  Matching is case-insensitive and ignores hyphens
#'   and spaces.
#' @param subtype A string with one of
#'
#'  * `"combinatorial"` (default): an extension of the seven standard Petra Sancta hatchings with systematically derived mixed-color hatchings.
#'  * `"fox-davies"`: the hatchings in Fox-Davies' *A Complete Guide to Heraldry*.
#'  * `"goodman"`: the hatchings in David Goodman's *Heraldic Tincture* reference.
#'  * `"unicode"`: the hatchings used in Unicode character charts.
#'
#'   The string is case-insensitive and hyphens and spaces are ignored.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   grid::grid.newpage()
#'   grid.pattern_hatch(x_hex, y_hex, type = "azure", colour = "blue")
#' }
#' if (capabilities("png") || guess_has_R4.1_features("masks")) {
#'   grid::grid.newpage()
#'   grid.pattern_hatch(x_hex, y_hex, type = "cendree", colour = "grey", spacing = 0.1)
#' }
#' print(names_hatch())
#' print(names_hatch("fox-davies"))
#' print(names_hatch("goodman"))
#' print(names_hatch("unicode"))
#' @seealso [grid.pattern_line()] for single-direction lines,
#'   [grid.pattern_crosshatch()] for perpendicular lines.
#'   `vignette("hatching", package = "gridpattern")` for a visual overview of all subtypes.
#'   <https://en.wikisource.org/wiki/A_Complete_Guide_to_Heraldry/Chapter_7#74> for Fox-Davies'
#'   _A Complete Guide to Heraldry_.
#'   <https://david.goodman.graphics/portfolio/item/crests-heraldry-and-coats-of-arms/> for
#'   David Goodman's Heraldic Tincture reference.
#' @export
grid.pattern_hatch <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	type = "gules",
	subtype = "combinatorial",
	colour = gp$col %||% "grey20",
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	lineend = gp$lineend %||% "round",
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	use_R4.1_masks = getOption(
		"ggpattern_use_R4.1_masks",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"hatch",
		x,
		y,
		id,
		colour = colour,
		type = type,
		subtype = subtype,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		lineend = lineend,
		linewidth = linewidth,
		use_R4.1_masks = use_R4.1_masks,
		png_device = png_device,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' @rdname grid.pattern_hatch
#' @param accent If `TRUE`, return tincture names using their traditional
#'   accented spellings where applicable (e.g. `"tenn\u00e9"`, `"brun\u00e2tre"`).
#'   Defaults to `FALSE`.
#' @export
names_hatch <- function(
	subtype = c("combinatorial", "fox-davies", "goodman", "unicode"),
	accent = FALSE
) {
	subtype <- tolower(subtype)
	subtype <- match.arg(subtype)
	nms <- if (subtype == "fox-davies") {
		sort(names(HATCH_FOX_DAVIES))
	} else if (subtype == "unicode") {
		sort(unname(HATCH_COLORS[names(HATCH_UNICODE)]))
	} else if (subtype == "goodman") {
		sort(names(HATCH_GOODMAN))
	} else {
		sort(unname(HATCH_COLORS[names(HATCH_COMBINATORIAL)]))
	}
	if (accent) {
		accented <- HATCH_ACCENTS[nms]
		nms[!is.na(accented)] <- accented[!is.na(accented)]
	}
	nms
}

# Accented spellings for tincture names that have them.
HATCH_ACCENTS <- c(
	`bleu celeste` = "bleu c\u00e9leste",
	cendree = "cendr\u00e9e",
	tenne = "tenn\u00e9",
	brunatre = "brun\u00e2tre"
)

# Internal: hatching specs per subtype.
# Each entry is one of:
#   list(special = "plain")   -- no pattern (argent)
#   list(special = "solid")   -- solid colour fill (unicode sable)
#   list(angles, linetypes)   -- one or two sets of stroked lines

# Combinatorial Petra Sancta: systematically extends the seven standard tinctures.
# Three derivation rules:
#   white + color  → dashed lines at that color's angle     (lightened tints)
#   yellow + color → dotdash lines at that color's angle    (Munsell YR and GY secondaries)
#   color + color  → crossing solid lines at both angles    (Munsell BG, PB, RP secondaries + others)
HATCH_COMBINATORIAL <- list(
	# === Base seven tinctures (standard Petra Sancta) ===
	argent = list(special = "plain"),
	or = list(special = "circle"),
	azure = list(angles = 0, linetypes = "solid"),
	gules = list(angles = 90, linetypes = "solid"),
	sable = list(angles = c(0, 90), linetypes = c("solid", "solid")),
	vert = list(angles = 135, linetypes = "solid"),
	purpure = list(angles = 45, linetypes = "solid"),
	# === White + color: dashed lines ===
	`bleu celeste` = list(angles = 0, linetypes = "dashed"), # argent + azure
	carnation = list(angles = 90, linetypes = "dashed"), # argent + gules
	cendree = list(special = "cendree"), # argent + sable
	mint = list(angles = 135, linetypes = "dashed"), # argent + vert (new)
	lavender = list(angles = 45, linetypes = "dashed"), # argent + purpure (new)
	# === Yellow + color: dotdash lines ===
	orange = list(angles = 90, linetypes = "dotdash"), # or + gules (YR)
	lime = list(angles = 135, linetypes = "dotdash"), # or + vert (GY)
	olive = list(special = "olive"), # or + sable: alternating circles and plus signs
	rose = list(angles = 45, linetypes = "dotdash"), # or + purpure (YP)
	# === Color × color: crossing lines ===
	brunatre = list(
		angles = c(90, 0, 135),
		linetypes = c("solid", "solid", "solid"),
		spacings = c(1, 1, 1 / sqrt(2))
	), # gules × azure × vert
	eisenfarbe = list(angles = c(45, 135), linetypes = c("solid", "solid")), # purpure × vert
	sanguine = list(angles = c(90, 45), linetypes = c("solid", "solid")), # gules × purpure (RP)
	tenne = list(angles = c(90, 135), linetypes = c("solid", "solid")), # gules × vert
	teal = list(angles = c(0, 135), linetypes = c("solid", "solid")), # azure × vert (BG, new)
	violet = list(angles = c(0, 45), linetypes = c("solid", "solid")) # azure × purpure (PB, new)
)

# Fox-Davies is the standard Petra Sancta tinctures plus the German heraldry extensions,
# all of which are defined in HATCH_COMBINATORIAL.
HATCH_FOX_DAVIES <- c(
	HATCH_COMBINATORIAL[c(
		"argent",
		"or",
		"azure",
		"gules",
		"sable",
		"vert",
		"purpure",
		"eisenfarbe",
		"sanguine",
		"tenne",
		"brunatre",
		"carnation",
		"cendree",
		"orange",
		"bleu celeste"
	)],
	list(proper = list(special = "proper"))
)

# Goodman system (david.goodman.graphics v2.0 2024-02-14).
# Shares most specs with fox-davies; key differences:
#   murrey   = eisenfarbe spec (45° + 135°) — not fox-davies sanguine
#   sanguine = teal spec (0° + 135°)        — different from fox-davies sanguine (90° + 45°)
#   rose     = carnation spec (90° dashed)  — rose and carnation share one hatching
# Rare metals (steel, copper, bronze, lead) are unique to this system.
HATCH_GOODMAN <- c(
	HATCH_COMBINATORIAL[c(
		"argent",
		"or",
		"azure",
		"gules",
		"sable",
		"vert",
		"purpure",
		"cendree",
		"brunatre",
		"bleu celeste",
		"carnation",
		"orange",
		"tenne"
	)],
	list(
		murrey = HATCH_COMBINATORIAL[["eisenfarbe"]], # 45° + 135° (dark red-purple)
		sanguine = HATCH_COMBINATORIAL[["teal"]], # 0° + 135° (blood red)
		rose = HATCH_COMBINATORIAL[["carnation"]], # 90° dashed (rose/carnation share hatching)
		steel = list(special = "steel"), # + signs in square grid
		copper = list(special = "c_hex"), # c in hex grid
		bronze = list(special = "c_hex"), # c in hex grid
		lead = list(special = "c_hex") # c in hex grid
	)
)

HATCH_UNICODE <- c(
	HATCH_COMBINATORIAL[c("argent", "or", "azure", "gules", "vert", "purpure", "tenne")],
	list(
		`bleu celeste` = list(angles = 0, linetypes = "dotdash"),
		carnation = HATCH_COMBINATORIAL[["rose"]], # unicode "pink" heart uses 45° dotdash
		cendree = list(special = "checker"),
		orange = list(angles = c(0, 45), linetypes = c("solid", "solid")),
		sable = list(special = "solid")
	)
)

# Modern color equivalents for tincture names.
# For the unicode subtype, values match the canonical Unicode emoji color names.
HATCH_COLORS <- c(
	`bleu celeste` = "light blue",
	argent = "white",
	azure = "blue",
	bronze = "bronze",
	brunatre = "umbre",
	carnation = "pink",
	cendree = "grey",
	copper = "copper",
	eisenfarbe = "slate",
	gules = "red",
	lavender = "lavender",
	lead = "lead grey",
	lime = "lime green",
	mint = "mint green",
	murrey = "mulberry",
	olive = "olive",
	or = "yellow",
	orange = "orange",
	proper = "color of nature",
	purpure = "purple",
	rose = "rose",
	sable = "black",
	sanguine = "magenta",
	steel = "steel grey",
	teal = "teal",
	tenne = "brown",
	vert = "green",
	violet = "violet"
)


# Map from normalized user input to canonical tincture name.
# Normalization: tolower() + remove hyphens and spaces.
HATCH_NAME_MAP <- c(
	argent = "argent",
	white = "argent",
	silver = "argent",
	azure = "azure",
	blue = "azure",
	cobalt = "azure",
	bleuceleste = "bleu celeste",
	"bleuc\u00e9leste" = "bleu celeste",
	skyblue = "bleu celeste",
	lightblue = "bleu celeste",
	celeste = "bleu celeste",
	"c\u00e9leste" = "bleu celeste",
	ciel = "bleu celeste",
	bronze = "bronze",
	brunatre = "brunatre",
	"brun\u00e2tre" = "brunatre",
	darkbrown = "brunatre",
	umbre = "brunatre",
	umber = "brunatre",
	carnation = "carnation",
	flesh = "carnation",
	pink = "carnation",
	cendree = "cendree",
	"cendr\u00e9e" = "cendree",
	grey = "cendree",
	gray = "cendree",
	ash = "cendree",
	ashgray = "cendree",
	ashgrey = "cendree",
	copper = "copper",
	slate = "eisenfarbe",
	irongrey = "eisenfarbe",
	irongray = "eisenfarbe",
	iron = "eisenfarbe",
	eisenfarbe = "eisenfarbe",
	gules = "gules",
	red = "gules",
	crimson = "gules",
	lavender = "lavender",
	lilac = "lavender",
	lead = "lead",
	leadgrey = "lead",
	leadgray = "lead",
	lime = "lime",
	chartreuse = "lime",
	limegreen = "lime",
	yellowgreen = "lime",
	mint = "mint",
	mintgreen = "mint",
	seafoam = "mint",
	murrey = "murrey",
	mulberry = "murrey",
	olive = "olive",
	or = "or",
	yellow = "or",
	gold = "or",
	orange = "orange",
	colorofnature = "proper",
	proper = "proper",
	natural = "proper",
	purpure = "purpure",
	purple = "purpure",
	rose = "rose",
	sable = "sable",
	black = "sable",
	sanguine = "sanguine",
	bloodred = "sanguine",
	magenta = "sanguine",
	steel = "steel",
	steelgrey = "steel",
	steelgray = "steel",
	teal = "teal",
	cyan = "teal",
	aqua = "teal",
	aquamarine = "teal",
	turquoise = "teal",
	bluegreen = "teal",
	tenne = "tenne",
	"tenn\u00e9" = "tenne",
	tawny = "tenne",
	brown = "tenne",
	vert = "vert",
	green = "vert",
	violet = "violet",
	bluepurple = "violet",
	violetblue = "violet"
)

create_pattern_hatch <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	type_norm <- gsub("[- ]", "", tolower(params$pattern_type))
	subtype_norm <- gsub("[- ]", "", tolower(params$pattern_subtype %||% "combinatorial"))
	is_unicode <- subtype_norm == "unicode"
	is_fox_davies <- subtype_norm == "foxdavies"
	is_goodman <- subtype_norm == "goodman"
	canonical <- HATCH_NAME_MAP[type_norm]
	if (is.na(canonical)) {
		stop("Unknown hatching type '", params$pattern_type, "'.")
	}

	spec <- if (is_unicode) {
		HATCH_UNICODE[[canonical]]
	} else if (is_fox_davies) {
		HATCH_FOX_DAVIES[[canonical]]
	} else if (is_goodman) {
		HATCH_GOODMAN[[canonical]]
	} else {
		HATCH_COMBINATORIAL[[canonical]]
	}

	if (is.null(spec)) {
		stop(
			"Hatching type '",
			params$pattern_type,
			"' is not supported ",
			"by the '",
			params$pattern_subtype %||% "combinatorial",
			"' subtype."
		)
	}

	if (!is.null(spec$special)) {
		if (spec$special == "plain") {
			return(grid::nullGrob())
		}
		if (spec$special == "solid") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			default.units <- "bigpts"
			bd <- convert_polygon_df_units(boundary_df, default.units)
			return(convert_polygon_df_to_polygon_grob(
				bd,
				default.units = default.units,
				gp = gpar(fill = col, col = NA)
			))
		}
		if (spec$special == "circle") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			spacing_bigpts <- convertX(
				unit(params$pattern_spacing, params$pattern_units),
				"bigpts",
				valueOnly = TRUE
			)
			density <- params$pattern_linewidth * .pt / spacing_bigpts
			return(patternGrob(
				"circle",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				colour = NA,
				fill = col,
				spacing = params$pattern_spacing,
				density = density,
				grid = "hex_circle"
			))
		}
		if (spec$special == "olive") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			spacing_bigpts <- convertX(
				unit(params$pattern_spacing, params$pattern_units),
				"bigpts",
				valueOnly = TRUE
			)
			lwd_bigpts <- params$pattern_linewidth * .pt
			density_dot <- lwd_bigpts / spacing_bigpts * 0.6
			density_plus <- min(lwd_bigpts / spacing_bigpts * 2, 0.9)
			return(patternGrob(
				"pch",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				colour = col,
				fill = col,
				shape = c(16, 3),
				spacing = params$pattern_spacing,
				density = c(density_dot, density_plus),
				angle = 0,
				type = "diagonal",
				subtype = 2L,
				grid = "square"
			))
		}
		if (spec$special == "steel") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			spacing_bigpts <- convertX(
				unit(params$pattern_spacing, params$pattern_units),
				"bigpts",
				valueOnly = TRUE
			)
			lwd_bigpts <- params$pattern_linewidth * .pt
			density <- min(lwd_bigpts / spacing_bigpts * 2.5, 0.9)
			return(patternGrob(
				"pch",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				colour = col,
				fill = col,
				shape = 3,
				spacing = params$pattern_spacing,
				density = density,
				angle = 0,
				grid = "square"
			))
		}
		if (spec$special == "c_hex") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			return(patternGrob(
				"text",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				colour = col,
				shape = "c",
				spacing = params$pattern_spacing,
				angle = 0,
				grid = "hex_circle"
			))
		}
		if (spec$special == "checker") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			return(patternGrob(
				"polygon_tiling",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				type = "square",
				spacing = params$pattern_spacing,
				angle = 0,
				colour = NA,
				fill = c(col, "transparent")
			))
		}
		if (spec$special == "cendree") {
			# Build segments directly at exact grid intersections (no lty approximation).
			# At each (m, j) grid point, draw an H dash if (m+j) is even, V dash if odd.
			# Dashes have length exactly S so centers are precisely at grid positions.
			default.units <- "bigpts"
			boundary_df_pts <- convert_polygon_df_units(boundary_df, default.units)
			params_pts <- convert_params_units(params, default.units)
			vpm <- get_vp_measurements(default.units)
			col <- update_alpha(params_pts$pattern_colour, params_pts$pattern_alpha)
			lwd <- params_pts$pattern_linewidth * .pt
			gp_seg <- gpar(
				col = col,
				lwd = lwd,
				lty = "solid",
				lineend = params_pts$pattern_lineend
			)
			S <- params_pts$pattern_spacing
			xo <- params_pts$pattern_xoffset
			yo <- params_pts$pattern_yoffset
			half <- S / 2
			# Grid indices covering viewport + margin (same extent as get_xy_grid)
			idx_seq <- seq_robust(from = 0, to = vpm$length, by = S)
			idx <- round(c(rev(-idx_seq[-1L]), idx_seq) / S) # ..., -2, -1, 0, 1, 2, ...
			x_grid <- xo + vpm$x + idx * S
			y_grid <- yo + vpm$y + idx * S
			ng <- length(idx)
			m_mat <- matrix(idx, nrow = ng, ncol = ng, byrow = FALSE)
			j_mat <- matrix(idx, nrow = ng, ncol = ng, byrow = TRUE)
			x_mat <- matrix(x_grid, nrow = ng, ncol = ng, byrow = FALSE)
			y_mat <- matrix(y_grid, nrow = ng, ncol = ng, byrow = TRUE)
			is_H <- (m_mat + j_mat) %% 2L == 0L
			x_H <- as.vector(x_mat[is_H])
			y_H <- as.vector(y_mat[is_H])
			x_V <- as.vector(x_mat[!is_H])
			y_V <- as.vector(y_mat[!is_H])
			maskee <- grid::grobTree(
				segmentsGrob(
					x_H - half,
					y_H,
					x_H + half,
					y_H,
					default.units = default.units,
					gp = gp_seg
				),
				segmentsGrob(
					x_V,
					y_V - half,
					x_V,
					y_V + half,
					default.units = default.units,
					gp = gp_seg
				)
			)
			masker <- convert_polygon_df_to_polygon_grob(
				boundary_df_pts,
				default.units = default.units,
				gp = gpar(fill = "white", col = NA, lwd = 0)
			)
			return(alphaMaskGrob(
				maskee,
				masker,
				use_R4.1_masks = params$pattern_use_R4.1_masks,
				png_device = params$pattern_png_device,
				res = params$pattern_res
			))
		}
		if (spec$special == "proper") {
			col <- update_alpha(params$pattern_colour, params$pattern_alpha)
			spacing_bigpts <- convertX(
				unit(params$pattern_spacing, params$pattern_units),
				"bigpts",
				valueOnly = TRUE
			)
			density <- params$pattern_linewidth * .pt / spacing_bigpts
			return(patternGrob(
				"wave",
				x = boundary_df$x,
				y = boundary_df$y,
				id = boundary_df$id,
				colour = NA,
				fill = c(col, "transparent"),
				angle = 135,
				spacing = params$pattern_spacing,
				density = density,
				type = "triangle"
			))
		}
	}

	# Build all line-segment grobs and apply a single mask.  Using separate
	# alphaMaskGrobs in a grobTree breaks compositing (full-viewport rasters
	# overwrite each other), so we combine the maskees first.
	default.units <- "bigpts"
	boundary_df_pts <- convert_polygon_df_units(boundary_df, default.units)
	params_pts <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	spacings <- spec$spacings %||% rep(1, length(spec$angles))
	maskees <- mapply(
		function(angle, lty, spacing_mult) {
			p <- params_pts
			p$pattern_angle <- angle
			p$pattern_linetype <- lty
			p$pattern_stagger <- (lty != "solid")
			p$pattern_spacing <- p$pattern_spacing * spacing_mult
			create_line_maskee(p, vpm, default.units)
		},
		spec$angles,
		spec$linetypes,
		spacings,
		SIMPLIFY = FALSE
	)
	maskee <- if (length(maskees) == 1L) maskees[[1L]] else do.call(grid::grobTree, maskees)
	masker <- convert_polygon_df_to_polygon_grob(
		boundary_df_pts,
		default.units = default.units,
		gp = gpar(fill = "white", col = NA, lwd = 0)
	)
	alphaMaskGrob(
		maskee,
		masker,
		use_R4.1_masks = params$pattern_use_R4.1_masks,
		png_device = params$pattern_png_device,
		res = params$pattern_res
	)
}

#' Square pattern matrix
#'
#' `pattern_square()` returns an integer matrix indicating where each
#' color (or other graphical element) should be drawn on a rectangular grid
#' for a specified square pattern type and subtype.
#' `names_square` lists the currently supported square `type`s (excluding those in `names_weave`).
#'
#' \describe{
#' \item{"horizontal", "vertical"}{"horizontal" and "vertical" simply cycle through the colors
#'   either horizontally or vertically.
#'   Use `subtype` to indicate the (integer) number of colors (or other graphical elements).
#'   "horizontal" will produce horizontal stripes of color whereas "vertical" will produce vertical stripes.}
#' \item{"diagonal", "diagonal_skew"}{"diagonal" and "diagonal_skew" simply cycle through the colors
#'   both horizontally and vertically.
#'   Use `subtype` to indicate the (integer) number of colors (or other graphical elements).
#'   If two colors are requested this provides the standard two-color checkerboard pattern.
#'   If there are more than three colors than "diagonal" will have colored diagonals
#'   going from top left to bottom right while "diagonal_skew" will have them
#'   going form bottom left to top right.}
#' \item{"square"}{"square" attempts a uniform coloring using "square_tiling" before falling
#'                 falling back on "diagonal".  If `subtype` is `1L`, `2L`, `3L`, or `4L` uses "square_tiling"
#'                 else uses "diagonal".}
#' \item{"square_tiling"}{"square_tiling" supports
#'   uniform coloring for (non-staggered) square tilings.
#'   Use `subtype` to either indicate the (integer) number of colors
#'   or a string with four integers such as `"1231"`
#'   (will fill in a 2x2 matrix by row which will then be tiled).
#'   Supports up to a max of four colors.}
#' \item{any pattern from `names_weave`}{
#'   We simply convert the logical matrix returned by [pattern_weave()] into an
#'   integer matrix by having any `TRUE` set to `1L` and `FALSE` set to `2L`.
#'   Hence the various weave patterns only support (up to) two-color patterns.
#'   See [pattern_weave()] for more details about supported `type` and `subtype`.}
#' }
#'
#' @param type Either "diagonal" (default), "diagonal_skew", "horizontal", "vertical",
#'             or any `type` in `names_weave`.  See Details.
#' @param subtype See Details.  For "diagonal", "diagonal_skew", "horizontal", or "vertical"
#'                an integer of the desired number of colors (or other graphical elements).
#' @param nrow Number of rows (height).
#' @param ncol Number of columns (width).
#' @return A matrix of integer values indicating where the each color
#'         (or other graphical element) should be drawn on a rectangular grid.
#'         Indices `[1,1]` of the matrix corresponds to the bottom-left of the grid
#'         while indices `[1,ncol]` corresponds to the bottom-right of the grid.
#'         This matrix has a "pattern_square" subclass which supports a special `print()` method.
#' @examples
#'  # supported square names
#'  print(names_square)
#'
#'  # (main) diagonal has colors going from top left to bottom right
#'  diagonal <- pattern_square("diagonal", 4L, nrow = 7L, ncol = 9L)
#'  print(diagonal)
#'
#'  # skew diagonal has colors going from bottom left to top right
#'  skew <- pattern_square("diagonal_skew", 4L, nrow = 7L, ncol = 9L)
#'  print(skew)
#'
#'  horizontal <- pattern_square("horizontal", 4L, nrow = 8L, ncol = 8L)
#'  print(horizontal)
#'
#'  vertical <- pattern_square("vertical", 4L, nrow = 8L, ncol = 8L)
#'  print(vertical)
#'
#'  # uniform coloring using 4 colors
#'  color4 <- pattern_square("square_tiling", 4L, nrow = 7L, ncol = 9L)
#'  print(color4)
#'
#'  # uniform coloring using 3 colors
#'  color3 <- pattern_square("square_tiling", 3L, nrow = 7L, ncol = 9L)
#'  print(color3)
#'
#'  # also supports the various 'weave' patterns
#'  zigzag <- pattern_square("twill_zigzag", nrow = 15L, ncol = 9L)
#'  print(zigzag)
#'
#' @seealso [grid.pattern_regular_polygon()] for drawing to a graphics device
#'           polygons in multiple color/size/shape patterns.
#'           [pattern_weave()] for more information on "weave" patterns.
#' @export
pattern_square <- function(type = "diagonal", subtype = NULL, nrow = 5L, ncol = 5L) {
	if (type %in% names_weave) {
		v <- as.integer(!pattern_weave(type, subtype, nrow, ncol)) + 1L
		m <- matrix(v, nrow = nrow, ncol = ncol)
	} else {
		m <- switch(
			type,
			diagonal = pattern_diagonal(subtype, nrow, ncol),
			diagonal_skew = pattern_diagonal(subtype, nrow, ncol, skew = TRUE),
			horizontal = pattern_horizontal(subtype, nrow, ncol),
			square = pattern_square_type(subtype, nrow, ncol),
			square_tiling = pattern_square_tiling(subtype, nrow, ncol),
			vertical = pattern_vertical(subtype, nrow, ncol),
			abort(paste("Don't recognize square pattern type", type))
		)
	}
	class(m) <- c("pattern_square", "matrix", "array")
	m
}

#' @rdname pattern_square
#' @export
names_square <- c("diagonal", "diagonal_skew", "horizontal", "square", "square_tiling", "vertical")

pattern_diagonal <- function(subtype = NULL, nrow = 5L, ncol = 5L, skew = FALSE) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 3L
	}
	stopifnot(is_integer(subtype))
	m <- matrix(1L, nrow = nrow, ncol = ncol)
	n <- as.integer(subtype)
	if (n == 1L) {
		return(m)
	}
	s <- seq.int(n)
	for (e in s) {
		step <- ifelse(skew, -(e - 1L), e - 1L)
		v <- rep(cycle_elements(s, step), length.out = ncol)
		for (i in seq(e, nrow, n)) {
			m[i, ] <- v
		}
	}
	m
}

pattern_horizontal <- function(subtype = NULL, nrow = 5L, ncol = 5L) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 3L
	}
	stopifnot(is_integer(subtype))
	n <- as.integer(subtype)
	if (nrow > 2L && n > 1L) {
		v1 <- rev(rep(c(seq.int(n, 2L, -1L), 1L), length.out = nrow %/% 2))
		v2 <- rep(seq.int(n), length.out = (nrow %/% 2) + (nrow %% 2))
		v <- c(v1, v2)
	} else {
		s <- seq.int(n)
		v <- rep(s, length.out = nrow)
	}
	v <- rep.int(v, ncol)
	matrix(v, nrow = nrow, ncol = ncol)
}

pattern_square_type <- function(subtype, nrow, ncol) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 3L
	}
	stopifnot(is_integer(subtype))
	n <- as.integer(subtype)
	if (n <= 4) {
		pattern_square_tiling(n, nrow, ncol)
	} else {
		pattern_diagonal(n, nrow, ncol)
	}
}

pattern_square_tiling <- function(subtype, nrow, ncol) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 3L
	}
	stopifnot(is_integer(subtype))
	if (is.character(subtype)) {
		subtype <- strsplit(subtype, "")[[1]]
	}
	m <- matrix(1L, nrow = nrow, ncol = ncol)
	n <- as.integer(subtype)
	if (all(n == 1L)) {
		return(m)
	}
	if (length(n) == 1L) {
		n <- switch(
			as.character(subtype),
			`1` = c(1L, 1L, 1L, 1L),
			`2` = c(2L, 1L, 1L, 2L),
			`3` = c(1L, 2L, 3L, 1L),
			`4` = 1:4,
			n
		)
	}
	n <- rep_len(n, 4)
	vt <- rep_len(n[1:2], ncol)
	vb <- rep_len(n[3:4], ncol)
	for (i in seq_len(nrow)) {
		if (i %% 2 == 1) {
			m[i, ] <- vb
		} else {
			m[i, ] <- vt
		}
	}
	m
}

pattern_vertical <- function(subtype = NULL, nrow = 5L, ncol = 5L) {
	if (is.null(subtype) || is.na(subtype)) {
		subtype <- 3L
	}
	stopifnot(is_integer(subtype))
	n <- as.integer(subtype)
	if (ncol > 2L && n > 1L) {
		v1 <- rev(rep(c(seq.int(n, 2L, -1L), 1L), length.out = ncol %/% 2))
		v2 <- rep(seq.int(n), length.out = (ncol %/% 2) + (ncol %% 2))
		v <- c(v1, v2)
	} else {
		s <- seq.int(n)
		v <- rep(s, length.out = ncol)
	}
	v <- rep.int(v, nrow)
	matrix(v, nrow = nrow, ncol = ncol, byrow = TRUE)
}

#' @export
print.pattern_square <- function(x, ...) {
	d <- dim(x)
	x <- matrix(int_to_char(x), nrow = d[1], ncol = d[2])
	cat("/", rep("-", ncol(x)), "\\", "\n")
	for (i in rev(seq_len(nrow(x)))) {
		cat("|", x[i, ], "|", "\n")
	}
	cat("\\", rep("-", ncol(x)), "/", "\n")
	invisible(NULL)
}

is_pattern_square <- function(type) {
	(type %in% names_weave) || (type %in% names_square)
}

int_to_char <- function(x) {
	stopifnot(max(x) < 36L)
	char <- as.character(x)
	indices <- which(x > 9L)
	char[indices] <- LETTERS[x[indices] - 9L]
	char
}

test_that("square patterns work as expected", {
	pss <- function(...) print.pattern_square(pattern_square(...))

	expect_error(pattern_square("foobar"), "Don't recognize square pattern type foobar")

	verify_output("../text_diagrams/diagonal.txt", pss("diagonal", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/diagonal_1.txt", pss("diagonal", 1L, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/diagonal_skew.txt", pss("diagonal_skew", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/horizontal.txt", pss("horizontal", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/square_twill.txt", pss("twill", nrow = 7, ncol = 9))
	verify_output("../text_diagrams/vertical.txt", pss("vertical", nrow = 7, ncol = 9))
	verify_output(
		"../text_diagrams/square_1122.txt",
		pss("square_tiling", "1122", nrow = 7, ncol = 9)
	)
	verify_output("../text_diagrams/square_3.txt", pss("square", NULL, nrow = 7, ncol = 9))
	verify_output(
		"../text_diagrams/square_tiling_3.txt",
		pss("square_tiling", NULL, nrow = 7, ncol = 9)
	)
	verify_output("../text_diagrams/square_4.txt", pss("square", 4L, nrow = 7, ncol = 9))
	verify_output("../text_diagrams/square_5.txt", pss("square", 5L, nrow = 7, ncol = 9))
})

#' Stripe patterned grobs
#'
#' `grid.pattern_stripe()` draws a stripe pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_circle
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_stripe(x_hex, y_hex, colour = "black",
#'                     fill = c("red", "blue"), density = 0.4)
#'
#' # Can alternatively use "gpar()" to specify colour and line attributes
#' grid::grid.newpage()
#' grid.pattern_stripe(x_hex, y_hex, density = 0.3,
#'                     gp = grid::gpar(col = "blue", fill = "yellow"))
#' @seealso `[grid.pattern_crosshatch()]` and `[grid.pattern_weave()]` for overlaying stripes.
#'          Use [grid.pattern_line()] for stroked lines that support all native `linetype` values.
#'          Use [grid.pattern_wave()] for wavy bands instead of straight ones.
#' @export
grid.pattern_stripe <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	grid = "square",
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"stripe",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		grid = grid,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_stripes_via_sf <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	create_crosshatch_via_sf_helper(params, boundary_df, add_top_hatch = FALSE)
}

#' Regular polygon patterned grobs
#'
#' `grid.pattern_regular_polygon()` draws a regular polygon pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_circle
#' @param rot Angle to rotate regular polygon (degrees, counter-clockwise).
#' @param scale For star polygons, multiplier (between 0 and 1)
#'              applied to exterior radius to get interior radius.
#' @param shape Either "convex" or "star" followed by the number of exterior vertices
#'              or alternatively "circle", "square", "null", "rhombille_rhombus",
#'              "tetrakis_left", or "tetrakis_right".
#'              For example "convex5" corresponds to a pentagon
#'              and "star6" corresponds to a six-pointed star.
#'              The "square" shape is larger than the "convex4" shape and is rotated an extra 45 degrees,
#'              it can be used to generate a multi-colored \dQuote{checkers} effect when density is 1.
#'              The "null" shape is not drawn, it can be used to create holes within multiple-element patterns.
#'              The "rhombille_rhombus" shape draws a rhombus while the
#'              "tetrakis_left" or "tetrakis_right" shapes draw an isosceles right triangle.
#'              These latter three non-regular-polygon shapes are
#'              intended to help generate rhombille and tetrakis square tilings.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @seealso [grid.pattern_circle()] for a special case of this pattern.
#'          The tiling vignette features more examples of regular polygon tiling using
#'          this function `vignette("tiling", package = "gridpattern")`.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'
#' # 'density', 'rot', and 'shape' are vectorized
#' grid.pattern_regular_polygon(x_hex, y_hex, colour = "black",
#'                              fill = c("blue", "yellow", "red"),
#'                              shape = c("convex4", "star8", "circle"),
#'                              density = c(0.45, 0.42, 0.4),
#'                              spacing = 0.08, angle = 0)
#'
#' # checker pattern using "square" shape
#' grid::grid.newpage()
#' grid.pattern_regular_polygon(x_hex, y_hex, shape = "square",
#'                              colour = "transparent",
#'                              fill = c("black", "red", "blue", "yellow"),
#'                              angle = 0, density = 1.0, spacing = 0.2)
#'
#' # checker pattern using the default "convex4" shape
#' grid::grid.newpage()
#' grid.pattern_regular_polygon(x_hex, y_hex, density = 1.0,
#'                              colour = "black", fill = "blue")
#'
#' # using a "twill_zigzag" 'weave' pattern
#' grid::grid.newpage()
#' grid.pattern_regular_polygon(x_hex, y_hex, fill = c("blue", "yellow"),
#'                              shape = c("circle", "star8"),
#'                              density = c(0.5, 0.6), type = "twill_zigzag")
#'
#' # hexagon tiling
#' grid::grid.newpage()
#' grid.pattern_regular_polygon(x_hex, y_hex, color = "transparent",
#'                              fill = c("white", "grey", "black"),
#'                              density = 1.0, spacing = 0.1,
#'                              shape = "convex6", grid = "hex")
#'
#' # triangle tiling
#' grid::grid.newpage()
#' grid.pattern_regular_polygon(x_hex, y_hex, fill = "green",
#'                              density = 1.0, spacing = 0.1,
#'                              shape = "convex3", grid = "hex")
#' @export
grid.pattern_regular_polygon <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	scale = 0.5,
	shape = "convex4",
	grid = "square",
	type = NULL,
	subtype = NULL,
	rot = 0,
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"regular_polygon",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		scale = scale,
		shape = shape,
		grid = grid,
		type = type,
		subtype = subtype,
		rot = rot,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_regular_polygon_via_sf <- function(
	params,
	boundary_df,
	aspect_ratio,
	legend = FALSE
) {
	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	spacing <- params$pattern_spacing
	grid <- params$pattern_grid

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm)

	# construct grobs using subsets if certain inputs are vectorized
	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype

	density <- params$pattern_density
	rot <- params$pattern_rot
	shape <- params$pattern_shape
	assert_rp_shape(shape)

	n_par <- max(lengths(list(fill, col, lwd, lty, density, rot, shape)))

	fill <- rep_len_fill(fill, n_par)
	col <- rep_len(col, n_par)
	lwd <- rep_len(lwd, n_par)
	lty <- rep_len(lty, n_par)
	density <- rep_len(density, n_par)
	rot <- rep_len(rot, n_par)
	shape <- rep_len(shape, n_par)

	density <- ifelse(
		shape %in% c("square", "tetrakis_left", "tetrakis_right"),
		1.414 * density,
		density
	)
	# avoid overlap errors when density == 1 due to machine precision issues
	if (grid == "square") {
		density <- ifelse(nigh(density, 1), 0.9999, density)
	}
	if (grepl("^hex", grid) && n_par < 3L) {
		density <- ifelse(nigh(density, 1), 0.994, density)
	}
	density_max <- max(density)

	# compute regular polygon relative coordinates which we will center on points
	radius_mult <- switch(grid, hex = 0.578, 0.5)
	radius_max <- radius_mult * spacing * density_max

	#### add fudge factor?
	boundary_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = 0)
	expanded_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = radius_max)
	contracted_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = -radius_max)

	# compute pattern matrix of graphical elements (e.g. fill colors)
	if (is.null(params$pattern_type) || is.na(params$pattern_type)) {
		params$pattern_type <- switch(grid, square = "square", "hex")
	}
	m_pat <- get_pattern_matrix(params$pattern_type, params$pattern_subtype, grid_xy, n_par)

	gl <- gList()
	for (i_par in seq(n_par)) {
		if (shape[i_par] == "null") {
			next
		}
		radius_outer <- radius_mult * spacing * density[i_par]
		xy_polygon <- get_xy_polygon(shape[i_par], params, radius_outer, rot[i_par])
		xy_par <- get_xy_par(grid_xy, i_par, m_pat, grid, spacing)
		if (length(xy_par$x) == 0) {
			next
		}

		# rotate by 'angle'
		xy_par <- rotate_xy(xy_par$x, xy_par$y, params$pattern_angle, vpm$x, vpm$y)

		# test if polygons within/near boundary
		points_sf <- sf::st_multipoint(as.matrix(as.data.frame(xy_par)))
		all_points_sf <- sf::st_intersection(expanded_sf, points_sf)
		interior_points_sf <- sf::st_intersection(all_points_sf, contracted_sf)
		exterior_points_sf <- sf::st_difference(all_points_sf, contracted_sf)

		gp <- gpar(fill = fill[[i_par]], col = col[i_par], lwd = lwd[i_par], lty = lty[i_par])

		# create grob for interior polygons
		name <- paste0("interior.", i_par)
		if (shape[i_par] == "circle") {
			grob <- sf_points_to_circle_grob(
				interior_points_sf,
				radius_outer,
				gp,
				default.units,
				name
			)
		} else {
			grob <- sf_points_to_polygon_grob(
				interior_points_sf,
				xy_polygon,
				gp,
				default.units,
				name
			)
		}
		gl <- append_gList(gl, grob)

		# create grob for exterior polygons
		polygons_sf <- sf_points_to_sf_multipolygon(exterior_points_sf, xy_polygon)
		exterior_multipolygon <- sf::st_intersection(polygons_sf, boundary_sf)
		name <- paste0("boundary.", i_par)
		grob <- sf_multipolygon_to_polygon_grob(exterior_multipolygon, gp, default.units, name)
		gl <- append_gList(gl, grob)
	}
	gTree(children = gl, name = "regular_polygon")
}

get_pattern_matrix <- function(type, subtype, grid_xy, n_par) {
	nrow <- length(grid_xy$y)
	ncol <- length(grid_xy$x)
	if (is_pattern_square(type)) {
		if (is.null(subtype) || is.na(subtype)) {
			if (type %in% names_weave) {
				subtype <- NULL
			} else {
				subtype <- n_par
			}
		}
		if (type %in% names_weave && n_par > 2) {
			abort(c(
				glue("pattern_type '{type}' can't arrange more than two elements"),
				i = glue("We detected {n_par} elements requested")
			))
		}
		m_pat <- pattern_square(type, subtype, nrow = nrow, ncol = ncol)
	} else {
		if (is.null(subtype) || is.na(subtype)) {
			subtype <- n_par
		}
		m_pat <- pattern_hex(type, subtype, nrow = nrow, ncol = ncol)
	}
	m_pat
}

get_xy_par <- function(grid_xy, i_par, m_pat, grid, spacing) {
	if (grid == "square") {
		get_xy_par_square(grid_xy, i_par, m_pat)
	} else if (grid == "elongated_triangle") {
		get_xy_par_el_tri(grid_xy, i_par, m_pat, spacing)
	} else {
		get_xy_par_hex(grid_xy, i_par, m_pat, spacing)
	}
}
get_xy_par_square <- function(grid_xy, i_par, m_pat) {
	x <- numeric(0)
	y <- numeric(0)
	for (i in seq_along(grid_xy$y)) {
		indices_x <- which(m_pat[i, ] == i_par)
		x <- c(x, grid_xy$x[indices_x])
		y <- c(y, rep(grid_xy$y[i], length(indices_x)))
	}
	list(x = x, y = y)
}
get_xy_par_hex <- function(grid_xy, i_par, m_pat, spacing = 1) {
	x <- numeric(0)
	y <- numeric(0)
	for (i in seq_along(grid_xy$y)) {
		indices_x <- which(m_pat[i, ] == i_par)
		if (i %% 2) {
			x_offset <- 0
		} else {
			x_offset <- -0.5 * spacing
		}
		x <- c(x, x_offset + grid_xy$x[indices_x])
		y <- c(y, rep(grid_xy$y[i], length(indices_x)))
	}
	list(x = x, y = y)
}
get_xy_par_el_tri <- function(grid_xy, i_par, m_pat, spacing = 1) {
	x <- numeric(0)
	y <- numeric(0)
	for (i in seq_along(grid_xy$y)) {
		indices_x <- which(m_pat[i, ] == i_par)
		if (i %% 4 == 3 || i %% 4 == 0) {
			x_offset <- 0
		} else {
			x_offset <- -0.5 * spacing
		}
		x <- c(x, x_offset + grid_xy$x[indices_x])
		y <- c(y, rep(grid_xy$y[i], length(indices_x)))
	}
	list(x = x, y = y)
}

# create grid of points large enough to cover viewport no matter the angle
get_xy_grid <- function(params, vpm, wavelength = FALSE) {
	xoffset <- params$pattern_xoffset
	yoffset <- params$pattern_yoffset
	if (wavelength) {
		h_spacing <- params$pattern_wavelength
	} else {
		h_spacing <- params$pattern_spacing
	}

	gm <- 1.00 # seems to need to be this big so {ggpattern} legends render correctly
	x_adjust <- switch(
		params$pattern_grid,
		hex = 0.5 * h_spacing,
		elongated_triangle = 0.5 * h_spacing,
		0
	)
	x_seq <- seq_robust(from = 0, to = gm * vpm$length + x_adjust, by = h_spacing)
	x <- xoffset + vpm$x + c(rev(tail(-x_seq, -1L)), x_seq)
	if (length(x) == 1L) {
		x_min <- vpm$x - vpm$length
		x_max <- vpm$x + vpm$length
	} else {
		x_min <- min(x)
		x_max <- max(x)
	}

	# adjust vertical spacing for "hex" pattern
	if (params$pattern_grid == "square") {
		v_spacing <- params$pattern_spacing
	} else if (params$pattern_grid == "elongated_triangle") {
		v_spacing <- (0.5 + 0.25 * sqrt(3)) * params$pattern_spacing
	} else {
		v_spacing <- 0.5 * sqrt(3) * params$pattern_spacing
	}
	y_seq <- seq_robust(from = 0, to = gm * vpm$length, by = v_spacing)
	# ensure middle y point in a hex grid is an odd number so we don't accidentally offset it
	if (params$pattern_grid != "square" && (length(y_seq) %% 2L == 0L)) {
		y_seq <- c(y_seq, y_seq[length(y_seq)] + v_spacing)
	}
	y <- yoffset + vpm$y + c(rev(tail(-y_seq, -1L)), y_seq)
	if (params$pattern_grid == "elongated_triangle") {
		y <- y + rep(c(0, -0.15 * v_spacing), length.out = length(y))
	}
	if (length(y) == 1L) {
		y_min <- vpm$y - vpm$length
		y_max <- vpm$y + vpm$length
	} else {
		y_min <- min(y)
		y_max <- max(y)
	}

	list(
		x = x,
		y = y,
		x_min = x_min,
		x_max = x_max,
		y_min = y_min,
		y_max = y_max,
		h_spacing = h_spacing,
		v_spacing = v_spacing
	)
}

get_xy_polygon <- function(shape, params, radius_outer, rot) {
	if (shape %in% c("square", "tetrakis_left", "tetrakis_right")) {
		rot <- rot + 45
	}
	if (shape == "square") {
		shape <- "convex4"
	}
	polygon_angle <- 90 + rot + params$pattern_angle
	if (shape == "circle") {
		# grid::grobPoints.circle() defaults to regular polygon with 100 vertices
		convex_xy(100, polygon_angle, radius_outer)
	} else if (grepl("convex", shape)) {
		n_vertices <- get_n_vertices(shape)
		convex_xy(n_vertices, polygon_angle, radius_outer)
	} else if (shape == "rhombille_rhombus") {
		rhombus_xy(polygon_angle, radius_outer)
	} else if (shape == "tetrakis_left") {
		tetrakis_left_xy(polygon_angle, radius_outer)
	} else if (shape == "tetrakis_right") {
		tetrakis_right_xy(polygon_angle, radius_outer)
	} else {
		n_vertices <- get_n_vertices(shape)
		radius_inner <- params$pattern_scale * radius_outer
		concave_xy(n_vertices, polygon_angle, radius_outer, radius_inner)
	}
}

assert_rp_shape <- function(shape) {
	tf <- grepl(
		"^convex[[:digit:]]+$|^star[[:digit:]]+$|^square$|^circle$|^null$|^tetrakis_left$|^tetrakis_right$|^rhombille_rhombus$",
		shape
	)
	if (all(tf)) {
		invisible(NULL)
	} else {
		shape <- shape[which(!tf)[1]]
		msg <- c(
			paste("Unknown shape", shape),
			i = 'See `help("grid.pattern_regular_polygon")` for supported shapes'
		)
		abort(msg)
	}
}

context("geometry")
test_that("geometry helpers work as expected", {
	xy <- rotate_xy(c(0, 1), c(0, 1), 90)
	expect_equal(xy$x, c(1, 0))
	expect_equal(xy$y, c(0, 1))
})
test_that("geometry patterns work as expected", {
	png_file <- tempfile(fileext = ".png")
	png(png_file)
	expect_error(grid.pattern_crosshatch(x, y, density = 1.1))
	expect_error(grid.pattern_stripe(x, y, density = 1.1))
	dev.off()
	unlink(png_file)

	skip_if_not_installed("vdiffr")
	skip_on_ci()
	library("vdiffr")

	expect_doppelganger("default", grid.pattern)

	x <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	y <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	expect_doppelganger("circle", function() {
		grid.pattern_circle(x, y, color = "blue", fill = "yellow", size = 2, density = 0.5)
	})

	expect_doppelganger("crosshatch", function() {
		grid.pattern_crosshatch(
			x,
			y,
			color = "black",
			fill = "blue",
			fill2 = "yellow",
			density = 0.5
		)
	})

	expect_doppelganger("fill", function() {
		grid.pattern_fill(x, y, fill = "blue", alpha = 0.5)
	})

	expect_doppelganger("none", function() {
		grid.pattern_none(x, y)
	})

	expect_error(assert_rp_shape(1), "Unknown shape 1")
	expect_null(assert_rp_shape(c("square", "convex4")))
	expect_null(assert_rp_shape(c("star5", "circle", "null")))
	expect_doppelganger("regular_polygon", function() {
		grid.pattern_regular_polygon(x, y, color = "black", fill = "blue", density = 0.5)
	})

	expect_doppelganger("hexagon", function() {
		grid.pattern_regular_polygon(
			x,
			y,
			color = "transparent",
			fill = c("white", "grey", "black"),
			density = 1.0,
			shape = "convex6",
			grid = "hex"
		)
	})

	expect_doppelganger("square", function() {
		grid.pattern_regular_polygon(
			x,
			y,
			color = "black",
			fill = c("white", "grey"),
			density = 1.0,
			shape = "square"
		)
	})

	expect_doppelganger("eight_sided_star", function() {
		grid.pattern_regular_polygon(
			x,
			y,
			colour = "black",
			fill = c("blue", "yellow"),
			density = 1.0,
			spacing = 0.1,
			shape = "star8"
		)
	})
	expect_doppelganger("stripe", function() {
		grid.pattern_stripe(x, y, color = "black", fill = c("yellow", "blue"), density = 0.5)
	})

	expect_doppelganger("stripe_gpar", function() {
		x <- c(0.1, 0.6, 0.8, 0.3)
		y <- c(0.2, 0.3, 0.8, 0.5)
		grid.pattern("stripe", x, y, gp = gpar(col = "blue", fill = "red", lwd = 2))
	})

	expect_doppelganger("wave_wavy", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "wavy",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_indented", function() {
		grid.pattern_wave(
			x,
			y,
			color = "black",
			fill = "yellow",
			type = "indented",
			density = 0.5,
			spacing = 0.15
		)
	})

	expect_doppelganger("wave_sawtooth", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "sawtoothed",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_sawtooth_reverse", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "sawtoothed",
			reverse = TRUE,
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "embattled",
			fill = c("red", "blue"),
			density = 0.6,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_embattled", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "embattled",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_wavy_stagger", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "wavy",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15,
			stagger = TRUE
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			fill = "yellow",
			type = "indented",
			density = 0.5,
			spacing = 0.15,
			stagger = TRUE
		),
		"overlap"
	)

	expect_doppelganger("wave_indented_stagger", function() {
		grid.pattern_wave(
			x,
			y,
			color = "black",
			fill = "yellow",
			type = "indented",
			density = 0.3,
			spacing = 0.15,
			amplitude = 0.03,
			stagger = TRUE
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "urdy",
			fill = c("red", "blue"),
			density = 0.6,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_urdy", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "urdy",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "dovetailed",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_dovetailed", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "dovetailed",
			fill = c("red", "blue"),
			density = 0.2,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_engrailed", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "engrailed",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_invected", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "invected",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "nebuly",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_nebuly", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "nebuly",
			fill = c("red", "blue"),
			density = 0.2,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "potenty",
			fill = c("red", "blue"),
			density = 0.4,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_potenty", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "potenty",
			fill = c("red", "blue"),
			density = 0.2,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "embattled_grady",
			fill = c("red", "blue"),
			density = 0.5,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_embattled_grady", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "embattled_grady",
			fill = c("red", "blue"),
			density = 0.2,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_error(
		grid.pattern_wave(
			x,
			y,
			type = "raguly",
			fill = c("red", "blue"),
			density = 0.5,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		),
		"overlap"
	)

	expect_doppelganger("wave_raguly", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "raguly",
			fill = c("red", "blue"),
			density = 0.3,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("wave_raguly_reverse", function() {
		grid.pattern_wave(
			x,
			y,
			colour = "black",
			type = "raguly",
			reverse = TRUE,
			fill = c("red", "blue"),
			density = 0.3,
			spacing = 0.15,
			angle = 0,
			amplitude = 0.05,
			frequency = 1 / 0.15
		)
	})

	expect_doppelganger("weave", function() {
		grid.pattern_weave(
			x,
			y,
			color = "black",
			fill = "yellow",
			fill2 = "blue",
			type = "twill",
			density = 0.5
		)
	})

	centroid_dot_pattern <- function(params, boundary_df, aspect_ratio, legend) {
		boundary_sf <- convert_polygon_df_to_polygon_sf(boundary_df)
		centroid <- sf::st_centroid(boundary_sf)
		grid::pointsGrob(
			x = centroid[1],
			y = centroid[2],
			pch = params$pattern_shape,
			size = unit(params$pattern_size, 'char'),
			default.units = "npc",
			gp = grid::gpar(col = update_alpha(params$pattern_fill, params$pattern_alpha))
		)
	}
	options(ggpattern_geometry_funcs = list(centroid = centroid_dot_pattern))
	x <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	y <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
	expect_doppelganger("centroid", function() {
		grid.pattern("centroid", x, y, fill = "blue", size = 5)
	})

	x <- c(0, 0, 0.5, 0.5, 0.5, 0.5, 1, 1)
	y <- c(0, 0.5, 0.5, 0, 0.5, 1, 1, 0.5)
	id <- rep(1:2, each = 4L)
	expect_doppelganger("two_id", function() {
		grid.pattern(x = x, y = y, id = id)
	})
})
test_that("get_xy_grid() bounding box is non-degenerate when spacing exceeds viewport (issue #95)", {
	# In ggpattern legend keys, `pattern_spacing` (after unit conversion) can exceed
	# `vpm$length`. When that happens `x_seq` collapses to `c(0)`, so `x = c(vpm$x)` and
	# `x_min == x_max`, producing a zero-width rectangle that `sf::st_intersection()` silently
	# discards — the pattern disappears entirely.
	params <- get_params(spacing = 20, grid = "square") # spacing already in same units as vpm
	vpm <- list(width = 10, height = 10, length = 10, x = 5, y = 5)

	grid_xy <- get_xy_grid(params, vpm)

	expect_gt(grid_xy$x_max - grid_xy$x_min, 0)
	expect_gt(grid_xy$y_max - grid_xy$y_min, 0)
})

#' Text character patterned grobs
#'
#' `grid.pattern_text()` draws a text character pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_regular_polygon
#' @inheritParams alphaMaskGrob
#' @param shape A character or expression vector.
#'              See `label` argument of [grid::textGrob()] for more details.
#' @param size Fontsize
#' @param fontfamily The font family.  See [grid::gpar()] for more details.
#' @param fontface The font face.  See [grid::gpar()] for more details.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' if (capabilities("png") &&
#'     gridpattern:::device_supports_unicode()) {
#'   x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'   y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'
#'   playing_card_symbols <- c("\u2660", "\u2665", "\u2666", "\u2663")
#'   grid.pattern_text(x_hex, y_hex,
#'                    shape = playing_card_symbols,
#'                    colour = c("black", "red", "red", "black"),
#'                    size = 18, spacing = 0.1, angle = 0)
#' }
#' @export
grid.pattern_text <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	angle = 30,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	scale = 0.5,
	shape = "X",
	grid = "square",
	type = NULL,
	subtype = NULL,
	rot = 0,
	alpha = gp$alpha %||% NA_real_,
	size = gp$fontsize %||% 12,
	fontfamily = gp$fontfamily %||% "sans",
	fontface = gp$fontface %||% "plain",
	use_R4.1_masks = getOption(
		"ggpattern_use_R4.1_masks",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"text",
		x,
		y,
		id,
		colour = colour,
		angle = angle,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		scale = scale,
		shape = shape,
		grid = grid,
		type = type,
		subtype = subtype,
		rot = rot,
		alpha = alpha,
		size = size,
		fontfamily = fontfamily,
		fontface = fontface,
		use_R4.1_masks = use_R4.1_masks,
		png_device = png_device,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_text <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	spacing <- params$pattern_spacing
	grid <- params$pattern_grid

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm)

	# vectorize fill, col, lwd, lty, density, rot, and shape
	col <- update_alpha_col(params$pattern_colour, params$pattern_alpha)
	fontsize <- params$pattern_size
	fontfamily <- params$pattern_fontfamily
	fontface <- params$pattern_fontface

	rot <- params$pattern_rot + params$pattern_angle
	shape <- params$pattern_shape

	n_par <- max(lengths(list(col, fontsize, fontfamily, fontface, rot, shape)))

	col <- rep_len(col, n_par)
	fontsize <- rep_len(fontsize, n_par)
	fontfamily <- rep_len(fontfamily, n_par)
	fontface <- rep_len(fontface, n_par)
	rot <- rep_len(rot, n_par)
	shape <- rep_len(shape, n_par)

	# compute pattern matrix of graphical elements (e.g. fill colors)
	if (is.null(params$pattern_type) || is.na(params$pattern_type)) {
		params$pattern_type <- switch(grid, square = "square", "hex")
	}
	m_pat <- get_pattern_matrix(params$pattern_type, params$pattern_subtype, grid_xy, n_par)

	gl <- gList()
	for (i_par in seq(n_par)) {
		if (shape[i_par] == "null") {
			next
		}
		xy_par <- get_xy_par(grid_xy, i_par, m_pat, grid, spacing)
		if (length(xy_par$x) == 0) {
			next
		}

		# rotate by 'angle'
		xy_par <- rotate_xy(xy_par$x, xy_par$y, params$pattern_angle, vpm$x, vpm$y)

		gp <- gpar(
			col = col[i_par],
			fontsize = fontsize[i_par],
			fontfamily = fontfamily[i_par],
			fontface = fontface[i_par]
		)

		# create grob for interior polygons
		name <- paste0("text.", i_par)

		grob <- textGrob(
			label = shape[i_par],
			x = xy_par$x,
			y = xy_par$y,
			rot = rot[i_par],
			just = "center",
			default.units = "bigpts",
			name = name,
			gp = gp
		)

		gl <- append_gList(gl, grob)
	}
	maskee <- gTree(children = gl)
	masker <- convert_polygon_df_to_polygon_grob(
		boundary_df,
		default.units = "bigpts",
		gp = gpar(fill = "white", col = NA, lwd = 0)
	)
	png_device <- params$pattern_png_device
	alphaMaskGrob(
		maskee,
		masker,
		use_R4.1_masks = params$pattern_use_R4.1_masks,
		png_device = png_device,
		res = params$pattern_res,
		name = "text"
	)
}

#' Compute regular star polygon scale or angles
#'
#' `star_scale()` computes star `scale` value given
#' an internal or external angle.  `star_angle()` computes
#' star angle (internal or external) given a `scale` value.
#'
#' [grid.pattern_regular_polygon()] parameterizes regular star polygons
#' with the number of its external vertices and a `scale` that equals the
#' fraction of the radius of the circle that circumscribes the interior vertices
#' divided by the radius of the circle that circumscribes the exterior vertices.
#' These helper functions help convert between that parameterization
#' and either the internal or external angle of the regular star polygon.
#' @param n_vertices Number of exterior vertices.
#' @param angle Angle in degrees.
#' @param scale Scale from 0 to 1.
#' @param external If `TRUE` angle should be considered an external angle.
#' @return `star_scale()` returns a numeric value between 0 and 1 intended
#'         for use as the `scale` argument in [grid.pattern_regular_polygon()].
#'         `star_angle()` returns a numeric value between 0 and 360 (degrees).
#' @examples
#'   # |8/3| star has internal angle 45 degrees and external angle 90 degrees
#'   scale <- star_scale(8, 45)
#'   scale2 <- star_scale(8, 90, external = TRUE)
#'   all.equal(scale, scale2)
#'   star_angle(8, scale)
#'   star_angle(8, scale, external = TRUE)
#'
#'   grid.pattern_regular_polygon(shape = "star8", scale = scale, angle = 0,
#'                                spacing = 0.2, density = 0.8)
#' @export
star_scale <- function(n_vertices, angle, external = FALSE) {
	if (external) {
		angle <- external_to_internal(n_vertices, angle)
	}
	if (n_vertices == 2) {
		return(star_scale2(angle))
	}
	stopifnot(angle >= 0, angle <= 180 * (1 - 2 / n_vertices))
	# we'll work with external degree
	angle <- internal_to_external(n_vertices, angle)
	t <- 360 / n_vertices
	xy1 <- list(x = 1, y = 0)
	xy2 <- list(x = to_x(t, 1), y = to_y(t, 1))
	xyc <- list(x = mean(c(xy1$x, xy2$x)), y = mean(c(xy1$y, xy2$y)))
	xyf <- list(x = to_x(t / 2, 1), y = to_y(t / 2, 1))
	dist_f <- dist(xyf, xyc)
	a2 <- dist(xy1, xyc)
	beta <- (180 - angle) / 2
	b <- a2 * sin(to_radians(beta)) / sin(to_radians(angle / 2))
	r <- 1 - b - dist_f
	stopifnot(r >= 0)
	r
}

star_scale2 <- function(angle) {
	stopifnot(angle >= 0, angle <= 90)
	a1 <- angle / 2
	a2 <- 180 - 90 - a1
	r <- sin(to_radians(a1)) / sin(to_radians(a2))
	stopifnot(r >= 0)
	r
}

#' @rdname star_scale
#' @export
star_angle <- function(n_vertices, scale, external = FALSE) {
	stopifnot(scale >= 0, scale <= 1)
	if (n_vertices == 2) {
		return(star_angle2(scale, external))
	}
	t <- 360 / n_vertices
	xy1 <- list(x = 1, y = 0)
	xy2 <- list(x = to_x(t, 1), y = to_y(t, 1))
	xyv <- list(x = to_x(t / 2, scale), y = to_y(t / 2, scale))
	xyc <- list(x = mean(c(xy1$x, xy2$x)), y = mean(c(xy1$y, xy2$y)))
	a2 <- dist(xy1, xyc)
	c <- dist(xyv, xy1)
	d <- to_degrees(2 * asin(a2 / c))
	if (!external) {
		d <- external_to_internal(n_vertices, d)
	}
	d
}

star_angle2 <- function(scale, external = FALSE) {
	d <- sqrt(1 + scale^2)
	d <- 2 * to_degrees(asin(scale / d))
	if (external) {
		d <- internal_to_external(2, d)
	}
	d
}

external_to_internal <- function(n_vertices, external) {
	n <- 2 * n_vertices # exterior plus interior vertices
	total <- (n - 2) * 180
	inverse <- 360 - external
	internal <- (total - n_vertices * inverse) / n_vertices
	internal
}

internal_to_external <- function(n_vertices, internal) {
	n <- 2 * n_vertices # exterior plus interior vertices
	total <- (n - 2) * 180
	inverse <- (total - n_vertices * internal) / n_vertices
	external <- 360 - inverse
	external
}

dist <- function(p1, p2) sqrt((p2$x - p1$x)^2 + (p2$y - p1$y)^2)

# c() and append() don't directly work with grid::gList() lists
append_gList <- function(gl, grob) {
	gl[[length(gl) + 1L]] <- grob
	gl
}

rep_len_fill <- function(x, length.out) {
	if (inherits(x, "GridPattern")) {
		rep_len(list(x), length.out)
	} else {
		rep_len(x, length.out)
	}
}

# get width, height, length, and center cooordinates of the viewport in `units` units
get_vp_measurements <- function(units = "bigpts") {
	width <- convertWidth(unit(1, "npc"), units, valueOnly = TRUE)
	height <- convertHeight(unit(1, "npc"), units, valueOnly = TRUE)
	length <- max(width, height)
	x <- convertX(unit(0.5, "npc"), units, valueOnly = TRUE)
	y <- convertY(unit(0.5, "npc"), units, valueOnly = TRUE)
	list(width = width, height = height, length = length, x = x, y = y)
}

#' Crosshatch patterned grobs
#'
#' `grid.pattern_crosshatch()` draws a crosshatch pattern onto the graphic device.
#'
#' @inheritParams grid.pattern_circle
#' @param fill2 The fill colour for the \dQuote{top} crosshatch lines.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_crosshatch(x_hex, y_hex, colour = "black", fill = "blue",
#'                         fill2 = "yellow", density = 0.5)
#' grid::grid.newpage()
#' grid.pattern_crosshatch(x_hex, y_hex, density = 0.3,
#'                         gp = grid::gpar(col = "blue", fill = "yellow"))
#' @seealso [grid.pattern_weave()] which interweaves two sets of lines.
#'        For a single set of lines use [grid.pattern_stripe()].
#' @export
grid.pattern_crosshatch <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	fill2 = fill,
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	grid = "square",
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"crosshatch",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		fill2 = fill2,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		grid = grid,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

create_pattern_crosshatch_via_sf <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	create_crosshatch_via_sf_helper(params, boundary_df, add_top_hatch = TRUE)
}

create_crosshatch_via_sf_helper <- function(params, boundary_df, add_top_hatch = TRUE) {
	if (abs(params$pattern_density - 1) < .Machine$double.eps^0.5) {
		params$pattern_density <- 1 - 1e-6
	}
	stopifnot(params$pattern_density <= 1)

	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm)

	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype
	gp <- gpar(col = col, fill = fill, lwd = lwd, lty = lty, lineend = 'square')

	boundary_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = 0)

	stripes_sf_bot <- create_h_stripes_sf(params, grid_xy, vpm)
	clipped_stripes_sf_bot <- sf::st_intersection(stripes_sf_bot, boundary_sf)
	grob <- sf_multipolygon_to_polygon_grob(clipped_stripes_sf_bot, gp, default.units, "stripe")

	if (add_top_hatch) {
		gp$fill <- update_alpha(params$pattern_fill2, params$pattern_alpha)

		stripes_sf_top <- create_v_stripes_sf(params, grid_xy, vpm)
		clipped_stripes_sf_top <- sf::st_intersection(stripes_sf_top, boundary_sf)
		grob_top <- sf_multipolygon_to_polygon_grob(
			clipped_stripes_sf_top,
			gp,
			default.units,
			"top"
		)

		grob <- editGrob(grob, name = "bottom")
		grob <- grobTree(grob, grob_top, name = "crosshatch")
	}
	grob
}

# build sf multipolygon 'rect' for each grid_xy$y value
create_h_stripes_sf <- function(params, grid_xy, vpm) {
	halfwidth <- 0.5 * grid_xy$v_spacing * params$pattern_density
	l_rects <- lapply(grid_xy$y, function(y0) {
		x <- c(grid_xy$x_min, grid_xy$x_min, grid_xy$x_max, grid_xy$x_max)
		y <- y0 + c(-1, 1, 1, -1) * halfwidth
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_rects)
}

# build sf multipolygon 'rect' for each grid_xy$x value
create_v_stripes_sf <- function(params, grid_xy, vpm) {
	halfwidth <- 0.5 * grid_xy$h_spacing * params$pattern_density
	l_rects <- lapply(grid_xy$x, function(x0) {
		x <- x0 + c(-1, 1, 1, -1) * halfwidth
		y <- c(grid_xy$y_min, grid_xy$y_min, grid_xy$y_max, grid_xy$y_max)
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_rects)
}

# ---
# repo: trevorld/gridpattern
# file: standalone-guess_has_R4.1_features.R
# last-updated: 2023-08-02
# license: https://spdx.org/licenses/MIT-0.html
# imports: [grDevices, utils]
# ---
#
# nocov start
#
# You may copy this source into your own R package
# by either using `usethis::use_standalone("trevorld/gridpattern", "standalone-guess_has_R4.1_features.R")`
# or simply copying this file into your `R` directory and adding `grDevices` and `utils` to the `Imports` of
# your `DESCRIPTION` file.

#' Guess whether "active" graphics device supports
#' the grid graphics features introduced in R v4.1.
#'
#' `guess_has_R4.1_features()` guesses whether "active" graphics device supports
#' the grid graphics features introduced in R v4.1.  If it guesses it does
#' it returns `TRUE` else `FALSE`.
#'
#' @param features Character vector of features to guess support for.
#'                 Will return `TRUE` only if guesses support for all requested features.\describe{
#'                 \item{"clippingPaths"}{Supports clipping path feature}
#'                 \item{"gradients"}{Supports (both linear and radial) gradient feature}
#'                 \item{"masks"}{Supports (alpha) mask feature}
#'                 \item{"patterns"}{Supports (tiling) pattern feature}
#'                 }
#' @seealso \url{https://www.stat.auckland.ac.nz/~paul/Reports/GraphicsEngine/definitions/definitions.html}
#'   for more info about the new grid graphics features introduced in R v4.1.
#' @return `TRUE` if we guess all `features` are supported else `FALSE`
#' @examples
#'   # If R version (weakly) greater than 4.1 should be TRUE
#'   pdf(tempfile(fileext = ".pdf"))
#'   print(guess_has_R4.1_features())
#'   invisible(dev.off())
#'
#'   # Should be FALSE
#'   postscript(tempfile(fileext = ".ps"))
#'   print(guess_has_R4.1_features())
#'   invisible(dev.off())
#'
#' @keywords internal
#' @noRd
guess_has_R4.1_features <- function(
	features = c("clippingPaths", "gradients", "masks", "patterns")
) {
	if (getRversion() < "4.1.0") {
		return(FALSE)
	}

	# In R 4.2 `dev.capabilities()` can confirm/deny R 4.1 graphic feature support
	# if active graphics device has implemented this feature
	if (getRversion() >= "4.2.0") {
		dev_capabilities <- grDevices::dev.capabilities()
		if (confirm_via_dev_capabilities(features, dev_capabilities)) {
			return(TRUE)
		}
		if (deny_via_dev_capabilities(features, dev_capabilities)) {
			return(FALSE)
		}
	}

	device <- names(grDevices::dev.cur())
	if (device %in% c("cairo_pdf", "cairo_ps", "pdf", "svg", "X11cairo")) {
		TRUE
	} else if (device %in% c("bmp", "jpeg", "png", "tiff")) {
		# on unix non-"cairo" type have different device names from "cairo" type
		# but on Windows can't distinguish between `type = "windows"` or `type = "cairo"`
		.Platform$OS.type == "unix"
	} else if (device %in% c("agg_jpeg", "agg_ppm", "agg_png", "agg_tiff")) {
		utils::packageVersion("ragg") >= "1.2.0"
	} else if (device == "devSVG") {
		# `vdiffr:::svglite()` has name "devSVG_vdiffr" since v1.0.6
		utils::packageVersion("svglite") >= "2.1.0"
	} else {
		FALSE
	}
}

# Will always return FALSE if called within R 4.1
# or if graphics device hasn't been updated to provide this information
# even if the device had been updated to provide R 4.1 graphic feature support
confirm_via_dev_capabilities <- function(
	features = c("clippingPaths", "gradients", "masks", "patterns"),
	dev_capabilities = grDevices::dev.capabilities()
) {
	for (feature in features) {
		if (!confirm_feature(feature, dev_capabilities)) {
			return(FALSE)
		}
	}
	TRUE
}

confirm_feature <- function(feature, dev_capabilities) {
	switch(
		feature,
		clippingPaths = isTRUE(dev_capabilities$clippingPaths),
		gradients = all(c("LinearGradient", "RadialGradient") %in% dev_capabilities$patterns),
		masks = "alpha" %in% dev_capabilities$masks,
		patterns = "TilingPattern" %in% dev_capabilities$patterns
	)
}

# Will return `TRUE` if `dev.capabilities()` explicitly indicates
# the given features are not supported (versus merely missing a positive indication)
deny_via_dev_capabilities <- function(
	features = c("clippingPaths", "gradients", "masks", "patterns"),
	dev_capabilities = grDevices::dev.capabilities()
) {
	for (feature in features) {
		if (deny_feature(feature, dev_capabilities)) {
			return(TRUE)
		}
	}
	FALSE
}

deny_feature <- function(feature, dev_capabilities) {
	switch(
		feature,
		clippingPaths = isFALSE(dev_capabilities$clippingPaths),
		gradients = !is.na(dev_capabilities$patterns) &&
			!all(c("LinearGradient", "RadialGradient") %in% dev_capabilities$patterns),
		masks = !is.na(dev_capabilities$masks) && !("alpha" %in% dev_capabilities$masks),
		patterns = !is.na(dev_capabilities$patterns) &&
			!("TilingPattern" %in% dev_capabilities$patterns)
	)
}

# nocov end

#' Wave patterned grobs
#'
#' `grid.pattern_wave()` draws a wave pattern onto the graphic device.
#' `names_wave` is a character vector of supported `type` values.
#'
#' @inheritParams grid.pattern_circle
#' @param units [grid::unit()] units for `amplitude`, `frequency`, `spacing`, `xoffset`, and `yoffset` parameters.
#' @param amplitude Wave amplitude (in `units` units)
#' @param frequency Linear frequency (in inverse `units` units)
#' @param type One of the following (see `names_wave` for the canonical list):
#'   \describe{
#'     \item{`"dovetailed"`}{A wave with diagonal strokes connecting the crests and troughs.  Alias: `"dovetail"`.}
#'     \item{`"embattled"`}{Square wave. Alias: `"square"`.}
#'     \item{`"embattled_grady"`}{Graduated stepped wave: two ascending steps followed by two descending steps per period.}
#'     \item{`"engrailed"`}{Repeating arches curving downward (non-positive half of a sine wave per period).}
#'     \item{`"indented"` (default)}{Triangular wave with equal rise and fall. Alias: `"triangle"`.}
#'     \item{`"invected"`}{Repeating arches curving upward (non-negative half of a sine wave per period).}
#'     \item{`"nebuly"`}{Smooth cloud-like wave.}
#'     \item{`"potenty"`}{A stepped wave with T-shaped crenellations.}
#'     \item{`"raguly"`}{Oblique stepped wave.  Use `reverse = TRUE` for the horizontally mirror image.}
#'     \item{`"sawtoothed"`}{Sawtooth wave with a gradual rise and sharp fall.  Use `reverse = TRUE` for a sharp rise and gradual fall.  Aliases: `"sawlike"`, `"sawtooth"`.}
#'     \item{`"urdy"`}{A wave with pointed crests and troughs.}
#'     \item{`"wavy"`}{Smooth sinusoidal wave. Aliases: `"sine"`, `"undy"`.}
#'   }
#' @param reverse If `TRUE`, horizontally mirror the wave.  Currently affects `"sawtoothed"` and `"raguly"` only.  Default `FALSE`.
#' @param stagger If `TRUE`, alternate wave rows are shifted by half a wavelength so that
#'   crests of one row align with troughs of adjacent rows, creating an interlocking effect.
#'   Default `FALSE`.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#' print(names_wave)
#'
#' # visual table of all wave types
#' grid::grid.newpage()
#' n <- length(names_wave)
#' nc <- 2L
#' nr <- ceiling(n / nc)
#' grid::pushViewport(grid::viewport(layout = grid::grid.layout(nr, nc)))
#' for (i in seq_len(n)) {
#'     grid::pushViewport(grid::viewport(
#'         layout.pos.row = (i - 1L) %/% nc + 1L,
#'         layout.pos.col = (i - 1L) %% nc + 1L
#'     ))
#'     grid.pattern_wave(colour = "black", fill = c("gold", "steelblue"),
#'                       type = names_wave[i], density = 0.18, spacing = 0.45,
#'                       angle = 0, amplitude = 0.100, frequency = 1 / 0.45)
#'     grid::grid.rect(x = 0.5, y = 0.86, width = 0.5, height = 0.28,
#'                     just = "centre", gp = grid::gpar(fill = "grey80", col = "black"))
#'     grid::grid.text(names_wave[i], x = 0.5, y = 0.88,
#'                     gp = grid::gpar(fontsize = 11))
#'     grid::grid.rect(gp = grid::gpar(fill = "transparent", col = "black", lwd = 6))
#'     grid::popViewport()
#' }
#' grid::popViewport()
#'
#' # stagger shifts alternate rows by half a wavelength
#' grid::grid.newpage()
#' x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#' grid.pattern_wave(x_hex, y_hex, colour = "black", type = "urdy",
#'                   fill = c("red", "blue"), density = 0.3,
#'                   spacing = 0.15, angle = 0,
#'                   amplitude = 0.045, frequency = 1 / 0.15, stagger = TRUE)
#' @seealso Use [grid.pattern_stripe()] for straight filled bands or [grid.pattern_line()] for stroked lines instead of waves.
#'   See <https://en.wikipedia.org/wiki/Line_(heraldry)> and <https://en.wikipedia.org/wiki/Waveform> for more information about the supported wave types.
#' @export
grid.pattern_wave <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	colour = gp$col %||% "grey20",
	fill = gp$fill %||% "grey80",
	angle = 30,
	density = 0.2,
	spacing = 0.05,
	xoffset = 0,
	yoffset = 0,
	units = "snpc",
	amplitude = 0.5 * spacing,
	frequency = 1 / spacing,
	alpha = gp$alpha %||% NA_real_,
	linetype = gp$lty %||% 1,
	linewidth = size %||% gp$lwd %||% 1,
	size = NULL,
	grid = "square",
	type = "indented",
	reverse = FALSE,
	stagger = FALSE,
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	if (missing(colour) && hasName(l <- list(...), "color")) {
		colour <- l$color
	}
	grid.pattern(
		"wave",
		x,
		y,
		id,
		colour = colour,
		fill = fill,
		angle = angle,
		density = density,
		spacing = spacing,
		xoffset = xoffset,
		yoffset = yoffset,
		units = units,
		amplitude = amplitude,
		frequency = frequency,
		alpha = alpha,
		linetype = linetype,
		linewidth = linewidth,
		grid = grid,
		type = type,
		reverse = reverse,
		stagger = stagger,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

#' @rdname grid.pattern_wave
#' @export
names_wave <- c(
	"dovetailed",
	"embattled",
	"embattled_grady",
	"engrailed",
	"indented",
	"invected",
	"nebuly",
	"potenty",
	"raguly",
	"sawtoothed",
	"urdy",
	"wavy"
)

create_pattern_wave_via_sf <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
	if (abs(params$pattern_density - 1) < .Machine$double.eps^0.5) {
		params$pattern_density <- 1 - 1e-6
	}
	stopifnot(params$pattern_density <= 1)

	# work in 'bigpts' instead 'npc' / 'snpc' units so we don't worry about the aspect ratio
	default.units <- "bigpts"
	boundary_df <- convert_polygon_df_units(boundary_df, default.units)
	params <- convert_params_units(params, default.units)
	vpm <- get_vp_measurements(default.units)

	# create grid of points large enough to cover viewport no matter the angle
	grid_xy <- get_xy_grid(params, vpm, wavelength = TRUE)

	fill <- update_alpha(params$pattern_fill, params$pattern_alpha)
	col <- update_alpha(params$pattern_colour, params$pattern_alpha)
	lwd <- params$pattern_linewidth * .pt
	lty <- params$pattern_linetype
	density <- params$pattern_density

	n_par <- max(lengths(list(fill, col, lwd, lty, density)))

	if (isTRUE(params$pattern_stagger) && n_par %% 2L == 1L) {
		halfwidth <- 0.5 * grid_xy$v_spacing * params$pattern_density
		if (2 * (params$pattern_amplitude + halfwidth) > n_par * grid_xy$v_spacing) {
			abort(c(
				"Wave stagger bands overlap between adjacent rows.",
				i = "Reduce `amplitude` or `density`, increase `spacing`, or use an even number of fill/colour values."
			))
		}
	}

	fill <- rep_len_fill(fill, n_par)
	col <- rep_len(col, n_par)
	lwd <- rep_len(lwd, n_par)
	lty <- rep_len(lty, n_par)
	density <- rep_len(density, n_par)

	gl <- gList()
	for (i_par in seq_len(n_par)) {
		gp <- gpar(
			col = col[i_par],
			fill = fill[[i_par]],
			lwd = lwd[i_par],
			lty = lty[i_par],
			lineend = 'square'
		)

		boundary_sf <- convert_polygon_df_to_polygon_sf(boundary_df, buffer_dist = 0)

		waves_sf <- create_waves_sf(params, grid_xy, vpm, i_par, n_par)
		clipped_waves_sf_bot <- tryCatch(
			sf::st_intersection(waves_sf, boundary_sf),
			error = function(e) {
				if (
					grepl(
						"TopologyException|Self-intersection|invalid",
						conditionMessage(e),
						ignore.case = TRUE
					)
				) {
					abort(
						c(
							paste0(
								"Wave pattern ",
								dQuote(params$pattern_type),
								" produced a self-intersecting polygon."
							),
							i = "Try reducing `density`, increasing `amplitude`, decreasing `frequency`, and/or increasing `spacing`."
						),
						call = NULL
					)
				}
				stop(e)
			}
		)
		name <- paste0("wave.", i_par)
		grob <- sf_multipolygon_to_polygon_grob(clipped_waves_sf_bot, gp, default.units, name)
		gl <- append_gList(gl, grob)
	}
	gTree(children = gl, name = "regular_polygon")
}

create_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	switch(
		params$pattern_type,
		sawlike = ,
		sawtooth = ,
		sawtoothed = create_sawtooth_waves_sf(params, grid_xy, vpm, i_par, n_par),
		dovetail = ,
		dovetailed = create_dovetail_waves_sf(params, grid_xy, vpm, i_par, n_par),
		embattled = ,
		square = create_stepped_waves_sf(params, grid_xy, vpm, i_par, n_par, n_steps = 1L),
		embattled_grady = create_stepped_waves_sf(params, grid_xy, vpm, i_par, n_par, n_steps = 2L),
		engrailed = create_sine_waves_sf(params, grid_xy, vpm, i_par, n_par),
		invected = create_sine_waves_sf(params, grid_xy, vpm, i_par, n_par),
		nebuly = create_nebuly_waves_sf(params, grid_xy, vpm, i_par, n_par),
		potenty = create_potenty_waves_sf(params, grid_xy, vpm, i_par, n_par),
		raguly = create_raguly_waves_sf(params, grid_xy, vpm, i_par, n_par),
		indented = ,
		triangle = create_triangle_waves_sf(params, grid_xy, vpm, i_par, n_par),
		urdy = create_urdy_waves_sf(params, grid_xy, vpm, i_par, n_par),
		wavy = ,
		sine = ,
		undy = create_sine_waves_sf(params, grid_xy, vpm, i_par, n_par),
		abort(paste("Don't know how to create wave pattern", dQuote(params$pattern_type)))
	)
}

create_sine_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	halfwidth <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	n_s <- 180L
	type <- params$pattern_type
	is_half_arch <- type %in% c("engrailed", "invected")
	if (is_half_arch) {
		# One arch per wavelength: theta spans only half a sine period (0 to π)
		theta <- to_radians(seq(0, by = 180L / n_s, length.out = n_s))
		# Use 2a amplitude and center at zero so the arch spans [-a, +a], matching
		# other wave types. Without the -/+a offset the arch baseline sits at y0±a.
		y_s_base <- if (type == "invected") 2 * a * sin(theta) - a else -2 * a * sin(theta) + a
	} else {
		theta <- to_radians(seq(0, by = 360L / n_s, length.out = n_s))
		y_s_base <- a * sin(theta)
	}
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)
	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_this <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L
		if (stagger_this) {
			y_s <- if (is_half_arch) {
				n_half <- n_s %/% 2L
				c(y_s_base[(n_half + 1L):n_s], y_s_base[1L:n_half])
			} else {
				-y_s_base
			}
		} else {
			y_s <- y_s_base
		}
		# Phase alignment: invected peak (y=+a) → x_min+λ/4 (matches sine peak, shift n_s/4).
		# Engrailed dent (y=−a) → x_min+3λ/4 (matches sine trough, shift 3*n_s/4).
		# Full sine already peaks at x_min+λ/4; no shift needed for it.
		if (is_half_arch) {
			phase_shift <- if (type == "engrailed") 3L * (n_s %/% 4L) else n_s %/% 4L
			y_s <- cycle_elements(y_s, phase_shift)
		}
		n_x <- length(grid_xy$x)
		xc <- seq(grid_xy$x_min, grid_xy$x_max, length.out = n_s * (n_x - 1L) + 1L)
		yc <- y0 + rep(y_s, length.out = n_s * (n_x - 1L) + 1L)
		yt <- yc + halfwidth
		yb <- yc - halfwidth
		x <- c(xc, rev(xc))
		y <- c(yt, rev(yb))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

# Generalised stepped-wave constructor used by "square"/"embattled" (n_steps = 1) and
# "embattled_grady" (n_steps = 2). Phase is chosen so the top step is centred at λ/4,
# aligning with the sine/triangle wave peak. n_steps=1 starts HIGH; n_steps=2 starts MID.
create_stepped_waves_sf <- function(params, grid_xy, vpm, i_par, n_par, n_steps = 1L) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)

	# n_steps transitions per half-period; step height = 2a/n_steps; step width = wavelength/(2*n_steps).
	# Self-intersection: hw >= half the step width (band corners from adjacent steps overlap).
	step_height <- 2 * a / n_steps
	step_width <- wavelength / (2L * n_steps)
	half_step <- 0.5 * step_width
	if (hw >= half_step) {
		abort(c(
			paste0(
				toupper(substring(params$pattern_type, 1L, 1L)),
				substring(params$pattern_type, 2L),
				" wave: band corners overlap between adjacent steps."
			),
			i = "Reduce `density` or increase `spacing`/`frequency`."
		))
	}
	# Phase alignment: shift x_trans back by (n_steps %/% 2) half-steps so the top
	# step is centred at λ/4 (the sine peak).  For n_steps=1 the shift is 0 (unchanged);
	# for n_steps=2 the shift is half_step, moving the first transition to x_min+λ/8.
	phase_shift <- (n_steps %/% 2L) * half_step
	x_trans <- seq(x_min + step_width - phase_shift, x_max + wavelength, by = step_width)
	n_iter <- sum(x_trans <= x_max + hw)
	# sign convention: +1 = descent (high→low), -1 = ascent (low→high)
	base_signs <- cycle_elements(c(rep(1L, n_steps), rep(-1L, n_steps)), -(n_steps %/% 2L))

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L
		# init_level: level at x_min, offset from HIGH by the (n_steps %/% 2) leading ascents
		base_init <- a - step_height * (n_steps %/% 2L)
		init_level <- if (stagger_row) -base_init else base_init
		signs <- rep(
			if (stagger_row) -base_signs else base_signs,
			length.out = n_iter
		)
		n_alloc <- 2L * n_iter + 2L
		top_xs <- numeric(n_alloc)
		top_ys <- numeric(n_alloc)
		bot_xs <- numeric(n_alloc)
		bot_ys <- numeric(n_alloc)
		top_xs[1L] <- x_min
		top_ys[1L] <- y0 + init_level + hw
		bot_xs[1L] <- x_min
		bot_ys[1L] <- y0 + init_level - hw
		top_i <- 2L
		bot_i <- 2L

		for (k in seq_len(n_iter)) {
			xt <- x_trans[k]
			s <- signs[k]
			top_step <- xt + s * hw
			bot_step <- xt - s * hw
			delta_y <- -s * step_height
			if (top_step <= x_max) {
				top_xs[top_i] <- top_step
				top_ys[top_i] <- top_ys[top_i - 1L]
				top_i <- top_i + 1L
				top_xs[top_i] <- top_step
				top_ys[top_i] <- top_ys[top_i - 1L] + delta_y
				top_i <- top_i + 1L
			}
			if (bot_step >= x_min && bot_step <= x_max) {
				bot_xs[bot_i] <- bot_step
				bot_ys[bot_i] <- bot_ys[bot_i - 1L]
				bot_i <- bot_i + 1L
				bot_xs[bot_i] <- bot_step
				bot_ys[bot_i] <- bot_ys[bot_i - 1L] + delta_y
				bot_i <- bot_i + 1L
			}
		}
		top_xs[top_i] <- x_max
		top_ys[top_i] <- top_ys[top_i - 1L]
		bot_xs[bot_i] <- x_max
		bot_ys[bot_i] <- bot_ys[bot_i - 1L]

		x <- c(top_xs[seq_len(top_i)], rev(bot_xs[seq_len(bot_i)]))
		y <- c(top_ys[seq_len(top_i)], rev(bot_ys[seq_len(bot_i)]))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

create_triangle_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	halfwidth <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	# Scale halfwidth so triangle's perpendicular stroke width matches sawtooth's.
	# The triangle diagonal has slope 4a/L vs sawtooth's 2a/L; dividing by the
	# steeper slope correction makes both waves appear equally thick on screen.
	halfwidth <- halfwidth * sqrt((1 + (4 * a / wavelength)^2) / (1 + (2 * a / wavelength)^2))
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)
	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		half_period_shape <- if (isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L) {
			c(0, -a, 0, a)
		} else {
			c(0, a, 0, -a)
		}
		n_x <- length(grid_xy$x)
		xc <- seq(grid_xy$x_min, grid_xy$x_max, length.out = 4L * (n_x - 1L) + 1L)
		yc <- y0 + rep(half_period_shape, length.out = 4L * (n_x - 1L) + 1L)
		yt <- yc + halfwidth
		yb <- yc - halfwidth
		x <- c(xc, rev(xc))
		y <- c(yt, rev(yb))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

# Build a piecewise-linear edge (for sawtooth bands) with a vertical step of `step_y` at each
# x in `x_steps`. The edge starts at `y_init` at `x_min` and follows `slope` between steps.
sawtooth_edge_xy <- function(x_min, x_max, x_steps, slope, y_init, step_y) {
	valid <- x_steps[x_steps > x_min & x_steps <= x_max]
	n_valid <- length(valid)
	n_pts <- 2L * n_valid + 2L
	xs <- numeric(n_pts)
	ys <- numeric(n_pts)
	xs[1L] <- x_min
	ys[1L] <- y_init
	current_x <- x_min
	current_y <- y_init
	for (i in seq_len(n_valid)) {
		x_s <- valid[i]
		y_at_step <- current_y + slope * (x_s - current_x)
		j <- 2L * i
		xs[j] <- x_s
		ys[j] <- y_at_step
		xs[j + 1L] <- x_s
		ys[j + 1L] <- y_at_step + step_y
		current_x <- x_s
		current_y <- y_at_step + step_y
	}
	xs[n_pts] <- x_max
	ys[n_pts] <- current_y + slope * (x_max - current_x)
	list(x = xs, y = ys)
}

create_sawtooth_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)
	# reverse = FALSE: gradual rise, sharp fall (ramp up)
	# reverse = TRUE:  sharp rise, gradual fall (ramp down)
	up <- !isTRUE(params$pattern_reverse)

	# One transition per period; stagger shifts by half a wavelength
	x_trans <- seq(x_min + wavelength, x_max + wavelength, by = wavelength)

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L
		x_t <- if (stagger_row) x_trans - 0.5 * wavelength else x_trans

		if (up) {
			slope <- 2 * a / wavelength
			# gradual rise then sharp fall; stagger starts mid-ramp (center = y0)
			init_top <- if (stagger_row) y0 + hw else y0 - a + hw
			init_bot <- if (stagger_row) y0 - hw else y0 - a - hw
			# top drops at x_t + hw; bottom drops at x_t - hw
			top <- sawtooth_edge_xy(x_min, x_max, x_t + hw, slope, init_top, -2 * a)
			bot <- sawtooth_edge_xy(x_min, x_max, x_t - hw, slope, init_bot, -2 * a)
		} else {
			slope <- -2 * a / wavelength
			# sharp rise then gradual fall; stagger starts mid-ramp (center = y0)
			init_top <- if (stagger_row) y0 + hw else y0 + a + hw
			init_bot <- if (stagger_row) y0 - hw else y0 + a - hw
			# top rises at x_t - hw; bottom rises at x_t + hw
			top <- sawtooth_edge_xy(x_min, x_max, x_t - hw, slope, init_top, 2 * a)
			bot <- sawtooth_edge_xy(x_min, x_max, x_t + hw, slope, init_bot, 2 * a)
		}

		x <- c(top$x, rev(bot$x))
		y <- c(top$y, rev(bot$y))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

create_urdy_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)

	# Each half-period: two diagonals separated by a vertical section.
	# Vertical section height = 2a/3 (one-third of total amplitude 2a).
	# Each diagonal spans wavelength/4 horizontally and 2a/3 vertically.
	v_ht <- 2 * a / 3
	d_wid <- wavelength / 4
	m_slope <- v_ht / d_wid # slope = 8a/(3L)

	# hw_y: y-offset for diagonals so perpendicular band width = hw = horizontal band width.
	# delta: miter-join offset at diagonal-vertical junctions.
	#   The diagonal band edge is extended to x ± hw (the vertical x), landing at y ± delta
	#   from the junction centre. This avoids the "backward step" that causes self-intersection.
	if (hw >= d_wid) {
		abort(c(
			"Urdy wave: band corners overlap between adjacent steps.",
			i = "Reduce `density` or increase `spacing`/`frequency`."
		))
	}
	hw_y <- hw * sqrt(1 + m_slope^2)
	hw_x <- hw
	delta <- hw_y - m_slope * hw_x # = hw * (sqrt(1+m²) - m)

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L

		# Phase: shift x_ps back by d_wid (= λ/4) so the peak lands at x_min + λ/4,
		# aligning with the sine/triangle peak.
		x_ps <- x_min - wavelength - d_wid
		if (stagger_row) {
			x_ps <- x_ps - 0.5 * wavelength
		}
		n_periods <- ceiling((x_max - x_ps) / wavelength) + 2L

		# Top edge = higher-y boundary; bot edge = lower-y boundary.
		# At each diagonal-vertical junction we use a miter join:
		#   the diagonal edge is extended to the vertical x (± hw_x) at y ± delta.
		# Ascending vertical: top edge is on the LEFT  (x - hw_x); bot on the RIGHT (x + hw_x).
		# Descending vertical: top edge is on the RIGHT (x + hw_x); bot on the LEFT  (x - hw_x).
		top_x <- x_ps
		top_y <- y0 - a + hw_y
		bot_x <- x_ps
		bot_y <- y0 - a - hw_y

		for (k in seq_len(n_periods)) {
			x0 <- x_ps + (k - 1L) * wavelength

			# One period: trough at x0, peak at x0+L/2, trough at x0+L.
			# Per-period top-edge points (6 new, starting from the in-array trough):
			#  miter entry/exit ascending-vertical left:  x0+dw-hw_x, y = y0±a/3 + delta
			#  peak:                                      x0+2dw,     y = y0+a+hw_y
			#  miter entry/exit descending-vertical right:x0+3dw+hw_x,y = y0±a/3 + delta
			#  next trough:                               x0+L,       y = y0-a+hw_y
			top_x <- c(
				top_x,
				x0 + d_wid - hw_x,
				x0 + d_wid - hw_x,
				x0 + 2 * d_wid,
				x0 + 3 * d_wid + hw_x,
				x0 + 3 * d_wid + hw_x,
				x0 + wavelength
			)
			top_y <- c(
				top_y,
				y0 - a / 3 + delta,
				y0 + a / 3 + delta,
				y0 + a + hw_y,
				y0 + a / 3 + delta,
				y0 - a / 3 + delta,
				y0 - a + hw_y
			)

			# Bot-edge points (vertical sections on the opposite sides):
			#  miter entry/exit ascending-vertical right: x0+dw+hw_x, y = y0±a/3 - delta
			#  peak:                                      x0+2dw,     y = y0+a-hw_y
			#  miter entry/exit descending-vertical left: x0+3dw-hw_x,y = y0±a/3 - delta
			#  next trough:                               x0+L,       y = y0-a-hw_y
			bot_x <- c(
				bot_x,
				x0 + d_wid + hw_x,
				x0 + d_wid + hw_x,
				x0 + 2 * d_wid,
				x0 + 3 * d_wid - hw_x,
				x0 + 3 * d_wid - hw_x,
				x0 + wavelength
			)
			bot_y <- c(
				bot_y,
				y0 - a / 3 - delta,
				y0 + a / 3 - delta,
				y0 + a - hw_y,
				y0 + a / 3 - delta,
				y0 - a / 3 - delta,
				y0 - a - hw_y
			)
		}

		x <- c(top_x, rev(bot_x))
		y <- c(top_y, rev(bot_y))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

create_dovetail_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)

	# Dovetail per period: trough (2dw), ascending diagonal (1dw horiz, 2a vert),
	# crest (4dw), descending diagonal, trough (2dw).  dw = wavelength/6.
	#
	# Acute triangular corners: the diagonal outer/inner edges are NOT mitered
	# but extended to their natural intersection with the horizontal band boundary,
	# producing sharp pointed tips.  T_tip is the x-extension of each tip from the
	# nominal junction centre:
	#   T_tip = hw * (D + dw) / (2a),   D = sqrt(dw² + (2a)²)
	#
	# When T_tip >= dw the ascending and descending outer tips cross each other
	# (self-intersecting polygon), so we abort early.
	dw <- wavelength / 6
	D <- sqrt(dw^2 + (2 * a)^2)
	T_tip <- if (a == 0) 0 else hw * (D + dw) / (2 * a)

	if (T_tip >= dw) {
		abort(c(
			"Dovetailed wave: acute corner tips overlap between adjacent dovetails.",
			i = "Reduce `density`, increase `amplitude`, or increase `spacing`."
		))
	}

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L
		# Phase: shift x_ps back by λ/4 so the crest centre (x0+3dw, dw=λ/6) lands at λ/4.
		x_ps <- x_min - wavelength - wavelength / 4
		if (stagger_row) {
			x_ps <- x_ps - 0.5 * wavelength
		}
		n_periods <- ceiling((x_max - x_ps) / wavelength) + 2L

		# Top edge = higher-y boundary; bot edge = lower-y boundary.
		# Per period, 5 new vertices each (the period-start point is already in the array):
		#   top: inner ascending trough tip   (x0+2dw-T, y0-a+hw)
		#        outer ascending crest tip    (x0+ dw-T, y0+a+hw)
		#        outer descending crest tip   (x0+5dw+T, y0+a+hw)
		#        inner descending trough tip  (x0+4dw+T, y0-a+hw)
		#        period end                   (x0+6dw,   y0-a+hw)
		#   bot: outer ascending trough tip   (x0+2dw+T, y0-a-hw)
		#        inner ascending crest tip    (x0+ dw+T, y0+a-hw)
		#        inner descending crest tip   (x0+5dw-T, y0+a-hw)
		#        outer descending trough tip  (x0+4dw-T, y0-a-hw)
		#        period end                   (x0+6dw,   y0-a-hw)
		top_x <- x_ps
		top_y <- y0 - a + hw
		bot_x <- x_ps
		bot_y <- y0 - a - hw

		for (k in seq_len(n_periods)) {
			x0 <- x_ps + (k - 1L) * wavelength
			top_x <- c(
				top_x,
				x0 + 2 * dw - T_tip,
				x0 + dw - T_tip,
				x0 + 5 * dw + T_tip,
				x0 + 4 * dw + T_tip,
				x0 + 6 * dw
			)
			top_y <- c(top_y, y0 - a + hw, y0 + a + hw, y0 + a + hw, y0 - a + hw, y0 - a + hw)
			bot_x <- c(
				bot_x,
				x0 + 2 * dw + T_tip,
				x0 + dw + T_tip,
				x0 + 5 * dw - T_tip,
				x0 + 4 * dw - T_tip,
				x0 + 6 * dw
			)
			bot_y <- c(bot_y, y0 - a - hw, y0 + a - hw, y0 + a - hw, y0 - a - hw, y0 - a - hw)
		}

		x <- c(top_x, rev(bot_x))
		y <- c(top_y, rev(bot_y))
		xy <- rotate_xy(x, y, params$pattern_angle, vpm$x, vpm$y)
		m_mat <- as.matrix(as.data.frame(xy))
		list(rbind(m_mat, m_mat[1L, ]))
	})
	sf::st_multipolygon(l_waves)
}

create_nebuly_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)

	# Nebuly = potenty band with rounded corners: separate outer and inner boundary
	# curves, each a piecewise quadratic Bézier arc that goes from the midpoint of
	# one band segment through the corner vertex (control point) to the midpoint of
	# the next segment.  Building outer/inner separately (like potenty) prevents the
	# band-edge crossing that a simple vertical-offset centerline polygon can produce.
	#
	# dw = wavelength/4; potenty centerline corners P1..P8 relative to (x0,y0):
	#   P1=(3dw,-a), P2=(3dw,0), P3=(2dw,0), P4=(2dw,+a),
	#   P5=(5dw,+a), P6=(5dw,0), P7=(4dw,0), P8=(4dw,-a)
	# Outer (left/CCW) corner offsets T1..T8:
	#   T1=(3dw-hw,-a+hw), T2=(3dw-hw,-hw),  T3=(2dw-hw,-hw),  T4=(2dw-hw,a+hw)
	#   T5=(5dw+hw,a+hw),  T6=(5dw+hw,-hw),  T7=(4dw+hw,-hw),  T8=(4dw+hw,-a+hw)
	# Inner (right/CW) corner offsets B1..B8:
	#   B1=(3dw+hw,-a-hw), B2=(3dw+hw,hw),   B3=(2dw+hw,hw),   B4=(2dw+hw,a-hw)
	#   B5=(5dw-hw,a-hw),  B6=(5dw-hw,hw),   B7=(4dw-hw,hw),   B8=(4dw-hw,-a-hw)
	dw <- wavelength / 4
	if (hw > dw / 2) {
		abort(c(
			"Nebuly wave: band corners overlap between adjacent steps.",
			i = "Reduce `density` or increase `spacing`/`frequency`."
		))
	}
	tc_dx <- c(
		3 * dw - hw,
		3 * dw - hw,
		2 * dw - hw,
		2 * dw - hw,
		5 * dw + hw,
		5 * dw + hw,
		4 * dw + hw,
		4 * dw + hw
	)
	tc_dy <- c(-a + hw, -hw, -hw, a + hw, a + hw, -hw, -hw, -a + hw)
	bc_dx <- c(
		3 * dw + hw,
		3 * dw + hw,
		2 * dw + hw,
		2 * dw + hw,
		5 * dw - hw,
		5 * dw - hw,
		4 * dw - hw,
		4 * dw - hw
	)
	bc_dy <- c(-a - hw, hw, hw, a - hw, a - hw, hw, hw, -a - hw)

	# Arc endpoints = midpoints of consecutive outer/inner band corner vertices.
	# MT0_next (index 9) has the same x-offset as MT0 shifted by one wavelength:
	# MT0..MT8 for outer, MB0..MB8 for inner.
	mt_dx <- c(
		1.5 * dw,
		3 * dw - hw,
		2.5 * dw - hw,
		2 * dw - hw,
		3.5 * dw,
		5 * dw + hw,
		4.5 * dw + hw,
		4 * dw + hw,
		5.5 * dw
	)
	mt_dy <- c(-a + hw, -a / 2, -hw, a / 2, a + hw, a / 2, -hw, -a / 2, -a + hw)
	mb_dx <- c(
		1.5 * dw,
		3 * dw + hw,
		2.5 * dw + hw,
		2 * dw + hw,
		3.5 * dw,
		5 * dw - hw,
		4.5 * dw - hw,
		4 * dw - hw,
		5.5 * dw
	)
	mb_dy <- c(-a - hw, -a / 2, hw, a / 2, a - hw, a / 2, hw, -a / 2, -a - hw)
	n_seg <- 8L

	n_s <- 12L
	tv <- seq(0, 1, length.out = n_s + 1L)[seq_len(n_s)]
	q0 <- (1 - tv)^2
	q1 <- 2 * tv * (1 - tv)
	q2 <- tv^2

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L

		# Phase: shift x_ps back by 5λ/8 so the crest centre (x0+3.5dw, dw=λ/4) lands at λ/4.
		x_ps <- x_min - wavelength - 5 * wavelength / 8
		if (stagger_row) {
			x_ps <- x_ps - 0.5 * wavelength
		}
		n_periods <- ceiling((x_max - x_ps) / wavelength) + 2L

		n_pts <- n_periods * n_seg * n_s + 2L
		top_x <- numeric(n_pts)
		top_y <- numeric(n_pts)
		bot_x <- numeric(n_pts)
		bot_y <- numeric(n_pts)

		# Lead-in at x_ps (left-end cap)
		top_x[1L] <- x_ps
		top_y[1L] <- y0 + mt_dy[1L]
		bot_x[1L] <- x_ps
		bot_y[1L] <- y0 + mb_dy[1L]
		pos <- 2L

		for (k in seq_len(n_periods)) {
			x0 <- x_ps + (k - 1L) * wavelength
			for (seg in seq_len(n_seg)) {
				# Outer arc: MT_{seg} → TC_{seg} → MT_{seg+1}
				ax <- x0 + mt_dx[seg]
				ay <- y0 + mt_dy[seg]
				bx <- x0 + tc_dx[seg]
				by <- y0 + tc_dy[seg]
				if (seg < n_seg) {
					cx <- x0 + mt_dx[seg + 1L]
					cy <- y0 + mt_dy[seg + 1L]
				} else {
					cx <- x0 + mt_dx[9L]
					cy <- y0 + mt_dy[9L]
				}
				top_x[pos:(pos + n_s - 1L)] <- q0 * ax + q1 * bx + q2 * cx
				top_y[pos:(pos + n_s - 1L)] <- q0 * ay + q1 * by + q2 * cy
				# Inner arc: MB_{seg} → BC_{seg} → MB_{seg+1}
				ax <- x0 + mb_dx[seg]
				ay <- y0 + mb_dy[seg]
				bx <- x0 + bc_dx[seg]
				by <- y0 + bc_dy[seg]
				if (seg < n_seg) {
					cx <- x0 + mb_dx[seg + 1L]
					cy <- y0 + mb_dy[seg + 1L]
				} else {
					cx <- x0 + mb_dx[9L]
					cy <- y0 + mb_dy[9L]
				}
				bot_x[pos:(pos + n_s - 1L)] <- q0 * ax + q1 * bx + q2 * cx
				bot_y[pos:(pos + n_s - 1L)] <- q0 * ay + q1 * by + q2 * cy
				pos <- pos + n_s
			}
		}
		# Right-end cap
		top_x[n_pts] <- x_ps + n_periods * wavelength + mt_dx[1L]
		top_y[n_pts] <- y0 + mt_dy[1L]
		bot_x[n_pts] <- x_ps + n_periods * wavelength + mb_dx[1L]
		bot_y[n_pts] <- y0 + mb_dy[1L]

		x <- c(top_x, rev(bot_x))
		y <- c(top_y, rev(bot_y))
		xy <- rotate_xy(x, y, params$pattern_angle, vpm$x, vpm$y)
		m_mat <- as.matrix(as.data.frame(xy))
		list(rbind(m_mat, m_mat[1L, ]))
	})
	sf::st_multipolygon(l_waves)
}

create_potenty_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)

	# One period: right 3dw, up a, left dw, up a, right 3dw, down a, left dw, down a.
	# Total advance = 3dw - dw + 3dw - dw = 4dw = wavelength, so dw = wavelength/4.
	# The top plateau overshoots by dw (landing at x0+5dw = x0+wl+dw), which is fine
	# because the per-row polygon is clipped to the boundary by sf::st_intersection.
	dw <- wavelength / 4
	if (hw > dw / 2) {
		abort(c(
			"Potenty wave: band corners overlap between adjacent steps.",
			i = "Reduce `density` or increase `spacing`/`frequency`."
		))
	}

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L

		# Phase: shift x_ps back by 5λ/8 so the crest centre (x0+3.5dw, dw=λ/4) lands at λ/4.
		x_ps <- x_min - wavelength - 5 * wavelength / 8
		if (stagger_row) {
			x_ps <- x_ps - 0.5 * wavelength
		}
		n_periods <- ceiling((x_max - x_ps) / wavelength) + 2L

		# 9 vertices per period (P1-P8 + next-period anchor) plus initial and trailing.
		n_pts <- 9L * n_periods + 2L
		top_x <- numeric(n_pts)
		top_y <- numeric(n_pts)
		bot_x <- numeric(n_pts)
		bot_y <- numeric(n_pts)
		top_x[1L] <- x_ps
		top_y[1L] <- y0 - a + hw
		bot_x[1L] <- x_ps
		bot_y[1L] <- y0 - a - hw

		for (k in seq_len(n_periods)) {
			x0 <- x_ps + (k - 1L) * wavelength
			i <- (k - 1L) * 9L + 2L

			# Left boundary: outer corners at P1, P2, P7, P8; inner at P3, P4, P5, P6.
			# Each corner = intersection of the two adjacent offset lines (left side of travel).
			top_x[i:(i + 8L)] <- c(
				x0 + 3 * dw - hw, # outer P1 (→ to ↑): left of → at y0-a, left of ↑ at x0+3dw
				x0 + 3 * dw - hw, # outer P2 (↑ to ←): left of ↑, left of ← at y0 — same x, vertical
				x0 + 2 * dw - hw, # inner P3 (← to ↑): left of ← at y0, left of ↑ at x0+2dw
				x0 + 2 * dw - hw, # inner P4 (↑ to →): left of ↑, left of → at y0+a — same x, vertical
				x0 + 5 * dw + hw, # inner P5 (→ to ↓): left of → at y0+a, left of ↓ at x0+5dw
				x0 + 5 * dw + hw, # inner P6 (↓ to ←): left of ↓, left of ← at y0 — same x, vertical
				x0 + 4 * dw + hw, # outer P7 (← to ↓): left of ← at y0, left of ↓ at x0+4dw
				x0 + 4 * dw + hw, # outer P8 (↓ to →): left of ↓, left of → at y0-a — same x, vertical
				x0 + 4 * dw # L1 of next period
			)
			top_y[i:(i + 8L)] <- c(
				y0 - a + hw, # P1
				y0 - hw, # P2
				y0 - hw, # P3
				y0 + a + hw, # P4
				y0 + a + hw, # P5
				y0 - hw, # P6
				y0 - hw, # P7
				y0 - a + hw, # P8
				y0 - a + hw # L1 next
			)

			# Right boundary: inner corners at P1, P2, P7, P8; outer at P3, P4, P5, P6.
			bot_x[i:(i + 8L)] <- c(
				x0 + 3 * dw + hw, # inner P1
				x0 + 3 * dw + hw, # inner P2 — same x, vertical
				x0 + 2 * dw + hw, # outer P3
				x0 + 2 * dw + hw, # outer P4 — same x, vertical
				x0 + 5 * dw - hw, # outer P5
				x0 + 5 * dw - hw, # outer P6 — same x, vertical
				x0 + 4 * dw - hw, # inner P7
				x0 + 4 * dw - hw, # inner P8 — same x, vertical
				x0 + 4 * dw # R1 of next period
			)
			bot_y[i:(i + 8L)] <- c(
				y0 - a - hw, # P1
				y0 + hw, # P2
				y0 + hw, # P3
				y0 + a - hw, # P4
				y0 + a - hw, # P5
				y0 + hw, # P6
				y0 + hw, # P7
				y0 - a - hw, # P8
				y0 - a - hw # R1 next
			)
		}

		top_x[n_pts] <- x_max
		top_y[n_pts] <- top_y[n_pts - 1L]
		bot_x[n_pts] <- x_max
		bot_y[n_pts] <- bot_y[n_pts - 1L]

		x <- c(top_x, rev(bot_x))
		y <- c(top_y, rev(bot_y))
		polygon_ring(x, y, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

create_raguly_waves_sf <- function(params, grid_xy, vpm, i_par, n_par) {
	hw <- 0.5 * grid_xy$v_spacing * params$pattern_density
	a <- params$pattern_amplitude
	wavelength <- grid_xy$h_spacing
	x_min <- grid_xy$x_min
	x_max <- grid_xy$x_max
	n_y <- length(grid_xy$y)
	indices_y <- seq(from = i_par, to = n_y, by = n_par)
	# reverse = FALSE: diagonal rises rightward (default)
	# reverse = TRUE:  diagonal rises leftward (mirror image)
	up <- !isTRUE(params$pattern_reverse)

	# Raguly per period (wavelength = 4*dw):
	#   up (reverse=FALSE): S1 low flat (+2dw), S2 diagonal-up-right (+2dw), S3 high flat (+2dw),
	#                       S4 diagonal-down-left (-2dw). Net +4dw per period.
	#   down (reverse=TRUE): S1 low flat (+2dw), S2 diagonal-up-left (-2dw), S3 high flat (+2dw),
	#                        S4 diagonal-down-right (+2dw). Net +4dw per period.
	#
	# Built as a single polygon per wave row.  The backward diagonal (S4 for raguly_up,
	# S2 for raguly_down) produces an acute-angle corner where it meets the adjacent
	# horizontal flat.  Two tip offsets are used:
	#   mg     = hw*(R1-dw)/a  — miter for forward-diagonal junctions (S1/S2, S2/S3)
	#   T_tip  = hw*(R1+dw)/a  — tip extension for backward-diagonal junctions (S3/S4, S4/S1)
	# Self-intersection check: hw >= max_hw = a*dw/R1.
	dw <- wavelength / 4
	if (a > 0) {
		R1 <- sqrt(dw^2 + a^2)
		max_hw <- a * dw / R1
		if (hw >= max_hw) {
			abort(c(
				"Raguly wave: outer diagonal overlaps the inner horizontal boundary.",
				i = "Reduce `density`, increase `amplitude`, or increase `spacing`/`frequency`."
			))
		}
		mg <- hw * (R1 - dw) / a
		T_tip <- hw * (R1 + dw) / a
	} else {
		mg <- 0
		T_tip <- 0
	}

	l_waves <- lapply(seq_along(indices_y), function(j) {
		y0 <- grid_xy$y[indices_y[j]]
		stagger_row <- isTRUE(params$pattern_stagger) && indices_y[j] %% 2L == 0L
		x_ps <- x_min - wavelength
		if (stagger_row) {
			x_ps <- x_ps - 0.5 * wavelength
		}
		n_periods <- ceiling((x_max - x_ps) / wavelength) + 2L

		yL <- y0 - a + hw # outer low level
		yH <- y0 + a + hw # outer high level
		yL2 <- y0 - a - hw # inner low level
		yH2 <- y0 + a - hw # inner high level

		# 4 vertices per period × 2 passes (forward outer + backward inner) + 2 endpoints.
		n_verts <- 8L * n_periods + 2L
		vx <- numeric(n_verts)
		vy <- numeric(n_verts)

		if (up) {
			# raguly_up: S4 is the backward diagonal (down-left).
			# Forward pass traverses the outer boundary: S1_outer→S2_outer→S3_outer→S4_outer,
			#   where S4_outer produces a sharp acute corner at J3 (x0+6dw+T_tip, yH).
			# Backward pass traverses the inner boundary: S4_inner→S3_inner→S2_inner→S1_inner.
			# Vertices per period forward: J1=(x0+2dw-mg,yL), J2=(x0+4dw-mg,yH),
			#   J3=(x0+6dw+T_tip,yH) [acute corner], J4=(x0+4dw+T_tip,yL).
			# Vertices per period backward: J3i=(x0+6dw-T_tip,yH2), J2i=(x0+4dw+mg,yH2),
			#   J1i=(x0+2dw+mg,yL2), J0i=(x0-T_tip,yL2).
			vx[1L] <- x_ps + T_tip
			vy[1L] <- yL

			for (k in seq_len(n_periods)) {
				x0 <- x_ps + (k - 1L) * wavelength
				i <- (k - 1L) * 4L + 2L
				vx[i:(i + 3L)] <- c(
					x0 + 2 * dw - mg,
					x0 + 4 * dw - mg,
					x0 + 6 * dw + T_tip,
					x0 + 4 * dw + T_tip
				)
				vy[i:(i + 3L)] <- c(yL, yH, yH, yL)
			}

			vx[4L * n_periods + 2L] <- x_ps + (n_periods - 1L) * wavelength + 4 * dw - T_tip
			vy[4L * n_periods + 2L] <- yL2

			for (j in seq_len(n_periods)) {
				x0 <- x_ps + (n_periods - j) * wavelength
				i <- 4L * n_periods + 4L * j - 1L
				vx[i:(i + 3L)] <- c(
					x0 + 6 * dw - T_tip,
					x0 + 4 * dw + mg,
					x0 + 2 * dw + mg,
					x0 - T_tip
				)
				vy[i:(i + 3L)] <- c(yH2, yH2, yL2, yL2)
			}
		} else {
			# raguly_down: S2 is the backward diagonal (up-left).
			# Forward pass traverses the outer boundary: S1_outer→S2_outer→S3_outer→S4_outer,
			#   where S2_outer produces sharp acute corners at J1 (x0+2dw-T_tip, yL) and
			#   J2 (x0-T_tip, yH).
			# Backward pass traverses the inner boundary: S4_inner→S3_inner→S2_inner→S1_inner.
			# Vertices per period forward: J1=(x0+2dw-T_tip,yL) [acute], J2=(x0-T_tip,yH) [acute],
			#   J3=(x0+2dw+mg,yH), J4=(x0+4dw+mg,yL).
			# Vertices per period backward: J3i=(x0+2dw-mg,yH2), J2i=(x0+T_tip,yH2),
			#   J1i=(x0+2dw+T_tip,yL2), J0i=(x0-mg,yL2).
			vx[1L] <- x_ps + mg
			vy[1L] <- yL

			for (k in seq_len(n_periods)) {
				x0 <- x_ps + (k - 1L) * wavelength
				i <- (k - 1L) * 4L + 2L
				vx[i:(i + 3L)] <- c(
					x0 + 2 * dw - T_tip,
					x0 - T_tip,
					x0 + 2 * dw + mg,
					x0 + 4 * dw + mg
				)
				vy[i:(i + 3L)] <- c(yL, yH, yH, yL)
			}

			vx[4L * n_periods + 2L] <- x_ps + (n_periods - 1L) * wavelength + 4 * dw - mg
			vy[4L * n_periods + 2L] <- yL2

			for (j in seq_len(n_periods)) {
				x0 <- x_ps + (n_periods - j) * wavelength
				i <- 4L * n_periods + 4L * j - 1L
				vx[i:(i + 3L)] <- c(x0 + 2 * dw - mg, x0 + T_tip, x0 + 2 * dw + T_tip, x0 - mg)
				vy[i:(i + 3L)] <- c(yH2, yH2, yL2, yL2)
			}
		}

		polygon_ring(vx, vy, params$pattern_angle, vpm)
	})
	sf::st_multipolygon(l_waves)
}

# # build sf multipolygon 'rect' for each grid_xy$y value
# create_h_stripes_sf <- function(params, grid_xy, vpm) {
#     halfwidth <- 0.5 * grid_xy$v_spacing * params$pattern_density
#     l_rects <- lapply(grid_xy$y,
#                       function(y0) {
#                           x <- c(grid_xy$x_min, grid_xy$x_min, grid_xy$x_max, grid_xy$x_max)
#                           y <- y0 + c(-1, 1, 1, -1) * halfwidth
#                           xy <- rotate_xy(x, y, params$pattern_angle, vpm$x, vpm$y)
#                           m <- as.matrix(as.data.frame(xy))
#                           list(rbind(m, m[1,]))
#                       })
#     sf::st_multipolygon(l_rects)
# }
#
# # build sf multipolygon 'rect' for each grid_xy$x value
# create_v_stripes_sf <- function(params, grid_xy, vpm) {
#     halfwidth <- 0.5 * grid_xy$h_spacing * params$pattern_density
#     l_rects <- lapply(grid_xy$x,
#                       function(x0) {
#                           x <- x0 + c(-1, 1, 1, -1) * halfwidth
#                           y <- c(grid_xy$y_min, grid_xy$y_min, grid_xy$y_max, grid_xy$y_max)
#                           xy <- rotate_xy(x, y, params$pattern_angle, vpm$x, vpm$y)
#                           m <- as.matrix(as.data.frame(xy))
#                           list(rbind(m, m[1,]))
#                       })
#     sf::st_multipolygon(l_rects)
# }

fill_types <- c('fit', 'expand', 'squish', 'none', 'tile')

#' Resize image, preserving aspect, such that one of target width or height is achieved without distoring or cropping.
#'
#' The canvas of the image will be expanded to cover the full width/height but
#' will be transparent.
#'
#' @inheritParams fill_area_with_img
#'
#' @return magick image of the required dimensions
#'
#' @examples
#' try({
#'   filename <- system.file("img", "Rlogo.png", package="png")
#'   img <- magick::image_read(filename)
#'   fill_area_with_img_fit(img, 100, 400)
#' })
#' @noRd
fill_area_with_img_fit <- function(img, width, height, gravity = 'Center', filter = 'lanczos') {
	geometry <- magick::geometry_size_pixels(width = width, height = height, preserve_aspect = TRUE)
	img <- magick::image_resize(img, geometry = geometry, filter = filter)
	img <- magick::image_extent(img, geometry, gravity = gravity)

	img
}

#' Resize image, ignoring aspect, such that both the target width and height is achieved.
#'
#' Unless the width and height are carefully chosen, this operation will distort
#' the image to force it to fit the dimensions
#'
#' @inheritParams fill_area_with_img
#'
#' @return magick image of the required dimensions
#'
#' @examples
#' try({
#'   filename <- system.file("img", "Rlogo.png", package="png")
#'   img <- magick::image_read(filename)
#'   fill_area_with_img_squish(img, 100, 400)
#' })
#' @noRd
fill_area_with_img_squish <- function(img, width, height, filter = 'lanczos') {
	geometry <- magick::geometry_size_pixels(
		width = width,
		height = height,
		preserve_aspect = FALSE
	)
	img <- magick::image_resize(img, geometry = geometry, filter = filter)
	img
}


#' Resize image, preserving aspect, such that when cropped, the target width and height is achieved.
#'
#' Unless the width and height are carefully chosen, this operation will distort
#' the image to force it to fit the dimensions
#'
#' @inheritParams fill_area_with_img
#'
#' @return magick image of the required dimensions
#'
#' @examples
#' try({
#'   filename <- system.file("img", "Rlogo.png", package="png")
#'   img <- magick::image_read(filename)
#'   fill_area_with_img_expand(img, 100, 400)
#' })
#' @noRd
fill_area_with_img_expand <- function(img, width, height, gravity = 'Center', filter = 'lanczos') {
	img_info <- magick::image_info(img)
	img_aspect <- img_info$width / img_info$height
	area_aspect <- width / height
	if (area_aspect > img_aspect) {
		scale_width <- width
		scale_height <- width / img_aspect
	} else if (area_aspect < img_aspect) {
		scale_height <- height
		scale_width <- height * img_aspect
	} else {
		scale_height <- height
		scale_width <- width
	}

	# expand
	geometry <- magick::geometry_size_pixels(
		width = scale_width,
		height = scale_height,
		preserve_aspect = FALSE
	)
	img <- magick::image_resize(img, geometry = geometry, filter = filter)

	# crop
	geometry <- magick::geometry_size_pixels(
		width = width,
		height = height,
		preserve_aspect = FALSE
	)
	img <- magick::image_crop(img, geometry = geometry, gravity = gravity)

	img
}


#' Resize image canvas to the target width/height, and use gravity to place the undistorted image
#'
#' Unless the width and height are carefully chosen, this operation will distort
#' the image to force it to fit the dimensions
#'
#' @inheritParams fill_area_with_img
#'
#' @return magick image of the required dimensions
#'
#' @examples
#' try({
#'   filename <- system.file("img", "Rlogo.png", package="png")
#'   img <- magick::image_read(filename)
#'   fill_area_with_img_none(img, 100, 400)
#' })
#' @noRd
fill_area_with_img_none <- function(
	img,
	width,
	height,
	gravity = 'Center',
	filter = 'lanczos',
	scale = 1
) {
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Scale if requested
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	scale <- update_scale(scale, img, width, height)
	if (scale != 1) {
		geometry <- magick::geometry_size_percent(width = scale * 100)
		img <- magick::image_resize(img, geometry, filter = filter)
	}

	geometry <- magick::geometry_size_pixels(width = width, height = height, preserve_aspect = TRUE)
	img <- magick::image_extent(img, geometry, gravity = gravity)

	img
}

# if `scale == -1` scale to img width, if `scale == -2` scale to img height
update_scale <- function(scale, img, width, height) {
	if (scale == -1) {
		info <- magick::image_info(img)
		width / info$width
	} else if (scale == -2) {
		info <- magick::image_info(img)
		height / info$height
	} else {
		scale
	}
}

#' Tile image to fill the specified area
#'
#' @return magick image of the required dimensions
#' @noRd
fill_area_with_img_tile <- function(
	img,
	width,
	height,
	gravity = "SouthWest",
	filter = filter,
	scale = 1
) {
	scale <- update_scale(scale, img, width, height)
	if (scale != 1) {
		geometry <- magick::geometry_size_percent(width = scale * 100)
		img <- magick::image_resize(img, geometry, filter = filter)
	}

	img_info <- magick::image_info(img)
	n_width <- width %/% img_info$width + as.integer(width %% img_info$width > 0)
	n_height <- height %/% img_info$height + as.integer(height %% img_info$height > 0)
	rows <- magick::image_append(rep(img, n_width))
	tiled <- magick::image_append(rep(rows, n_height), stack = TRUE)

	geometry <- magick::geometry_size_pixels(
		width = width,
		height = height,
		preserve_aspect = FALSE
	)

	cropped <- magick::image_crop(tiled, geometry = geometry, gravity = gravity)

	cropped
}

#' Fill an area with a magick image
#'
#' Support fill types
#'
#' \itemize{
#' \item{\code{fit   }}{ - Fill image canvas, preserving aspect, such that one of target width or height is achieved without distoring or cropping.}
#' \item{\code{squish}}{ - Fill image canvas, ignoring aspect, such that both the target width and height is achieved.}
#' \item{\code{expand}}{ - Fill image canvas, by scaling image and preserving aspect, such that when cropped, the target width and height is achieved.}
#' \item{\code{none  }}{ - Fill image canvas canvas to the target width/height, and use gravity to place the undistorted image}
#' \item{\code{tile  }}{ - Tile the image canvas with image.}
#' }
#'
#' @param img magick image
#' @param width,height target dimensions for final image
#' @param type how to resize the image. 'none', 'fit', 'squish', 'expand' or 'tile'. default: squish
#' @param gravity how to anchor the image during the operation. See \code{magick::gravity_types()}
#' @param filter magick filter type. default: lanczos. See \code{magick::filter_types()}
#'        for more information.  This option has no effect for \code{type = 'none'}.
#' @param scale scale the source image before tiling. default: 1
#' @param filter default: lanczos
#'
#' @return magick image of the required dimensions
#'
#' @examples
#' try({
#'   filename <- system.file("img", "Rlogo.png", package="png")
#'   img <- magick::image_read(filename)
#'   fill_area_with_img(img, 100, 400, type = 'squish')
#' })
#' @noRd
fill_area_with_img <- function(
	img,
	width,
	height,
	type = 'squish',
	gravity = 'Center',
	filter = 'lanczos',
	scale = 1
) {
	if (
		length(width) != 1 ||
			length(height) != 1 ||
			is.na(width) ||
			is.na(height) ||
			width <= 0 ||
			height <= 0
	) {
		return(img)
	}

	type <- check_default(type, fill_types)

	switch(
		type,
		fit = fill_area_with_img_fit(img, width, height, gravity = gravity, filter = filter),
		expand = fill_area_with_img_expand(img, width, height, gravity = gravity, filter = filter),
		squish = fill_area_with_img_squish(img, width, height, filter = filter),
		none = fill_area_with_img_none(
			img,
			width,
			height,
			gravity = gravity,
			filter = filter,
			scale = scale
		),
		tile = fill_area_with_img_tile(
			img,
			width,
			height,
			gravity = gravity,
			filter = filter,
			scale = scale
		),
		{
			warn("fill_area_with_img(): resize not understood: '", type, "', using 'squish'")
			fill_area_with_img_squish(img, width, height)
		}
	)
}

#' Placeholder image patterned grobs
#'
#' `grid.pattern_placeholder()` draws a placeholder image pattern onto the graphic device.
#' `names_placeholder` are character vectors of supported placeholder types.
#'
#' @inheritParams grid.pattern_plasma
#' @param type Image source.  `names_placeholder` is a vector of supported values.
#'             If you would like only greyscale images append `bw` to the name.
#' @return A grid grob object invisibly.  If `draw` is `TRUE` then also draws to the graphic device as a side effect.
#' @examples
#'   if (requireNamespace("magick")) {
#'     x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'     # requires internet connection to download from placeholder image websites
#'     try(grid.pattern_placeholder(x_hex, y_hex, type="bear"))
#'   }
#'
#'   print(names_placeholder)
#' @seealso [reset_image_cache()] resets the image cache used by [grid.pattern_image()] and `grid.pattern_placeholder()`.
#' @export
grid.pattern_placeholder <- function(
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	type = "bear",
	alpha = gp$alpha %||% NA_real_,
	aspect_ratio = 1,
	key_scale_factor = 1,
	res = getOption("ggpattern_res", 72),
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grid.pattern(
		"placeholder",
		x,
		y,
		id,
		type = type,
		alpha = alpha,
		aspect_ratio = aspect_ratio,
		key_scale_factor = key_scale_factor,
		res = res,
		default.units = default.units,
		name = name,
		gp = gp,
		draw = draw,
		vp = vp
	)
}

## All placeholder names

#' @rdname grid.pattern_placeholder
#' @export
names_placeholder <- c(
	"bear",
	"bearbw",
	"beard",
	"beardbw",
	"cage",
	"cagebw",
	"dummy",
	"dummybw",
	"flickr",
	"flickrbw",
	"keanu",
	"keanubw",
	"kitten",
	"kittenbw",
	"murray",
	"murraybw",
	"picsum",
	"picsumbw",
	"placeholder",
	"placeholderbw",
	"seagal",
	"seagalbw"
)

#' Fetch a placeholder image of the correct dimensions
#'
#' Fetch a placeholder image from one of the sites used by web designers.  Append
#' 'bw' to the name to fetch greyscale images instead of colour.
#'
#' \describe{
#'   \item{`bear`}{ - Bears from \url{https://placebear.com/}}
#'   \item{`beard`}{ - Beards! \url{https://placebeard.it/}}
#'   \item{`cage`}{ - Nicholas Cage images from \url{https://placecage.lucidinternets.com}}
#'   \item{`dummy`}{ - Numeric placeholder images from \url{https://dummyimage.com}}
#'   \item{`flickr`}{ - Images from Flickr \url{https://loremflickr.com/}}
#'   \item{`keanu`}{ - Keanu Reeves images from \url{https://placekeanu.com}}
#'   \item{`kitten`}{ - Kittens from \url{https://placecats.com/}}
#'   \item{`murray`}{ - Bill Murrary images from \url{https://fillmurray.lucidinternets.com}}
#'   \item{`picsum`}{ - Images from \url{https://picsum.photos/}}
#'   \item{`placeholder`}{ - Numeric placeholder images from \url{placehold.co}}
#'   \item{`seagal`}{ - Steven Seagal images from \url{https://stevensegallery.lucidinternets.com/}}
#' }
#'
#' @param width,height image dimensions
#' @param type specify the server from which to fetch images. default: 'kitten'
#'
#' @noRd
fetch_placeholder_img <- function(width = 100, height = 100, type = 'dummy') {
	width <- as.integer(width)
	height <- as.integer(height)

	img_url <- switch(
		type,
		bear = glue("https://placebear.com/{width}/{height}"),
		bearbw = glue("https://placebear.com/g/{width}/{height}"),
		beard = glue("https://placebeard.it/{width}/{height}"),
		beardbw = glue("https://placebeard.it/g/{width}/{height}"),
		cage = glue("https://placecage.lucidinternets.com/{width}/{height}"),
		cagebw = glue("https://placecage.lucidinternets.com/g/{width}/{height}"),
		dummy = glue("https://dummyimage.com/{width}x{height}"),
		dummybw = glue("https://dummyimage.com/{width}x{height}/fff/000"),
		flickr = glue("https://loremflickr.com/{width}/{height}"),
		flickrbw = glue("https://loremflickr.com/g/{width}/{height}/all"),
		keanu = glue("https://placekeanu.com/{width}/{height}"),
		keanubw = glue("https://placekeanu.com/{width}/{height}/g"),
		kitten = glue("https://placecats.com/{width}/{height}"),
		kittenbw = glue("https://placecats.com/g/{width}/{height}"),
		murray = glue("https://fillmurray.lucidinternets.com/{width}/{height}"),
		murraybw = glue("https://fillmurray.lucidinternets.com/g/{width}/{height}"),
		picsum = glue("https://picsum.photos/{width}/{height}"),
		picsumbw = glue("https://picsum.photos/{width}/{height}?grayscale"),
		placeholderbw = glue("https://placehold.co/{width}x{height}/000000/FFFFFF/png"),
		placeholder = glue("https://placehold.co/{width}x{height}/png"),
		seagal = glue("https://www.stevensegallery.lucidinternets.com/{width}/{height}"),
		seagalbw = glue("https://www.stevensegallery.lucidinternets.com/g/{width}/{height}"),
		{
			warn(glue("Unknown placeholder type '{type}' using 'bear' instead"))
			glue("https://dummyimage.com/{width}x{height}")
		}
	)

	img_read_memoised(filename = img_url)
}

#' Fetch a placeholder image of the correct size and return as an array
#'
#' @inheritParams create_gradient_as_array
#'
#' @return RGBA array
#'
#' @noRd
fetch_placeholder_array <- function(width, height, params, legend) {
	assert_suggested("magick", "placeholder")

	if (legend) {
		img <- magick::image_blank(width, height)
		return(convert_img_to_array(img))
	}

	placeholder_type <- check_default(
		as.character(params$pattern_type),
		default = 'kitten',
		type = 'char'
	)
	img <- fetch_placeholder_img(width = width, height = height, type = placeholder_type)

	convert_img_to_array(img)
}

#' Reset 'gridpattern' image cache
#'
#' [grid.pattern_image()] and [grid.pattern_placeholder()] store images in a cache
#' (so we won't download image URLs over and over).
#' `reset_image_cache()` resets this cache.
#' @export
reset_image_cache <- function() {
	memoise::forget(img_read_memoised)
}

#' Create patterned grobs by pattern name
#'
#' `grid.pattern()` draws patterned shapes onto the graphic device.
#' `patternGrob()` returns the grid grob objects.
#' `names_pattern` is a character vector of builtin patterns.
#'
#' Here is a list of the various patterns supported:
#'
#' \describe{
#' \item{ambient}{Noise array patterns onto the graphic device powered by the `ambient` package.
#'                See [grid.pattern_ambient()] for more information.}
#' \item{aRtsy}{Patterns powered by the `aRtsy` package.
#'              See [grid.pattern_aRtsy()] for more information.}
#' \item{circle}{Circle geometry patterns.
#'               See [grid.pattern_circle()] for more information.}
#' \item{crosshatch}{Crosshatch geometry patterns.
#'                   See [grid.pattern_crosshatch()] for more information.}
#' \item{gradient}{Gradient array/geometry patterns.
#'                 See [grid.pattern_gradient()] for more information.}
#' \item{hatch}{Heraldic hatching patterns.
#'              See [grid.pattern_hatch()] for more information.}
#' \item{image}{Image array patterns.
#'              See [grid.pattern_image()] for more information.}
#' \item{line}{Line geometry patterns.
#'             See [grid.pattern_line()] for more information.}
#' \item{magick}{`imagemagick` array patterns.
#'               See [grid.pattern_magick()] for more information.}
#' \item{none}{Does nothing.
#'             See [grid::grid.null()] for more information.}
#' \item{pch}{Plotting character geometry patterns.
#'            See [grid.pattern_pch()] for more information.}
#' \item{placeholder}{Placeholder image array patterns.
#'                    See [grid.pattern_placeholder()] for more information.}
#' \item{plasma}{Plasma array patterns.
#'               See [grid.pattern_plasma()] for more information.}
#' \item{polygon_tiling}{Polygon tiling patterns.
#'                        See [grid.pattern_polygon_tiling()] for more information.}
#' \item{regular_polygon}{Regular polygon patterns.
#'                        See [grid.pattern_regular_polygon()] for more information.}
#' \item{rose}{Rose array/geometry patterns.
#'             See [grid.pattern_rose()] for more information.}
#' \item{stripe}{Stripe geometry patterns.
#'               See [grid.pattern_stripe()] for more information.}
#' \item{text}{Text array/geometry patterns.
#'             See [grid.pattern_text()] for more information.}
#' \item{wave}{Wave geometry patterns.
#'               See [grid.pattern_wave()] for more information.}
#' \item{weave}{Weave geometry patterns.
#'               See [grid.pattern_weave()] for more information.}
#' \item{Custom geometry-based patterns}{See \url{https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html} for more information.}
#' \item{Custom array-based patterns}{See \url{https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html} for more information.}
#' }
#'
#' @inheritParams grid::polygonGrob
#' @param pattern Name of pattern.  See Details section for a list of supported patterns.
#' @param x A numeric vector or unit object specifying x-locations of the pattern boundary.
#' @param y A numeric vector or unit object specifying y-locations of the pattern boundary.
#' @param id A numeric vector used to separate locations in x, y into multiple boundaries.
#'           All locations within the same `id` belong to the same boundary.
#' @param ... Pattern parameters.
#' @param legend Whether this is intended to be drawn in a legend or not.
#' @param prefix Prefix to prepend to the name of each of the pattern parameters in `...`.
#'               For compatibility with `ggpattern` most underlying functions assume parameters beginning with `pattern_`.
#' @param default.units A string indicating the default units to use if `x` or `y`
#'                      are only given as numeric vectors.
#' @return A grid grob object (invisibly in the case of `grid.pattern()`).
#'         If `draw` is `TRUE` then `grid.pattern()` also draws to the graphic device as a side effect.
#' @examples
#'  print(names_pattern)
#'  \donttest{# May take more than 5 seconds on CRAN servers
#'  x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'  y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
#'
#'  # geometry-based patterns
#'  # 'stripe' pattern
#'  grid::grid.newpage()
#'  grid.pattern("stripe", x_hex, y_hex,
#'               colour="black", fill=c("yellow", "blue"), density = 0.5)
#'
#'  # Can alternatively use "gpar()" to specify colour and line attributes
#'  grid::grid.newpage()
#'  grid.pattern("stripe", x_hex, y_hex,
#'               gp = grid::gpar(col="blue", fill="red", lwd=2))
#'
#'  # 'weave' pattern
#'  grid::grid.newpage()
#'  grid.pattern("weave", x_hex, y_hex, type = "satin",
#'               colour = "black", fill = "lightblue", fill2 =  "yellow",
#'               density = 0.3)
#'
#'  # 'regular_polygon' pattern
#'  grid::grid.newpage()
#'  grid.pattern_regular_polygon(x_hex, y_hex, colour = "black",
#'                               fill = c("blue", "yellow", "red"),
#'                               shape = c("convex4", "star8", "circle"),
#'                               density = c(0.45, 0.42, 0.4),
#'                               spacing = 0.08, angle = 0)
#'
#'  # can be used to achieve a variety of 'tiling' effects
#'  grid::grid.newpage()
#'  grid.pattern_regular_polygon(x_hex, y_hex, color = "transparent",
#'                               fill = c("white", "grey", "black"),
#'                               density = 1.0, spacing = 0.1,
#'                               shape = "convex6", grid = "hex")
#'  if (suppressPackageStartupMessages(requireNamespace("magick", quietly = TRUE))) {
#'    # array-based patterns
#'    # 'image' pattern
#'    logo_filename <- system.file("img", "Rlogo.png" , package="png")
#'    grid::grid.newpage()
#'    grid.pattern("image", x_hex, y_hex, filename=logo_filename, type="fit")
#'  }
#'  if (suppressPackageStartupMessages(requireNamespace("magick", quietly = TRUE))) {
#'    # 'plasma' pattern
#'    grid::grid.newpage()
#'    grid.pattern("plasma", x_hex, y_hex, fill="green")
#'  }
#'  }
#' @seealso \url{https://coolbutuseless.github.io/package/ggpattern/index.html}
#'          for more details on the `ggpattern` package.
#' @export
grid.pattern <- function(
	pattern = "stripe",
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	legend = FALSE,
	prefix = "pattern_",
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	grob <- patternGrob(
		pattern,
		x,
		y,
		id,
		...,
		legend = legend,
		prefix = prefix,
		default.units = default.units,
		name = name,
		gp = gp,
		vp = vp
	)
	if (draw) {
		grid.draw(grob)
	}
	invisible(grob)
}

#' @rdname grid.pattern
#' @export
names_pattern <- c(
	"ambient",
	"aRtsy",
	"circle",
	"crosshatch",
	"fill",
	"gradient",
	"hatch",
	"image",
	"line",
	"magick",
	"none",
	"pch",
	"placeholder",
	"plasma",
	"polygon_tiling",
	"regular_polygon",
	"rose",
	"stripe",
	"text",
	"wave",
	"weave"
)

#' @rdname grid.pattern
#' @export
patternGrob <- function(
	pattern = "stripe",
	x = c(0, 0, 1, 1),
	y = c(1, 0, 0, 1),
	id = 1L,
	...,
	legend = FALSE,
	prefix = "pattern_",
	default.units = "npc",
	name = NULL,
	gp = gpar(),
	draw = TRUE,
	vp = NULL
) {
	params <- get_params(..., pattern = pattern, prefix = prefix, gp = gp)
	if (!inherits(x, "unit")) {
		x <- unit(x, default.units)
	}
	if (!inherits(y, "unit")) {
		y <- unit(y, default.units)
	}

	gTree(
		pattern = pattern,
		x = x,
		y = y,
		id = id,
		params = params,
		legend = legend,
		name = name,
		gp = gp,
		vp = vp,
		cl = "pattern"
	)
}

#' @export
makeContent.pattern <- function(x) {
	# avoid weird errors with array patterns if there is an active device open
	current_dev <- grDevices::dev.cur()
	on.exit(grDevices::dev.set(current_dev))

	xp <- convertX(x$x, "npc", valueOnly = TRUE)
	yp <- convertY(x$y, "npc", valueOnly = TRUE)
	id <- x$id
	boundary_df <- create_polygon_df(xp, yp, id)

	if (!is.na(x$params$pattern_aspect_ratio)) {
		aspect_ratio <- x$params$pattern_aspect_ratio
	} else {
		width <- convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)
		height <- convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
		aspect_ratio <- width / height
	}

	# needs to be called within active graphics device to guess R4.1 capabilities
	params <- get_R4.1_params(x$params)

	fn <- get_pattern_fn(x$pattern)
	grob <- fn(params, boundary_df, aspect_ratio, x$legend)
	gl <- gList(grob)
	setChildren(x, gl)
}

get_pattern_fn <- function(pattern) {
	user_geometry_fns <- getOption("ggpattern_geometry_funcs")
	user_array_fns <- getOption("ggpattern_array_funcs")
	assert_patterns_unique(user_geometry_fns, user_array_fns)
	geometry_fns <- c(
		list(
			aRtsy = create_pattern_aRtsy,
			circle = create_pattern_circle_via_sf,
			crosshatch = create_pattern_crosshatch_via_sf,
			fill = create_pattern_fill,
			gradient = create_pattern_gradient,
			hatch = create_pattern_hatch,
			line = create_pattern_line,
			none = create_pattern_none,
			pch = create_pattern_pch,
			polygon_tiling = create_pattern_polygon_tiling,
			regular_polygon = create_pattern_regular_polygon_via_sf,
			rose = create_pattern_rose,
			stripe = create_pattern_stripes_via_sf,
			text = create_pattern_text,
			wave = create_pattern_wave_via_sf,
			weave = create_pattern_weave_via_sf
		),
		user_geometry_fns
	)
	array_fns <- c(
		list(
			ambient = create_pattern_ambient,
			image = img_read_as_array_wrapper,
			magick = create_magick_pattern_as_array,
			placeholder = fetch_placeholder_array,
			plasma = create_magick_plasma_as_array
		),
		user_array_fns
	)
	array_fns <- lapply(array_fns, function(fn) {
		function(...) create_pattern_array(..., array_fn = fn)
	})
	fns <- c(geometry_fns, array_fns)
	fns[[pattern]] %||% abort(paste("Don't know the function for pattern", pattern))
}

assert_patterns_unique <- function(user_geometry_fns, user_array_fns) {
	names_geometry <- names(user_geometry_fns)
	names_array <- names(user_array_fns)
	msg_geometry <- '`options("ggpattern_geometry_funcs")` sets custom "geometry" patterns'
	msg_array <- '`options("ggpattern_array_funcs")` sets custom "array" patterns'
	# check pattern names not duplicated within custom pattern types
	duplicated_geometry <- duplicated(names_geometry)
	if (any(duplicated_geometry)) {
		name <- names_geometry[which(duplicated_geometry)[1]]
		msg <- c(
			glue('There are multiple custom "geometry" patterns named "{name}"'),
			i = msg_geometry
		)
		abort(msg)
	}
	duplicated_array <- duplicated(names_array)
	if (any(duplicated_array)) {
		name <- names_array[which(duplicated_array)[1]]
		msg <- c(glue('There are multiple custom "array" patterns named "{name}"'), i = msg_array)
		abort(msg)
	}
	# check pattern names not duplicated between custom pattern types
	match_user <- match(names_geometry, names_array)
	if (any(!is.na(match_user))) {
		index <- which(!is.na(match_user))[1]
		name <- names_geometry[index]
		msg <- c(
			glue(
				'There is a custom "geometry" pattern and custom "array" pattern both named "{name}"'
			),
			i = msg_geometry,
			i = msg_array
		)
		abort(msg)
	}
	# check pattern names not duplicated between custom patterns and builtin patterns
	match_geometry <- match(names_geometry, names_pattern)
	if (any(!is.na(match_geometry))) {
		index <- which(!is.na(match_geometry))[1]
		name <- names_geometry[index]
		msg <- c(
			glue(
				'There is a custom "geometry" pattern and builtin {{gridpattern}} pattern both named "{name}"'
			),
			i = msg_geometry
		)
		abort(msg)
	}
	match_array <- match(names_array, names_pattern)
	if (any(!is.na(match_array))) {
		index <- which(!is.na(match_array))[1]
		name <- names_array[index]
		msg <- c(
			glue(
				'There is a custom "array" pattern and builtin {{gridpattern}} pattern both named "{name}"'
			),
			i = msg_array
		)
		abort(msg)
	}
	invisible(NULL)
}

#' Create a \code{polygon_df} object from the given coordinates
#'
#' code using \code{polygon_df} should not assume that the first and last point
#' within each id are the same.  i.e. they may have to manually set a final
#' point equal to the initial point if that is what their graphics system
#' desires
#'
#' @param x,y coordinates of polygon. not necessarily closed.
#' @param id a numeric vector used to separate locations in x,y into multiple polygons
#'
#' @return data.frame with x, y, id columns.
#'
#' @noRd
create_polygon_df <- function(x, y, id = 1L) {
	data_frame(
		x = x,
		y = y,
		id = id
	)
}

# Convert units from 'npc' to another {grid} unit
convert_polygon_df_units <- function(df, units = "bigpts") {
	df$x <- convertX(unit(df$x, "npc"), units, valueOnly = TRUE)
	df$y <- convertY(unit(df$y, "npc"), units, valueOnly = TRUE)
	df
}

#' Convert a \code{polygon_df} to \code{grid::polygonGrob} object
#'
#' @param polygon_df polygon_df data.frame
#' @param default.units 'npc
#' @param gp default: gpar()
#'
#' @return sf polygon object
#'
#' @noRd
convert_polygon_df_to_polygon_grob <- function(polygon_df, default.units = 'npc', gp = gpar()) {
	if (is.null(polygon_df) || nrow(polygon_df) < 3) {
		return(grid::nullGrob())
	}

	grid::polygonGrob(
		x = polygon_df$x,
		y = polygon_df$y,
		id = polygon_df$id,
		default.units = default.units,
		gp = gp
	)
}

#' Convert a \code{polygon_df} to an \code{sf} POLYGON/MULTIPOLYGON
#'
#' @param polygon_df Polygon data.frame
#' @param buffer_dist buffer the polygon by the given distance
#'
#' @return sf polygon object
#' @noRd
convert_polygon_df_to_polygon_sf <- function(polygon_df, buffer_dist = 0) {
	if (is.null(polygon_df) || nrow(polygon_df) < 3 || anyNA(polygon_df$x) || anyNA(polygon_df$y)) {
		return(sf::st_polygon())
	}

	polys <- split(polygon_df, polygon_df$id)

	create_coords <- function(poly) {
		xs <- poly$x
		ys <- poly$y

		# {sf} wants explicitly closed polygons, so set the last point
		# to be the same as the first
		if (xs[1] != tail(xs, 1) || ys[1] != tail(ys, 1)) {
			xs <- c(xs, xs[1])
			ys <- c(ys, ys[1])
		}

		list(cbind(xs, ys))
	}

	all_coords <- lapply(polys, create_coords)

	res <- sf::st_multipolygon(all_coords)

	# perform a zero-buffer operation to remove self-intersection
	# As suggested here:
	#  - https://gis.stackexchange.com/questions/163445/getting-topologyexception-input-geom-1-is-invalid-which-is-due-to-self-intersec#163480
	# https://gis.stackexchange.com/questions/223252/how-to-overcome-invalid-input-geom-and-self-intersection-when-intersecting-shape
	res <- sf::st_buffer(res, buffer_dist)
	res
}

#' Convert a sf GEOMETRYCOLLECTION/POLYGON/MULTIPOLYGON into a polygon_df
#'
#' @param mp sf GEOMETRYCOLLECTION, POLYGON, or MULTIPOLYGON object
#'
#' @return polygon_df data.frame object
#'
#' @noRd
convert_polygon_sf_to_polygon_df <- function(mp) {
	mat <- as.matrix(mp)
	if (sf::st_is_empty(mp)) {
		return(mat)
	}
	if (
		!inherits(
			mp,
			c(
				'POLYGON',
				'MULTIPOLYGON',
				'GEOMETRYCOLLECTION',
				'MULTILINESTRING',
				'MULTIPOINT',
				'LINESTRING',
				'POINT'
			)
		)
	) {
		warn(paste(
			"convert_polygon_sf_to_polygon_df():",
			"Not GEOMETRYCOLLECTION, POLYGON, or MULTIPOLYGON:",
			deparse(class(mp))
		))
	}
	if (!inherits(mp, c('POLYGON', 'MULTIPOLYGON', 'GEOMETRYCOLLECTION'))) {
		return(NULL)
	}

	poly_lengths <- get_poly_lengths(mp)
	id <- rep.int(seq_along(poly_lengths), times = poly_lengths)

	create_polygon_df(x = mat[, 1], y = mat[, 2], id = id)
}

get_poly_lengths <- function(sf_object) {
	if (inherits(sf_object, c('POLYGON', 'LINESTRING', 'POINT'))) {
		nrow(as.matrix(sf_object))
	} else if (inherits(sf_object, c('MULTIPOLYGON', 'MULTILINESTRING', 'MULTIPOINT'))) {
		if (max(lengths(sf_object)) > 1L) {
			abort("There is a MULTIPOLYGON/MULTILINESTRING/MULTIPOINT with length greater than 1")
		}
		vapply(sf_object, function(x) nrow(as.matrix(x[[1]])), integer(1))
	} else if (inherits(sf_object, 'GEOMETRYCOLLECTION')) {
		vapply(sf_object, get_poly_lengths, integer(1))
	} else {
		abort(paste(
			"convert_polygon_sf_to_polygon_df():",
			"Not GEOMETRYCOLLECTION, POLYGON, or MULTIPOLYGON:",
			deparse(class(sf_object))
		))
	}
}

#' Convert a polygon to an alpha mask
#'
#' Alpha values of 1 correspond to pixels within the polygon, all other
#' values will be zero.
#'
#' @param polygon_df polygon_df object
#' @param width,height dimensions of the alpha channel matrix to return
#'
#' @return numeric matrix with all values with 0 or 1
#'
#' @noRd
convert_polygon_df_to_alpha_channel <- function(polygon_df, width, height) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Convert the polygon to an actual grob, coloured 'black'
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	gp <- gpar(fill = 'black')
	boundary_grob <- convert_polygon_df_to_polygon_grob(polygon_df, gp = gp)

	# Note `ragg::agg_capture()`'s non-"native" format is a matrix of color strings
	# while `png::readPNG()`'s non-"native" format is an array of numeric values
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Save the grob as an image of the given size
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	png_file <- tempfile(fileext = ".png")
	png(png_file, width = width, height = height)
	grid.draw(boundary_grob)
	dev.off()

	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	# Load the file and convert o a numeric matrix with values 0/1 depending
	# on whether the pixel is white or black.
	#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	alpha_channel <- png::readPNG(png_file, native = FALSE)
	alpha_channel <- alpha_channel[,, 1] < 0.5
	storage.mode(alpha_channel) <- 'numeric'

	# t(alpha_channel)
	alpha_channel
}

sf_multipolygon_to_polygon_grob <- function(
	multipolygons_sf,
	gp = gpar(),
	default.units = "npc",
	name = NULL
) {
	df <- convert_polygon_sf_to_polygon_df(multipolygons_sf)
	if (is.null(df)) {
		nullGrob()
	} else {
		polygonGrob(
			x = df$x,
			y = df$y,
			id = df$id,
			default.units = default.units,
			gp = gp,
			name = name
		)
	}
}

# build a circle of radius `r` centered on each point in `sf_points`
sf_points_to_circle_grob <- function(
	sf_points,
	r,
	gp = gpar(),
	default.units = "npc",
	name = NULL
) {
	points_mat <- as.matrix(sf_points)
	if (is.null(points_mat) || nrow(points_mat) == 0) {
		nullGrob()
	} else {
		circleGrob(
			x = points_mat[, 1],
			y = points_mat[, 2],
			r = r,
			default.units = default.units,
			gp = gp,
			name = name
		)
	}
}

# `xy_polygon` has `x` and `y` elements which will be added to each point in `sf_points`
sf_points_to_polygon_grob <- function(
	sf_points,
	xy_polygon,
	gp = gpar(),
	default.units = "npc",
	name = NULL
) {
	points_mat <- as.matrix(sf_points)
	df_polygon <- as.data.frame(xy_polygon)
	l_xy <- lapply(seq(nrow(points_mat)), function(i_r) {
		x0 <- points_mat[i_r, 1]
		y0 <- points_mat[i_r, 2]
		df <- df_polygon
		df$x <- df$x + x0
		df$y <- df$y + y0
		df
	})
	df <- do.call(rbind, l_xy)
	if (is.null(df)) {
		nullGrob()
	} else {
		df$id <- rep(seq(nrow(points_mat)), each = nrow(df_polygon))
		polygonGrob(
			x = df$x,
			y = df$y,
			id = df$id,
			default.units = default.units,
			gp = gp,
			name = name
		)
	}
}

# `xy_polygon` has `x` and `y` elements which will be added to each point in `sf_points`
sf_points_to_sf_multipolygon <- function(sf_points, xy_polygon) {
	points_mat <- as.matrix(sf_points)
	df_polygon <- as.data.frame(xy_polygon)
	df_polygon <- rbind(df_polygon, df_polygon[1L, ])
	l_xy <- lapply(seq(nrow(points_mat)), function(i_r) {
		x0 <- points_mat[i_r, 1]
		y0 <- points_mat[i_r, 2]
		df <- df_polygon
		df$x <- df$x + x0
		df$y <- df$y + y0
		list(as.matrix(df))
	})
	sf::st_multipolygon(l_xy)
}

#' Create patterned fills by pattern name
#'
#' `patternFill()` returns [grid::pattern()] fill objects.
#' It is a wrapper around [patternGrob()].
#'
#' @param ... Passed to [patternGrob()].
#' @param x,y,width,height  The size of the [grid::pattern()] tile.
#' @param default.units  The default [grid::unit()] unit to use for `x`, `y`, `width`, and `height`.
#' @param just,hjust,vjust  The justification of the tile relative to its location.
#' @param group A logical indicating whether the pattern is relative to the bounding box of the grob or whether it is relative to individual shapes within the grob.  Ignored if R is less than version 4.2.
#' @examples
#' if (guess_has_R4.1_features("patterns") &&
#'     require("grid", quietly = TRUE)) {
#'   grid.newpage()
#'   stripe_fill <- patternFill("stripe", fill = c("red", "blue"))
#'   grid.circle(gp = gpar(fill = stripe_fill))
#' }
#'
#' if (guess_has_R4.1_features("patterns") &&
#'     require("ggplot2", quietly = TRUE) &&
#'     (getRversion() >= "4.2")) {
#'   grid.newpage()
#'   weave_fill <- patternFill("weave", fill = "red", fill2 = "blue",
#'                             colour = "transparent")
#'   hex_fill <- patternFill("polygon_tiling", type = "hexagonal",
#'                           fill = c("black", "white", "grey"),
#'                           colour = "transparent")
#'   df <- data.frame(trt = c("a", "b"), outcome = c(1.9, 3.2))
#'   gg <- ggplot(df, aes(trt, outcome)) +
#'     geom_col(fill = list(weave_fill, hex_fill))
#'   plot(gg)
#' }
#' @return A [grid::pattern()] fill object.
#' @export
patternFill <- function(
	...,
	x = 0.5,
	y = 0.5,
	width = 1,
	height = 1,
	default.units = "npc",
	just = "centre",
	hjust = NULL,
	vjust = NULL,
	group = TRUE
) {
	stopifnot(getRversion() >= "4.1.0")
	args <- list(
		grob = patternGrob(...),
		x = x,
		y = y,
		width = width,
		height = height,
		default.units = default.units,
		just = just,
		hjust = hjust,
		vjust = vjust
	)
	# `group` was introduced in R 4.2
	if (getRversion() >= "4.2.0") {
		args$group <- group
	}
	do.call(grid::pattern, args)
}

#' Clip grob using another grob to specify the clipping path
#'
#' `clippingPathGrob()` clips a grob using another grob to specify the clipping path
#'
#' @param clippee Grob to be clipped
#' @param clipper Grob that defines clipping region
#' @param use_R4.1_clipping If `TRUE` use the grid clipping path feature introduced in R v4.1.0.
#'                          If `FALSE` do a `rasterGrob` approximation.
#'                          If `NULL` try to guess an appropriate choice.
#'                          Note not all graphic devices support the grid clipping path feature
#'                          and the grid clipping path feature does not nest.
#' @param png_device \dQuote{png} graphics device to save intermediate raster data with if `use_R4.1_clipping` is `FALSE`.
#'                   If `NULL` and suggested package `ragg` is available
#'                   and versions are high enough we directly capture clipped raster via [ragg::agg_capture()].
#'                   Otherwise we will use `png_device`
#'                   (default [ragg::agg_png()] if available else [grDevices::png()]) and [png::readPNG()]
#'                   to manually compute a clipped raster.
#' @param res Resolution of desired `rasterGrob` in pixels per inch if `use_R4.1_clipping` is `FALSE`.
#' @return A `grid` grob
#' @inheritParams grid::polygonGrob
#' @examples
#'   if (capabilities("png") && require("grid")) {
#'     clippee <- patternGrob("circle", gp = gpar(col = "black", fill = "yellow"),
#'                            spacing = 0.1, density = 0.5)
#'     angle <- seq(2 * pi / 4, by = 2 * pi / 6, length.out = 7)
#'     x_hex_outer <- 0.5 + 0.5 * cos(angle)
#'     y_hex_outer <- 0.5 + 0.5 * sin(angle)
#'     x_hex_inner <- 0.5 + 0.25 * cos(rev(angle))
#'     y_hex_inner <- 0.5 + 0.25 * sin(rev(angle))
#'     clipper <- grid::pathGrob(x = c(x_hex_outer, x_hex_inner),
#'                               y = c(y_hex_outer, y_hex_inner),
#'                               id = rep(1:2, each = 7),
#'                               rule = "evenodd")
#'     clipped <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = FALSE)
#'     grid.newpage()
#'     grid.draw(clipped)
#'   }
#' @export
clippingPathGrob <- function(
	clippee,
	clipper,
	use_R4.1_clipping = getOption(
		"ggpattern_use_R4.1_clipping",
		getOption("ggpattern_use_R4.1_features")
	),
	png_device = NULL,
	res = getOption("ggpattern_res", 72),
	name = NULL,
	gp = gpar(),
	vp = NULL
) {
	gTree(
		clippee = clippee,
		clipper = clipper,
		use_R4.1_clipping = use_R4.1_clipping,
		res = res,
		png_device = png_device,
		name = name,
		gp = gp,
		vp = vp,
		cl = "clipping_path"
	)
}

#' @export
makeContent.clipping_path <- function(x) {
	current_dev <- grDevices::dev.cur()
	on.exit(grDevices::dev.set(current_dev))

	use_R4.1_clipping <- x$use_R4.1_clipping
	if (is.null(use_R4.1_clipping)) {
		use_R4.1_clipping <- guess_has_R4.1_features("clippingPaths")
	} else {
		use_R4.1_clipping <- as.logical(use_R4.1_clipping)
	}

	stopifnot(getRversion() >= '4.1.0' || !use_R4.1_clipping)

	if (use_R4.1_clipping) {
		grob <- grobTree(x$clippee, vp = viewport(clip = x$clipper), name = "clip")
	} else if (
		is.null(x$png_device) &&
			getRversion() >= '4.1.0' &&
			requireNamespace("ragg", quietly = TRUE) &&
			packageVersion("ragg") >= '1.2.0'
	) {
		grob <- gridpattern_clip_agg_capture(x$clippee, x$clipper, x$res)
	} else {
		png_device <- x$png_device %||% default_png_device()
		if (device_supports_clipping(png_device)) {
			grob <- gridpattern_clip_raster_straight(x$clippee, x$clipper, x$res, png_device)
		} else {
			grob <- gridpattern_clip_raster_manual(x$clippee, x$clipper, x$res, png_device)
		}
	}

	gl <- gList(grob)
	setChildren(x, gl)
}

device_supports_clipping <- function(png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	png_file <- tempfile(fileext = ".png")
	on.exit(unlink(png_file), add = TRUE)
	png_device(png_file)
	value <- guess_has_R4.1_features("clippingPaths")
	dev.off()
	value
}

gridpattern_clip_agg_capture <- function(clippee, clipper, res) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	ragg::agg_capture(height = height, width = width, res = res, bg = "transparent")
	grob <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = TRUE)
	grid.draw(grob)
	raster_clipped <- dev.capture(native = FALSE)
	dev.off()
	grid::rasterGrob(raster_clipped)
}

gridpattern_clip_raster_straight <- function(clippee, clipper, res, png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	png_clipped <- tempfile(fileext = ".png")
	on.exit(unlink(png_clipped), add = TRUE)
	png_device(png_clipped, height = height, width = width, res = res, bg = "transparent")
	grob <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = TRUE)
	grid.draw(grob)
	dev.off()

	raster_clipped <- png::readPNG(png_clipped, native = FALSE)
	grid::rasterGrob(raster_clipped)
}

gridpattern_clip_raster_manual <- function(clippee, clipper, res, png_device) {
	current_dev <- grDevices::dev.cur()
	if (current_dev > 1) {
		on.exit(grDevices::dev.set(current_dev))
	}
	height <- res * convertHeight(unit(1, "npc"), "in", valueOnly = TRUE)
	width <- res * convertWidth(unit(1, "npc"), "in", valueOnly = TRUE)

	png_clippee <- tempfile(fileext = ".png")
	on.exit(unlink(png_clippee), add = TRUE)
	png_device(png_clippee, height = height, width = width, res = res, bg = "transparent")
	grid.draw(clippee)
	dev.off()

	png_clipper <- tempfile(fileext = ".png")
	on.exit(unlink(png_clipper), add = TRUE)
	png_device(png_clipper, height = height, width = width, res = res, bg = "transparent")
	pushViewport(viewport(gp = gpar(lwd = 0, col = NA, fill = "black")))
	grid.draw(clipper)
	popViewport()
	dev.off()

	raster_clippee <- png::readPNG(png_clippee, native = FALSE)
	raster_clipper <- png::readPNG(png_clipper, native = FALSE)
	clip_region <- apply(raster_clipper, c(1, 2), function(x) any(x > 0))
	if (length(dim(raster_clippee) == 2)) {
		raster_clippee[!clip_region] <- 0
	} else {
		for (j in seq_len(dim(raster_clippee)[3])) {
			raster_clippee[!clip_region, j] <- 0
		}
	}
	rasterGrob(raster_clippee, name = "clip")
}

---
title: "Polygon Tiling Examples"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{Polygon Tiling Examples}
    %\VignetteEngine{knitr::rmarkdown}
    \usepackage[utf8]{inputenc}
---

### Table of Contents

[Uniform tilings](#uniform)

* [Triangular tiling](#triangle)
* [Square tiling](#square)
* [Hexagonal tiling](#hexagon)
* [Elongated triangular tiling](#elongatedtriangle)
* [Snub square tiling](#snubsquare)
* [Truncated square tiling](#truncatedsquare)
* [Truncated hexagonal tiling](#truncatedhex)
* [Trihexagonal tiling](#trihexagon)
* [Snub trihexagonal tiling](#snubtrihex)
* [Truncated trihexagonal tiling](#truncatedtrihexagon)
* [Rhombitrihexagonal tiling](#rhombitrihexagon)
* [Additional uniform tilings with star polygons](#star)

[Non-uniform tilings](#non)

* [Herringbone tiling](#herringbone)
* [Pythagorean tiling](#pythagorean)
* [Rhombille tiling](#rhombille)
* [Tetrakis square tiling](#tetrakis)
* [Additional tilings using rhombi](#rhombi)

Several [uniform regular polygon tiling patterns](https://en.wikipedia.org/wiki/Uniform_tiling) can be achieved by use of `grid.pattern_regular_polygon()` plus occasionally `grid.polygon()` to set a background color.  This vignette highlights several such tiling patterns plus a couple notable non-uniform tiling patterns.

```{r}
library("grid")
library("gridpattern")
add_borders <- function() {
    pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
    pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
    grid.rect(gp = gpar(fill = NA, col = "white", lwd = 6))
    popViewport()
}
x_sq <- c(0, 0, 1, 1)
y_sq <- c(0, 1, 1, 0)
# colorblind accessible scheme https://jfly.uni-koeln.de/color/
blue <- grDevices::rgb(0.35, 0.70, 0.90)
yellow <- grDevices::rgb(0.95, 0.90, 0.25)
red <- grDevices::rgb(0.80, 0.40, 0.00)
green <- grDevices::rgb(0.00, 0.60, 0.50)
orange <- grDevices::rgb(0.90, 0.60, 0.00)
```

<a name="uniform">Uniform tilings</a>
=====================================

<a name="triangle">Triangular tiling</a>
-----------------------------------------

When we consider the background to be a triangle fill color we can straight-forwardly
achieve several of the [uniform colorings of a triangular tiling](https://en.wikipedia.org/wiki/Triangular_tiling#Uniform_colorings) using the "convex3" shape with the "hex" grid.

```{r, fig.alt = "Four different uniform colorings of a triangular tiling"}
grid.triangular_tiling <- function(...) {
    grid.pattern_regular_polygon(..., shape = "convex3", density = 1, grid = "hex",
                                 spacing = 0.2, angle = 0, rot = 180)
}
# 1 color uniform triangular tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 2 color uniform triangular tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 3 color uniform triangular tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
grid.triangular_tiling(gp = gp)
popViewport()

# 4 color uniform triangular tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(red, blue, green), col = "black")
grid.triangular_tiling(gp = gp)
popViewport()
add_borders()
```

<a name="square">Square tiling</a>
----------------------------------

We can achieve the nine non-staggered [uniform colorings of a square tiling](https://en.wikipedia.org/wiki/Square_tiling#Uniform_colorings) using the "square" shape and the appropriate "type" (and perhaps "subtype").

```{r, fig.alt = "Four different uniform colorings of a square tiling"}
# 2 color uniform square tiling (1212)
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, 
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 2 color uniform square tiling (1122)
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, type = "horizontal",
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 3 color uniform square tiling (1213)
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1,
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
# 4 color uniform square tiling (1234)
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue, green), col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1,
                             spacing = 0.2, angle = 0, gp = gp)
popViewport()
add_borders()
```

<a name="hexagon">Hexagonal tiling</a>
--------------------------------------------

We can achieve all the [uniform colorings of a hexagonal tiling](https://en.wikipedia.org/wiki/Hexagonal_tiling#Uniform_colorings) using the "convex6" shape with the "hex" grid and the appropriate "type".  Replacing the "convex6" shape with an appropriately scaled "star6" shape results in a uniform tiling using star polygons.

```{r, fig.alt = "Four different uniform colorings of a hexagonal tiling"}
grid.hexagonal_tiling <- function(..., shape = "convex6", spacing = 0.2) {
    scale <- star_scale(6, 30)
    grid.pattern_regular_polygon(..., shape = shape, density = 1, grid = "hex",
                                 spacing = spacing, scale = scale, angle = 0)
}
# 1 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# One of several 2 color uniform hexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(red, yellow), col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# 3 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.hexagonal_tiling(gp = gp)
popViewport()

# (4.6_{π/6})^3
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.hexagonal_tiling(shape = "star6", spacing = 0.3, gp = gp)
popViewport()
add_borders()
```

<a name=elongatedtriangle>Elongated triangular tiling</a>
---------------------------------------------------------

We can achieve the [uniform colorings of a elongated triangular tiling](https://en.wikipedia.org/wiki/Elongated_triangular_tiling) using the "elongated_triangle" grid and the "convex4" and "convex3" shapes.

```{r, fig.alt = "Three different uniform colorings of an elongated triangular tiling"}
grid.elongated_triangular_tiling <- function(...) {
    grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
    grid.pattern_regular_polygon(..., shape = rep(c("convex4", "convex3"), each = 2), 
                                 density = rep(c(1.41, 1.15), each = 2), 
                                 grid = "elongated_triangle",
                                 type = "square_tiling", subtype = "3412",
                                 spacing = 0.2, angle = 0, 
                                 rot = rep(c(45, 0), each = 2))
}
# 1 color elongated triangular tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()

# 2 color elongated triangular tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = c(red, red, yellow, yellow), col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()

# 3 color elongated triangular tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = c(red, blue, yellow, yellow), col = "black")
grid.elongated_triangular_tiling(gp = gp)
popViewport()
add_borders()
```

<a name=snubsquare>Snub square tiling</a>
-----------------------------------------

We can achieve the two [uniform colorings of a snub square tiling](https://en.wikipedia.org/wiki/Snub_square_tiling) using a "star4" shape and a "convex4" shape.

```{r, fig.alt = "Three different uniform colorings of a snub square tiling"}
grid.snub_square <- function(..., gp_sq, gp_sq2 = gp_sq, gp_tri = gp_sq) {
    scale_star <- star_scale(4, 90 + 60, external = TRUE)
    grid.polygon(x_sq, y_sq, gp = gp_sq)
    grid.pattern_regular_polygon(shape = "star4", scale = scale_star,
                                 angle = 0, rot = 15, spacing = 0.2,
                                 density = 1.4, gp = gp_tri)
    grid.pattern_regular_polygon(shape = "convex4", scale = scale_star,
                                 angle = 0, rot = 60, spacing = 0.2,
                                 density = scale_star * 1.4, gp = gp_sq2)
}

# 2 color uniform snub square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp_y <- gpar(fill = yellow, col = "black")
gp_r <- gpar(fill = red, col = "black")
grid.snub_square(gp_sq = gp_y, gp_tri = gp_r)
popViewport()

# 3 color uniform snub square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp_b <- gpar(fill = blue, col = "black")
grid.snub_square(gp_sq = gp_r, gp_sq2 = gp_b, gp_tri = gp_y)
popViewport()

# 1 color uniform hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.snub_square(gp_sq = gp_y)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()
```

<a name=truncatedsquare>Truncated square tiling</a>
---------------------------------------------------

We can achieve the two [uniform colorings of a truncated square tiling](https://en.wikipedia.org/wiki/Truncated_square_tiling#Uniform_colorings) using the "convex8" shape rotated 22.5 degrees.  Adding in an extra (possibly alternating) "star8" shape results in a uniform tiling using star polygons.


```{r, fig.alt = "Four different uniform colorings of a truncated square tiling"}
# 2 color uniform truncated square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
popViewport()

# 3 color uniform truncated square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = blue, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
popViewport()

# 3 color uniform truncated square tiling with star
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
gp <- gpar(fill = orange, col = "black")
scale <- star_scale(8, 60, external = TRUE)
grid.pattern_regular_polygon(shape = "star8", density = 1.082, rot = 22.5, 
                             angle = 0, scale = scale, spacing = 0.3, gp = gp)
popViewport()

# 4 color uniform truncated square tiling with an alternating star
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = blue, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex8", density = 1.082, rot = 22.5, 
                             angle = 0, spacing = 0.3, gp = gp)
gp <- gpar(fill = orange, col = c("black"))
grid.pattern_regular_polygon(shape = c("null", "star8"), 
                             density = 1.082, rot = 22.5, 
                             angle = 0, scale = scale, spacing = 0.3, gp = gp)
popViewport()
add_borders()
```

<a name="truncatedhex">Truncated hexagonal tiling</a>
-----------------------------------------------------

We can achieve a [truncated hexagonal tiling](https://en.wikipedia.org/wiki/Truncated_hexagonal_tiling) using the "convex12" shape with a 15 degree rotation in a "hex_circle" grid.  The related "star12" variations are fun tilings as well.

```{r, fig.alt = "A truncated hexagonal tiling and a related 'star12' variation"}
# 2 color uniform truncated hexagonal tiling
grid.truncated_hexagonal_tiling <- function(..., shape = "convex12") {
    scale <- star_scale(12, 60, external = TRUE)
    grid.pattern_regular_polygon(..., shape = shape, density = 1.034, spacing = 0.3,
                                 angle = 0, rot = 15, scale = scale, 
                                 grid = "hex_circle")
}
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.truncated_hexagonal_tiling(gp = gp)
popViewport()

# Add in a "convex12" star within
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = red, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.truncated_hexagonal_tiling(gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.truncated_hexagonal_tiling(gp = gp, shape = "star12")
popViewport()
add_borders()
```

<a name="trihexagon">Trihexagonal tiling</a>
--------------------------------------------

We can achieve one of the [uniform colorings of a trihexagonal tiling](https://en.wikipedia.org/wiki/Trihexagonal_tiling#Uniform_colorings) using the "convex6" shape with a "hex_circle" grid and rotating the shape 30 degrees.  

There are topologically equivalent tilings using the "star6", "star3", and "convex3" shapes that also provides an interesting tiling effect.

```{r, fig.alt = "A trihexagonal tiling plus 'star6', 'star3', and 'convex3' variations"}
grid.trihexagonal_tiling <- function(..., shape = "convex6", density = 1) {
    grid.pattern_regular_polygon(..., shape = shape, density = density, grid = "hex_circle",
                                 spacing = 0.2, rot = 30, angle = 0)
}
# 2 color uniform trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.trihexagonal_tiling(gp = gp)
popViewport()

# 'star6' trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
scale <- star_scale(6, 120, external = TRUE)
grid.trihexagonal_tiling(shape = "star6", scale = scale, gp = gp)
popViewport()

# 'star3' trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
scale <- star_scale(3, 30)
grid.trihexagonal_tiling(shape = "star3", scale = scale, density=1.57, gp = gp)
popViewport()

# 'convex3' trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
grid.trihexagonal_tiling(shape = "convex3", density=1.33, gp = gp)
popViewport()
add_borders()
```

<a name=snubtrihex>Snub trihexagonal tiling</a>
-----------------------------------------

We can achieve the one [uniform colorings of a snub (tri)hexagonal tiling](https://en.wikipedia.org/wiki/Snub_trihexagonal_tiling) using a "star6" shape and a "convex6" shape.

```{r, fig.alt = "Three different colorings of a snub trihexagonal tiling"}
grid.snub_trihexagonal <- function(..., gp_tri, gp_tri2 = gp_tri, gp_hex = gp_tri) {
    scale_star <- star_scale(6, 60 + 60, external = TRUE)
    grid.polygon(x_sq, y_sq, gp = gp_tri)
    grid.pattern_regular_polygon(shape = "star6", scale = scale_star,
                                 grid = "hex_circle",
                                 angle = 0, rot = 19, spacing = 0.2,
                                 density = 1.305, gp = gp_tri2)
    grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle",
                                 angle = 0, rot = 19 + 30, spacing = 0.2,
                                 density = scale_star * 1.305, gp = gp_hex)
}

# 3 color uniform snub square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp_y <- gpar(fill = yellow, col = "black")
gp_r <- gpar(fill = red, col = "black")
gp_b <- gpar(fill = blue, col = "black")
grid.snub_trihexagonal(gp_tri = gp_b, gp_tri2 = gp_r, gp_hex = gp_y)
popViewport()

# 2 color snub trihexagonal tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.snub_trihexagonal(gp_tri = gp_y, gp_hex = gp_r)
popViewport()

# 1 color snub trihexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.snub_trihexagonal(gp_tri = gp_y)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()
```


<a name="truncatedtrihexagon">Truncated trihexagonal</a>
---------------------------------------------------------------------------------------

By carefully using `grid.pattern_stripe()` it is possible to achieve the [truncated trihexagonal tiling](https://en.wikipedia.org/wiki/Truncated_trihexagonal_tiling).

```{r, fig.alt = "Two different colorings of a truncated trihexagonal tiling"}
# truncated trihexagonal
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3,
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle", 
                             density = 0.75, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# truncated trihexagonal with 5 colors
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = green, col = NA))
gp <- gpar(fill = orange, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.42, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = c(yellow, red, blue), col = "black")
grid.pattern_regular_polygon(shape = "convex6", grid = "hex_circle", 
                             density = 0.75, spacing = 0.3, angle = 0, gp = gp)
popViewport()
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
popViewport()
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()
```

<a name="rhombitrihexagon">Rhombitrihexagonal tiling</a>
---------------------------------------------------------------------------------------

By carefully using `grid.pattern_stripe()` it is also possible to achieve the [rhombitrihexagonal tiling](https://en.wikipedia.org/wiki/Rhombitrihexagonal_tiling).  There are also variant tilings where we add in "star12" polygons.

```{r, fig.alt = "Two different colorings of a rhombitrihexagonal tiling plus a 'star12' variation"}
# rhombitrihexagonal
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# rhombitrihexagonal with 5 colors
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = green, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = orange, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
popViewport()

# 3.4.6.3.12* rhombitrihexagonal plus star
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 0, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = -60, gp = gp)
grid.pattern_stripe(grid = "hex_circle", density = 0.25, spacing = 0.3, 
                    angle = 60, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.pattern_regular_polygon(shape = "convex12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp)
gp <- gpar(fill = green, col = "black")
scale <- star_scale(12, 30)
grid.pattern_regular_polygon(shape = "star12", grid = "hex_circle", rot = 15,
                             density = 0.82, spacing = 0.3, angle = 0, gp = gp,
                             scale = scale)
popViewport()

pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
popViewport()
add_borders()
```

<a name="star">Additional uniform tilings with star polygons</a>
---------------------------------------------------------------

Some additional [uniform tilings with star polygons](https://en.wikipedia.org/wiki/Uniform_tiling#Uniform_tilings_using_star_polygons) not shown above.

```{r, fig.alt = "Four uniform tilings with star polygons"}
# 12.12.4*
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow, blue), col = "black")
grid.pattern_regular_polygon(shape="convex12", rot=15, density=1.035, 
                             angle=0, gp = gp, spacing=0.3)
popViewport()

# 12.3*.12.3*
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow), col = "black")
grid.pattern_regular_polygon(shape="convex12", density=1, grid = "hex_circle",
                             angle=0, gp = gp, spacing=0.4)
popViewport()

# 4.4*.4**
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = c(yellow, blue), col = "black")
gp <- gpar(fill=yellow)
scale <- star_scale(4, 30)
grid.pattern_regular_polygon(shape="star4", density=1.60, angle=0, scale=scale, 
                             rot=-9.5, spacing=0.3, gp = gp)
popViewport()

# 3.3.8*.4**.8*
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
sp <- 0.35
gp <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "square", density = 1, angle = 0, 
                             spacing = sp, gp = gp)
gp <- gpar(fill = yellow, col = "black")
scale <- star_scale(4, 60)
grid.pattern_regular_polygon(shape = "star4", density = 1.14, rot = 45, 
                             xoffset = sp / 2, yoffset = sp / 2,
                             angle = 0, scale = scale, spacing = sp, gp = gp)
gp <- gpar(fill = blue, col = "black")
scale <- star_scale(8, 15)
grid.pattern_regular_polygon(shape = "star8", density = 1.082, rot = 22.5, 
                             angle = 0, scale = 0.30, spacing = sp, gp = gp)
popViewport()
add_borders()
```

```{r, fig.alt = "Four more uniform tilings with star polygons"}
# 3.6*.6**
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
scale <- star_scale(6, 30)
grid.pattern_regular_polygon(shape="star6", density=1.30, angle=0, scale=scale, rot=-22.0, 
                             spacing=0.3, grid="hex", gp=gp)
popViewport()

# 4.6.4*.6
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
scale <- star_scale(4, 120, external = TRUE)
grid.pattern_regular_polygon(shape=c("star4", "convex4"), density=c(1.2, 0.8), 
                             scale = scale, 
                             spacing = 0.2, gp = gp, angle=45)
popViewport()

# 9.3.9.3*
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = blue, col = NA)
grid.pattern_stripe(spacing = 0.4, density = 0.35, angle = 0,
                    grid = "hex_circle", gp = gp, yoffset = 0.096)
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "convex9", density = 1.01, angle = 0, 
                             grid = "hex_circle", spacing = 0.4, gp = gp)
popViewport()

# 8.4*.8.4*
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = red, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = c("convex8", "null"), density = 1.41, 
                             angle = 0, spacing = 0.2, gp = gp)
popViewport()
add_borders()
```

```{r, fig.alt = "Two more uniform tilings with star polygons"}
# 18.18.3*
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = blue, col = NA))
gp <- gpar(fill = c(red, yellow), col = "black")
grid.pattern_regular_polygon(shape = "convex18", density = 1, 
                             grid = "hex_circle", angle = 0,
                             spacing = 0.2, gp = gp)
popViewport()

# Only uniform if you consider the larger squares to be four-pointed stars...
# 4.8*.4**.8*
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = blue, col = NA))
gp <- gpar(fill = c(yellow, red), col = "black")
scale <- star_scale(8, 90, external = TRUE)
grid.pattern_regular_polygon(shape = c("star8", "convex4"), scale = scale,
                             density = c(1.53, 1.2), rot = c(22.5, 0), 
                             angle = 0, spacing = 0.2, gp = gp)
popViewport()
add_borders()
```

<a name=non>Non-uniform tilings</a>
===================================

<a name="herringbone">Herringbone tiling</a>
--------------------------------------------

We can achieve a [herringbone tiling](https://en.wikipedia.org/wiki/Herringbone_pattern) by using `grid.pattern_weave()` and particular "twill" weaves of the subtype "n/n(1)".

```{r, fig.alt = "Four different herringbone tilings"}
grid.herringbone_tiling <- function(..., subtype = "2/2(1)") {
    grid.pattern_weave(..., type = "twill", subtype = subtype, density = 1, 
                       spacing = 0.1, angle = 45)
}
# 1 color 2:1 herringbone 
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp)
popViewport()

# 2 color 2:1 herringbone
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, fill2 = red)
popViewport()

# 1 color 3:1 herringbone
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, subtype = "3/3(1)")
popViewport()

# 2 color 3:1 herringbone
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = "black")
grid.herringbone_tiling(gp = gp, fill2 = red, subtype = "3/3(1)")
popViewport()
add_borders()
```

<a name="pythagorean">Pythagorean tiling</a>
--------------------------------------------

We can get a [Pythagorean tiling](https://en.wikipedia.org/wiki/Pythagorean_tiling) by setting a background color and rotating "convex4" shapes

```{r, fig.alt = "Four different colorings of a Pythagorean tiling"}
grid.pythagorean_tiling <- function(...) {
    grid.pattern_regular_polygon(..., shape = "convex4", density = 1.222, 
                                 rot = 15, spacing = 0.2, angle = -15)
}
# 1 color Pythagorean tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = yellow, col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 2 color Pythagorean tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = red, col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 3 color Pythagorean tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(blue, red), col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()

# 4 color Pythagorean tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.polygon(x_sq, y_sq, gp = gpar(fill = yellow, col = NA))
gp <- gpar(fill = c(red, blue, green), col = "black")
grid.pythagorean_tiling(gp = gp)
popViewport()
add_borders()
```

<a name="rhombille">Rhombille tiling</a>
----------------------------------------------

We can get a [rhombille tiling](https://en.wikipedia.org/wiki/Rhombille_tiling) by using the "rhombille_rhombus" shape:

```{r, fig.alt = "Three different colorings of a rhombille tiling"}
grid.rhombille_tiling <- function(gp1, gp2 = gp1, gp3 = gp2) {
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot = -120, spacing = 0.2,
                       grid = "hex", gp = gp3)
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot =  120, spacing = 0.2,
                       grid = "hex", gp = gp2)
    grid.pattern_regular_polygon(shape = "rhombille_rhombus", density = 1,
                       angle = 0, rot = 0, spacing = 0.2,
                       grid = "hex", gp = gp1)
}
gp1 <- gpar(fill = yellow, col = "black")
gp2 <- gpar(fill = blue, col = "black")
gp3 <- gpar(fill = red, col = "black")
# 1 color rhombille tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.rhombille_tiling(gp1)
popViewport()
# 2 color rhombille tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.rhombille_tiling(gp1, gp2)
popViewport()
# 3 color rhombille tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.rhombille_tiling(gp1, gp2, gp3)
popViewport()
add_borders()
```

<a name="tetrakis">Tetrakis square tiling</a>
---------------------------------------------

We can get a [tetrakis square tiling](https://en.wikipedia.org/wiki/Tetrakis_square_tiling) by using the "tetrakis_left" and "tetrakis_right" shapes:

```{r, fig.alt = "Four different colorings of a tetrakis square tiling"}
grid.tetrakis_tiling <- function(gp1, gp2 = gp1, gp3 = gp2, gp4 = gp1) {
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1, 
                                 angle = 0, rot=0, gp = gp1, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1, 
                                 angle = 0, rot=90, gp = gp4, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1,
                                 angle = 0, rot=180, gp = gp1, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_left", density = 1,
                                 angle = 0, rot=270, gp = gp4, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=0, gp = gp2, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=90, gp = gp3, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=180, gp = gp2, spacing = 0.25)
    grid.pattern_regular_polygon(shape = "tetrakis_right", density = 1,
                                 angle = 0, rot=270, gp = gp3, spacing = 0.25)

}
gp1 <- gpar(fill = yellow, col = "black")
gp2 <- gpar(fill = blue, col = "black")
gp3 <- gpar(fill = red, col = "black")
gp4 <- gpar(fill = green, col = "black")
# 1 color tetrakis square tiling
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1)
popViewport()
# 2 color tetrakis square tiling
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2)
popViewport()
# 3 color tetrakis square tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2, gp3)
popViewport()
# 4 color tetrakis square tiling
pushViewport(viewport(x=0.75, y=0.25, width=0.5, height=0.5))
grid.tetrakis_tiling(gp1, gp2, gp3, gp4)
popViewport()
add_borders()
```

<a name="rhombi">Additional tilings using rhombi</a>
----------------------------------------------------

A rhombi may be produced by the "star2" shape:

```{r, fig.alt = "Three different tilings using rhombi"}
# 2*.2**.2*.2**
pushViewport(viewport(x=0.25, y=0.75, width=0.5, height=0.5))
sp <- 0.3
ang <- 63.5
scale <- 0.5
gpy <- gpar(fill = yellow, col = "black")
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpy)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpy,
                             xoffset = 0.5 * sp)
gpr <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpr,
                             xoffset = 0.25 * sp, yoffset = 0.5 * sp)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = ang,
                             density = 1, spacing = sp, gp = gpr,
                             xoffset = 0.75 * sp, yoffset = 0.5 * sp)
popViewport()

# 4.2*.4.2**
pushViewport(viewport(x=0.75, y=0.75, width=0.5, height=0.5))
dens_sq <- 0.73
scale <- star_scale(2, 60)
dens_rh <- 0.88
gpy <- gpar(fill = yellow, col = "black")
gpb <- gpar(fill = blue, col = "black")
gpr <- gpar(fill = red, col = "black")
grid.pattern_regular_polygon(shape = "convex4", scale = scale, angle = 0,
                             rot = 60,
                             density = dens_sq, spacing = sp, gp = gpb)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = 0,
                             density = dens_rh, spacing = sp, gp = gpy,
                             rot = 45, xoffset = 0.5 * sp)
grid.pattern_regular_polygon(shape = "convex4", scale = scale, angle = 0,
                             rot = -60, yoffset = 0.5 * sp, xoffset = 0.5 * sp,
                             density = dens_sq, spacing = sp, gp = gpr)
grid.pattern_regular_polygon(shape = "star2", scale = scale, angle = 0,
                             density = dens_rh, spacing = sp, gp = gpy,
                             rot = -45, yoffset = 0.5 * sp)
popViewport()

# Use a "star12" instead of a "convex12" in truncated hexagonal tiling
pushViewport(viewport(x=0.25, y=0.25, width=0.5, height=0.5))
gp <- gpar(fill = yellow, col = NA)
grid.polygon(x_sq, y_sq, gp = gp)
gp <- gpar(fill = blue, col = "black")
grid.truncated_hexagonal_tiling(gp = gp, shape = "star12")
popViewport()
add_borders()
```

---
title: "Developing Patterns"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Developing Patterns}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
knitr::opts_chunk$set(
collapse = TRUE,
comment = "#>"
)
```
```{r setup}
library("gridpattern")
```

## Developing Patterns - overview

To develop a pattern for use with `{gridpattern}` (and packages that use it such as `{ggpattern}`) you will need to:

1. Decide whether this is a geometry-based pattern or an array-based pattern.
2. Create a function with the required arguments and return value
3. Use `options()` to let `{gridpattern}` know that a certain pattern name should be
   delegated to your function.

This vignette shows how to:

* write a `create_pattern()` function of the correct signature for a geometry-based pattern or array-based pattern
* Instruct `{gridpattern}` on where to find this user-defined pattern

### Table of Contents

* [The two pattern classes](#classes)
* [List of all pattern aesthetics](#aes)
* [Aesthetic use by `{ggpattern}` patterns](#aes-by-pattern)
* [Geometry-based pattern functions](#geometry-based)
* [Array-based pattern functions](#array-based)
* [Geometry-based example #1 (recreate polygon)](#geometry-example-1)
* [Geometry-based example #2 (using other patterns)](#geometry-example-2)
* [Array-based example](#array-example)
* [Other examples](#other-example)

## <a name="classes"></a> Pattern Classification - geometry-based and array-based

There are only 2 high-level classes of pattern supported by `gridpattern` - geometry-based
and array-based.

**Geometry-based** patterns create a series of geometry objects and trim
them (using `sf::st_intersection()`, `gridGeometry::polyclipGrob()`, `gridpattern::clippingPathGrob()`, etc)
to be within the boundary of the grob. 
For example, the 'stripes' pattern in `gridpattern` is a series of equally spaced rectangular polygons.

**Array-based** patterns are RGBA image arrays.  Any supplied image will be 
processed by `gridpattern` to ensure it is masked to only apply to the area 
within the grob's boundary.

## <a name="aes"></a> `{ggpattern}` Aesthetics - Descriptions and Defaults

Although custom `{gridpattern}` pattern parameters need not limit itself to the set of aesthetics provided by `{ggpattern}` doing
so may make your pattern more useful for others:

<details open = "true">
<summary> `{ggpattern}` aesthetic summary - click to open/close </summary>

| aesthetic                  | description                                   | default    | possible values                    |
|----------------------------|-----------------------------------------------|------------|------------------------------------|
| `pattern`                  | Name of the pattern to draw                   | 'stripe'   | `gridpattern::names_pattern`       |
| `pattern_alpha`            | Alpha                                         | 1          | value in range [0, 1] or `NA`      |
| `pattern_angle`            | Rotation angle (entire pattern)               | 30         | angle in degrees                   |
| `pattern_aspect_ratio`     | Aspect ratio adjustment                       | NA         | usual range [0.01, 10]             |
| `pattern_colour`           | Stroke colour                                 | 'grey20'   | colour                             |
| `pattern_density`          | Approx. fraction of area the pattern fills    | 0.2        | value in range [0, 1]              |
| `pattern_filename`         | Image filename/URL                            | ''         | Filename/URL                       |
| `pattern_fill2`            | Second fill colour                            | '#4169E1'  | colour                             |
| `pattern_fill`             | Fill colour                                   | 'grey80'   | colour                             |
| `pattern_filter`           | Image scaling filter                          | 'lanczos'  | `magick::filter_types`             |
| `pattern_frequency`        | Frequency                                     | 0.1        |                                    |
| `pattern_gravity`          | Image placement                               | 'center'   | `magick::gravity_types`            |
| `pattern_grid`             | Pattern grid type                             | 'square'   | 'square', 'hex', 'hex\_circle'     |
| `pattern_key_scale_factor` | Scale factor for pattern in legend            | 1          |                                    |
| `pattern_linetype`         | Stroke linetype                               | 1          | linetype                           |
| `pattern_linewidth`        | Stroke linewidth                              | 1          | linewidth                          |
| `pattern_option_1 - 5`     | Generic options for expansion                 | 0          |                                    |
| `pattern_orientation`      | Orientation                                   | 'vertical' | 'vertical', 'horizontal', 'radial' |
| `pattern_phase`            | Phase                                         | 0          |                                    |
| `pattern_res`              | Pattern resolution (pixels per inch)          | NA         |                                    |
| `pattern_rot`              | Rotation angle (shape within pattern)         | 0          | angle in degrees                   |
| `pattern_scale`            | Scale                                         | 1          | Multiplier                         |
| `pattern_shape`            | Plotting shape                                | 1          | shapes                             |
| `pattern_size`             | Size factor (e.g. fontsize)                   | 1          | fontsize                           |
| `pattern_spacing`          | Spacing between repetitions of pattern        | 0.05       | value in range [0, 1] (snpc units) |
| `pattern_subtype`          | Generic control option                        | NA         | pattern-dependent                  |
| `pattern_type`             | Generic control option                        | NA         | pattern-dependent                  |
| `pattern_xoffset`          | Shift pattern along x axis                    | 0          | value in range [0, 1] (snpc units) |
| `pattern_yoffset`          | Shift pattern along y axis                    | 0          | value in range [0, 1] (snpc units) |

</summary>
</details>

Note `{ggpattern}` may also pass other geom aesthetics of possible interest such as ``fill``.
Also note that `{ggpattern}` will only pass pattern aesthetics values of length one 
but if the pattern is called directly by `gridpattern::patternGrob()` then the pattern
may be passed pattern parameters of arbitrary length.

## <a name="geometry-based"></a> Geometry-based pattern functions - Formal Arguments and Return Values

All geometry-based pattern creation functions must:

1. Have the exact function signature: `function(params, boundary_df, aspect_ratio, legend)`

    * `params` - parameters for the pattern (the aesthetics) e.g `pattern_fill`
    * `boundary_df` - data.frame containing polygon information i.e. The `polygon_df` format.
    * `aspect_ratio` - the best guess that `{gridpattern}` / `{ggpattern}` is able to make as to the
      aspect ratio of the **viewport** in which this pattern is being rendered.
    * `legend` logical value to indicate whether or not this function is being called
      to render a key legend or the in-place geom fill.

2. Return a grid grob object. This can be any valid grob including a `grid::grobTree()`.
   The user should make sure it lies within the boundary represented by `boundary_df` either by clipping with functions
   like `sf::st_intersection()`, `gridGeometry::polyclipGrob()`, `gridpattern::clippingPathGrob()`, etc.
   or using bounded grob functions like `gridpattern::patternGrob()` or `grid::polygonGrob()`.

## <a name="array-based"></a> Array-based pattern functions - Formal Arguments and Return Values

All array-based pattern creation functions must:

1. Have the exact function signature: `function(width, height, params, legend)`
    * `width,height` - dimensions of the bounding box of the geom area
    * `params` - parameters from the geom (the aesthetics) e.g `pattern_fill`
    * `legend` logical value to indicate whether or not this function is being called
      to render a key legend or the in-place geom fill.
2. Return a 3D array of RGBA values (all values in the range [0, 1]).
   `gridpattern` itself will mask this image so that it only applies to the area within the grob's boundary.

## <a name="polygon_df"></a> The `polygon_df` data.frame format

The `polygon_df` is a very simple data.frame format to contain polygon values.  This is
used to pass the coordinates of the geom boundary from the geom to the
pattern generating function.

It contains only 'x' and 'y' columns for the coordinates, and an 'id' column 
used to signify which polygon the coordinates belong to.

The following `polygon_df` data.frame contains 2 polygons:

```{r echo = FALSE}
x <- read.csv(textConnection("
x,  y,  id
0,  0,   1
1,  0,   1
1,  1,   1
0,  1,   1
0,  0,   2
2,  0,   2
2,  1,   2
0,  1,   2"))

knitr::kable(x, caption = "example data in 'polygon_df' format")
```

## <a name="user-funcs"></a> Associating a function with  `{gridpattern}` pattern name

There are two global `option()` values which can be set - one for geometry-based 
patterns, and the other for array-based patterns.

The global values should point to a named list, where the names are the pattern 
names you want to use within `{gridpattern}`, and the named values are the
actual functions.  Note for backwards-compatibility with the original `{ggpattern}` system
these options start with `ggpattern` instead of `gridpattern`.


```{r eval = FALSE}
options(ggpattern_array_funcs    = list(your_pattern_name = your_pattern_function))
options(ggpattern_geometry_funcs = list(your_pattern_name = your_pattern_function))
```

Pattern names must be different from any of the builtin patterns included in `{gridpattern}`.

<a name="geometry-example-1"></a> Example geometry-based pattern #1 (recreate polygon)
---------------------------------------------------------------------------------------

All geometry-based pattern creation functions must:

1. Have the exact function signature: `function(params, boundary_df, aspect_ratio, legend)`
    * `params` - parameters from the geom (the aesthetics) e.g `pattern_fill`
    * `boundary_df` - data.frame containing polygon information i.e. The `polygon_df` format.
    * `aspect_ratio` - the best guess that `{gridpattern}` / `{ggpattern}` is able to make as to the
      aspect ratio of the **viewport** in which this pattern is being rendered.
    * `legend` logical value to indicate whether or not this function is being called
      to render a key legend or the in-place geom fill.
2. Return a grid grob bounded by the boundary represented by `boundary_df` 
   (including grid structures like a `grid::grobTree()`).

For this example we'll create a simple single color fill pattern based on `grid::polygonGrob()` called 'polygon'.

```{r}
create_pattern_polygon <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    x <- boundary_df$x
    y <- boundary_df$y
    id <- boundary_df$id

    alpha <- ifelse(is.na(params$pattern_alpha), 1, params$pattern_alpha)
    gp <- grid::gpar(alpha = alpha, 
                     col = params$pattern_colour,
                     fill = params$pattern_fill,
                     lty = params$pattern_linetype,
                     lwd = params$pattern_linewidth)

    grid::polygonGrob(x = x, y = y, id = id, default.units = "npc", gp = gp)
}
```

A global option `ggpattern_geometry_funcs` is a named list which contains 
geometry-based pattern creating functions to use outside of `ggpattern`.

The **name** used in this list corresponds to the `pattern` name used with 
the geom - in this case we will be using `pattern = 'polygon'`.

```{r}
options(ggpattern_geometry_funcs = list(polygon = create_pattern_polygon))
```

```{r, fig.alt = "Three rectangles each filled with a single color"}
grid.pattern("polygon", fill = "red", size = 4, linetype = "dashed", 
             x = c(0.05, 0.05, 0.305, 0.305), y = c(0.05, 0.305, 0.305, 0.05))
grid.pattern("polygon", fill = "green", alpha = 0.2, 
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("polygon", fill = "blue", colour = "grey",
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))
```

<a name="geometry-example-2"></a> Example geometry-based pattern function #2 (using other patterns)
---------------------------------------------------------------------------------------------------

All geometry-based pattern creation functions must:

1. Have the exact function signature: `function(params, boundary_df, aspect_ratio, legend)`
    * `params` - parameters from the geom (the aesthetics) e.g `pattern_fill`
    * `boundary_df` - data.frame containing polygon information i.e. The `polygon_df` format.
    * `aspect_ratio` - the best guess that `{gridpattern}` / `{ggpattern}` is able to make as to the
      aspect ratio of the **viewport** in which this pattern is being rendered.
    * `legend` logical value to indicate whether or not this function is being called
      to render a key legend or the in-place geom fill.
2. Return a grid grob bounded by the boundary represented by `boundary_df` 
   (including grid structures like a `grid::grobTree()`).

For this example we'll create an example that re-uses the pre-existing 
'stripe', 'circle', and 'gradient' patterns and combines them into a new 'complex' pattern.

```{r}
create_pattern_complex <- function(params, boundary_df, aspect_ratio, legend = FALSE) {
    args <- as.list(params)
    args <- args[grep("^pattern_", names(args))]
    args$x <- boundary_df$x
    args$y <- boundary_df$y
    args$id <- boundary_df$id
    args$prefix <- ""

    args_stripe <- args
    args_stripe$pattern <- "stripe"
    args_stripe$pattern_density <- 0.5 * args$pattern_density
    args_stripe$pattern_spacing <- 2 * args$pattern_spacing
    grob_stripe <- do.call(gridpattern::patternGrob, args_stripe)

    args_circle <- args
    args_circle$pattern <- "regular_polygon"
    args_circle$pattern_shape <- c("circle", "null")
    args_circle$pattern_yoffset <- args$pattern_spacing + args$pattern_yoffset
    args_circle$pattern_type = "horizontal"
    grob_circle <- do.call(gridpattern::patternGrob, args_circle)

    args_gradient <- args
    args_gradient$pattern <- "gradient"
    args_gradient$pattern_fill <- "#00000070"
    args_gradient$pattern_fill2 <- "#FFFFFF70"
    args_gradient$pattern_orientation <- "vertical"
    grob_gradient <- do.call(gridpattern::patternGrob, args_gradient)

    grid::grobTree(grob_stripe, grob_circle, grob_gradient)
}
```

A global option `ggpattern_geometry_funcs` is a named list which contains 
geometry-based pattern creating functions to use outside of `ggpattern`.

The **name** used in this list corresponds to the `pattern` name used with 
the geom - in this case we will be using `pattern = 'complex'`.

```{r}
options(ggpattern_geometry_funcs = list(complex = create_pattern_complex))
```

```{r, fig.alt = "Three rectangles each with a 'stripe', 'circle', and 'gradient' pattern"}
grid.pattern("complex", fill = "red", angle = 45, spacing = 0.05, density = 0.3,
             x = c(0.0, 0.0, 0.3, 0.3), y = c(0.0, 0.3, 0.3, 0.0))
grid.pattern("complex", fill = "green", angle = 45, spacing = 0.2, density = 0.2,
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("complex", fill = "blue", angle = 45, spacing = 0.1, density = 0.3,
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))
grid::grid.polygon(x = c(0.0, 0.0, 0.3, 0.3, 0.35, 0.35, 0.65, 0.65, 0.7, 0.7, 1.0, 1.0),
                   y = c(0.0, 0.3, 0.3, 0.0, 0.35, 0.65, 0.65, 0.35, 0.7, 1.0, 1.0, 0.7), 
                   id = rep(1:3, each = 4), 
                   gp = grid::gpar(col = "black", fill = NA, lwd=4))
```


<a name="array-example"></a> Example array-based pattern function
------------------------------------------------------------------------------

All array-based pattern creation functions must:

1. Have the exact function signature: `function(width, height, params, legend)`
    * `width,height` - dimensions of the bounding box of the geom area
    * `params` - parameters from the geom (the aesthetics) e.g `pattern_fill`
    * `legend` logical value to indicate whether or not this function is being called
      to render a key legend or the in-place geom fill.
2. Return a 3D array of RGBA values (all values in the range [0, 1]).

For this example we'll create a simple example that cycles through RGBA values.

Parameters for this pattern:

* `pattern_type` is used to distinguish between source data for the pattern 

**Note**: This pattern exploits vector recyling in the creation of the RGBA array, 
and as dimensions change the alignment of the R, G, B and A planes will not 
remain in a fixed relationship.  Thus if you change the shape of the rendered image,
you will change the nature of the pattern.

```{r}
create_pattern_simple <- function(width, height, params, legend) {
  
  # Ensure the selected pattern is sane.
  choice <- params$pattern_type
  if (is.null(choice) || is.na(choice) || !is.character(choice)) {
    choice <- 'a'
  }
  
  # Choose the values with which to fill the array
  values <- switch(
    choice,
    a = rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3),
    b = rep(c(1, 0, 0, 1, 0.5, 0.5, 1, 1, 0, 0, 0, 0, 0, 0.5), each = 7),
    c = rep(seq(0, 1, 0.05), each = 7),
    rep(c(0, 1, 0, 1, 1, 0, 0, 1, 1, 1), each = 3)
  )
  # Create an RGBA array of the requested dimensions
  simple_array <- array(values, dim = c(height, width, 4))

  simple_array
}
```

A global option `ggpattern_array_funcs` is a named list which contains 
geometry-based pattern creating functions to use outside of `ggpattern`.

The **name** used in this list corresponds to the `pattern` name used with 
the geom - in this case we will be using `pattern = 'simple'`.

```{r}
options(ggpattern_array_funcs = list(simple = create_pattern_simple))
```

```{r, fig.alt = "Three rectangles each filled with an 'array' pattern"}
grid::grid.polygon(x = c(0, 0, 1, 1), y = c(0, 1, 1, 0), 
                   gp = grid::gpar(col=NA, fill="grey"))
grid.pattern("simple", type = "a",
             x = c(0.0, 0.0, 0.3, 0.3), y = c(0.0, 0.3, 0.3, 0.0))
grid.pattern("simple", type = "b",
             x = c(0.35, 0.35, 0.65, 0.65), y = c(0.35, 0.65, 0.65, 0.35))
grid.pattern("simple", type = "c",
             x = c(0.7, 0.7, 1.0, 1.0), y = c(0.7, 1.0, 1.0, 0.7))
```

<a name="other-example"></a> Other examples
-------------------------------------------

The `{ggpattern}` package contains a vignette on developing `{gridpattern}` / `{ggpattern}` patterns with a few more examples:  https://coolbutuseless.github.io/package/ggpattern/articles/

Also the `{gridpattern}` source contains the full source code for  `r length(names_pattern)` patterns: https://github.com/trevorld/gridpattern/tree/main/R

---
title: "Heraldic Color Hatching"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Heraldic Color Hatching}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
can_render <- capabilities("png") || guess_has_R4.1_features("masks")
can_run_aqp <- requireNamespace("aqp", quietly = TRUE) &&
    (interactive() || identical(Sys.getenv("IN_PKGDOWN"), "true") || identical(Sys.getenv("NOT_CRAN"), "true"))
```

The `gridpattern` package supports heraldic color hatching via `grid.pattern_hatch()`.
Hatching encodes color information using patterns of lines and dots, allowing
images to be reproduced in black and white while retaining color identity.

Four systems are supported via the `subtype` argument:

- `"combinatorial"` (default): extends the seven standard Petra Sancta tinctures with systematically derived mixed-color combinations following three rules:

  1. white combined with a color is represented by dashed lines
  2. yellow combined with a color is represented by dotdash lines
  3. color combined with a color is represented by crossing solid lines

- `"fox-davies"`: contains the sixteen hatchings from Fox-Davies' [*A Complete Guide to Heraldry*](https://en.wikisource.org/wiki/A_Complete_Guide_to_Heraldry/Chapter_7#74) covering the seven standard Petra Sancta tinctures plus nine extensions from German heraldry.
- `"goodman"`: contains all the hatchings from David Goodman's [Heraldic Tincture](https://david.goodman.graphics/portfolio/item/crests-heraldry-and-coats-of-arms/) reference (v2.0, 2024).
- `"unicode"`: the system used in the official Unicode character chart pdfs to render "colored" (emoji) glyphs in black-and-white.

Use `names_hatch()` to query supported tincture names for a given subtype.  The "hatch" pattern will also coerce some of the more common color names to the right tincture e.g. "gold" and "yellow" will be both coerced to "or" and vice versa.

```{r setup}
library("grid")
library("gridpattern")
```

```{r names}
names_hatch()
names_hatch("fox-davies")
names_hatch("goodman")
names_hatch("unicode")
```

## Combinatorial color hatching

The default `"combinatorial"` subtype starts from the set of five [Munsell primary colors](https://en.wikipedia.org/wiki/Munsell_color_system) of red, yellow, green, blue, and purple plus black and white and the standard Petra Sancta color hatching system and then systematically derives additional color hatchings via three rules:

1. **white + color** → dashed lines
2. **yellow + color** → dotdash lines
3. **color + color** → crossing solid lines

```{r combinatorial-table, fig.alt = "Color table showing the Combinatorial Petra Sancta tinctures arranged by achromatic, primaries, secondaries, and notable combinations", fig.width = 7, fig.height = 8.5, eval = can_render && can_run_aqp, echo = FALSE}
# Munsell primary colors
p_col <- c(
    argent  = "#FFFFFF",
    sable   = "#000000",
    gules   = "#C83030", # 5R 4/14
    # or      = "#E8C840", # 5Y 8/12
    or      = "#D4B828", # 5Y 7/12
    azure   = "#0072B0", # 5B 4/10
    vert    = "#008060", # 5G 5/10
    # purpure = "#7B4090" # 5P 4/10
    purpure = "#9050C0" # 5P 6/12
)
# p_col <- c(
# 	argent  = aqp::parseMunsell("N 9.5/"),
# 	azure   = aqp::parseMunsell("5B 4/10"),
# 	gules   = aqp::parseMunsell("5R 4/14"),
# 	or      = aqp::parseMunsell("5Y 7/12"),
# 	sable   = aqp::parseMunsell("N 1/"),
#     purpure = aqp::parseMunsell("5P 6/12"),
#     vert    = aqp::parseMunsell("5G 5/10")
# )

# Five Munsell secondary hues via subtractive mixing
s_col <- c(
    orange   = mix_col(c(p_col["gules"],   p_col["or"])),       # YR: red + yellow
    lime     = mix_col(c(p_col["or"],       p_col["vert"])),     # GY: yellow + green
    teal     = mix_col(c(p_col["azure"],    p_col["vert"])),     # BG: blue + green
    violet   = mix_col(c(p_col["azure"],    p_col["purpure"])),  # PB: blue + purple
    sanguine = mix_col(c(p_col["gules"],    p_col["purpure"]))   # RP: red + purple
)

# Notable combination colors
w_col <- c(
    carnation      = mix_col(c(p_col["argent"],  p_col["gules"])),
    cendree        = mix_col(c(p_col["argent"],  p_col["sable"])),
    mint           = mix_col(c(p_col["argent"],  p_col["vert"])),
    `bleu celeste` = mix_col(c(p_col["argent"],  p_col["azure"])),
    lavender       = mix_col(c(p_col["argent"],  p_col["purpure"]))
)
o_col <- c(
    tenne          = mix_col(c(p_col["gules"],   p_col["vert"])),
    slate          = mix_col(c(p_col["purpure"], p_col["vert"])),
    olive          = mix_col(c(p_col["or"],       p_col["sable"])),
    rose           = mix_col(c(p_col["or"],       p_col["purpure"])),
    brunatre       = mix_col(c(p_col["azure"],    p_col["gules"],   p_col["vert"]))
)

groups <- list(
    list(
        label     = "Achromatic",
        tinctures = c("argent", "sable"),
        cols      = p_col[c("argent", "sable")],
        names     = c("white (W)", "black (K)")
    ),
    list(
        label     = "Munsell Primary Hues",
        tinctures = c("gules", "or", "vert", "azure", "purpure"),
        cols      = p_col[c("gules", "or", "vert", "azure", "purpure")],
        names     = c("red (R)", "yellow (Y)", "green (G)", "blue (B)", "purple (P)")
    ),
    list(
        label     = "Munsell Secondary Hues",
        tinctures = c("orange", "lime", "teal", "violet", "sanguine"),
        cols      = s_col,
        names     = c("orange (R+Y)", "lime (Y+G)", "teal (G+B)", "violet (B+P)", "magenta (P+R)")
    ),
    list(
        label     = "Combinations with White",
        tinctures = c("carnation", "cendree", "mint", "bleu celeste", "lavender"),
        cols      = w_col,
        names     = c("pink (R+W)", "grey (K+W)", "mint (G+W)", "light blue (B+W)", "lavender (P+W)")
    ),
    list(
        label     = "Other Combinations*",
        tinctures = c("tenne", "olive", "slate", "brunatre", "rose"),
        cols      = o_col[c("tenne", "olive", "slate", "brunatre", "rose")],
        names     = c("brown (R+G)", "olive (Y+K)", "slate (G+P)", "umbre (B+R+G)", "rose (Y+P)")
    )
)

rx <- c(0, 0, 1, 1)
ry <- c(1, 0, 0, 1)
ncols_fig <- 5L

row_heights <- unlist(lapply(groups, function(g) {
    n_sr <- ceiling(length(g$tinctures) / ncols_fig)
    c(0.28, rep(1, n_sr))
}))

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.97, height = 0.97))
grid.text(
    "Combinatorial Petra Sancta",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 31, fontface = "bold")
)

# Upper-right rules legend
pushViewport(viewport(
    x = unit(1, "npc") - unit(0.2, "cm"),
	y = unit(1, "npc") - unit(1.3, "cm"),
    just = c("right", "top"),
    width = unit(10.0, "cm"), height = unit(3.1, "cm")
))
grid.rect(gp = gpar(fill = "grey98", col = "grey60", lwd = 0.8))
grid.text("Combination rules:", x = 0.01, y = 0.91, just = c("left", "top"),
          gp = gpar(fontsize = 14, fontface = "bold"))
legend_rules <- c(
    "1. Dashed lines — combined with white",
    "2. Dot-dash lines — combined with yellow",
    "3. Crossed solid lines — mixed colors (if not black)"
)
for (i in seq_along(legend_rules)) {
    grid.text(legend_rules[i], x = 0.01, y = 0.66 - (i - 1L) * 0.25,
              just = c("left", "top"), gp = gpar(fontsize = 11))
}
popViewport()

pushViewport(viewport(
    y = 0.49, height = 0.90,
    layout = grid.layout(length(row_heights), ncols_fig, heights = unit(row_heights, "null"))
))

layout_row <- 1L
for (g in groups) {
    pushViewport(viewport(layout.pos.row = layout_row, layout.pos.col = 1:ncols_fig))
    grid.text(g$label, x = 0.01, just = "left",
              gp = gpar(fontsize = 18, fontface = "bold", col = "black"))
    popViewport()
    layout_row <- layout_row + 1L

    n_sr <- ceiling(length(g$tinctures) / ncols_fig)
    for (sr in seq_len(n_sr)) {
        idx_from <- (sr - 1L) * ncols_fig + 1L
        idx_to   <- min(sr * ncols_fig, length(g$tinctures))
        for (ci in idx_from:idx_to) {
            t     <- g$tinctures[ci]
            col_i <- (ci - 1L) %% ncols_fig + 1L
            if (is.na(t)) next
            col   <- unname(g$cols[ci])
            nm    <- g$names[ci]
            display_col <- if (t == "argent") "grey55" else col

            pushViewport(viewport(layout.pos.row = layout_row, layout.pos.col = col_i))
            pushViewport(viewport(y = 0.58, width = 0.90, height = 0.72,
                                  layout = grid.layout(1, 2)))
            pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
            grid.rect(gp = gpar(fill = col, col = display_col, lwd = 1.5))
            popViewport()
            pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
            grid.pattern_hatch(rx, ry, type = t, color = display_col,
                               spacing = 0.18, linewidth = 0.8)
            grid.rect(gp = gpar(fill = NA, col = display_col, lwd = 1.5))
            popViewport()
            popViewport()
            grid.text(nm, y = unit(0.105, "npc"), gp = gpar(fontsize = 9, col = "black"))
            popViewport()
        }
        layout_row <- layout_row + 1L
    }
}

popViewport()

# Footnote
grid.text(
    "* Display colors are sensitive to the exact primary pigments chosen;\n  results are roughly consistent for saturated heraldic primaries with Munsell pigment mixing.",
    x = 0.01, y = 0.004, just = c("left", "bottom"),
    gp = gpar(fontsize = 10, col = "black", fontface = "italic")
)

popViewport()
```

**Note:** The mixed display colors shown above can be sensitive to the exact hex values chosen for the primaries.
The results are fairly consistent when the primaries are the saturated, high-chroma colors typical of heraldry combined with Munsell pigment mixing
but softer or more neutral primaries can shift some secondaries noticeably
(for example, mixing yellow and blue can yield anything from olive-grey to muted purple depending on the blue's hue angle).

## Heraldic tincture hatching

### Fox-Davies

The `"fox-davies"` hatching subtype includes the seven standard tinctures plus nine extensions from German heraldry whose hatchings were included in Fox-Davies' [*A Complete Guide to Heraldry*](https://en.wikisource.org/wiki/A_Complete_Guide_to_Heraldry/Chapter_7#74).

```{r fox-davies-shields, fig.alt = "Heraldic shields showing the Fox-Davies hatching tinctures", fig.width = 7, fig.height = 6.0, eval = can_render, echo = FALSE}
# Approximate display color for each tincture
tincture_col <- c(
    argent         = "grey40",
    azure          = "#003399",
    `bleu celeste` = "#4499CC",
    brunatre       = "#7B3A10",
    carnation      = "#CC6688",
    cendree        = "#708090",
    gules          = "#CC0000",
    eisenfarbe     = "#708090",
    proper         = "#228B22",
    or             = "#DAA520",
    orange         = "#EE7700",
    purpure        = "#660099",
    sable          = "#111111",
    sanguine       = "#880000",
    tenne          = "#BB6600",
    vert           = "#006400"
)
color_equiv <- c(
    argent         = "white/silver",
    azure          = "blue",
    `bleu celeste` = "light blue",
    brunatre       = "(earth) brown",
    carnation      = "carnation",
    cendree        = "ash grey",
    gules          = "red",
    eisenfarbe     = "iron grey",
    proper         = "color of nature",
    or             = "yellow/gold",
    orange         = "orange",
    purpure        = "purple",
    sable          = "black",
    sanguine       = "blood red",
    tenne          = "(tawny) brown",
    vert           = "green"
)

# Heater shield polygon (normalised to [0,1] x [0,1])
sx <- c(0.0, 0.0, 0.5, 1.0, 1.0)
sy <- c(1.0, 0.35, 0.0, 0.35, 1.0)

tinctures <- names_hatch("fox-davies")
tincture_labels <- names_hatch("fox-davies", accent = TRUE)
n <- length(tinctures)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.97, height = 0.97))
grid.text(
    "Heraldic Hatching (Petra Sancta + German Heraldry Extensions)",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(y = 0.47, height = 0.90, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    t <- tinctures[i]
    col <- tincture_col[t]
    row_i <- ((i - 1L) %/% ncols) + 1L
    col_i <- ((i - 1L) %% ncols) + 1L

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(y = 0.60, width = 0.78, height = 0.70))

    grid.polygon(sx, sy, gp = gpar(fill = "white", col = NA))
    grid.pattern_hatch(sx, sy, type = t, subtype = "fox-davies", color = col, spacing = 0.12, linewidth = 1.0)
    grid.polygon(sx, sy, gp = gpar(fill = NA, col = col, lwd = 1.5))

    popViewport()

    grid.text(tincture_labels[i], y = unit(0.20, "npc"), gp = gpar(fontsize = 8.5, col = "grey20"))
    grid.text(color_equiv[t], y = unit(0.06, "npc"), gp = gpar(fontsize = 7.5, col = col))

    popViewport()
}

popViewport()
popViewport()
```

### Goodman

The `"goodman"` hatching subtype includes all the hatchings in David Goodman's [Heraldic Tincture](https://david.goodman.graphics/portfolio/item/crests-heraldry-and-coats-of-arms/) reference (v2.0, 2024).  This shares most hatchings with Fox-Davies but differs in few ways:

- Goodman's **sanguine** hatching instead uses horizontal plus diagonal `\` crossing lines.
- Goodman also has a distinct **murrey** hatching which uses crossing diagonal lines (Fox-Davies' instead uses this hatching for the eisenfarbe (iron grey) hatching).
- New **rose** with the same hatching as carnation (which Goodman also lists).
- New **steel** metal rendered as plus signs in a square grid.
- New **copper**, **bronze**, and **lead** metals which are each rendered as the letter "c" in a hex grid.
- Goodman omits the **proper** hatching that Fox-Davies included.

```{r goodman-shields, fig.alt = "Heraldic shields showing Goodman tinctures that differ from Fox-Davies", fig.width = 7, fig.height = 4.0, eval = can_render, echo = FALSE}
tincture_col <- c(
    sanguine = "#880000",
    murrey   = "#990055",
    steel    = "#708090",
    copper   = "#B87333"
)

sx <- c(0.0, 0.0, 0.5, 1.0, 1.0)
sy <- c(1.0, 0.35, 0.0, 0.35, 1.0)

tinctures <- names(tincture_col)
n <- length(tinctures)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.97, height = 0.97))
grid.text(
    "Goodman — New and Different Tinctures",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(y = 0.44, height = 0.85, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    t <- tinctures[i]
    col <- tincture_col[t]
    row_i <- ((i - 1L) %/% ncols) + 1L
    col_i <- ((i - 1L) %% ncols) + 1L

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(y = 0.60, width = 0.78, height = 0.70))

    grid.polygon(sx, sy, gp = gpar(fill = "white", col = NA))
    grid.pattern_hatch(sx, sy, type = t, subtype = "goodman", color = col, spacing = 0.12, linewidth = 0.8)
    grid.polygon(sx, sy, gp = gpar(fill = NA, col = col, lwd = 1.5))

    popViewport()

    grid.text(t, y = unit(0.20, "npc"), gp = gpar(fontsize = 18, col = "black"))
    popViewport()
}

popViewport()
popViewport()
```

## Unicode color hatching

The `"unicode"` hatching subtype provides each of the hatching used in the official Unicode character chart pdfs to assign a distinct pattern to each color to render "colored" (emoji) glyphs in black-and-white.
Notably Unicode has twelve different [colored heart emoji](https://unicode.org/emoji/charts/full-emoji-list.html#heart) (red, blue, green, yellow, purple, black, white, brown, orange, light blue, grey, pink) that each needed a separate hatching.

```{r unicode-hearts, fig.alt = "Twelve Unicode colored hearts rendered with hatching patterns", fig.width = 7, fig.height = 6, eval = requireNamespace("Unicode", quietly = TRUE) && can_render, echo = FALSE}
library("Unicode")

# The 12 Unicode colored hearts in codepoint order
heart_codepoints <- c(
    red          = 0x2764L, # HEAVY BLACK HEART (displays as red via emoji VS)
    blue         = 0x1F499L,
    green        = 0x1F49AL,
    yellow       = 0x1F49BL,
    purple       = 0x1F49CL,
    black        = 0x1F5A4L,
    white        = 0x1F90DL,
    brown        = 0x1F90EL,
    orange       = 0x1F9E1L,
    `light blue` = 0x1FA75L,
    grey         = 0x1FA76L,
    pink         = 0x1FA77L
)

# Approximate display colors
heart_col <- c(
    red          = "#CC0000",
    blue         = "#0055CC",
    green        = "#006400",
    yellow       = "#CCAA00",
    purple       = "#6600AA",
    black        = "#111111",
    white        = "#999999", # grey stroke so argent pattern is visible
    brown        = "#7B3A10",
    orange       = "#FF8000",
    `light blue` = "#4499CC",
    grey         = "#666666",
    pink         = "#DD4488"
)

uchars <- as.u_char(as.integer(heart_codepoints))
labels <- u_char_name(uchars)
heart_shape <- "♥" # U+2665 BLACK HEART SUIT — uniform shape template

n <- length(heart_codepoints)
ncols <- 4L
nrows <- ceiling(n / ncols)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.95, height = 0.95))
grid.text(
    "Unicode Colored Hearts with Hatching",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 22, fontface = "bold")
)
pushViewport(viewport(y = 0.48, height = 0.90, layout = grid.layout(nrows, ncols)))

for (i in seq_len(n)) {
    col_i <- ((i - 1L) %% ncols) + 1L
    row_i <- ((i - 1L) %/% ncols) + 1L
    col <- heart_col[i]

    pushViewport(viewport(layout.pos.row = row_i, layout.pos.col = col_i))
    pushViewport(viewport(width = 0.85, height = 0.85))

    pfill <- patternFill(
        "hatch",
        type      = names(heart_codepoints)[i],
        subtype   = "unicode",
        color    = col,
        spacing   = 0.14,
        linewidth = 0.8
    )
    grid.draw(
        fillStrokeGrob(
            textGrob(heart_shape, gp = gpar(fontsize = 84)),
            gp = gpar(fill = pfill, col = col)
        )
    )

    grid.text(labels[i], y = unit(0.12, "npc"),
              gp = gpar(fontsize = 12, col = "black"))
    grid.text(sprintf("U+%04X", heart_codepoints[i]), y = unit(0.00, "npc"),
              gp = gpar(fontsize = 10, col = "black"))

    popViewport()
    popViewport()
}

popViewport()
popViewport()
```

## Okabe-Ito hatching

One of the techniques to meet Web Content Accessibility Guidelines (WCAG) is to [use color and pattern](https://www.w3.org/WAI/WCAG21/Techniques/general/G111) to ensure things are accessible to the color-blind.

The [Okabe-Ito palette](https://jfly.uni-koeln.de/color/) is a widely used colorblind-friendly palette. Here is an example of adding a simple hatching scheme to go with this palette to provide visual redundancy:

  * "yellow", "blue", and "white" are given their standard Petra Sancta hatchings
  + "reddish purple" is given a "purple" Petra Sancta hatching
  * "bluish green" is given a "green" Petra Sancta hatching
  * "vermillion" (red orange) is given a "red" Petra Sancta hatching
  + we use a simple black fill for "black" instead of a "sable" crosshatch
  * "orange" and "sky blue" are given the hatchings from German heraldry (see Fox-Davies' section above)

```{r okabe-ito, fig.alt = "Table of Okabe-Ito palette colors paired with heraldic hatching patterns", fig.width = 6, fig.height = 6, eval = can_render}
oi_names <- c(
    "black", "orange", "sky blue", "bluish green",
    "yellow", "blue", "vermilion", "reddish purple", "white"
)
oi_hex <- c(
    "#000000", "#E69F00", "#56B4E9", "#009E73",
    "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#FFFFFF"
)
oi_hatch <- c(
    NA, "orange", "bleu celeste", "vert",
    "or", "azure", "gules", "purpure", NA
)
sx <- c(0, 0, 1, 1)
sy <- c(1, 0, 0, 1)
n <- length(oi_names)

grid.newpage()
grid.rect(gp = gpar(fill = "white", col = NA))
pushViewport(viewport(width = 0.90, height = 0.94))
grid.text(
    "Okabe-Ito Palette with Heraldic Hatching",
    y = unit(1, "npc") - unit(0.25, "cm"),
    just = "top",
    gp = gpar(fontsize = 13, fontface = "bold")
)
pushViewport(viewport(
    y = 0.46, height = 0.88,
    layout = grid.layout(
        n, 3,
        widths = unit(c(3, 2.5, 4), "null"),
        heights = unit(rep(1, n), "null")
    )
))

for (i in seq_len(n)) {
    grid.text(oi_names[i], x = 0.90, just = "right",
              gp = gpar(fontsize = 12, col = "black"),
	          vp = viewport(layout.pos.row = i, layout.pos.col = 1))

    grid.text(oi_hex[i],
              gp = gpar(fontsize = 12, fontfamily = "mono", col = "black"),
	          vp = viewport(layout.pos.row = i, layout.pos.col = 2))

    pushViewport(viewport(layout.pos.row = i, layout.pos.col = 3))
    grid.rect(gp = gpar(fill = oi_hex[i], col = "black", lwd = 3.0))
    if (!is.na(oi_hatch[i])) {
        grid.pattern_hatch(sx, sy, type = oi_hatch[i],
                           colour = "black", spacing = 0.18, linewidth = 0.8)
    }
    popViewport()
}

popViewport()
popViewport()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-pattern.R
\name{grid.pattern}
\alias{grid.pattern}
\alias{names_pattern}
\alias{patternGrob}
\title{Create patterned grobs by pattern name}
\usage{
grid.pattern(
  pattern = "stripe",
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  legend = FALSE,
  prefix = "pattern_",
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_pattern

patternGrob(
  pattern = "stripe",
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  legend = FALSE,
  prefix = "pattern_",
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{pattern}{Name of pattern.  See Details section for a list of supported patterns.}

\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Pattern parameters.}

\item{legend}{Whether this is intended to be drawn in a legend or not.}

\item{prefix}{Prefix to prepend to the name of each of the pattern parameters in \code{...}.
For compatibility with \code{ggpattern} most underlying functions assume parameters beginning with \code{pattern_}.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object (invisibly in the case of \code{grid.pattern()}).
If \code{draw} is \code{TRUE} then \code{grid.pattern()} also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern()} draws patterned shapes onto the graphic device.
\code{patternGrob()} returns the grid grob objects.
\code{names_pattern} is a character vector of builtin patterns.
}
\details{
Here is a list of the various patterns supported:

\describe{
\item{ambient}{Noise array patterns onto the graphic device powered by the \code{ambient} package.
See \code{\link[=grid.pattern_ambient]{grid.pattern_ambient()}} for more information.}
\item{aRtsy}{Patterns powered by the \code{aRtsy} package.
See \code{\link[=grid.pattern_aRtsy]{grid.pattern_aRtsy()}} for more information.}
\item{circle}{Circle geometry patterns.
See \code{\link[=grid.pattern_circle]{grid.pattern_circle()}} for more information.}
\item{crosshatch}{Crosshatch geometry patterns.
See \code{\link[=grid.pattern_crosshatch]{grid.pattern_crosshatch()}} for more information.}
\item{gradient}{Gradient array/geometry patterns.
See \code{\link[=grid.pattern_gradient]{grid.pattern_gradient()}} for more information.}
\item{hatch}{Heraldic hatching patterns.
See \code{\link[=grid.pattern_hatch]{grid.pattern_hatch()}} for more information.}
\item{image}{Image array patterns.
See \code{\link[=grid.pattern_image]{grid.pattern_image()}} for more information.}
\item{line}{Line geometry patterns.
See \code{\link[=grid.pattern_line]{grid.pattern_line()}} for more information.}
\item{magick}{\code{imagemagick} array patterns.
See \code{\link[=grid.pattern_magick]{grid.pattern_magick()}} for more information.}
\item{none}{Does nothing.
See \code{\link[grid:grid.null]{grid::grid.null()}} for more information.}
\item{pch}{Plotting character geometry patterns.
See \code{\link[=grid.pattern_pch]{grid.pattern_pch()}} for more information.}
\item{placeholder}{Placeholder image array patterns.
See \code{\link[=grid.pattern_placeholder]{grid.pattern_placeholder()}} for more information.}
\item{plasma}{Plasma array patterns.
See \code{\link[=grid.pattern_plasma]{grid.pattern_plasma()}} for more information.}
\item{polygon_tiling}{Polygon tiling patterns.
See \code{\link[=grid.pattern_polygon_tiling]{grid.pattern_polygon_tiling()}} for more information.}
\item{regular_polygon}{Regular polygon patterns.
See \code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} for more information.}
\item{rose}{Rose array/geometry patterns.
See \code{\link[=grid.pattern_rose]{grid.pattern_rose()}} for more information.}
\item{stripe}{Stripe geometry patterns.
See \code{\link[=grid.pattern_stripe]{grid.pattern_stripe()}} for more information.}
\item{text}{Text array/geometry patterns.
See \code{\link[=grid.pattern_text]{grid.pattern_text()}} for more information.}
\item{wave}{Wave geometry patterns.
See \code{\link[=grid.pattern_wave]{grid.pattern_wave()}} for more information.}
\item{weave}{Weave geometry patterns.
See \code{\link[=grid.pattern_weave]{grid.pattern_weave()}} for more information.}
\item{Custom geometry-based patterns}{See \url{https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html} for more information.}
\item{Custom array-based patterns}{See \url{https://trevorldavis.com/R/gridpattern/dev/articles/developing-patterns.html} for more information.}
}
}
\examples{
 print(names_pattern)
 \donttest{# May take more than 5 seconds on CRAN servers
 x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
 y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))

 # geometry-based patterns
 # 'stripe' pattern
 grid::grid.newpage()
 grid.pattern("stripe", x_hex, y_hex,
              colour="black", fill=c("yellow", "blue"), density = 0.5)

 # Can alternatively use "gpar()" to specify colour and line attributes
 grid::grid.newpage()
 grid.pattern("stripe", x_hex, y_hex,
              gp = grid::gpar(col="blue", fill="red", lwd=2))

 # 'weave' pattern
 grid::grid.newpage()
 grid.pattern("weave", x_hex, y_hex, type = "satin",
              colour = "black", fill = "lightblue", fill2 =  "yellow",
              density = 0.3)

 # 'regular_polygon' pattern
 grid::grid.newpage()
 grid.pattern_regular_polygon(x_hex, y_hex, colour = "black",
                              fill = c("blue", "yellow", "red"),
                              shape = c("convex4", "star8", "circle"),
                              density = c(0.45, 0.42, 0.4),
                              spacing = 0.08, angle = 0)

 # can be used to achieve a variety of 'tiling' effects
 grid::grid.newpage()
 grid.pattern_regular_polygon(x_hex, y_hex, color = "transparent",
                              fill = c("white", "grey", "black"),
                              density = 1.0, spacing = 0.1,
                              shape = "convex6", grid = "hex")
 if (suppressPackageStartupMessages(requireNamespace("magick", quietly = TRUE))) {
   # array-based patterns
   # 'image' pattern
   logo_filename <- system.file("img", "Rlogo.png" , package="png")
   grid::grid.newpage()
   grid.pattern("image", x_hex, y_hex, filename=logo_filename, type="fit")
 }
 if (suppressPackageStartupMessages(requireNamespace("magick", quietly = TRUE))) {
   # 'plasma' pattern
   grid::grid.newpage()
   grid.pattern("plasma", x_hex, y_hex, fill="green")
 }
 }
}
\seealso{
\url{https://coolbutuseless.github.io/package/ggpattern/index.html}
for more details on the \code{ggpattern} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-array-ambient.R
\name{grid.pattern_ambient}
\alias{grid.pattern_ambient}
\title{Ambient patterned grobs}
\usage{
grid.pattern_ambient(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  type = "simplex",
  fill = gp$fill \%||\% "grey80",
  fill2 = "#4169E1",
  frequency = 0.01,
  interpolator = "quintic",
  fractal = switch(type, worley = "none", "fbm"),
  octaves = 3,
  lacunarity = 2,
  gain = 0.5,
  pertubation = "none",
  pertubation_amplitude = 1,
  value = "cell",
  distance_ind = c(1, 2),
  jitter = 0.45,
  res = getOption("ggpattern_res", 72),
  alpha = NA_real_,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{type}{Either cubic, perlin, simplex, value, white, or worley}

\item{fill}{Colour.}

\item{fill2}{Second colour.}

\item{frequency}{Determines the granularity of the features in the noise.}

\item{interpolator}{How should values between sampled points be calculated?
Either \code{'linear'}, \code{'hermite'}, or \code{'quintic'} (default), ranging from lowest
to highest quality.}

\item{fractal}{The fractal type to use. Either \code{'none'}, \code{'fbm'} (default),
\code{'billow'}, or \code{'rigid-multi'}. It is suggested that you experiment with the
different types to get a feel for how they behaves.}

\item{octaves}{The number of noise layers used to create the fractal noise.
Ignored if \code{fractal = 'none'}. Defaults to \code{3}.}

\item{lacunarity}{The frequency multiplier between successive noise layers
when building fractal noise. Ignored if \code{fractal = 'none'}. Defaults to \code{2}.}

\item{gain}{The relative strength between successive noise layers when
building fractal noise. Ignored if \code{fractal = 'none'}. Defaults to \code{0.5}.}

\item{pertubation}{The pertubation to use. Either \code{'none'} (default),
\code{'normal'}, or \code{'fractal'}. Defines the displacement (warping) of the noise,
with \code{'normal'} giving a smooth warping and \code{'fractal'} giving a more eratic
warping.}

\item{pertubation_amplitude}{The maximal pertubation distance from the
origin. Ignored if \code{pertubation = 'none'}. Defaults to \code{1}.}

\item{value}{The noise value to return. Either
\itemize{
\item \code{'value'} (default) A random value associated with the closest point
\item \code{'distance'} The distance to the closest point
\item \code{'distance2'} The distance to the nth closest point (n given by
\code{distance_ind[1]})
\item \code{'distance2add'} Addition of the distance to the nth and mth closest point given in \code{distance_ind}
\item \code{'distance2sub'} Substraction of the distance to the nth and mth closest point given in \code{distance_ind}
\item \code{'distance2mul'} Multiplication of the distance to the nth and mth closest point given in \code{distance_ind}
\item \code{'distance2div'} Division of the distance to the nth and mth closest point given in \code{distance_ind}
}}

\item{distance_ind}{Reference to the nth and mth closest points that should
be used when calculating \code{value}.}

\item{jitter}{The maximum distance a point can move from its start position
during sampling of cell points.}

\item{res}{Assumed resolution (in pixels per graphic device inch) to use when creating array pattern.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_ambient()} draws noise patterns onto the graphic device powered by the \code{ambient} package.
}
\examples{
 if (requireNamespace("ambient", quietly = TRUE)) {
   x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
   y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
   grid.pattern_ambient(x_hex, y_hex, fill = "green", fill2 = "blue")
 }
 if (requireNamespace("ambient")) {
   grid::grid.newpage()
   grid.pattern_ambient(x_hex, y_hex, fill = "green", fill2 = "blue", type = "cubic")
 }
}
\seealso{
For more information about the noise types please see the relevant \code{ambient} documentation:
\code{\link[ambient:noise_cubic]{ambient::noise_cubic()}}, \code{\link[ambient:noise_perlin]{ambient::noise_perlin()}}, \code{\link[ambient:noise_simplex]{ambient::noise_simplex()}},
\code{\link[ambient:noise_value]{ambient::noise_value()}}, \code{\link[ambient:noise_white]{ambient::noise_white()}}, and \code{\link[ambient:noise_worley]{ambient::noise_worley()}}.
\code{\link[=grid.pattern_plasma]{grid.pattern_plasma()}} provides an alternative noise pattern that depends on \code{magick}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-array-placeholder.R
\name{grid.pattern_placeholder}
\alias{grid.pattern_placeholder}
\alias{names_placeholder}
\title{Placeholder image patterned grobs}
\usage{
grid.pattern_placeholder(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  type = "bear",
  alpha = gp$alpha \%||\% NA_real_,
  aspect_ratio = 1,
  key_scale_factor = 1,
  res = getOption("ggpattern_res", 72),
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_placeholder
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{type}{Image source.  \code{names_placeholder} is a vector of supported values.
If you would like only greyscale images append \code{bw} to the name.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{aspect_ratio}{Override aspect ratio.}

\item{key_scale_factor}{Additional scale factor for legend.}

\item{res}{Assumed resolution (in pixels per graphic device inch) to use when creating array pattern.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_placeholder()} draws a placeholder image pattern onto the graphic device.
\code{names_placeholder} are character vectors of supported placeholder types.
}
\examples{
  if (requireNamespace("magick")) {
    x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
    y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
    # requires internet connection to download from placeholder image websites
    try(grid.pattern_placeholder(x_hex, y_hex, type="bear"))
  }

  print(names_placeholder)
}
\seealso{
\code{\link[=reset_image_cache]{reset_image_cache()}} resets the image cache used by \code{\link[=grid.pattern_image]{grid.pattern_image()}} and \code{grid.pattern_placeholder()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern_hex.R
\name{pattern_hex}
\alias{pattern_hex}
\alias{names_hex}
\title{Hex pattern matrix}
\usage{
pattern_hex(type = "hex", subtype = NULL, nrow = 5L, ncol = 5L)

names_hex
}
\arguments{
\item{type}{Currently just supports "hex".}

\item{subtype}{An integer indicating number of colors (or other graphical elements).}

\item{nrow}{Number of rows (height).}

\item{ncol}{Number of columns (width).}
}
\value{
A matrix of integer values indicating where the each color
or other graphical elements should be drawn on a horizontal hex grid
(i.e. hexagons are assumed to be pointy side up).
Indices \verb{[1,1]} of the matrix corresponds to the bottom-left of the grid
while indices \verb{[1,ncol]} corresponds to the bottom-right of the grid.
The even rows are assumed to be on the \strong{left} of the ones on the odd rows
(for those in the same column in the matrix).
This matrix has a "pattern_hex" subclass which supports a special \code{print()} method.
}
\description{
\code{pattern_hex()} returns an integer matrix indicating where each
color (or other graphical element) should be drawn on a (horizontal) hex grid
for a specified hex pattern type and subtype.
\code{names_hex} lists the currently supported hex \code{type}s.
}
\details{
\describe{
\item{"hex"}{Attempts to use a uniform coloring if it exists.
For subtype \code{1L}, \code{2L}, and \code{3L} we use the "hex1" pattern.
For subtype \code{4L} we use the "hex2" pattern.
For subtype \code{7L} we use the "hex3" pattern.
Else a uniform coloring does not exist and we use the "hex_skew" pattern.}
\item{"hex1"}{Provides the 1-uniform colorings of a hexagonal tiling.  Only exists for \code{subtype} \code{1L}, \code{2L}, or \code{3L}.}
\item{"hex2"}{Provides the 2-uniform colorings of a hexagonal tiling.  Only exists for \code{subtype} \code{2L} or \code{4L}.}
\item{"hex3"}{Provides the 3-uniform colorings of a hexagonal tiling.  Only exists for \code{subtype} \code{2L} or \code{7L}.}
\item{"hex_skew"}{For the "hex_skew" \code{type} we cycle through \code{subtype} elements on the horizontal line and "main" diagonal line.
For some \code{subtype} numbers this may lead to noticeable color repeats on the "skew" diagonal line.
If \code{subtype} is strictly greater than \code{2L} then a hexagon should never touch another hexagon of the same color.}
}
}
\examples{
 # supported hex names
 print(names_hex)

 # 1-uniform 3-color
 hex_3color <- pattern_hex("hex1", 3L, nrow = 7L, ncol = 9L)
 print(hex_3color)

 # 2-uniform 4-color
 hex_4color <- pattern_hex("hex2", 4L, nrow = 7L, ncol = 9L)
 print(hex_4color)

}
\seealso{
\code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} for drawing to a graphics device
hexagons, triangles, circles, etc. in hexagon patterns.
The tiling vignette features several examples of regular polygon tiling using
this both the "hex" and "hex_circle" types \code{vignette("tiling", package = "gridpattern")}.
For more information on uniform colorings of a hexagonal tiling see
\url{https://en.wikipedia.org/wiki/Hexagonal_tiling#Uniform_colorings}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-geometry-weave.R
\name{grid.pattern_weave}
\alias{grid.pattern_weave}
\title{Weave patterned grobs}
\usage{
grid.pattern_weave(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  colour = gp$col \%||\% "grey20",
  fill = gp$fill \%||\% "grey80",
  fill2 = fill,
  angle = 30,
  density = 0.2,
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  alpha = gp$alpha \%||\% NA_real_,
  linetype = gp$lty \%||\% 1,
  linewidth = size \%||\% gp$lwd \%||\% 1,
  size = NULL,
  grid = "square",
  type = "plain",
  subtype = NA,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{colour}{Stroke colour(s).}

\item{fill}{The fill colour for the horizontal "weft" lines.}

\item{fill2}{The fill colour for the vertical "warp" lines.}

\item{angle}{Rotation angle in degrees.}

\item{density}{Approx. fraction of area the pattern fills.}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{linetype}{Stroke linetype.}

\item{linewidth}{Stroke linewidth.}

\item{size}{For backwards compatibility can be used to set \code{linewidth}.}

\item{grid}{Adjusts placement and density of certain graphical elements.
\code{"square"} (default) is a square grid.
\code{"hex"} is a hexagonal grid suitable for hexagonal and triangular tiling.
\code{"hex_circle"} is a hexagonal grid suitable for circle packing.
\code{"elongated_triangle"} is a grid used for the "elongated triangle" tiling.}

\item{type}{The weave type.  See \code{\link[=pattern_weave]{pattern_weave()}} for more details.}

\item{subtype}{The weave subtype.  See \code{\link[=pattern_weave]{pattern_weave()}} for more details.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_weave()} draws a weave pattern onto the graphic device.
}
\examples{
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
gp <- grid::gpar(colour = "black", fill = "lightblue", lwd=0.5)

# Plain weave (default weave)
grid.pattern_weave(x_hex, y_hex, fill2 = "yellow",
                   gp = gp, spacing = 0.1, density = 0.3)

# Irregular matt weave
grid::grid.newpage()
grid.pattern_weave(x_hex, y_hex,  type = "matt_irregular",
                   fill2 = "yellow", gp = gp, spacing = 0.1, density = 0.3)

# Twill weave
grid::grid.newpage()
grid.pattern_weave(x_hex, y_hex, type = "twill",
                   fill2 = "yellow", gp = gp, spacing = 0.1, density = 0.3)

# Zig-zag twill
grid::grid.newpage()
grid.pattern_weave(x_hex, y_hex, type = "twill_zigzag",
                   fill2 = "yellow", gp = gp, spacing = 0.05, density = 0.7)

# Herringbone twill with density 1
grid::grid.newpage()
gp$col <- NA
grid.pattern_weave(x_hex, y_hex, type = "twill_herringbone",
                   fill2 = "yellow", gp = gp, spacing = 0.05, density = 1.0)
}
\seealso{
\code{\link[=pattern_weave]{pattern_weave()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-pattern-fill.R
\name{patternFill}
\alias{patternFill}
\title{Create patterned fills by pattern name}
\usage{
patternFill(
  ...,
  x = 0.5,
  y = 0.5,
  width = 1,
  height = 1,
  default.units = "npc",
  just = "centre",
  hjust = NULL,
  vjust = NULL,
  group = TRUE
)
}
\arguments{
\item{...}{Passed to \code{\link[=patternGrob]{patternGrob()}}.}

\item{x, y, width, height}{The size of the \code{\link[grid:pattern]{grid::pattern()}} tile.}

\item{default.units}{The default \code{\link[grid:unit]{grid::unit()}} unit to use for \code{x}, \code{y}, \code{width}, and \code{height}.}

\item{just, hjust, vjust}{The justification of the tile relative to its location.}

\item{group}{A logical indicating whether the pattern is relative to the bounding box of the grob or whether it is relative to individual shapes within the grob.  Ignored if R is less than version 4.2.}
}
\value{
A \code{\link[grid:pattern]{grid::pattern()}} fill object.
}
\description{
\code{patternFill()} returns \code{\link[grid:pattern]{grid::pattern()}} fill objects.
It is a wrapper around \code{\link[=patternGrob]{patternGrob()}}.
}
\examples{
if (guess_has_R4.1_features("patterns") &&
    require("grid", quietly = TRUE)) {
  grid.newpage()
  stripe_fill <- patternFill("stripe", fill = c("red", "blue"))
  grid.circle(gp = gpar(fill = stripe_fill))
}

if (guess_has_R4.1_features("patterns") &&
    require("ggplot2", quietly = TRUE) &&
    (getRversion() >= "4.2")) {
  grid.newpage()
  weave_fill <- patternFill("weave", fill = "red", fill2 = "blue",
                            colour = "transparent")
  hex_fill <- patternFill("polygon_tiling", type = "hexagonal",
                          fill = c("black", "white", "grey"),
                          colour = "transparent")
  df <- data.frame(trt = c("a", "b"), outcome = c(1.9, 3.2))
  gg <- ggplot(df, aes(trt, outcome)) +
    geom_col(fill = list(weave_fill, hex_fill))
  plot(gg)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-geometry-none.R
\name{grid.pattern_none}
\alias{grid.pattern_none}
\title{Grobs without any pattern}
\usage{
grid.pattern_none(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_none()} draws nothing onto the graphic device.
}
\examples{
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
grid.pattern_none(x_hex, y_hex)
}
\seealso{
\code{\link[grid:grid.null]{grid::grid.null()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-array-image.R
\name{grid.pattern_image}
\alias{grid.pattern_image}
\title{Image patterned grobs}
\usage{
grid.pattern_image(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  filename = "",
  type = "fit",
  scale = 1,
  gravity = switch(type, tile = "southwest", "center"),
  filter = "lanczos",
  alpha = gp$alpha \%||\% NA_real_,
  aspect_ratio = 1,
  key_scale_factor = 1,
  res = getOption("ggpattern_res", 72),
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{filename}{Image of filename or URL}

\item{type}{Image scaling type}

\item{scale}{Extra scaling}

\item{gravity}{Position of image within area.  \code{magick::gravity_types()} returns a vector of supported values.}

\item{filter}{Filter to use when scaling. \code{magick::filter_types()} returns a vector of supported values.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{aspect_ratio}{Override aspect ratio.}

\item{key_scale_factor}{Additional scale factor for legend.}

\item{res}{Assumed resolution (in pixels per graphic device inch) to use when creating array pattern.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_image()} draws an image pattern onto the graphic device.
}
\details{
Here is a description of the \code{type} arguments:
\describe{
\item{expand}{Scale the image beyond the bounding box and crop it such that
the image fully covers the width and the height of the region.}
\item{fit}{Scale the image such that either the width or the height of the image fits in the bounding box.
Affected by \code{gravity}}
\item{none}{Position a single image in the region without attempting to scale to the bounding box size.
Affected by \code{scale} and \code{gravity}.}
\item{squish}{Distort the image to cover the bounding box of the region.}
\item{tile}{Repeat the image to cover the bounding box.  Affected by \code{tile}.}
}
}
\examples{
  \donttest{# May emit a "CPU time > 2.5 times elapsed time" NOTE in a CRAN check
  if (requireNamespace("magick")) {
    x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
    y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
    logo_filename <- system.file("img", "Rlogo.png" , package = "png")
    grid.pattern_image(x_hex, y_hex, filename = logo_filename, type = "fit")
  }
  if (requireNamespace("magick")) {
    # "tile" `type` image pattern depends on `magick` functionality
    # which is not reliable across platforms
    grid::grid.newpage()
    try(grid.pattern_image(x_hex, y_hex, filename = logo_filename,
                           type = "tile"))
  }
  }
}
\seealso{
\code{\link[=grid.pattern_placeholder]{grid.pattern_placeholder()}} is an image pattern that uses images
downloaded from the internet.
\code{\link[=reset_image_cache]{reset_image_cache()}} resets the image cache used by \code{grid.pattern_image()}
and \code{\link[=grid.pattern_placeholder]{grid.pattern_placeholder()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reset_image_cache.R
\name{reset_image_cache}
\alias{reset_image_cache}
\title{Reset 'gridpattern' image cache}
\usage{
reset_image_cache()
}
\description{
\code{\link[=grid.pattern_image]{grid.pattern_image()}} and \code{\link[=grid.pattern_placeholder]{grid.pattern_placeholder()}} store images in a cache
(so we won't download image URLs over and over).
\code{reset_image_cache()} resets this cache.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-geometry-circle.R
\name{grid.pattern_circle}
\alias{grid.pattern_circle}
\title{Circle patterned grobs}
\usage{
grid.pattern_circle(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  colour = gp$col \%||\% "grey20",
  fill = gp$fill \%||\% "grey80",
  angle = 30,
  density = 0.2,
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  alpha = gp$alpha \%||\% NA_real_,
  linetype = gp$lty \%||\% 1,
  linewidth = size \%||\% gp$lwd \%||\% 1,
  size = NULL,
  grid = "square",
  type = NULL,
  subtype = NULL,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{colour}{Stroke colour(s).}

\item{fill}{Fill colour(s) or \code{\link[grid:pattern]{grid::pattern()}} / gradient object(s).}

\item{angle}{Rotation angle in degrees.}

\item{density}{Approx. fraction of area the pattern fills.}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{linetype}{Stroke linetype.}

\item{linewidth}{Stroke linewidth.}

\item{size}{For backwards compatibility can be used to set \code{linewidth}.}

\item{grid}{Adjusts placement and density of certain graphical elements.
\code{"square"} (default) is a square grid.
\code{"hex"} is a hexagonal grid suitable for hexagonal and triangular tiling.
\code{"hex_circle"} is a hexagonal grid suitable for circle packing.
\code{"elongated_triangle"} is a grid used for the "elongated triangle" tiling.}

\item{type}{Adjusts the repeating of certain aesthetics such as color.
Can use any type in \code{names_hex}, \code{names_square}, or \code{names_weave}.
See for \code{\link[=pattern_hex]{pattern_hex()}}, \code{\link[=pattern_square]{pattern_square()}}, and \code{\link[=pattern_weave]{pattern_weave()}} for
more information about supported \code{type} arguments.}

\item{subtype}{See for \code{\link[=pattern_hex]{pattern_hex()}}, \code{\link[=pattern_square]{pattern_square()}}, and \code{\link[=pattern_weave]{pattern_weave()}} for
more information about supported \code{subtype} arguments.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_circle()} draws a circle pattern onto the graphic device.
}
\examples{
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
grid.pattern_circle(x_hex, y_hex, fill = c("blue", "yellow"), density = 0.5)
grid::grid.newpage()
grid.pattern_circle(x_hex, y_hex, density = 0.8, grid = "hex_circle",
                    gp = grid::gpar(fill = c("blue", "yellow", "red")))
grid::grid.newpage()
grid.pattern_circle(x_hex, y_hex, density = 1.2, grid = "hex_circle",
                    gp = grid::gpar(fill = c("blue", "yellow", "red")))
# using a "twill_zigzag" 'weave' pattern
grid::grid.newpage()
grid.pattern_circle(x_hex, y_hex, fill = "blue", density = 0.5, type = "twill_zigzag")
}
\seealso{
See \code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} for a more general case of this pattern.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-pattern-aRtsy.R
\name{grid.pattern_aRtsy}
\alias{grid.pattern_aRtsy}
\alias{names_aRtsy}
\title{Grobs with patterns powered by the aRtsy package}
\usage{
grid.pattern_aRtsy(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  type = "strokes",
  fill = gp$fill \%||\% "grey80",
  alpha = gp$alpha \%||\% NA_real_,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_aRtsy()
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored}

\item{type}{Name of pattern.}

\item{fill}{Passed to the underlying \code{aRtsy} function's \code{color} / \code{colors} argument.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_aRtsy()} draws patterns powered by the \code{aRtsy} package.
\code{names_aRtsy()} returns character vector of supported types.
}
\examples{
if (requireNamespace("aRtsy", quietly = TRUE)) {
  print(names_aRtsy())
}

\donttest{# Make take more than 5 seconds on CRAN servers
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
if (requireNamespace("aRtsy", quietly = TRUE) &&
    guess_has_R4.1_features("patterns")) {
  grid::grid.newpage()
  grid.pattern_aRtsy(x_hex, y_hex, type = "forest",
                     fill = c("black", "white", "grey"))
}
}
}
\seealso{
\url{https://koenderks.github.io/aRtsy/} for more information about the \code{aRtsy} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-color.R
\name{mean_col}
\alias{mean_col}
\title{Compute average color}
\usage{
mean_col(...)
}
\arguments{
\item{...}{Colors to average}
}
\value{
A color string of 9 characters: \code{"#"} followed by the
red, blue, green, and alpha values in hexadecimal.
}
\description{
\code{mean_col()} computes an average color.
}
\details{
We currently compute an average color
by using the quadratic mean of the colors' RGBA values.
}
\examples{
 mean_col("black", "white")
 mean_col(c("black", "white"))
 mean_col("red", "blue")
}
\seealso{
\code{\link[=mix_col]{mix_col()}} for subtractive (pigment) mixing via Munsell color space.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/star_scale.R
\name{star_scale}
\alias{star_scale}
\alias{star_angle}
\title{Compute regular star polygon scale or angles}
\usage{
star_scale(n_vertices, angle, external = FALSE)

star_angle(n_vertices, scale, external = FALSE)
}
\arguments{
\item{n_vertices}{Number of exterior vertices.}

\item{angle}{Angle in degrees.}

\item{external}{If \code{TRUE} angle should be considered an external angle.}

\item{scale}{Scale from 0 to 1.}
}
\value{
\code{star_scale()} returns a numeric value between 0 and 1 intended
for use as the \code{scale} argument in \code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}}.
\code{star_angle()} returns a numeric value between 0 and 360 (degrees).
}
\description{
\code{star_scale()} computes star \code{scale} value given
an internal or external angle.  \code{star_angle()} computes
star angle (internal or external) given a \code{scale} value.
}
\details{
\code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} parameterizes regular star polygons
with the number of its external vertices and a \code{scale} that equals the
fraction of the radius of the circle that circumscribes the interior vertices
divided by the radius of the circle that circumscribes the exterior vertices.
These helper functions help convert between that parameterization
and either the internal or external angle of the regular star polygon.
}
\examples{
  # |8/3| star has internal angle 45 degrees and external angle 90 degrees
  scale <- star_scale(8, 45)
  scale2 <- star_scale(8, 90, external = TRUE)
  all.equal(scale, scale2)
  star_angle(8, scale)
  star_angle(8, scale, external = TRUE)

  grid.pattern_regular_polygon(shape = "star8", scale = scale, angle = 0,
                               spacing = 0.2, density = 0.8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern_weave.R
\name{pattern_weave}
\alias{pattern_weave}
\alias{names_weave}
\title{Weave pattern matrix}
\usage{
pattern_weave(type = "plain", subtype = NULL, nrow = 5L, ncol = 5L)

names_weave
}
\arguments{
\item{type}{Type of weave.  See Details.}

\item{subtype}{Subtype of weave.  See Details.}

\item{nrow}{Number of rows (length of warp).}

\item{ncol}{Number of columns (length of weft).}
}
\value{
A matrix of logical values indicating where the "warp"
is "up" (if \code{TRUE}) or "down" (if \code{FALSE}).
Indices \verb{[1,1]} of the matrix corresponds to the bottom-left of the weave
while indices \verb{[1,ncol]} corresponds to the bottom-right of the weave.
This matrix has a "pattern_weave" subclass which supports a special \code{print()} method.
}
\description{
\code{pattern_weave()} returns a logical matrix indicating where the warp lines should
be "up" for a specified weave pattern type and subtype.
\code{names_weave} is a character vector listing supported weave pattern types.
}
\details{
Here is a list of the various weave \code{type}s supported:

\describe{
\item{basket}{A simple criss-cross pattern using two threads at a time.
Same as the "matt_irregular" weave but with a default \code{subtype} of \code{2L}.}
\item{matt}{A simple criss-cross pattern using 3 (or more) threads at a time.
Same as the "matt_irregular" weave but with a default \code{subtype} of \code{3L}.}
\item{matt_irregular}{A generalization of the "plain" weave.
A character \code{subtype} \code{"U/D(L+R)"} is a standard matt weave specification:
\code{U} indicates number warp up, \code{D} indicates number warp down,
\code{L} indicates number of warp up in repeat, and
\code{R} indicates number of warp down in repeat.
An integer \code{subtype} \code{N} will be interpreted as a \code{"N/N(N+N)"} irregular matt weave.
A character \code{subtype} \code{"U/D"} will be interpreted as a \code{"U/D(U+D)"} irregular matt weave.
Has a default \code{subtype} of \code{"3/2(4+2)"}.}
\item{plain}{A simple criss-cross pattern.
Same as the "matt_irregular" weave but with a default \code{subtype} of \code{1L}.}
\item{rib_warp}{A plain weave variation that emphasizes vertical lines.
An integer \code{subtype} \code{N} will be interpreted as a "matt_irregular" \code{"N/N(1+1)"} weave.
A character \code{subtype} \code{"U/D"} will be interpreted as a "matt_irregular" \code{"U/D(1+1)"} weave.
Default \code{subtype} of \code{2L}.}
\item{satin}{A "regular" satin weave is a special type of the elongated twill weave
with a move number carefully chosen so no twill line is distinguishable.
Same as the "twill_elongated" weave but with a default \code{subtype} of \code{5L}.}
\item{twill}{A simple diagonal pattern.
Same as the "twill_elongated" weave but with a default \code{subtype} of \code{"2/1"}.}
\item{twill_elongated}{A generalization of the "twill" weave.
A character \code{subtype} \code{"U/D(M)"} is a standard twill weave specification:
\code{U} indicates number warp up, \code{D} indicates number warp down,
and \code{M} indicates the "move" number.
A character \code{subtype} \code{"U/D"} will be interpreted as a \code{"U/D(1)"} elongated twill weave.
An integer \code{subtype} \code{N} will provide a \code{"{N-1}/1(1)"} elongated twill weave
if \code{N} is less than 5, 6, or greater than 14 otherwise it will
provide a \code{"{N-1}/1(M)"} weave where \code{M} is the largest
possible regular "satin" move number.
Default \code{subtype} of \code{"4/3(2)"}.}
\item{twill_herringbone}{Adds a (vertical) "herringbone" effect to
the specified "twill_elongated" weave.
Default \code{subtype} of \code{"4/3(2)"}.}
\item{twill_zigzag}{Adds a (vertical) "zig-zag" effect to the specified "twill_elongated" weave.
Default \code{subtype} of \code{"4/3(2)"}.}
}
For both "matt" and "twill" weaves the \code{U/D} part of the subtype can be further extended
to \code{U1/D1*U2/D2}, \code{U1/D1*U2/D2*U3/D3}, etc.
For the "matt" weave the "(L+R)" part of the subtype can be further extended
to \code{(L1+R1+L2+R2)}, \code{(L1+R1+L2+R2+L3+R3)}, etc.
}
\examples{
 # supported weave names
 print(names_weave)

 plain <- pattern_weave("plain", nrow = 7, ncol = 9)
 print(plain)

 matt_irregular <- pattern_weave("matt_irregular", nrow = 9, ncol = 11)
 print(matt_irregular)

 satin <- pattern_weave("satin", nrow = 9, ncol = 11)
 print(satin)

 twill <- pattern_weave("twill", nrow = 9, ncol = 11)
 print(twill)

 twill_zigzag <- pattern_weave("twill_zigzag", nrow = 18, ncol = 11)
 print(twill_zigzag)

}
\seealso{
\code{\link[=grid.pattern_weave]{grid.pattern_weave()}} for drawing weaves onto a graphics device.
See \url{https://textilestudycenter.com/derivatives-of-plain-weave/}
for further information on the "matt" family of weaves,
\url{https://textilelearner.net/twill-weave-features-classification-derivatives-and-uses/}
for further information on the "twill" family of weaves, and
\url{https://texwiz101.blogspot.com/2012/03/features-and-classification-of-satin.html}
for further information on "satin" weaves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-color.R
\name{mix_col}
\alias{mix_col}
\title{Mix colors via Munsell color space}
\usage{
mix_col(..., w = NULL, mixingMethod = "adaptive")
}
\arguments{
\item{...}{Colors to mix.  Can be individual color strings or character vectors;
all are combined into a single vector.}

\item{w}{A numeric vector of weights or proportions the same length as the combined color vector.
Defaults to equal weights.}

\item{mixingMethod}{Mixing method passed to \code{\link[aqp:mixMunsell]{aqp::mixMunsell()}}.}
}
\value{
A single R color string in hex notation.
}
\description{
\code{mix_col()} simulates subtractive (pigment) color mixing by converting
input colors to Munsell notation via \code{\link[aqp:col2Munsell]{aqp::col2Munsell()}}, mixing them with
\code{\link[aqp:mixMunsell]{aqp::mixMunsell()}}, and converting the result back to an R color string
via \code{\link[aqp:parseMunsell]{aqp::parseMunsell()}}.
}
\examples{
should_run_ex <- requireNamespace("aqp", quietly = TRUE) &&
    (interactive() || identical(Sys.getenv("IN_PKGDOWN"), "true"))
if (should_run_ex) {
  mix_col(c("red", "blue"))
}
if (should_run_ex) {
  mix_col("red", "yellow", "blue", w = c(2, 1, 1))
}
}
\seealso{
\code{\link[=mean_col]{mean_col()}} for a simpler quadratic-mean RGB approach (no extra packages required).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-both-hatch.R
\name{grid.pattern_hatch}
\alias{grid.pattern_hatch}
\alias{names_hatch}
\title{Heraldic color hatching patterned grobs}
\usage{
grid.pattern_hatch(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  type = "gules",
  subtype = "combinatorial",
  colour = gp$col \%||\% "grey20",
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  alpha = gp$alpha \%||\% NA_real_,
  lineend = gp$lineend \%||\% "round",
  linewidth = size \%||\% gp$lwd \%||\% 1,
  size = NULL,
  use_R4.1_masks = getOption("ggpattern_use_R4.1_masks",
    getOption("ggpattern_use_R4.1_features")),
  png_device = NULL,
  res = getOption("ggpattern_res", 72),
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_hatch(
  subtype = c("combinatorial", "fox-davies", "goodman", "unicode"),
  accent = FALSE
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{type}{A tincture or color name.  \code{names_hatch()} lists supported values.
Both traditional tincture names (e.g. \code{"gules"}) and modern color equivalents
(e.g. \code{"red"}) are accepted.  Matching is case-insensitive and ignores hyphens
and spaces.}

\item{subtype}{A string with one of
\itemize{
\item \code{"combinatorial"} (default): an extension of the seven standard Petra Sancta hatchings with systematically derived mixed-color hatchings.
\item \code{"fox-davies"}: the hatchings in Fox-Davies' \emph{A Complete Guide to Heraldry}.
\item \code{"goodman"}: the hatchings in David Goodman's \emph{Heraldic Tincture} reference.
\item \code{"unicode"}: the hatchings used in Unicode character charts.
}

The string is case-insensitive and hyphens and spaces are ignored.}

\item{colour}{Stroke colour(s).}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{lineend}{Line end style, one of \code{"round"} (default), \code{"butt"}, or \code{"square"}.}

\item{linewidth}{Stroke linewidth.}

\item{size}{For backwards compatibility can be used to set \code{linewidth}.}

\item{use_R4.1_masks}{If \code{TRUE} use the grid mask feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation.
If \code{NULL} try to guess an appropriate choice.
Note not all graphic devices support the grid mask feature.}

\item{png_device}{\dQuote{png} graphics device to save intermediate raster data with if \code{use_R4.1_masks} is \code{FALSE}.
If \code{NULL} and suggested package \code{ragg} is available
and versions are high enough we directly capture masked raster via \code{\link[ragg:agg_capture]{ragg::agg_capture()}}.
Otherwise we will use \code{png_device}
(default \code{\link[ragg:agg_png]{ragg::agg_png()}} if available else \code{\link[grDevices:png]{grDevices::png()}}) and \code{\link[png:readPNG]{png::readPNG()}}
to manually compute a masked raster.}

\item{res}{Resolution of desired \code{rasterGrob} in pixels per inch if \code{use_R4.1_masks} is \code{FALSE}.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}

\item{accent}{If \code{TRUE}, return tincture names using their traditional
accented spellings where applicable (e.g. \code{"tenn\\u00e9"}, \code{"brun\\u00e2tre"}).
Defaults to \code{FALSE}.}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_hatch()} draws a heraldic color hatching patterns onto the graphic device.
\code{names_hatch()} returns a character vector of supported \code{type} values.
}
\examples{
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
if (capabilities("png") || guess_has_R4.1_features("masks")) {
  grid::grid.newpage()
  grid.pattern_hatch(x_hex, y_hex, type = "azure", colour = "blue")
}
if (capabilities("png") || guess_has_R4.1_features("masks")) {
  grid::grid.newpage()
  grid.pattern_hatch(x_hex, y_hex, type = "cendree", colour = "grey", spacing = 0.1)
}
print(names_hatch())
print(names_hatch("fox-davies"))
print(names_hatch("goodman"))
print(names_hatch("unicode"))
}
\seealso{
\code{\link[=grid.pattern_line]{grid.pattern_line()}} for single-direction lines,
\code{\link[=grid.pattern_crosshatch]{grid.pattern_crosshatch()}} for perpendicular lines.
\code{vignette("hatching", package = "gridpattern")} for a visual overview of all subtypes.
\url{https://en.wikisource.org/wiki/A_Complete_Guide_to_Heraldry/Chapter_7#74} for Fox-Davies'
\emph{A Complete Guide to Heraldry}.
\url{https://david.goodman.graphics/portfolio/item/crests-heraldry-and-coats-of-arms/} for
David Goodman's Heraldic Tincture reference.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-geometry-tiling.R
\name{grid.pattern_polygon_tiling}
\alias{grid.pattern_polygon_tiling}
\alias{names_polygon_tiling}
\title{Polygon tiling patterned grobs}
\usage{
grid.pattern_polygon_tiling(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  colour = gp$col \%||\% "grey20",
  fill = gp$fill \%||\% "grey80",
  angle = 30,
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  type = "square",
  alpha = gp$alpha \%||\% NA_real_,
  linetype = gp$lty \%||\% 1,
  linewidth = size \%||\% gp$lwd \%||\% 1,
  size = NULL,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_polygon_tiling
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{colour}{Stroke colour(s).}

\item{fill}{Fill colour(s) or \code{\link[grid:pattern]{grid::pattern()}} / gradient object(s).}

\item{angle}{Rotation angle in degrees.}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{type}{Name of polygon tiling to draw.  See Details.}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).
Not supported for all polygon tiling \code{type}.}

\item{linetype}{Stroke linetype.}

\item{linewidth}{Stroke linewidth.}

\item{size}{For backwards compatibility can be used to set \code{linewidth}.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_polygon_tiling()} draws a specified polygon tiling pattern onto the graphic device.
\code{names_polygon_tiling} lists all supported types.
}
\details{
\code{grid.pattern_polygon_tiling()} supports 1, 2, or 3 \code{fill} colors with the first colors (weakly)
covering a larger area.  Size of the pattern is controlled by \code{spacing}.
We support the following polygon tiling \code{type}s:

\describe{
\item{\code{elongated_triangular}}{Creates an elongated triangular tiling made of squares and triangles.}
\item{\code{herringbone}}{Creates a herringbone tiling made of rectangles.}
\item{\code{hexagonal}}{Creates a hexagonal tiling made of hexagons.}
\item{\code{pythagorean}}{Creates a Pythagorean tiling made of squares of two different sizes.}
\item{\code{rhombille}}{Creates a rhombille tiling made of rhombi.}
\item{\code{rhombitrihexagonal}}{Creates a rhombitrihexagonal tiling made out of
dodecagons, hexagons, and squares.}
\item{\code{snub_square}}{Creates a snub square tiling made of squares and triangles.}
\item{\code{snub_trihexagonal}}{Creates a snub trihexagonal tiling made of hexagons and triangles.}
\item{\code{square}}{Creates a square tiling made of squares.}
\item{\code{tetrakis_square}}{Creates a tetrakis square tiling made of isosceles right triangles.}
\item{\code{triangular}}{Creates a triangular tiling made of equilateral triangles.}
\item{\code{trihexagonal}}{Creates a trihexagonal tiling made of hexagons and triangles.}
\item{\code{truncated_square}}{Creates a truncated square tiling made of octagons and squares.}
\item{\code{truncated_hexagonal}}{Creates a truncated hexagonal tiling made of dodecagons and triangles.}
\item{\code{truncated_trihexagonal}}{Creates a truncated trihexagonal tiling made of hexagons, squares, and triangles.}
\item{\verb{2*.2**.2*.2**}}{Creates a polygon tiling made of rhombi.}
\item{\verb{2**.3**.12*}}{Creates a polygon tiling made of rhombi, triangles, and twelve-pointed stars.}
\item{\verb{3.3.3.3**}}{Creates a polygon tiling made of triangles.}
\item{\verb{3.3*.3.3**}}{Creates a regular (star) polygon tiling made of triangles and three-pointed stars.}
\item{\verb{3.3.3.12*.3.3.12*}}{Creates a regular (star) polygon tiling made of triangles and twelve-pointed stars.}
\item{\verb{3.3.8*.3.4.3.8*}}{Creates a regular (star) polygon tiling made of triangles, squares, and eight-pointed stars.}
\item{\verb{3.3.8*.4**.8*}}{Creates a regular (star) polygon tiling made of triangles, four-pointed stars, and eight-pointed stars.}
\item{\verb{3.4.6.3.12*}}{Creates a regular (star) polygon tiling made of triangles, squares, hexagons, and twelve-pointed stars.}
\item{\verb{3.4.8.3.8*}}{Creates a regular (star) polygon tiling made of triangles, squares, octagons, and eight-pointed stars.}
\item{\verb{3.6*.6**}}{Creates a regular (star) polygon tiling made of triangles and six-pointed stars.}
\item{\verb{4.2*.4.2**}}{Creates a polygon tiling made of squares and rhombi.}
\item{\verb{4.4*.4**}}{Creates a regular (star) polygon tiling made of squares and four-pointed stars.}
\item{\verb{4.6.4*.6}}{Creates a regular (star) polygon tiling made of squares, hexagons, and four-pointed stars.}
\item{\verb{4.6*.4.6*.4.6*}}{Creates a regular (star) polygon tiling made of squares and six-pointed stars.}
\item{\verb{4.8*.4**.8*}}{Creates a polygon tiling of squares and eight-pointed stars.}
\item{\verb{6.6*.6.6*}}{Creates a regular (star) polygon tiling made of hexagons and six-pointed stars.}
\item{\verb{8.4*.8.4*}}{Creates a regular (star) polygon tiling made of octagons and four-pointed stars.}
\item{\verb{9.3.9.3*}}{Creates a regular (star) polygon tiling made of triangles, nonagons, and three-pointed stars.}
\item{\verb{12.3*.12.3*}}{Creates a regular (star) polygon tiling made of dodecagons and three-pointed stars.}
\item{\verb{12.12.4*}}{Creates a regular (star) polygon tiling made of dodecagons and four-pointed stars.}
\item{\verb{18.18.3*}}{Creates a regular (star) polygon tiling made of eighteen-sided polygons and three-pointed stars.}
}
}
\examples{
print(names_polygon_tiling)

x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
gp1 <- grid::gpar(fill = "yellow", col = "black")
gp2 <- grid::gpar(fill = c("yellow", "red"), col = "black")
gp3 <- grid::gpar(fill = c("yellow", "red", "blue"), col = "black")

grid.pattern_polygon_tiling(x_hex, y_hex, type = "herringbone", gp = gp1)

grid::grid.newpage()
grid.pattern_polygon_tiling(x_hex, y_hex, type = "hexagonal",
                            spacing = 0.2, gp = gp3)

grid::grid.newpage()
grid.pattern_polygon_tiling(x_hex, y_hex, type = "pythagorean",
                            spacing = 0.2, gp = gp2)

grid::grid.newpage()
grid.pattern_polygon_tiling(x_hex, y_hex, type = "snub_trihexagonal",
                            spacing = 0.2, gp = gp3)

grid::grid.newpage()
grid.pattern_polygon_tiling(x_hex, y_hex, type = "rhombille",
                              spacing = 0.2, gp = gp3)
}
\seealso{
The tiling vignette \code{vignette("tiling", package = "gridpattern")} for more
information about these tilings as well as more
examples of polygon tiling using the \code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/z_update_alpha_docs.R
\name{update_alpha}
\alias{update_alpha}
\title{Update colour and/or pattern transparency}
\usage{
update_alpha(fill, alpha)
}
\arguments{
\item{fill}{A fill colour given as a \code{character} or \code{integer} vector, or as a
(list of) \verb{<GridPattern>} object(s) and/or colour(s).}

\item{alpha}{A transparency value between 0 (transparent) and 1 (opaque),
parallel to \code{fill}.}
}
\value{
A \code{character} vector of colours or list of \verb{<GridPattern>} objects.
}
\description{
\code{update_alpha()} modifies the transparency of colours and/or patterns.
}
\details{
\itemize{
\item This is a fork of pattern utilities mainly added to \code{{ggplot2}} by Teun van den Brand.
\item \code{update_alpha()} does not depend on \code{{ggplot2}} or \code{{scales}}.
\item Like \code{\link[ggplot2:fill_alpha]{ggplot2::fill_alpha()}} but unlike \code{\link[scales:alpha]{scales::alpha()}} it also attempts
to set the transparency of \verb{<GridPattern>} objects.
\item Unlike \code{\link[ggplot2:fill_alpha]{ggplot2::fill_alpha()}} it will work on a list of length one
containing a vector of color strings.
}
}
\section{Usage in other packages}{


To avoid taking a dependency on \code{gridpattern} you may copy the source of \code{update_alpha()}
into your own package under the permissive MIT license.  Either use
\code{usethis::use_standalone("trevorld/gridpattern", "standalone-update_alpha.R")}
or copy the file \code{update_alpha.R} into your \code{R} directory and
add \code{grDevices}, \code{grid}, and \code{rlang} to the \code{Imports} of your \code{DESCRIPTION} file.
}

\examples{
# Typical color input
update_alpha("red", 0.5)

# Pattern input
if (getRversion() >= "4.2" && requireNamespace("grid", quietly = TRUE)) {
  update_alpha(list(grid::linearGradient()), 0.5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-geometry-wave.R
\name{grid.pattern_wave}
\alias{grid.pattern_wave}
\alias{names_wave}
\title{Wave patterned grobs}
\usage{
grid.pattern_wave(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  colour = gp$col \%||\% "grey20",
  fill = gp$fill \%||\% "grey80",
  angle = 30,
  density = 0.2,
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  amplitude = 0.5 * spacing,
  frequency = 1/spacing,
  alpha = gp$alpha \%||\% NA_real_,
  linetype = gp$lty \%||\% 1,
  linewidth = size \%||\% gp$lwd \%||\% 1,
  size = NULL,
  grid = "square",
  type = "indented",
  reverse = FALSE,
  stagger = FALSE,
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)

names_wave
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{colour}{Stroke colour(s).}

\item{fill}{Fill colour(s) or \code{\link[grid:pattern]{grid::pattern()}} / gradient object(s).}

\item{angle}{Rotation angle in degrees.}

\item{density}{Approx. fraction of area the pattern fills.}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{amplitude}, \code{frequency}, \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{amplitude}{Wave amplitude (in \code{units} units)}

\item{frequency}{Linear frequency (in inverse \code{units} units)}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{linetype}{Stroke linetype.}

\item{linewidth}{Stroke linewidth.}

\item{size}{For backwards compatibility can be used to set \code{linewidth}.}

\item{grid}{Adjusts placement and density of certain graphical elements.
\code{"square"} (default) is a square grid.
\code{"hex"} is a hexagonal grid suitable for hexagonal and triangular tiling.
\code{"hex_circle"} is a hexagonal grid suitable for circle packing.
\code{"elongated_triangle"} is a grid used for the "elongated triangle" tiling.}

\item{type}{One of the following (see \code{names_wave} for the canonical list):
\describe{
\item{\code{"dovetailed"}}{A wave with diagonal strokes connecting the crests and troughs.  Alias: \code{"dovetail"}.}
\item{\code{"embattled"}}{Square wave. Alias: \code{"square"}.}
\item{\code{"embattled_grady"}}{Graduated stepped wave: two ascending steps followed by two descending steps per period.}
\item{\code{"engrailed"}}{Repeating arches curving downward (non-positive half of a sine wave per period).}
\item{\code{"indented"} (default)}{Triangular wave with equal rise and fall. Alias: \code{"triangle"}.}
\item{\code{"invected"}}{Repeating arches curving upward (non-negative half of a sine wave per period).}
\item{\code{"nebuly"}}{Smooth cloud-like wave.}
\item{\code{"potenty"}}{A stepped wave with T-shaped crenellations.}
\item{\code{"raguly"}}{Oblique stepped wave.  Use \code{reverse = TRUE} for the horizontally mirror image.}
\item{\code{"sawtoothed"}}{Sawtooth wave with a gradual rise and sharp fall.  Use \code{reverse = TRUE} for a sharp rise and gradual fall.  Aliases: \code{"sawlike"}, \code{"sawtooth"}.}
\item{\code{"urdy"}}{A wave with pointed crests and troughs.}
\item{\code{"wavy"}}{Smooth sinusoidal wave. Aliases: \code{"sine"}, \code{"undy"}.}
}}

\item{reverse}{If \code{TRUE}, horizontally mirror the wave.  Currently affects \code{"sawtoothed"} and \code{"raguly"} only.  Default \code{FALSE}.}

\item{stagger}{If \code{TRUE}, alternate wave rows are shifted by half a wavelength so that
crests of one row align with troughs of adjacent rows, creating an interlocking effect.
Default \code{FALSE}.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_wave()} draws a wave pattern onto the graphic device.
\code{names_wave} is a character vector of supported \code{type} values.
}
\examples{
print(names_wave)

# visual table of all wave types
grid::grid.newpage()
n <- length(names_wave)
nc <- 2L
nr <- ceiling(n / nc)
grid::pushViewport(grid::viewport(layout = grid::grid.layout(nr, nc)))
for (i in seq_len(n)) {
    grid::pushViewport(grid::viewport(
        layout.pos.row = (i - 1L) \%/\% nc + 1L,
        layout.pos.col = (i - 1L) \%\% nc + 1L
    ))
    grid.pattern_wave(colour = "black", fill = c("gold", "steelblue"),
                      type = names_wave[i], density = 0.18, spacing = 0.45,
                      angle = 0, amplitude = 0.100, frequency = 1 / 0.45)
    grid::grid.rect(x = 0.5, y = 0.86, width = 0.5, height = 0.28,
                    just = "centre", gp = grid::gpar(fill = "grey80", col = "black"))
    grid::grid.text(names_wave[i], x = 0.5, y = 0.88,
                    gp = grid::gpar(fontsize = 11))
    grid::grid.rect(gp = grid::gpar(fill = "transparent", col = "black", lwd = 6))
    grid::popViewport()
}
grid::popViewport()

# stagger shifts alternate rows by half a wavelength
grid::grid.newpage()
x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
grid.pattern_wave(x_hex, y_hex, colour = "black", type = "urdy",
                  fill = c("red", "blue"), density = 0.3,
                  spacing = 0.15, angle = 0,
                  amplitude = 0.045, frequency = 1 / 0.15, stagger = TRUE)
}
\seealso{
Use \code{\link[=grid.pattern_stripe]{grid.pattern_stripe()}} for straight filled bands or \code{\link[=grid.pattern_line]{grid.pattern_line()}} for stroked lines instead of waves.
See \url{https://en.wikipedia.org/wiki/Line_(heraldry)} and \url{https://en.wikipedia.org/wiki/Waveform} for more information about the supported wave types.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clippingPathGrob.R
\name{clippingPathGrob}
\alias{clippingPathGrob}
\title{Clip grob using another grob to specify the clipping path}
\usage{
clippingPathGrob(
  clippee,
  clipper,
  use_R4.1_clipping = getOption("ggpattern_use_R4.1_clipping",
    getOption("ggpattern_use_R4.1_features")),
  png_device = NULL,
  res = getOption("ggpattern_res", 72),
  name = NULL,
  gp = gpar(),
  vp = NULL
)
}
\arguments{
\item{clippee}{Grob to be clipped}

\item{clipper}{Grob that defines clipping region}

\item{use_R4.1_clipping}{If \code{TRUE} use the grid clipping path feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation.
If \code{NULL} try to guess an appropriate choice.
Note not all graphic devices support the grid clipping path feature
and the grid clipping path feature does not nest.}

\item{png_device}{\dQuote{png} graphics device to save intermediate raster data with if \code{use_R4.1_clipping} is \code{FALSE}.
If \code{NULL} and suggested package \code{ragg} is available
and versions are high enough we directly capture clipped raster via \code{\link[ragg:agg_capture]{ragg::agg_capture()}}.
Otherwise we will use \code{png_device}
(default \code{\link[ragg:agg_png]{ragg::agg_png()}} if available else \code{\link[grDevices:png]{grDevices::png()}}) and \code{\link[png:readPNG]{png::readPNG()}}
to manually compute a clipped raster.}

\item{res}{Resolution of desired \code{rasterGrob} in pixels per inch if \code{use_R4.1_clipping} is \code{FALSE}.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A \code{grid} grob
}
\description{
\code{clippingPathGrob()} clips a grob using another grob to specify the clipping path
}
\examples{
  if (capabilities("png") && require("grid")) {
    clippee <- patternGrob("circle", gp = gpar(col = "black", fill = "yellow"),
                           spacing = 0.1, density = 0.5)
    angle <- seq(2 * pi / 4, by = 2 * pi / 6, length.out = 7)
    x_hex_outer <- 0.5 + 0.5 * cos(angle)
    y_hex_outer <- 0.5 + 0.5 * sin(angle)
    x_hex_inner <- 0.5 + 0.25 * cos(rev(angle))
    y_hex_inner <- 0.5 + 0.25 * sin(rev(angle))
    clipper <- grid::pathGrob(x = c(x_hex_outer, x_hex_inner),
                              y = c(y_hex_outer, y_hex_inner),
                              id = rep(1:2, each = 7),
                              rule = "evenodd")
    clipped <- clippingPathGrob(clippee, clipper, use_R4.1_clipping = FALSE)
    grid.newpage()
    grid.draw(clipped)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/z_guess_has_R4.1_features_docs.R
\name{guess_has_R4.1_features}
\alias{guess_has_R4.1_features}
\title{Guess whether "active" graphics device supports
the grid graphics features introduced in R v4.1.}
\usage{
guess_has_R4.1_features(
  features = c("clippingPaths", "gradients", "masks", "patterns")
)
}
\arguments{
\item{features}{Character vector of features to guess support for.
Will return \code{TRUE} only if guesses support for all requested features.\describe{
\item{"clippingPaths"}{Supports clipping path feature}
\item{"gradients"}{Supports (both linear and radial) gradient feature}
\item{"masks"}{Supports (alpha) mask feature}
\item{"patterns"}{Supports (tiling) pattern feature}
}}
}
\value{
\code{TRUE} if we guess all \code{features} are supported else \code{FALSE}
}
\description{
\code{guess_has_R4.1_features()} guesses whether "active" graphics device supports
the grid graphics features introduced in R v4.1.  If it guesses it does
it returns \code{TRUE} else \code{FALSE}.
}
\section{Usage in other packages}{


To avoid taking a dependency on \code{gridpattern} you may copy the source of \code{guess_has_R4.1_features()}
into your own package under the permissive MIT No Attribution (MIT-0) license.  Either use
\code{usethis::use_standalone("trevorld/gridpattern", "standalone-guess_has_R4.1_features.R")}
or copy the file \code{standalone-guess_has_R4.1_features.R} into your \code{R} directory and
add \code{grDevices} and \code{utils} to the \code{Imports} of your \code{DESCRIPTION} file.
}

\examples{
  # If R version (weakly) greater than 4.1 should be TRUE
  pdf(tempfile(fileext = ".pdf"))
  print(guess_has_R4.1_features())
  invisible(dev.off())

  # Should be FALSE
  postscript(tempfile(fileext = ".ps"))
  print(guess_has_R4.1_features())
  invisible(dev.off())

}
\seealso{
\url{https://www.stat.auckland.ac.nz/~paul/Reports/GraphicsEngine/definitions/definitions.html} for more info about the new grid graphics
features introduced in R v4.1.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern_square.R
\name{pattern_square}
\alias{pattern_square}
\alias{names_square}
\title{Square pattern matrix}
\usage{
pattern_square(type = "diagonal", subtype = NULL, nrow = 5L, ncol = 5L)

names_square
}
\arguments{
\item{type}{Either "diagonal" (default), "diagonal_skew", "horizontal", "vertical",
or any \code{type} in \code{names_weave}.  See Details.}

\item{subtype}{See Details.  For "diagonal", "diagonal_skew", "horizontal", or "vertical"
an integer of the desired number of colors (or other graphical elements).}

\item{nrow}{Number of rows (height).}

\item{ncol}{Number of columns (width).}
}
\value{
A matrix of integer values indicating where the each color
(or other graphical element) should be drawn on a rectangular grid.
Indices \verb{[1,1]} of the matrix corresponds to the bottom-left of the grid
while indices \verb{[1,ncol]} corresponds to the bottom-right of the grid.
This matrix has a "pattern_square" subclass which supports a special \code{print()} method.
}
\description{
\code{pattern_square()} returns an integer matrix indicating where each
color (or other graphical element) should be drawn on a rectangular grid
for a specified square pattern type and subtype.
\code{names_square} lists the currently supported square \code{type}s (excluding those in \code{names_weave}).
}
\details{
\describe{
\item{"horizontal", "vertical"}{"horizontal" and "vertical" simply cycle through the colors
either horizontally or vertically.
Use \code{subtype} to indicate the (integer) number of colors (or other graphical elements).
"horizontal" will produce horizontal stripes of color whereas "vertical" will produce vertical stripes.}
\item{"diagonal", "diagonal_skew"}{"diagonal" and "diagonal_skew" simply cycle through the colors
both horizontally and vertically.
Use \code{subtype} to indicate the (integer) number of colors (or other graphical elements).
If two colors are requested this provides the standard two-color checkerboard pattern.
If there are more than three colors than "diagonal" will have colored diagonals
going from top left to bottom right while "diagonal_skew" will have them
going form bottom left to top right.}
\item{"square"}{"square" attempts a uniform coloring using "square_tiling" before falling
falling back on "diagonal".  If \code{subtype} is \code{1L}, \code{2L}, \code{3L}, or \code{4L} uses "square_tiling"
else uses "diagonal".}
\item{"square_tiling"}{"square_tiling" supports
uniform coloring for (non-staggered) square tilings.
Use \code{subtype} to either indicate the (integer) number of colors
or a string with four integers such as \code{"1231"}
(will fill in a 2x2 matrix by row which will then be tiled).
Supports up to a max of four colors.}
\item{any pattern from \code{names_weave}}{
We simply convert the logical matrix returned by \code{\link[=pattern_weave]{pattern_weave()}} into an
integer matrix by having any \code{TRUE} set to \code{1L} and \code{FALSE} set to \code{2L}.
Hence the various weave patterns only support (up to) two-color patterns.
See \code{\link[=pattern_weave]{pattern_weave()}} for more details about supported \code{type} and \code{subtype}.}
}
}
\examples{
 # supported square names
 print(names_square)

 # (main) diagonal has colors going from top left to bottom right
 diagonal <- pattern_square("diagonal", 4L, nrow = 7L, ncol = 9L)
 print(diagonal)

 # skew diagonal has colors going from bottom left to top right
 skew <- pattern_square("diagonal_skew", 4L, nrow = 7L, ncol = 9L)
 print(skew)

 horizontal <- pattern_square("horizontal", 4L, nrow = 8L, ncol = 8L)
 print(horizontal)

 vertical <- pattern_square("vertical", 4L, nrow = 8L, ncol = 8L)
 print(vertical)

 # uniform coloring using 4 colors
 color4 <- pattern_square("square_tiling", 4L, nrow = 7L, ncol = 9L)
 print(color4)

 # uniform coloring using 3 colors
 color3 <- pattern_square("square_tiling", 3L, nrow = 7L, ncol = 9L)
 print(color3)

 # also supports the various 'weave' patterns
 zigzag <- pattern_square("twill_zigzag", nrow = 15L, ncol = 9L)
 print(zigzag)

}
\seealso{
\code{\link[=grid.pattern_regular_polygon]{grid.pattern_regular_polygon()}} for drawing to a graphics device
polygons in multiple color/size/shape patterns.
\code{\link[=pattern_weave]{pattern_weave()}} for more information on "weave" patterns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pattern-both-text.R
\name{grid.pattern_text}
\alias{grid.pattern_text}
\title{Text character patterned grobs}
\usage{
grid.pattern_text(
  x = c(0, 0, 1, 1),
  y = c(1, 0, 0, 1),
  id = 1L,
  ...,
  colour = gp$col \%||\% "grey20",
  angle = 30,
  spacing = 0.05,
  xoffset = 0,
  yoffset = 0,
  units = "snpc",
  scale = 0.5,
  shape = "X",
  grid = "square",
  type = NULL,
  subtype = NULL,
  rot = 0,
  alpha = gp$alpha \%||\% NA_real_,
  size = gp$fontsize \%||\% 12,
  fontfamily = gp$fontfamily \%||\% "sans",
  fontface = gp$fontface \%||\% "plain",
  use_R4.1_masks = getOption("ggpattern_use_R4.1_masks",
    getOption("ggpattern_use_R4.1_features")),
  png_device = NULL,
  res = getOption("ggpattern_res", 72),
  default.units = "npc",
  name = NULL,
  gp = gpar(),
  draw = TRUE,
  vp = NULL
)
}
\arguments{
\item{x}{A numeric vector or unit object specifying x-locations of the pattern boundary.}

\item{y}{A numeric vector or unit object specifying y-locations of the pattern boundary.}

\item{id}{A numeric vector used to separate locations in x, y into multiple boundaries.
All locations within the same \code{id} belong to the same boundary.}

\item{...}{Currently ignored.}

\item{colour}{Stroke colour(s).}

\item{angle}{Rotation angle in degrees.}

\item{spacing}{Spacing between repetitions of pattern (in \code{units} units).}

\item{xoffset}{Shift pattern along x axis (in \code{units} units).}

\item{yoffset}{Shift pattern along y axis (in \code{units} units).}

\item{units}{\code{\link[grid:unit]{grid::unit()}} units for \code{spacing}, \code{xoffset}, and \code{yoffset} parameters.}

\item{scale}{For star polygons, multiplier (between 0 and 1)
applied to exterior radius to get interior radius.}

\item{shape}{A character or expression vector.
See \code{label} argument of \code{\link[grid:textGrob]{grid::textGrob()}} for more details.}

\item{grid}{Adjusts placement and density of certain graphical elements.
\code{"square"} (default) is a square grid.
\code{"hex"} is a hexagonal grid suitable for hexagonal and triangular tiling.
\code{"hex_circle"} is a hexagonal grid suitable for circle packing.
\code{"elongated_triangle"} is a grid used for the "elongated triangle" tiling.}

\item{type}{Adjusts the repeating of certain aesthetics such as color.
Can use any type in \code{names_hex}, \code{names_square}, or \code{names_weave}.
See for \code{\link[=pattern_hex]{pattern_hex()}}, \code{\link[=pattern_square]{pattern_square()}}, and \code{\link[=pattern_weave]{pattern_weave()}} for
more information about supported \code{type} arguments.}

\item{subtype}{See for \code{\link[=pattern_hex]{pattern_hex()}}, \code{\link[=pattern_square]{pattern_square()}}, and \code{\link[=pattern_weave]{pattern_weave()}} for
more information about supported \code{subtype} arguments.}

\item{rot}{Angle to rotate regular polygon (degrees, counter-clockwise).}

\item{alpha}{Alpha (between 0 and 1) or \code{NA} (default, preserves colors' alpha value).}

\item{size}{Fontsize}

\item{fontfamily}{The font family.  See \code{\link[grid:gpar]{grid::gpar()}} for more details.}

\item{fontface}{The font face.  See \code{\link[grid:gpar]{grid::gpar()}} for more details.}

\item{use_R4.1_masks}{If \code{TRUE} use the grid mask feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation.
If \code{NULL} try to guess an appropriate choice.
Note not all graphic devices support the grid mask feature.}

\item{png_device}{\dQuote{png} graphics device to save intermediate raster data with if \code{use_R4.1_masks} is \code{FALSE}.
If \code{NULL} and suggested package \code{ragg} is available
and versions are high enough we directly capture masked raster via \code{\link[ragg:agg_capture]{ragg::agg_capture()}}.
Otherwise we will use \code{png_device}
(default \code{\link[ragg:agg_png]{ragg::agg_png()}} if available else \code{\link[grDevices:png]{grDevices::png()}}) and \code{\link[png:readPNG]{png::readPNG()}}
to manually compute a masked raster.}

\item{res}{Resolution of desired \code{rasterGrob} in pixels per inch if \code{use_R4.1_masks} is \code{FALSE}.}

\item{default.units}{A string indicating the default units to use if \code{x} or \code{y}
are only given as numeric vectors.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{draw}{A logical value indicating whether graphics output
    should be produced.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A grid grob object invisibly.  If \code{draw} is \code{TRUE} then also draws to the graphic device as a side effect.
}
\description{
\code{grid.pattern_text()} draws a text character pattern onto the graphic device.
}
\examples{
if (capabilities("png") &&
    gridpattern:::device_supports_unicode()) {
  x_hex <- 0.5 + 0.5 * cos(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))
  y_hex <- 0.5 + 0.5 * sin(seq(2 * pi / 4, by = 2 * pi / 6, length.out = 6))

  playing_card_symbols <- c("\u2660", "\u2665", "\u2666", "\u2663")
  grid.pattern_text(x_hex, y_hex,
                   shape = playing_card_symbols,
                   colour = c("black", "red", "red", "black"),
                   size = 18, spacing = 0.1, angle = 0)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gridpattern-package.R
\docType{package}
\name{gridpattern-package}
\alias{gridpattern}
\alias{gridpattern-package}
\title{gridpattern: 'grid' Pattern Grobs}
\description{
\if{html}{\figure{logo.svg}{options: style='float: right' alt='logo' width='120'}}

Provides 'grid' grobs that fill in a user-defined area with various patterns. Includes enhanced versions of the geometric and image-based patterns originally contained in the 'ggpattern' package as well as original 'pch', 'polygon_tiling', 'regular_polygon', 'rose', 'text', 'wave', and 'weave' patterns plus support for custom user-defined patterns.
}
\section{Package options}{

The following \code{gridpattern} options may be set globally via \code{\link[base:options]{base::options()}}:
\describe{
\item{ggpattern_array_funcs}{Set custom \dQuote{array} pattern functions.}
\item{ggpattern_geometry_funcs}{Set custom \dQuote{geometry} pattern functions.}
\item{ggpattern_res}{Set custom raster image resolution (pixels per inch) for certain patterns.}
\item{ggpattern_use_R4.1_clipping}{If \code{TRUE} use the grid clipping path feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation of the clipped pattern.
If \code{NULL} try to guess an appropriate choice.}
\item{ggpattern_use_R4.1_features}{If \code{TRUE} sets the default for all the other
\verb{ggpattern_use_R4.1_*} options arguments to \code{TRUE}.
If \code{FALSE} sets them to \code{FALSE}.}
\item{ggpattern_use_R4.1_gradients}{If \code{TRUE} use the grid gradient feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation of the gradient pattern.
If \code{NULL} try to guess an appropriate choice.}
\item{ggpattern_use_R4.1_masks}{If \code{TRUE} use the grid mask feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation of the masked pattern.
If \code{NULL} try to guess an appropriate choice.}
\item{ggpattern_use_R4.1_patterns}{If \code{TRUE} use the grid pattern feature introduced in R v4.1.0.
Currently only used by a couple of examples.}
}
Note to use the R v4.1.0 features one needs R be (at least) version 4.1 and not all graphic devices
support any/all these features.  See \url{https://www.stat.auckland.ac.nz/~paul/Reports/GraphicsEngine/definitions/definitions.html} for more information on these features.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://trevorldavis.com/R/gridpattern/}
  \item \url{https://github.com/trevorld/gridpattern}
  \item Report bugs at \url{https://github.com/trevorld/gridpattern/issues}
}

}
\author{
\strong{Maintainer}: Trevor L. Davis \email{trevor.l.davis@gmail.com} (\href{https://orcid.org/0000-0001-6341-4639}{ORCID})

Authors:
\itemize{
  \item Trevor L. Davis \email{trevor.l.davis@gmail.com} (\href{https://orcid.org/0000-0001-6341-4639}{ORCID})
  \item Mike FC (Code/docs adapted from ggpattern)
}

Other contributors:
\itemize{
  \item ggplot2 authors (some utility functions copied from ggplot2) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alphaMaskGrob.R
\name{alphaMaskGrob}
\alias{alphaMaskGrob}
\title{Mask grob using another grob to specify the (alpha) mask}
\usage{
alphaMaskGrob(
  maskee,
  masker,
  use_R4.1_masks = getOption("ggpattern_use_R4.1_masks",
    getOption("ggpattern_use_R4.1_features")),
  png_device = NULL,
  res = getOption("ggpattern_res", 72),
  name = NULL,
  gp = gpar(),
  vp = NULL
)
}
\arguments{
\item{maskee}{Grob to be masked}

\item{masker}{Grob that defines masking region}

\item{use_R4.1_masks}{If \code{TRUE} use the grid mask feature introduced in R v4.1.0.
If \code{FALSE} do a \code{rasterGrob} approximation.
If \code{NULL} try to guess an appropriate choice.
Note not all graphic devices support the grid mask feature.}

\item{png_device}{\dQuote{png} graphics device to save intermediate raster data with if \code{use_R4.1_masks} is \code{FALSE}.
If \code{NULL} and suggested package \code{ragg} is available
and versions are high enough we directly capture masked raster via \code{\link[ragg:agg_capture]{ragg::agg_capture()}}.
Otherwise we will use \code{png_device}
(default \code{\link[ragg:agg_png]{ragg::agg_png()}} if available else \code{\link[grDevices:png]{grDevices::png()}}) and \code{\link[png:readPNG]{png::readPNG()}}
to manually compute a masked raster.}

\item{res}{Resolution of desired \code{rasterGrob} in pixels per inch if \code{use_R4.1_masks} is \code{FALSE}.}

\item{name}{ A character identifier. }

\item{gp}{An object of class \code{"gpar"}, typically the output
    from a call to the function \code{\link[grid]{gpar}}.  This is basically
    a list of graphical parameter settings.}

\item{vp}{A Grid viewport object (or NULL).}
}
\value{
A \code{grid} grob
}
\description{
\code{alphaMaskGrob()} masks a grob using another grob to specify the (alpha) mask.
}
\examples{
  \donttest{# May take more than 5 seconds on CRAN servers
  if (capabilities("png") && require("grid")) {
    maskee <- patternGrob("circle", gp = gpar(col = "black", fill = "yellow"),
                           spacing = 0.1, density = 0.5)
    angle <- seq(2 * pi / 4, by = 2 * pi / 6, length.out = 7)
    x_hex_outer <- 0.5 + 0.5 * cos(angle)
    y_hex_outer <- 0.5 + 0.5 * sin(angle)
    x_hex_inner <- 0.5 + 0.25 * cos(rev(angle))
    y_hex_inner <- 0.5 + 0.25 * sin(rev(angle))
    gp <- gpar(lwd = 0, col = NA, fill = "white")
    masker <- grid::pathGrob(x = c(x_hex_outer, x_hex_inner),
                             y = c(y_hex_outer, y_hex_inner),
                             id = rep(1:2, each = 7),
                             rule = "evenodd", gp = gp)
    masked <- alphaMaskGrob(maskee, masker, use_R4.1_masks = FALSE)
    grid.draw(masked)
  }
  if (capabilities("png") && require("grid")) {
    maskee_transparent <- rectGrob(gp = gpar(col = NA, fill = "blue"))
    gp <- gpar(lwd = 20, col = "black", fill = grDevices::rgb(0, 0, 0, 0.5))
    masker_transparent <- editGrob(masker, gp = gp)
    masked_transparent <- alphaMaskGrob(maskee_transparent,
                                        masker_transparent,
                                        use_R4.1_masks = FALSE)
    grid.newpage()
    grid.draw(masked_transparent)
  }
  }
}
